package edu.bu.signstream.codingScheme.ui.importLegacyCodingScheme;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.event.KeyEvent;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.*;

public class TreeDragSource implements DragSourceListener, DragGestureListener {

	DragSource source;

	DragGestureRecognizer recognizer;

	TransferableTreeNode transferable;

	DefaultMutableTreeNode oldNode;

	JTree sourceTree;


	public TreeDragSource(JTree tree, int actions) {      
      
		sourceTree = tree;
      
		source = new DragSource();
		recognizer = source.createDefaultDragGestureRecognizer(sourceTree,
				actions, this);

		sourceTree.addKeyListener(new java.awt.event.KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				int keyCode = e.getKeyCode();
				if (keyCode == 127) {
					TreePath path = sourceTree.getSelectionPath();
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) path
							.getLastPathComponent();
					((DefaultTreeModel) sourceTree.getModel())
							.removeNodeFromParent(node);
				}
			}
		});
	}
   
   public boolean isTreeEmpty() {
      DefaultTreeModel model = (DefaultTreeModel) sourceTree.getModel();
      DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
      int n = root.getChildCount();
      return n <= 0;               
   }

	/*
	 * Drag Gesture Handler
	 */
	@Override
	public void dragGestureRecognized(DragGestureEvent dge) {
		TreePath path = sourceTree.getSelectionPath();
		if ((path == null) || (path.getPathCount() <= 1)) {
			// We can't move the root node or an empty selection
			return;
		}
		oldNode = (DefaultMutableTreeNode) path.getLastPathComponent();
		transferable = new TransferableTreeNode(oldNode);
		//
		// System.out.println("Transferrable: path = " + path + ", transferable
		// = " + transferable);
		// 
		// source.startDrag(dge, DragSource.DefaultMoveNoDrop, transferable,
		// this);

		// If you support dropping the node anywhere, you should probably
		// start with a valid move cursor:
		source.startDrag(dge, DragSource.DefaultMoveDrop, transferable, this);
	}

	/*
	 * Drag Event Handlers
	 */
	@Override
	public void dragEnter(DragSourceDragEvent dsde) {
	}

	@Override
	public void dragExit(DragSourceEvent dse) {
	}

	@Override
	public void dragOver(DragSourceDragEvent dsde) {
	}

	@Override
	public void dropActionChanged(DragSourceDragEvent dsde) {
		System.out.println("Action: " + dsde.getDropAction());
		System.out.println("Target Action: " + dsde.getTargetActions());
		System.out.println("User Action: " + dsde.getUserAction());
	}

	@Override
	public void dragDropEnd(DragSourceDropEvent dsde) {
		/*
		 * to support move or copy, we have to check which occurred:
		 */

		  System.out.println("Drop Action: " + dsde.getDropAction());
      
		if (dsde.getDropSuccess()
				&& (dsde.getDropAction() == DnDConstants.ACTION_MOVE)) {
//
 //System.out.println("DropSuccess, actionMove");
//
			Object object = oldNode.getUserObject();
			DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) oldNode
					.getParent();
			Object parentObject = parentNode.getUserObject();

			if (object instanceof SS3FieldValue) {
				SS3FieldValue value = (SS3FieldValue) object;
				SS3Field field = (SS3Field) parentObject;
				field.remove(value);
			}

			if (object instanceof SS3Field) {
				SS3Field field = (SS3Field) object;
				SS3CodingScheme cScheme = (SS3CodingScheme) parentObject;
				cScheme.removeField(field);
			}

			((DefaultTreeModel) sourceTree.getModel())
					.removeNodeFromParent(oldNode);
		}

		/*
		 * to support move only... if (dsde.getDropSuccess()) {
		 * ((DefaultTreeModel)sourceTree.getModel()).removeNodeFromParent(oldNode); }
		 */
	}
   
   public SS3CodingScheme getSS3CodingScheme() {
      DefaultTreeModel model = (DefaultTreeModel) sourceTree.getModel();
      DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
      SS3CodingScheme cScheme = (SS3CodingScheme) root.getUserObject();
   
   //
   int n = root.getChildCount();
   //
      //
      System.out.println("child count = " + n + ", cScheme.fields = " + cScheme.getFields().size());
      //
      
      return cScheme;
   }
}
