package edu.bu.signstream.common.util.vo.ss3.codingScheme;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GesturePropertyCategoryObject;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeTemplates;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSPaletteItemGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteMenu;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureProperty;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureMenu;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.gestureMenus.GestureMenuGroup;
import edu.bu.signstream.grepresentation.view.LabelObject;

import java.awt.*;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

public class SS3CodingScheme {
	private String ID = null;
	private String superCodingSchemeID = null;

	private String ssVersion;
	private String dbVersion;

	private ArrayList cSchemeRef = new ArrayList();
	private ArrayList prtRef = new ArrayList();
	private ArrayList authors = new ArrayList();
	private HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
	private ArrayList<SS3Field> sortedFields = new ArrayList<SS3Field>();
	private CSHandShapeTemplates handShapeTemplates = null;
	private ArrayList<PaletteMenu> paletteMenus = new ArrayList<>();

	private ArrayList defaultSegment = new ArrayList();

	private ValueCollections vCollections;
	private Color color = Color.GRAY;

	public static char DOMINANT = 'D';
	public static char NON_DOMINANT = 'N';
	public static char FINGERSPELLING = 'L';

	private HashMap fTypes = new HashMap();
	
	private HashMap imgs = new HashMap();

	private List<Panel> panels = List.of();
    private Map<String, Panel> panelsMap = Collections.unmodifiableMap(new HashMap<>());

    private ViewMode defaultViewMode;
    private List<ViewMode> viewModes = List.of();
    private Map<String, ViewMode> viewModesMap = Collections.unmodifiableMap(new HashMap<>());

	private CSUtilityInformation utilityInfo = new CSUtilityInformation();

	private ArrayList<FieldsGroup> fieldsGroup = new ArrayList<FieldsGroup>();
	
	private HashMap<String, GestureMenu> gestureMenuHashMap;
	private HashMap<String, GestureProperty> gesturePropertyHashMap;
	private HashMap<String, GestureMenuGroup> gestureMenuGroupHashMap;

	public FieldsGroup getDependentNonmanualFieldsGroup(String fieldID) {
		for (int index = 0; index < fieldsGroup.size(); index++) {
			FieldsGroup group = fieldsGroup.get(index);
			if (fieldID.equalsIgnoreCase(group.getID()))
				return group;
		}
		return null;
	}

	public ArrayList<FieldsGroup> loadNonManualGroups() {
		fieldsGroup.clear();

		FieldsGroup headGroup = new FieldsGroup(Constants.HEAD_GROUP_NAME, Constants.HEAD_GROUP_ID);
		FieldsGroup faceGroup = new FieldsGroup(Constants.FACE_GROUP_NAME, Constants.FACE_GROUP_ID);
		FieldsGroup bodyGroup = new FieldsGroup(Constants.BODY_GROUP_NAME, Constants.BODY_GROUP_ID);
		FieldsGroup grmGroup = new FieldsGroup(Constants.GRAMMATICAL_GROUP_NAME, Constants.GRAMMATICAL_GROUP_ID);

		getSortedFields().stream().forEach(field -> {
			String group = field.getGroup();

			if (group != null && group.trim().length() >= Constants.HEAD_GROUP_ID) {
				if (group.equalsIgnoreCase(Constants.FACE_GROUP_NAME_CAPS)) {
					faceGroup.addFieldID(field.getId());
				} else if (group.equalsIgnoreCase(Constants.HEAD_GROUP_NAME_CAPS)) {
					headGroup.addFieldID(field.getId());
				} else if (group.equalsIgnoreCase(Constants.BODY_GROUP_NAME_CAPS)) {
					bodyGroup.addFieldID(field.getId());
				} else if (group.equalsIgnoreCase(Constants.GRAMMATICAL_GROUP_NAME_CAPS)) {
					grmGroup.addFieldID(field.getId());
				}
			}
		});

		fieldsGroup.add(headGroup);
		fieldsGroup.add(faceGroup);
		fieldsGroup.add(bodyGroup);
		fieldsGroup.add(grmGroup);
		return fieldsGroup;
	}

	public ArrayList<FieldsGroup> loadGlossFieldGroups() {

		ArrayList<String> fieldIDs = new ArrayList<String>();
		for (SS3Field field : getSortedFields()) {
			if (field.getGroup().equalsIgnoreCase(Constants.MANUAL_GROUP_NAME_CAPS)) {
				fieldIDs.add(field.getId());
			}
		}

		FieldsGroup manualGroup = new FieldsGroup(Constants.MANUAL_GROUP_NAME, Constants.MANUAL_GROUP_ID);
		for (String fieldID : fieldIDs) {
			manualGroup.addFieldID(fieldID);
		}

		var list = new ArrayList<FieldsGroup>();
		list.add(manualGroup);
		return list;
	}
	
	public ArrayList<FieldsGroup> loadCustomFieldGroups() {
		var speechGroup = new FieldsGroup(Constants.SPEECH_GROUP_NAME, Constants.SPEECH_GROUP_ID);
		var genericGroup = new FieldsGroup(Constants.GENERIC_GROUP_NAME, Constants.GENERIC_GROUP_ID);

		getSortedFields().stream()
            .forEach(f -> {
                if (f.getGroup().equalsIgnoreCase(Constants.SPEECH_GROUP_NAME_CAPS)) {
                    speechGroup.addFieldID(f.getId());
                } else if (f.getGroup().equalsIgnoreCase(Constants.GENERIC_GROUP_NAME_CAPS)) {
                    genericGroup.addFieldID(f.getId());
                }
            });

		var list = new ArrayList<FieldsGroup>();
		list.add(speechGroup);
		list.add(genericGroup);
		return list;
	}
	
	public ArrayList<FieldsGroup> loadGestureFieldGroups() {
		ArrayList<FieldsGroup> groups = new ArrayList<FieldsGroup>();
		
		FieldsGroup gestureGroup = new FieldsGroup(Constants.GESTURE_GROUP_NAME, Constants.GESTURE_GROUP_ID);
		FieldsGroup domGestureCategoryGroup  = new FieldsGroup(Constants.GESTURE_DOM_CATEGORY_GROUP_NAME,  Constants.GESTURE_DOM_CATEGORY_GROUP_ID);
		FieldsGroup ndomGestureCategoryGroup = new FieldsGroup(Constants.GESTURE_NDOM_CATEGORY_GROUP_NAME, Constants.GESTURE_NDOM_CATEGORY_GROUP_ID);

		ArrayList<String> fieldIDs = new ArrayList<String>();
		ArrayList<String> domGestCatFieldIDs = new ArrayList<String>();
		ArrayList<String> ndomGestCatFieldIDs = new ArrayList<String>();
		
		for (SS3Field field : getSortedFields()) {
			if (field.getGroup().equalsIgnoreCase(Constants.GESTURE_GROUP_NAME_CAPS)) {
				fieldIDs.add(field.getId());
			} else if (field.getGroup().equalsIgnoreCase(Constants.GESTURE_DOM_CATEGORY_GROUP_NAME_CAPS)) {
				domGestCatFieldIDs.add(field.getId());
			} else if (field.getGroup().equalsIgnoreCase(Constants.GESTURE_NDOM_CATEGORY_GROUP_NAME_CAPS)) {
				ndomGestCatFieldIDs.add(field.getId());
			}
		}

		for (String fieldID : fieldIDs) {
			gestureGroup.addFieldID(fieldID);
		}

		for (String fieldID : domGestCatFieldIDs) {
			domGestureCategoryGroup.addFieldID(fieldID);
		}

		for (String fieldID : ndomGestCatFieldIDs) {
			ndomGestureCategoryGroup.addFieldID(fieldID);
		}
		
		groups.add(gestureGroup);
		groups.add(domGestureCategoryGroup);
		groups.add(ndomGestureCategoryGroup);

		return groups;
	}
	
	public FieldsGroup loadGphFieldGroup() {
		FieldsGroup graphGroup = new FieldsGroup("Graph", 0);

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			SS3Field field = fields.get(key);
			String group = field.getGroup();

			if (group != null && group.trim().length() >= 1) {
				if (group.equalsIgnoreCase(Constants.TYPE_GRAPH)) {
					graphGroup.addFieldID(field.getId());
				}
			}
		}

		return graphGroup;
	}

	// clones only field objects. Use for importing legacy coding scheme only!
	public SS3CodingScheme shallowCloneCodingScheme() {
		SS3CodingScheme cs = new SS3CodingScheme();
		cs.setID("" + Util.getLongUniqueNumber());
		cs.setSuperCodingSchemeID(getID());

		Set set = fields.keySet();
		Iterator it = set.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object object = fields.get(key);
			SS3Field field = (SS3Field) object;
			SS3Field nField = field.cloneField();
			cs.addField(nField);
		}

		return cs;
	}

	/***
	 * @items - list of CSHandShapeItem objects
	 * @returns list of SS3FieldValue objects
	 */
	public ArrayList<SS3FieldValue> getCorrespondingFieldValues(
			ArrayList<CSPaletteItemGroup> itemGroups, String fieldType) {

		String fieldID = getFieldID(fieldType);
		SS3Field field = getField(fieldID);

		HashMap<String, SS3FieldValue> values = field.getValues();
		ArrayList<SS3FieldValue> selectedValues = new ArrayList<>();

		for (CSPaletteItemGroup itemGroup : itemGroups) {
			for (CSHandShapeItem item : itemGroup.getItems()) {
				String itemID = item.getId();
				SS3FieldValue value = values.get(itemID);

				if (value != null) {
					value.setCSHandShapeItem(item);
					selectedValues.add(value);
				}
			}

		}

		return selectedValues;
	}

	// items - an collection of HandShapeImgItem objects
	public ArrayList getCorrespondingImages(ArrayList items) {
		ArrayList images = new ArrayList();

		for (int i = 0; i < items.size(); i++) {
			HandShapeImgItem item = (HandShapeImgItem) items.get(i);
			SS3FieldTypeImg image = getImage(item.getId());
			images.add(image);
		}

		return images;
	}

	public String getSignStreamVersion() {
		return ssVersion;
	}

	public void setSignStreamVersion(String ssVersion) {
		this.ssVersion = ssVersion;
	}

	public String getDbVersion() {
		return dbVersion;
	}

	public void setDbVersion(String dbVersion) {
		this.dbVersion = dbVersion;
	}

	public void setValueCollections(ValueCollections vCollections) {
		this.vCollections = vCollections;
	}

	public ValueCollections getValueCollections() {
		return vCollections;
	}

	public ValueCollection getValueCollection(String cID) {
		return vCollections.getValueCollection(cID);
	}

	public SS3FieldValue getValueCollectionSS3FieldValue(String cID, String vID) {
		ValueCollection vCol = vCollections.getValueCollection(cID);
		return vCol.getValue(vID);
	}

	public HashMap<String, SS3Field> getDependentRootFields() {
		HashMap<String, SS3Field> ht = new HashMap<String, SS3Field>();
		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			SS3FieldType fType = getFieldType(type);
			String indType = "D";

			if (fType != null && indType.equalsIgnoreCase(fType.getDepType()))
				ht.put(field.getId(), field);
		}
		return ht;
	}

	public HashMap getIndependentFields() {
		HashMap ht = new HashMap();
		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			SS3FieldType fType = getFieldType(type);
			String indType = "I";

			if (fType != null && indType.equalsIgnoreCase(fType.getDepType()))
				ht.put(field.getId(), field);
		}
		return ht;
	}

	public ArrayList<SS3Field> getFields(String dType) {
		ArrayList<SS3Field> list = new ArrayList<SS3Field>();

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			if (type.equalsIgnoreCase(dType))
				list.add(field);
		}
		return list;
	}

	public ArrayList<SS3Field> getFields(String... dTypes) {
		ArrayList<SS3Field> results = new ArrayList<SS3Field>();
		for (String dType : dTypes) {
			results.addAll(getFields(dType));
		}
		return results;
	}

	public String getLeftHandFieldID() {
		return getFieldID(Constants.TYPE_NDOM_HAND);
	}

	public String getRightHandFieldID() {
		return getFieldID(Constants.TYPE_DOM_HAND);
	}

	public String getLeftHandShapeFieldID() {
		return getFieldID(Constants.TYPE_NDOM_HAND_SHAPE);
	}

	public String getRightHandShapeFieldID() {
		return getFieldID(Constants.TYPE_DOM_HAND_SHAPE);
	}

	public String getFieldID(String cType) {
		Set<String> keys = fields.keySet();

		for (String key : keys) {

			SS3Field field = fields.get(key);
			String type = field.getType();

			if (type.equalsIgnoreCase(cType)) {
				return field.getId();
			}
		}
		return null;
	}

	public ArrayList<SS3Field> getDependentFields(String fieldID) {
		ArrayList<SS3Field> list = new ArrayList<SS3Field>();

		SS3Field field = getField(fieldID);
		String type = field.getType();

		SS3FieldType fType = (SS3FieldType) fTypes.get(type);
		ArrayList<SS3FieldType> types = fType.containsFields();

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Object obj = fields.get(key);

			SS3Field fld = (SS3Field) obj;
			String t = fld.getType();

			for (int j = 0; j < types.size(); j++) {
				SS3FieldType ft = types.get(j);
				String tp = ft.getType();

				if (tp.equalsIgnoreCase(t))
					list.add(fld);
			}
		}
		return list;
	}

	public void addImage(SS3FieldTypeImg img) {
		imgs.put(img.getId(), img);
	}

	public SS3FieldTypeImg getImage(String id) {
		return (SS3FieldTypeImg) imgs.get(id);
	}

	public void setPanels(List<Panel> panels) {
	    this.panels = Collections.unmodifiableList(panels);
	    this.panelsMap = panels.stream()
            .collect(Collectors.toUnmodifiableMap(p -> p.id, p -> p));
	}
	public List<Panel> panels() { return panels; }
	public Panel panel(String id) { return panelsMap.get(id); }

	public ViewMode getDefaultViewMode() { return defaultViewMode; }
	public void setDefaultViewMode(ViewMode mode) { this.defaultViewMode = mode; }

	public void setViewModes(List<ViewMode> modes) {
	    this.viewModes = Collections.unmodifiableList(modes);
	    this.viewModesMap = modes.stream()
            .collect(Collectors.toUnmodifiableMap(ViewMode::getId, m -> m));
    }
    public List<ViewMode> viewModes() { return viewModes; }
    public ViewMode viewMode(String id) { return viewModesMap.get(id); }

	public SS3Field getField(String fieldID) {
		return fields.get(fieldID);
	}

	/**
	 * GD 2023-04-19 going forward use this in favor of SS3Field.getValuesAsList
	 * @param fieldID
	 * @return
	 */
	public HashMap<String, SS3FieldValue> getFieldValuesAsMap(String fieldID) {
		SS3Field field = fields.get(fieldID);

		HashMap<String, SS3FieldValue> fieldValues = new HashMap<String, SS3FieldValue>();
        
        if (field != null) {
        	for (SS3FieldValue val: field.getValuesAsList()) {
        		fieldValues.put(val.getId(), val);
        	}
        } else {
        	return null;
        }
        
        return fieldValues;
	}
	
	public SS3FieldValue getFieldsMapSS3FieldValue(String fieldID, String valueID) {
		SS3Field field = getField(fieldID);
		return field.getFieldValue(valueID); 
	}

	public SS3FieldType getFieldType(String id) {
		return (SS3FieldType) fTypes.get(id);
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(String clr) {
		this.color = Util.hexString2Color(clr);
	}

	public ArrayList getFieldImg(String fieldID) {
		SS3Field field = getField(fieldID);
		return field.getValuesAsList();
	}

	public void addFieldType(SS3FieldType type) {
		fTypes.put(type.getType(), type);
	}
	
	public int getMaxOrderAcrossAllFields() {
		int maxOrder = -1;
		
		if (fields.size() > 0) {
			for (SS3Field field: fields.values()) {
				int order = field.getOrder();
				if (order > maxOrder)
					maxOrder = order;
			}
		}
		
		return maxOrder;
	}
	
	public void addFieldAndOrder(SS3Field field) {
		fields.put(field.getId(), field);
		
		// figure the proper ordering of the newly added field relative to others by group
		ArrayList<FieldsGroup> groups = new ArrayList<FieldsGroup>();
		String groupStr = field.getGroup();
		if (groupStr.equals(Constants.HEAD_GROUP_NAME_CAPS) || groupStr.equals(Constants.FACE_GROUP_NAME_CAPS) 
				|| groupStr.equals(Constants.BODY_GROUP_NAME_CAPS) || groupStr.equals(Constants.GRAMMATICAL_GROUP_NAME_CAPS)) {
			 groups = loadNonManualGroups();
		} else if (groupStr.equals(Constants.SPEECH_GROUP_NAME_CAPS) || groupStr.equals(Constants.GENERIC_GROUP_NAME_CAPS)) {
			groups = loadNonManualGroups();
			groups.addAll(loadCustomFieldGroups());
		} else {
			return;		// ignore gloss/gesture for now
		}
		
		int highestOrder = 1;
		SS3Field fieldWithHighestOrder = null;
		for (FieldsGroup group: groups) {
			for (String fieldID: group.getFieldIDs()) {
				SS3Field currentField = fields.get(fieldID);
				int currentOrder = currentField.getOrder();
				if (currentOrder > highestOrder) {
					highestOrder = currentOrder;
					fieldWithHighestOrder = currentField;
				}
			}
			
			if (group.getGroupName().toUpperCase().equals(String.valueOf(groupStr))) {
				break;
			}
		}
		
		if (fieldWithHighestOrder == null)
			return;
		
		for (SS3Field currentField: fields.values()) {
			int currentOrder = currentField.getOrder();
			if (currentOrder > highestOrder) {
				currentField.setOrder(currentOrder + 1);
			}
		}
		field.setOrder(highestOrder + 1);
		
		resortFields();
	}
	
	public void addField(SS3Field field) {
		fields.put(field.getId(), field);
	}

	public void removeField(SS3Field field) {
		fields.remove(field.getId());
		
		int order = field.getOrder();
		for (SS3Field currentField: fields.values()) {
			int currentOrder = currentField.getOrder();
			if (currentOrder > order) {
				currentField.setOrder(currentOrder - 1);
			}			
		}
		
		resortFields();
	}
	
	public boolean moveFieldOrderTo(SS3Field fromField, SS3Field toField) {
		int fromIndex = -1, toIndex = -1;
		for (int i = 0; i < sortedFields.size(); i++) {
			SS3Field field = sortedFields.get(i);
			String fieldID = field.getId();
			
			if (fieldID.equals(fromField.getId()))
				fromIndex = i;
			else if (fieldID.equals(toField.getId()))
				toIndex = i;
		}
		
		if (fromIndex == -1 || toIndex == -1)
			return false;
		
		SS3Field removedField = sortedFields.remove(fromIndex);
		sortedFields.add(toIndex, removedField);
		
		int beginIndex = fromIndex, endIndex = toIndex;
		if (toIndex < fromIndex) {
			beginIndex = toIndex;
			endIndex = fromIndex;
		}
		
		plusOneBetweenSortedIndices(beginIndex, endIndex);
		
		resortFields();
		
		return true;
	}
	
	private void plusOneBetweenSortedIndices(int beginIndex, int endIndex) {
		for (int i = beginIndex; i <= endIndex; i++) {
			SS3Field field = sortedFields.get(i);
			int order;
			
			if (i == 0) {
				order = 1;
			} else {
				SS3Field prevField = sortedFields.get(i - 1);
				order = prevField.getOrder() + 1;
			}
			
			field.setOrder(order);
		}
	}
	
	private void resortFields() {
		sortedFields = (ArrayList<SS3Field>) fields.values().stream().sorted().collect(Collectors.toList());
	}

    public void sortLabelObjectList(List<LabelObject> orderList) {
    	ArrayList<SS3Field> fieldOrderList = getSortedFields();
    	
        Collections.sort(orderList, new Comparator<LabelObject>() {
            @Override
            public int compare(LabelObject o1, LabelObject o2) {
            	SS3Field field1 = getField(o1.getFieldID());
            	SS3Field field2 = getField(o2.getFieldID());
            	
                int idx1 = fieldOrderList.indexOf(field1);
                if (idx1 == -1) {
                    idx1 = Integer.MAX_VALUE;
                }
                int idx2 = fieldOrderList.indexOf(field2);
                if (idx2 == -1) {
                    idx2 = Integer.MAX_VALUE;
                }
                return idx1 - idx2; // when sorting, compare two objects by their positions in the sorted ordering
            }
        });
    }

    public void sortFieldWrapperList(List<FieldWrapper> orderList) {
    	ArrayList<SS3Field> fieldOrderList = getSortedFields();
    	
        Collections.sort(orderList, new Comparator<FieldWrapper>() {
            @Override
            public int compare(FieldWrapper o1, FieldWrapper o2) {
            	SS3Field field1 = getField(o1.getField().getFieldID());
            	SS3Field field2 = getField(o2.getField().getFieldID());
            	
                int idx1 = fieldOrderList.indexOf(field1);
                if (idx1 == -1) {
                    idx1 = Integer.MAX_VALUE;
                }
                int idx2 = fieldOrderList.indexOf(field2);
                if (idx2 == -1) {
                    idx2 = Integer.MAX_VALUE;
                }
                return idx1 - idx2; // when sorting, compare two objects by their positions in the sorted ordering
            }
        });
    }

	public ArrayList getAuthors() {
		return authors;
	}

	public void setAuthors(ArrayList list) {
		authors = list;
	}

	public HashMap<String, SS3Field> getFields() {
		return fields;
	}
	
	public ArrayList<SS3Field> getSortedFields() {
		if (sortedFields.isEmpty())
			resortFields();
		
		return sortedFields;
	}

	public void setFields(HashMap map) {
		fields = map;
	}

	public ArrayList getParticipantRef() {
		return prtRef;
	}

	public void setParticipantRef(ArrayList list) {
		prtRef = list;
	}

	public ArrayList getCodingSchemeRef() {
		return cSchemeRef;
	}

	public void setCodingSchemeRef(ArrayList list) {
		cSchemeRef = list;
	}

	@Override
	public String toString() {
		String name = utilityInfo.getCodingSchemeName();
		if (name == null)
			name = "Coding Schema";

		return name;
	}

	public ArrayList getDefaultSegment() {
		return defaultSegment;
	}

	public void setDefaultSegment(ArrayList list) {
		defaultSegment = list;
	}

	public void addFieldIDToDfltSegment(String fieldID) {
		defaultSegment.add(fieldID);
	}

	public CSUtilityInformation getCSUtilityInformation() {
		return utilityInfo;
	}

	public String getID() {
		return ID;
	}

	public void setID(String id) {
		ID = id;
	}

	public String getSuperCodingSchemeID() {
		return superCodingSchemeID;
	}

	public void setSuperCodingSchemeID(String superCodingSchemeID) {
		this.superCodingSchemeID = superCodingSchemeID;
	}

	/**
	 * @return
	 */
	public CSHandShapeTemplates getHandShapeTemplates() {
		return this.handShapeTemplates;
	}

	public ArrayList<PaletteMenu> getPaletteMenus() {
		return this.paletteMenus;
	}

	/**
	 * @param templates
	 */
	public void setHandShapeTemplates(CSHandShapeTemplates templates) {
		this.handShapeTemplates = templates;
	}

	public void setPaletteMenus(ArrayList<PaletteMenu> menus) {
		this.paletteMenus = menus;
	}


	public void setGestureMenus(HashMap<String, GestureMenu> gestureMenus) {
		this.gestureMenuHashMap = gestureMenus;
	}

	public void setGestureProperties(HashMap<String, GestureProperty> gestureProperties) {
		this.gesturePropertyHashMap = gestureProperties;
	}

	public void setGestureMenuGroups(HashMap<String, GestureMenuGroup> gestureMenuGroups) {
		this.gestureMenuGroupHashMap = gestureMenuGroups;
	}

	public HashMap<String, GestureMenuGroup> getGestureMenuGroupHashMap() {
		return this.gestureMenuGroupHashMap;
	}

	public HashMap<String, GestureProperty> getGesturePropertyHashMap() {
		return this.gesturePropertyHashMap;
	}

	public HashMap<String, GestureMenu> getGestureMenuHashMap() {
		return this.gestureMenuHashMap;
	}
}
