package edu.bu.signstream.common.util.vo.ss3.codingScheme;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;

import java.util.Collections;
import java.util.Set;

public class ViewMode {
    private final String id;
    private final String label;
    private final Set<Panel> panels;

    public ViewMode(String id, String label, Set<Panel> panels) {
        this.id = id;
        this.label = label;
        this.panels = Collections.unmodifiableSet(panels);
    }

    public ViewMode(Set<Panel> panels) {
        this.id = Util.uuid();
        this.label = id;
        this.panels = panels;
    }


    public boolean matches(ViewMode o) {
        return panels.containsAll(o.panels)
            && o.panels.containsAll(panels);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof ViewMode && ((ViewMode)obj).id.equals(id);
    }

    @Override
    public int hashCode() {
        return id.hashCode();
    }

    public String getId() {
        return this.id;
    }

    public String getLabel() {
        return this.label;
    }

    public Set<Panel> getPanels() {
        return this.panels;
    }

    public boolean containsPanel(String panelID) {
        for (Panel panel : panels) {
            if (panel.id.equals(panelID)) {
                return true;
            }
        }

        return false;
    }
    
    @Override
	public String toString() {
    	return "ViewMode, ID: " + id + ", Label: " + label + ", Panels: " + panels;
    }


}
