package edu.bu.signstream.grepresentation.fields.apexField;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.*;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Bound;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.popup.FieldValueSelectPopup;

public class ApexFieldWrapper extends FieldWrapper {
	protected final ApexField nmField;
	private ApexDblRightClickedDelegate rDblClickDelegate;
	private Point lastPoint = null;

	public ApexFieldWrapper(ApexField nmField, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(nmField, segmentPanel, timeRange);
		this.nmField = nmField;
		rDblClickDelegate = new ApexDblRightClickedDelegate(nmField);
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public void unselectAll() {
		nmField.unselect();
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	// @TODO: implement
	@Override
	public void addEventsEntity(EventsEntity entity) {
		nmField.addChainedApexEventEntity((ChainedApexEventsEntity) entity);
	}

	// select event or entity located at given point
	@Override
	public void mouseClicked(Point point, JPanel panel) {

		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		EventsEntity entity = nmField.getEntityAt(point.x);
		Event event = nmField.getEventAt(point.x);
		if (event != null) {
			event.select();
			mediaTb.selectEvent(event, entity, nmField);
		}
	}

	// not functional
	@Override
	public void mouseRightClicked(Point point) {
	}

	// not functional
	@Override
	public void mouseLeftClicked(Point point) {
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	// add new chained event
	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
	    // Apexes should only be created by right clicking on a gesture
		if (true) return;
		PresentationField parentField = nmField.getParentField();
		Event parentEvent = parentField.getEventAt(point.x);

        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

        // if outside the bounds of any utterance, do not proceed
		if (parentEvent == null) return;

		NewChainedApexEvent newEvent = new NewChainedApexEvent(time, segmentPanel);
		int numberOfEntities = nmField.getEntities().size();

		if (numberOfEntities == 0) {
			nmField.setNewChainedApexEvent(newEvent);
		} else {
			EventsEntity nextEntity = nmField.getNextEntity(time);
			EventsEntity prevEntity = nmField.getPreviousEntity(time);

			if (prevEntity != null && prevEntity.getEndTimeInfo().greaterThan(time)) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}

			newEvent.setNextEntity(nextEntity);
			newEvent.setPreviousEntity(prevEntity);

			nmField.setNewChainedApexEvent(newEvent);
		}

		SS3Singleton.getMediaPlayer().seek(time);
	}

	// change field ID
	@Override
	public void mouseDblLeftClicked(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
			final String fieldID = field.getFieldID();

			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

			SS3Field field = dbScheme.getField(fieldID);
			final ArrayList<SS3FieldValue> values = field.getValuesAsList();
			final JPopupMenu popup = new JPopupMenu();
			final ChainedApexEvent cEvent = (ChainedApexEvent) event;

			ChainedApexEventsEntity entity = (ChainedApexEventsEntity) SS3Singleton.getMediaToolBar(null)
					.selectedEntity();

			if (cEvent.equals(entity.getValue())) {
				for (int i = 0; i < values.size(); i++) {
					final SS3FieldValue value = values.get(i);

					final JLabel label = new JLabel(value.getName());
					label.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseClicked(MouseEvent arg0) {
							if (cEvent != null) {
								String id = value.getId();
								cEvent.setID(id);
								SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
								panel.repaint();
							}
							popup.setVisible(false);
						}

						@Override
						public void mouseEntered(MouseEvent arg0) {
							label.setForeground(Color.RED);
						}

						@Override
						public void mouseExited(MouseEvent arg0) {
							label.setForeground(Color.BLACK);
						}
					});

					popup.add(label);
				}

				/*final JLabel addNew = new JLabel("Add new value");
				addNew.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent arg0) {
						if (cEvent != null) {
							popup.setVisible(false);

							final JPopupMenu popup1 = new JPopupMenu();
							NewCSValuePanel valuePanel = new NewCSValuePanel(fieldID, cEvent,
									nmField.getSignStreamSegmentPanel(), popup1);

							popup1.add(valuePanel);
							Dimension d = popup1.getPreferredSize();
							int width = (int) d.getWidth();
							int x = (int) point.getX() - width / 2;
							int y = (int) point.getY();

							popup1.show(panel, x, y);
							popup1.setVisible(true);

							// SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup1);

							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
							panel.repaint();
						}
					}

					@Override
					public void mouseEntered(MouseEvent arg0) {
						addNew.setForeground(Color.RED);
					}

					@Override
					public void mouseExited(MouseEvent arg0) {
						addNew.setForeground(Color.BLACK);
					}
				});

				popup.add(addNew);*/
				popup.setPreferredSize(popup.getPreferredSize());
				popup.show(panel, (int) point.getX(), (int) point.getY());
				popup.setVisible(true);

				SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
			}
		}
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
	}

	// add new event (hold. onset, offset) to selected entity
	public void mouseCtrlAltClicked(Point point, JPanel panel) {
		EventsEntity ent = nmField.getEntityAt(point.x);

		if (ent == null)
			return;

		ApexChainedApexEventsEntity entity = (ApexChainedApexEventsEntity) ent;

		if (isEntitySelected()) {
			rDblClickDelegate.entityDblClick(entity, point, panel);
		} else {
			Event event = nmField.getEventAt(point.x);
			ChainedApexEvent cEvent = (ChainedApexEvent) event;
			rDblClickDelegate.dblClick(cEvent, entity, point, panel);
		}
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		panel.setCursor(new Cursor(getCursorType(point)));

		Event event = nmField.getEventAt(point.x);
		if (event != null) {
			panel.setToolTipText(event.getToolTipText());
		} else {
			panel.setToolTipText("");
		}
	}

	public int getCursorType(Point point) {
		/* Below Commented-out code changes the default mouse to Resizeable Mouse Cursor Icon.
		 * It has been been commented out since the Apex field is no longer modifiable through mouse drag, making this utility redundant  
		 */
//		Event event = nmField.getSelectedEvent();
//
//		if (event != null) {
//			double startCoord = event.getStartTimeCoordinate();
//			double endCoord = event.getEndTimeCoordinate();
//
//			if (startCoord == endCoord) {
//				if (startCoord - 3 <= point.x && point.x <= startCoord - 1) {
//					MOVE_DIRECTION = MOVE_START_POINT;
//					return Cursor.E_RESIZE_CURSOR;
//				} else if (startCoord + 1 <= point.x && point.x <= startCoord + 3) {
//					MOVE_DIRECTION = MOVE_END_POINT;
//					return Cursor.E_RESIZE_CURSOR;
//				} else {
//					return Cursor.DEFAULT_CURSOR;
//				}
//			} else {
//				if (startCoord - 3 < point.x && point.x < startCoord + 3) {
//					MOVE_DIRECTION = MOVE_START_POINT;
//					return Cursor.E_RESIZE_CURSOR;
//				} else if (endCoord - 3 < point.x && point.x < endCoord + 3) {
//					MOVE_DIRECTION = MOVE_END_POINT;
//					return Cursor.E_RESIZE_CURSOR;
//				} else {
//					return Cursor.DEFAULT_CURSOR;
//				}
//			}
//		}

		return Cursor.DEFAULT_CURSOR;
	}

    private boolean drawNewEvent(Time time) {
        var newEvent = nmField.getNewChainedApexEvent();
        if (newEvent != null) {
            EventsEntity nextEntity = newEvent.getNextEntity();
            EventsEntity prevEntity = newEvent.getPreviousEntity();

            if (prevEntity != null) {
                var prevEnd = prevEntity.getEndTimeInfo();
                if (time.lessThan(prevEnd))
                    time = prevEnd;
            }

            if (nextEntity != null) {
                var nextStart = nextEntity.getStartTimeInfo();
                if (time.greaterThan(nextStart))
                    time = nextStart;
            }

            newEvent.setEndTimeInfo(time);

            SS3Singleton.getMediaPlayer().seek(time);

            return true;
        }
        return false;
    }

	@Override
	public void mouseDragged(Point point, JPanel panel) {
		if (lastPoint != null && lastPoint.x == point.x)
			return;

		PresentationField parentField = nmField.getParentField();
		Event parentEvent = parentField.getEventAt(point.x);

        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

		// if outside the bounds of any utterance, do not proceed
		if (parentEvent == null) return;

		// if outside the bounds of the starting utterance, do not proceed
		if (lastPoint != null) {
			Event lastParentEvent = parentField.getEventAt(lastPoint.x);

			if (lastParentEvent != null && lastParentEvent != parentEvent)
				return;
		}

		lastPoint = point;

		boolean drawNewEvent = drawNewEvent(time);

		if (!drawNewEvent) {
			/*
			 * Below Commented out code adjusts the Apex field start & end values via mouse drag.
			 * Disabled due to issues when Apex field is selected and dragged beyond parent gesture event.
			 * See Gitlab Issue #372 for more details. 
			 */
//			ChainedApexEventsEntity entity = (ChainedApexEventsEntity) nmField.getSelectedEntity();
//			if (entity != null) {
//
//                if (MOVE_DIRECTION == MOVE_START_POINT) {
////                    if (parentEvent.getStartTimeInfo().greaterThan(time)) {
////                        SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
////                        return;
////                    }
//                	String resp = entity.setSelectedEventStartMovieTime(time);
//
//                    if (resp != null) {
//                        SS3Singleton.getErrorMessages().showErrorMessage(resp);
//                        return;
//                    }
//                }
//
//                if (MOVE_DIRECTION == MOVE_END_POINT) {
////                    if (parentEvent.getEndTimeInfo().lessThan(time)) {
////                        SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
////                        return;
////                    }
//
//                    String resp = entity.setSelectedEventEndMovieTime(time);
//
//                    if (resp != null) {
//                        SS3Singleton.getErrorMessages().showErrorMessage(resp);
//                        return;
//                    }
//                }
//			}
			SS3Singleton.getMediaPlayer().seek(time);
		}
	}

	@Override
	public void mouseReleased(final Point point, final JPanel panel) {
		// clear the lastPoint since the user has finished dragging the mouse
		lastPoint = null;

		NewChainedApexEvent newEvent = nmField.getNewChainedApexEvent();
		if (newEvent != null) {

			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

			Time start = newEvent.getStartTimeInfo();
			Time end = newEvent.getEndTimeInfo();

			if (end.lessThan(start)) {
				Time temp = end;
				end = start;
				start = temp;
			}

            var utterance = nmField.getParentField().getEventAt(point.x);
			if (start.equals(end) || !utterance.interval().contains(start.to(end), Bound.INCLUSIVE)) {
                nmField.setNewChainedApexEvent(null);
                return;
            }

			var event = new ChainedApexEvent(segmentPanel);
            event.setStartTimeInfo(start);
			event.setEndTimeInfo(end);
            event.select();

            var collection = new ChainedApexEventsCollection();
            collection.setValueEvent(event);

            var popup = new FieldValueSelectPopup(field, segmentPanel, id -> {
                event.setID(id);

                ChainedApexEventsEntity entity = new ChainedApexEventsEntity(collection, segmentPanel, nmField);
                entity.updateParentEntity();

                nmField.addChainedApexEventEntity(entity);
                SS3Singleton.getMediaToolBar(null).selectEvent(event, entity, nmField);

                SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
                nmField.setNewChainedApexEvent(null);
                segmentPanel.repaint();
            }, () -> {
                nmField.setNewChainedApexEvent(null);
                segmentPanel.repaint();
            });

            var loc = new Point((int)event.getEndTimeInfo().as(TimeUnit.COORDINATE, Rounding.NEAREST), field.getY() - 11);
            popup.display(panel, loc);
		}
	}

	@Override
	public void mouseHoldReleased(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
			final String fieldID = field.getFieldID();

			SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();

			SS3Field field = dbScheme.getField(fieldID);
			final ArrayList<SS3FieldValue> values = field.getValuesAsList();
			final JPopupMenu popup = new JPopupMenu();
			final ChainedApexEvent cEvent = (ChainedApexEvent) event;

			ChainedApexEventsEntity entity = (ChainedApexEventsEntity) SS3Singleton.getMediaToolBar(null)
					.selectedEntity();

			if (cEvent.equals(entity.getValue())) {
				for (int i = 0; i < values.size(); i++) {
					final SS3FieldValue value = values.get(i);

					final JLabel label = new JLabel(value.getName());
					label.addMouseListener(new MouseAdapter() {
						@Override
						public void mouseClicked(MouseEvent arg0) {
							if (cEvent != null) {
								String id = value.getId();
								cEvent.setID(id);
								SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
								panel.repaint();
							}
							popup.setVisible(false);
						}

						@Override
						public void mouseEntered(MouseEvent arg0) {
							label.setForeground(Color.RED);
						}

						@Override
						public void mouseExited(MouseEvent arg0) {
							label.setForeground(Color.BLACK);
						}
					});

					popup.add(label);
				}

				/*final JLabel addNew = new JLabel("Add new value");
				addNew.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent arg0) {
						if (cEvent != null) {
							popup.setVisible(false);

							final JPopupMenu popup1 = new JPopupMenu();
							NewCSValuePanel valuePanel = new NewCSValuePanel(fieldID, cEvent,
									nmField.getSignStreamSegmentPanel(), popup1);

							popup1.add(valuePanel);
							Dimension d = popup1.getPreferredSize();
							int width = (int) d.getWidth();
							int x = (int) point.getX() - width / 2;
							int y = (int) point.getY();

							popup1.show(panel, x, y);
							popup1.setVisible(true);

							SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup1);

							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
							panel.repaint();
						}
					}

					@Override
					public void mouseEntered(MouseEvent arg0) {
						addNew.setForeground(Color.RED);
					}

					@Override
					public void mouseExited(MouseEvent arg0) {
						addNew.setForeground(Color.BLACK);
					}
				});

				popup.add(addNew);*/
				popup.setPreferredSize(popup.getPreferredSize());
				popup.show(panel, (int) point.getX(), (int) point.getY());
				popup.setVisible(true);

				SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
			}
		}
	}
}
