package edu.bu.signstream.grepresentation.fields.apexField;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.UtteranceView;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;

import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

public class ApexFreeTextFieldWrapper extends ApexFieldWrapper {

    public ApexFreeTextFieldWrapper(ApexField field, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
        super(field, segmentPanel, timeRange);
    }

    @Override
    public void mousePressedAlt(final Point point, JPanel panel) {
        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR);

        EventsEntity entityAt = nmField.getEntityAt(point.x);
        if (entityAt != null) {
            return;
        }

        final JPopupMenu popup = new JPopupMenu();

        int width = 250;
        int height = 17;

        final JTextField labelTextField = new JTextField();

        labelTextField.setPreferredSize(new Dimension(width, height));
        labelTextField.setBounds(new Rectangle(0, 0, 0, 0));
        labelTextField.setBackground(UIPalette.getBackground());
        labelTextField.setForeground(Color.black);
        labelTextField.setSize(5, 10);
        popup.setBounds(0, 0, 0, 0);
        popup.setBorder(BorderFactory.createEmptyBorder());
        popup.setBackground(UIPalette.getBackground());
        popup.add(labelTextField);

        popup.addPopupMenuListener(new PopupMenuListener() {
            @Override
            public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
            }

            @Override
            public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
                onExit();
            }

            @Override
            public void popupMenuCanceled(PopupMenuEvent e) {
                onExit();
            }

            public void onExit() {
                handleNewFreeTextEvent(labelTextField.getText(), time);
            }
        });

        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                labelTextField.requestFocus();
            }
        });

        labelTextField.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(KeyEvent e) {
                if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                    popup.setVisible(false);
                    segmentPanel.repaint();
                }
            }
        });

        UtteranceView utteranceView = segmentPanel.getUtteranceView();

        int txtWidthL = utteranceView.getWidht() + utteranceView.getVisiblePoint().x;
        int txtWidth = txtWidthL - point.x;
        Dimension d = popup.getPreferredSize();
        d.setSize(txtWidth, d.getHeight());
        popup.setPreferredSize(d);

        int y = field.getY() - 11;

        SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

        popup.show(panel, point.x, y);
        popup.setVisible(true);

        labelTextField.addFocusListener(new FocusListener() {
            @Override
            public void focusGained(FocusEvent e) {
            }

            @Override
            public void focusLost(FocusEvent e) {
                popup.setVisible(false);
            }
        });
    }

    private void handleNewFreeTextEvent(String text, Time time) {
        if (!text.trim().isEmpty()) {
            NewChainedApexEvent newChainedApexEvent = new NewChainedApexEvent(time, segmentPanel);

            ChainedApexEventsCollection collection = new ChainedApexEventsCollection();
            ChainedApexEvent event = new ChainedApexEvent(segmentPanel);
            event.setStartTimeInfo(newChainedApexEvent.getStartTimeInfo());
            event.setEndTimeInfo(newChainedApexEvent.getEndTimeInfo());
            event.setText(text);

            collection.setValueEvent(event);
            event.select();

            ChainedApexEventsEntity entity = new ChainedApexEventsEntity(collection, segmentPanel, nmField);
            entity.updateParentEntity();
            entity.select();

            nmField.addChainedApexEventEntity(entity);

            ToolBarPanels panels = segmentPanel.getToolBarPanels();
            MediaToolBar mediaTb = panels.getMediaToolBar();
            mediaTb.selectEvent(event, entity, nmField);
        }
    }
}
