package edu.bu.signstream.grepresentation.fields.customField;

import java.awt.Cursor;
import java.awt.Point;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.ToolBarPanels;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.popup.TextEntryPopupGenerator;

public class CustomPointFieldWrapper extends FieldWrapper {
	protected final NonManualField nmField;

	public CustomPointFieldWrapper(NonManualField nmField, SignStreamSegmentPanel segmentPanel, TimeRange timeRange) {
		super(nmField, segmentPanel, timeRange);
		this.nmField = nmField;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public void unselectAll() {
		nmField.unselect();
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	// @TODO: implement
	@Override
	public void addEventsEntity(EventsEntity entity) {
		nmField.addChainedEventEntity((ChainedEventsEntity) entity);
	}

	// select event or entity located at given point
	@Override
	public void mouseClicked(Point point, JPanel panel) {
		ToolBarPanels panels = segmentPanel.getToolBarPanels();
		MediaToolBar mediaTb = panels.getMediaToolBar();

		EventsEntity entity = nmField.getEntityAt(point.x);
		Event event = nmField.getEventAt(point.x);
		if (event != null) {
			event.select();
			mediaTb.selectEvent(event, entity, nmField);
		}
	}

	@Override
	public void mouseRightClicked(Point point) {}

	@Override
	public void mouseLeftClicked(Point point) {}

	@Override
	public void mouseClickedAlt(Point point) {}

    @Override
    public void mouseReleased(final Point point, final JPanel panel) {}

    @Override
    public void mouseHoldReleased(final Point point, final JPanel panel) {}

    @Override
    public void mouseDblRightClicked(Point point, JPanel panel) {}

	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
//		PresentationField parentField = nmField.getParentField();
//		Event utterance = parentField.getEventAt(point.x);

        SS3Singleton.getMediaToolBar().deselect();

        var time = TimeRange.ABSOLUTE.time(point.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

//        // if outside the bounds of any utterance, do not proceed
//		if (utterance == null || !utterance.interval().contains(time, Bound.INCLUSIVE)) return;

        var event = new ChainedEvent(segmentPanel);
        event.setStartTimeInfo(time);
        event.setEndTimeInfo(time);
        event.select();

        var collection = new CustomChainedEventsCollection();
        collection.setValueEvent(event);

        Point loc = new Point((int)event.getEndTimeCoordinate(), field.getY() - 11);

        TextEntryPopupGenerator.getInstance("", panel, loc, label -> {
            event.setText(label);

            var entity = new CustomPointChainedEventsEntity(collection, segmentPanel, nmField);
//            entity.determineParentEntity();

            nmField.addChainedEventEntity(entity);

            var bar = SS3Singleton.getMediaToolBar(segmentPanel);
            bar.selectEvent(event, entity, nmField);

            SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
            nmField.setNewChainedEvent(null);
            segmentPanel.repaint();
        }, () -> {
            nmField.setNewChainedEvent(null);
            segmentPanel.repaint();
        });

        SS3Singleton.getMediaPlayer().seek(time);
    }

	@Override
	public void mouseDblLeftClicked(final Point point, final JPanel panel) {
		Event event = SS3Singleton.getMediaToolBar(null).selectedEvent();

		// if event is null by standard procedure, then try the procedure from
		// mouseClicked() before returning
        // TODO: ????
		if (event == null) {
			MediaToolBar mediaTb = SS3Singleton.getMediaToolBar(null);

			EventsEntity entity = nmField.getEntityAt(point.x);
			event = nmField.getEventAt(point.x);
			if (event != null) {
				event.select();
				mediaTb.selectEvent(event, entity, nmField);
			} else {
				return;
			}
		}

		if (!isEntitySelected()) {
            final ChainedEvent cEvent = (ChainedEvent) event;

            var entity = (CustomPointChainedEventsEntity) SS3Singleton.getMediaToolBar(null)
                .selectedEntity();
            
            if (cEvent.equals(entity.getValue())) {
                Point loc = new Point(point.x, this.field.getY() - 11);
                
            	TextEntryPopupGenerator.getInstance(cEvent.getText(), panel, loc, label -> {
                    cEvent.setText(label);

                    nmField.setNewChainedEvent(null);
                    segmentPanel.repaint();
                }, () -> {
                    nmField.setNewChainedEvent(null);
                    segmentPanel.repaint();
                });
            }
		}
	}

	public void mouseDraggedAlt(Point p) {
        var time = TimeRange.ABSOLUTE.time(p.x, TimeUnit.COORDINATE)
            .in(Util.getCurrentRange(getField()), Rounding.FLOOR)
            .clamp(SS3Singleton.getMediaPlayer().interval());

        ChainedEventsEntity entity = (ChainedEventsEntity) nmField.getSelectedEntity();
        if (entity != null) {
            String resp;
            if (MOVE_DIRECTION == MOVE_START_POINT) {
            	time = time.in(entity.getEndTimeInfo().range());
                resp = entity.canSetSelectedEventStartMovieTime(time);

                if (resp != null) {
                    SS3Singleton.getErrorMessages().showErrorMessage(resp);
                    return;
                }
                
                entity.setStartTimeInfo(time);
            } else if (MOVE_DIRECTION == MOVE_END_POINT) {
                time = time.in(entity.getStartTimeInfo().range());
                resp = entity.canSetSelectedEventEndMovieTime(time);

                if (resp != null) {
                    SS3Singleton.getErrorMessages().showErrorMessage(resp);
                    return;
                }
                entity.setEndTimeInfo(time);
            }
        }

        SS3Singleton.getMediaPlayer().seek(time);
	}

    @Override
    public void mouseMoved(Point point, JPanel panel) {
        panel.setCursor(new Cursor(getCursorType(point)));

        Event event = nmField.getEventAt(point.x);
        if (event != null) {
            panel.setToolTipText(event.getToolTipText());
        } else {
            panel.setToolTipText("");
        }
    }

    public int getCursorType(Point point) {
        Event event = nmField.getSelectedEvent();

        if (event != null) {
            double startCoord = event.getStartTimeCoordinate();
            double endCoord = event.getEndTimeCoordinate();

            if (startCoord == endCoord) {
                if (startCoord - 3 <= point.x && point.x <= startCoord - 1) {
                    MOVE_DIRECTION = MOVE_START_POINT;
                    return Cursor.E_RESIZE_CURSOR;
                } else if (startCoord + 1 <= point.x && point.x <= startCoord + 3) {
                    MOVE_DIRECTION = MOVE_END_POINT;
                    return Cursor.E_RESIZE_CURSOR;
                } else {
                    return Cursor.DEFAULT_CURSOR;
                }
            } else {
                if (startCoord - 3 < point.x && point.x < startCoord + 3) {
                    MOVE_DIRECTION = MOVE_START_POINT;
                    return Cursor.E_RESIZE_CURSOR;
                } else if (endCoord - 3 < point.x && point.x < endCoord + 3) {
                    MOVE_DIRECTION = MOVE_END_POINT;
                    return Cursor.E_RESIZE_CURSOR;
                } else {
                    return Cursor.DEFAULT_CURSOR;
                }
            }
        }

        return Cursor.DEFAULT_CURSOR;
    }
}
