package edu.bu.signstream.grepresentation.fields.gestureField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureEnteredText extends JPanel {

	private static final long serialVersionUID = 1L;
	private SignStreamSegmentPanel segmentPanel;
	private GestureField gestField;

	private ArrayList<GestureText> enteredText = new ArrayList<GestureText>();

	private int startSelectionCoord = -1;
	private int endSelectionCoord = -1;
	private boolean selectedSegment = false;

	private FontMetrics fontMetrics;

	private String text = "";

	public GestureEnteredText(SignStreamSegmentPanel segmentPanel, GestureField gestField) {
		this.segmentPanel = segmentPanel;
		this.gestField = gestField;

		fontMetrics = getFontMetrics(SegmentGraphReprUtil.getGraphReprFont());
	}

	public ArrayList<GestureText> getEnteredText() {
		return enteredText;
	}
	
	public ArrayList<GestureText> getEnteredTextBetween(Time startTimeInfo, Time endTimeInfo) {
		ArrayList<GestureText> enteredText = getEnteredText();
		ArrayList<GestureText> enteredTextBetween = new ArrayList<GestureText>();

		for (GestureText text: enteredText) {
			if (text.getStartTime() == null || text.getEndTime() == null)
				continue;
			
			Time entStartTimeInfo = text.getStartTime();
			Time entEndTimeInfo = text.getEndTime();
			
			if (startTimeInfo.lessThanOrEq(entStartTimeInfo) && entEndTimeInfo.lessThanOrEq(endTimeInfo)) {
				enteredTextBetween.add(text);
			}
		}
		
		return enteredTextBetween;
	}

	public void deleteText(GestureText textToDelete) {
		enteredText.remove(textToDelete);
	}

	public void addText(GestureText text) {
		enteredText.add(text);
	}

	public void setText(String text) {
		this.text = text;
	}
	
	public void removeText(ArrayList<GestureText> textToRemove) {
		for(GestureText txt : textToRemove) {
			enteredText.remove(txt);
		}
	}

	public GestureText getPrecedingText(int coord) {
		GestureText precedingText = null;

		for (GestureText entText : enteredText) {
			if (entText.getEndCoord() <= coord
					&& (precedingText == null || entText.getEndCoord() < precedingText
							.getEndCoord())) {
				precedingText = entText;
			}
		}
		return precedingText;
	}

	public GestureText getFollowingText(int coord) {
		GestureText followingText = null;

		for (GestureText entText : enteredText) {
			if (entText.getStartCoord() <= coord
					&& (followingText == null || entText.getEndCoord() < followingText
							.getStartCoord())) {
				followingText = entText;
			}
		}
		return followingText;
	}

	public ArrayList<GestureText> getSortedEnteredTextInInteval2(double startCoord, double endCoord) {
			ArrayList<GestureText> text = new ArrayList<GestureText>();
			for (GestureText entText : enteredText) {
				
				double txtStartCoord = entText.getStartCoord();
				double txtWidth = fontMetrics.stringWidth(entText.getText());
				double txtEndCoord = txtStartCoord + txtWidth;
				
				if (startCoord <= txtEndCoord && txtStartCoord < endCoord) {				
					text.add(entText);
				}
			}
			return text;
		}

	public ArrayList<GestureText> getSortedEnteredTextInInteval(double startCoord,
			double endCoord) {
		ArrayList<GestureText> text = new ArrayList<GestureText>();
		for (GestureText entText : enteredText) {
			if (startCoord <= entText.getStartCoord()) {
				if (endCoord == -1 || entText.getStartCoord() < endCoord) {
					text.add(entText);
				}
			}
		}
		return text;
	}

	public ArrayList<GestureText> getSelectedText() {

		ArrayList<GestureText> selectedText = new ArrayList<GestureText>();

		for (GestureText text : enteredText) {
			if (text.isSelected())
				selectedText.add(text);
			// return text;
		}
		return selectedText;
	}

	public String getSentence(ArrayList<GestureText> text) {
		StringBuffer sb = new StringBuffer();
		Iterator<GestureText> iterator = text.iterator();
		while (iterator.hasNext()) {
			sb.append(" ");
			GestureText txt = iterator.next();
			sb.append(txt.getText());
		}
		return sb.toString().trim();
	}

	public void synchronize(ArrayList<GestureText> enteredTextInInterval,
			double startCoord, double endCoord) {
		// remove text objects
		for (GestureText t : enteredTextInInterval) {
			enteredText.remove(t);
		}

		// create new objects
		StringTokenizer tk = new StringTokenizer(text);
		double tokenStartCoord = startCoord;
		boolean visible = true;

		boolean tokenStarted = false;
		String singleToken = null;

		while (tk.hasMoreTokens()) {
			String token = tk.nextToken();

			int charAt = token.lastIndexOf('"');

			if (charAt >= 0) {
				if (!tokenStarted) {
					singleToken = token;
					tokenStarted = true;
				} else {
					singleToken = singleToken + " " + token;
					tokenStarted = false;

					int width = fontMetrics.stringWidth(singleToken);
					double tokenEndCoord = tokenStartCoord + width;

					if (tokenEndCoord > endCoord) {
						visible = false;
					}

					GestureText text1 = new GestureText(singleToken, tokenStartCoord,
							tokenEndCoord);
					text1.setVisible(visible);

					enteredText.add(text1);

					tokenStartCoord = tokenEndCoord + 10;

					singleToken = null;
				}
			} else if (tokenStarted) {
				singleToken = singleToken + " " + token;
			} else {

				int width = fontMetrics.stringWidth(token);
				double tokenEndCoord = tokenStartCoord + width;

				if (tokenEndCoord > endCoord) {
					visible = false;
				}

				GestureText text1 = new GestureText(token, tokenStartCoord, tokenEndCoord);
				text1.setVisible(visible);

				enteredText.add(text1);

				tokenStartCoord = tokenEndCoord + 10;
			}
		}
		
		if (tokenStarted) {
			int width = fontMetrics.stringWidth(singleToken);
			double tokenEndCoord = tokenStartCoord + width;

			if (tokenEndCoord > endCoord) {
				visible = false;
			}

			GestureText text1 = new GestureText(singleToken, tokenStartCoord, tokenEndCoord);
			text1.setVisible(visible);
			
			enteredText.add(text1);

			tokenStartCoord = tokenEndCoord + 10;
		}

		text = "";
	}

	public void unselectAll() {
		for (GestureText text : enteredText) {
			text.setSelected(false);
		}
	}

	public GestureField getGestureField() {
		return gestField;
	}

	public GestureText getTextAt(int coord) {
		for (GestureText text : enteredText) {

			double startCoord = text.getStartCoord();
			double endCoord = text.getEndCoord();

			if (endCoord == startCoord && fontMetrics != null) {
				endCoord = startCoord + fontMetrics.stringWidth(text.getText());
			}
			
			if(startCoord == endCoord) {
				endCoord = startCoord + 10;
			}

			if (startCoord <= coord && endCoord >= coord) {
				return text;
			}
		}
		return null;
	}

	public void selectTextBetweenSelectedEvents() {

		int firstSelected = -1;
		int lastSelected = -1;

		for (int i = 0; i < enteredText.size(); i++) {
			GestureText text = enteredText.get(i);

			if (text.isSelected()) {
				if (firstSelected == -1) {
					firstSelected = i;
				} else {
					lastSelected = i;
				}
			}
		}

		if (lastSelected > -1) {
			for (int i = firstSelected; i < lastSelected + 1; i++) {
				GestureText text = enteredText.get(i);
				text.setSelected(true);
			}
		}
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		fontMetrics = getFontMetrics(font);
		segmentPanel.getPainter().drawEnteredGestureText(enteredText, g2d, Y,
				fontMetrics, gestField);
	}

	public int getEndSelectionCoord() {
		return endSelectionCoord;
	}

	public SignStreamSegmentPanel getSegmentPanel() {
		return segmentPanel;
	}

	public int getStartSelectionCoord() {
		return startSelectionCoord;
	}

	public void setEndSelectionCoord(int i) {
		endSelectionCoord = i;
	}

	public void setSegmentPanel(SignStreamSegmentPanel panel) {
		segmentPanel = panel;
	}

	public ArrayList<Event> createGestureChainedEvent(Time startTimeInfo, Time endTimeInfo) {
		ArrayList<Event> list = new ArrayList<Event>();

		ArrayList<GestureText> selectedText = getSelectedText();
		if (!selectedText.isEmpty()) {
			String text = "";

			for (GestureText t : selectedText) {
				text = text + t.getText() + " ";
			}

			text = text.trim();	// disallow entering whitespace at front or end upon newly adding event from entered text

			SS3Item item = new SS3Item();
			item.setId(Util.getLongUniqueNumber());

			var range = Util.getCurrentRange(gestField, SS3Singleton.getVideoTimeRange());
			item.setStartTimeInfo(startTimeInfo.in(range));
			item.setEndTimeInfo(endTimeInfo.in(range));
			item.setText(text); // selectedText.getText());

			GestureEvent gestEvent = new GestureEvent(item,
					segmentPanel);
			gestEvent.select();
			GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
			collection.setTextValueEvent(gestEvent);

			GestureEventsEntity entity = new GestureEventsEntity(
					collection, segmentPanel, gestField);
			entity.updateParentEntity();
			gestField.addGestureEventsEntity(entity);

			for (GestureText t1 : selectedText) {
				enteredText.remove(t1);
			}

			gestEvent.select();

			list.add(gestEvent);
			list.add(entity);
		}

		return list;
	}

	public void moveTextObjectsForward(int legacyStartCoord, int newStartCoord,
			int prevEventEndCoord) {

		// select Text objects that are in between legacyStartCoord and
		// maxEndCoord
		ArrayList<GestureText> textToMove = new ArrayList<GestureText>();
		for (GestureText text : enteredText) {
			if (prevEventEndCoord <= text.getEndCoord()
					|| text.getStartCoord() <= newStartCoord) {
				textToMove.add(text);
			}
		}

		// sort textToMove Text objects
		double[] startCoordsToSort = new double[textToMove.size()];
		for (int counter = 0; counter < textToMove.size(); counter++) {
			GestureText txt = textToMove.get(counter);
			startCoordsToSort[counter] = txt.getEndCoord();
		}
		Sorter.sort(startCoordsToSort);

		// create sorted list of Text objects to move
		ArrayList<GestureText> sortedText = new ArrayList<GestureText>();
		for (int i = 0; i < startCoordsToSort.length; i++) {
			double startCoord = startCoordsToSort[i];
			for (GestureText txt : textToMove) {
				if (txt.getStartCoord() == startCoord) {
					sortedText.add(txt);
					textToMove.remove(txt);
				}
			}
		}

		// move objects
		int txtEndCoord = newStartCoord - 10;
		boolean visible = true;
		for (int j = sortedText.size() - 1; j >= 0; j--) {
			GestureText t = sortedText.get(j);

			int width = fontMetrics.stringWidth(t.getText());
			int txtStartCoord = txtEndCoord - width;

			if (prevEventEndCoord >= txtStartCoord) {
				visible = false;
			}

			t.setStartCoord(txtStartCoord);
			t.setEndCoord(txtEndCoord);
			t.setVisible(visible);

			txtStartCoord = txtStartCoord - 10;
		}
	}

	public void setStartSelectionCoord(int coord) {
		startSelectionCoord = coord;
	}

	public boolean isSelectedSegment() {
		return selectedSegment;
	}

	public void setSelectedSegment(boolean b) {
		selectedSegment = b;
	}
}
