package edu.bu.signstream.grepresentation.fields.gestureField.dialog;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeTemplate;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeTemplates;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSPaletteItemGroup;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEntityProperties;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.DomNdomHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.palettes.GesturePaletteDialog;
import edu.bu.signstream.grepresentation.fields.palettes.doublePalettes.GeneralDoublePaletteDialog;
import edu.bu.signstream.grepresentation.fields.palettes.doublePalettes.GestureDoublePaletteDialog;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GesturePaletteInformationHelper;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.listener.WeakItemListener;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

public class GestureHandShapePanel extends JPanel implements ItemListener, ActionListener, GesturePaletteInformationHelper {

	private static final long serialVersionUID = 1L;

	private JDialog parentDialog = null;

	private final JRadioButton fingerspelled = new JRadioButton("Fingerspelled");
	private final JRadioButton loan = new JRadioButton("Loan sign");
	private final JRadioButton number = new JRadioButton("Number");
	private final JRadioButton lexical = new JRadioButton("Lexical sign");
	private final JRadioButton classifier = new JRadioButton("Classifier");
	private final JRadioButton gesture = new JRadioButton("Gesture");
	private final JCheckBox nameSign = new JCheckBox("Name sign");

	private JRadioButton oneHanded = new JRadioButton("One-handed");
	private JRadioButton twoHanded = new JRadioButton("Two-handed");

	private JRadioButton dominantHand = new JRadioButton("Dominant Hand");
	private JRadioButton nonDominantHand = new JRadioButton("Non-Dominant Hand");

	private JButton selectHandshapes = new JButton("Select handshapes");
	private JButton clearHandshapes = new JButton("Clear");

	private GestureInfoPanel gestureInfoPanel = null;

	private JCheckBox oneFrameGloss = new JCheckBox("One-Frame Gesture");

	private JCheckBox markedForm = new JCheckBox("Marked # of Hands");
	private JCheckBox passiveBaseArm = new JCheckBox("Passive base arm");

	private JComboBox<GestureDomNdomHandshapesType> sameDifDomNdomHandshapes = new JComboBox<>(GestureDomNdomHandshapesType.values());
	private JComboBox<GestureStartEndHandshapesType> sameDifStartEndHandshapes = new JComboBox<>(GestureStartEndHandshapesType.values());
	private JComboBox<GestureStartEndHandshapesType> sameDifStartEndHandshapesNdom = new JComboBox<>(GestureStartEndHandshapesType.values());
	private JComboBox<GestureStartEndHandshapesType> sameDifStartEndHandshapesDom = new JComboBox<>(GestureStartEndHandshapesType.values());

	private final int ONE_HANDED = 1;
	private final int TWO_HANDED = 2;
	private final int ONE_HND_R = 3;
	private final int ONE_HND_L = 4;
	private final int FINGERSPELLED = 5;
	private final int LOAN = 6;
	private final int LEXICAL = 7;
	private final int CLASSIFIER = 8;
	private final int GESTURE = 9;
	private final int NUMBER = 10;

	private final int SELECT_HANDSHAPES = 0;
	private final int CLEAR_HANDSHAPES = 1;

	private ButtonGroup grp1 = new ButtonGroup();
	private ButtonGroup oneHndGrp = new ButtonGroup();

	// contains entered information. Loaded from preprocessor
	private HandShapeFormBean hshFormBean;
	// contains updated information. Will be used by postprocessor
	private HandShapeFormBean bean;

	private GestureEventsEntity selectedEntity = null;
	private GestureFieldWrapper wrapper = null;
	// private Segment segment = null;

	private JPanel southPanel = null;

    private SelectedGestureHandshapesPanel ndomHPanel = null;
	private SelectedGestureHandshapesPanel domHPanel = null;

	private SameHandshapesDomNdomComboBoxListener sameDifDomNdomHandshapesComboBoxListener = null;
	private SameHandshapesStartEndComboBoxListener sameDifStartEndHandshapesComboBoxListener = null;
	private SameHandshapesStartEndDomComboBoxListener sameDifStartEndHandshapesDomComboBoxListener = null;
	private SameHandshapesStartEndNdomComboBoxListener sameDifStartEndHandshapesNdomComboBoxListener = null;

	private String paletteID;

	public GestureHandShapePanel(GestureInfoPanel gestureInfoPanel) {
		this.gestureInfoPanel = gestureInfoPanel;
		this.parentDialog = gestureInfoPanel.getParentDialog();

		this.hshFormBean = gestureInfoPanel.getGestureHandShapeFormBean();
		this.bean = new HandShapeFormBean();

		this.bean.setSelectedDomHandShapeImages(hshFormBean.getSelectedDomHandShapeImages());
		this.bean.setSelectedNdomHandShapeImages(hshFormBean.getSelectedNdomHandShapeImages());
		this.bean.setType(hshFormBean.getType());
		this.bean.setRightHandSelected(hshFormBean.isRightHandSelected());

		this.wrapper = hshFormBean.getGestFieldWrapper();
		this.selectedEntity = (GestureEventsEntity) SS3Singleton.getMediaToolBar(null).selectedEntity();
		this.paletteID = Util.DEFAULT_GESTURE_HANDSHAPE_PALETTE_GROUP_ID;
		//associatedFieldsPanel.getEnterButton().setEnabled(false);

		oneHanded.addItemListener(new WeakItemListener(this, oneHanded));
		twoHanded.addItemListener(new WeakItemListener(this, twoHanded));
		dominantHand.addItemListener(new WeakItemListener(this, dominantHand));
		nonDominantHand.addItemListener(new WeakItemListener(this, nonDominantHand));

		dominantHand.setActionCommand("" + ONE_HND_R);
		nonDominantHand.setActionCommand("" + ONE_HND_L);
		oneHanded.setActionCommand("" + ONE_HANDED);
		twoHanded.setActionCommand("" + TWO_HANDED);

		fingerspelled.setActionCommand("" + FINGERSPELLED);
		loan.setActionCommand("" + LOAN);
		lexical.setActionCommand("" + LEXICAL);
		classifier.setActionCommand("" + CLASSIFIER);
		gesture.setActionCommand("" + GESTURE);
		number.setActionCommand("" + NUMBER);

		fingerspelled.addActionListener(new WeakActionListener(this, fingerspelled));
		loan.addActionListener(new WeakActionListener(this, loan));
		number.addActionListener(new WeakActionListener(this, number));
		lexical.addActionListener(new WeakActionListener(this, lexical));
		classifier.addActionListener(new WeakActionListener(this, classifier));
		gesture.addActionListener(new WeakActionListener(this, gesture));

		selectHandshapes.setActionCommand("" + SELECT_HANDSHAPES);
		clearHandshapes.setActionCommand("" + CLEAR_HANDSHAPES);
		selectHandshapes.addActionListener(new WeakActionListener(this, selectHandshapes));
		clearHandshapes.addActionListener(new WeakActionListener(this, clearHandshapes));

		sameDifDomNdomHandshapes.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapes.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapesNdom.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapesDom.setRenderer(new CenteredTextCellRenderer());

		java.awt.Font cFont = sameDifDomNdomHandshapes.getFont();
		java.awt.Font font = new java.awt.Font(cFont.getName(), cFont.getStyle(), 10);

		sameDifDomNdomHandshapes.setFont(font);
		sameDifStartEndHandshapes.setFont(font);
		sameDifStartEndHandshapesNdom.setFont(font);
		sameDifStartEndHandshapesDom.setFont(font);

		Dimension d1 = sameDifDomNdomHandshapes.getPreferredSize();

		int w0 = sameDifStartEndHandshapesDom.getPreferredSize().width;
		int w1 = sameDifStartEndHandshapesNdom.getPreferredSize().width;

		int w = w0 > w1 ? w0 : w1;

		Dimension d = new Dimension(2 * w + 10, d1.height);
		Dimension dd = new Dimension(w + 5, d1.height);

		sameDifDomNdomHandshapes.setPreferredSize(d);
		sameDifDomNdomHandshapes.setMaximumSize(d);
		sameDifDomNdomHandshapesComboBoxListener = new SameHandshapesDomNdomComboBoxListener(this);
		sameDifDomNdomHandshapes.addActionListener(new WeakActionListener(sameDifDomNdomHandshapesComboBoxListener, sameDifDomNdomHandshapes));

		sameDifStartEndHandshapes.setPreferredSize(d);
		sameDifStartEndHandshapes.setMinimumSize(d);
		sameDifStartEndHandshapesComboBoxListener = new SameHandshapesStartEndComboBoxListener(this);
		sameDifStartEndHandshapes.addActionListener(new WeakActionListener(sameDifStartEndHandshapesComboBoxListener, sameDifStartEndHandshapes));

		sameDifStartEndHandshapesDom.setRenderer(new DefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				JLabel comp = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				comp.setText(value.toString() + " (dom)");
				return comp;
			}
		});

		sameDifStartEndHandshapesDom.setMinimumSize(dd);
		sameDifStartEndHandshapesDom.setSize(dd);
		sameDifStartEndHandshapesDom.setPreferredSize(dd);
		sameDifStartEndHandshapesDomComboBoxListener = new SameHandshapesStartEndDomComboBoxListener(this);
		sameDifStartEndHandshapesDom.addActionListener(new WeakActionListener(sameDifStartEndHandshapesDomComboBoxListener, sameDifStartEndHandshapesDom));

		sameDifStartEndHandshapesNdom.setRenderer(new DefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				JLabel comp = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				comp.setText(value.toString() + " (ndom)");
				return comp;
			}
		});
		sameDifStartEndHandshapesNdom.setMinimumSize(dd);
		sameDifStartEndHandshapesNdom.setSize(dd);
		sameDifStartEndHandshapesNdom.setPreferredSize(dd);
		sameDifStartEndHandshapesNdomComboBoxListener = new SameHandshapesStartEndNdomComboBoxListener(this);
		sameDifStartEndHandshapesNdom.addActionListener(new WeakActionListener(sameDifStartEndHandshapesNdomComboBoxListener, sameDifStartEndHandshapesNdom));

		dominantHand.setMinimumSize(dd);
		dominantHand.setSize(dd);
		dominantHand.setPreferredSize(dd);
		dominantHand.setHorizontalAlignment(SwingConstants.CENTER);

		nonDominantHand.setMinimumSize(dd);
		nonDominantHand.setSize(dd);
		nonDominantHand.setPreferredSize(dd);
		nonDominantHand.setHorizontalAlignment(SwingConstants.CENTER);

		grp1.add(oneHanded);
		grp1.add(twoHanded);

		oneHndGrp.add(nonDominantHand);
		oneHndGrp.add(dominantHand);

		ButtonGroup classifierGroup = new ButtonGroup();
		classifierGroup.add(fingerspelled);
		classifierGroup.add(loan);
		classifierGroup.add(number);
		classifierGroup.add(lexical);
		classifierGroup.add(classifier);
		classifierGroup.add(gesture);


		setPreferredSize(new Dimension(500, 340));

		populate();
		init();
	}

	public void update() {
		this.hshFormBean = gestureInfoPanel.getGestureHandShapeFormBean();
		this.bean.setSelectedDomHandShapeImages(hshFormBean.getSelectedDomHandShapeImages());
		this.bean.setSelectedNdomHandShapeImages(hshFormBean.getSelectedNdomHandShapeImages());
		this.bean.setSelectedDomOrientationFingerTipImages(hshFormBean.getSelectedDomOrientationFingerTipImages());
		this.bean.setSelectedNdomOrientationFingerTipImages(hshFormBean.getSelectedNdomOrientationFingerTipImages());
		this.bean.setSelectedDomPalmOrientationImages(hshFormBean.getSelectedDomOrientationPalmImages());
		this.bean.setSelectedNdomPalmOrientationImages(hshFormBean.getSelectedNdomOrientationPalmImages());

		this.bean.setType(hshFormBean.getType());
		this.bean.setRightHandSelected(hshFormBean.isRightHandSelected());
		this.wrapper = hshFormBean.getGestFieldWrapper();
		this.selectedEntity = (GestureEventsEntity) SS3Singleton.getMediaToolBar(null).selectedEntity();

        ArrayList<HandShapeImage> rhImages =  new ArrayList<>(hshFormBean.getSelectedDomHandShapeImages());
        ArrayList<HandShapeImage> lhImages =  new ArrayList<>(hshFormBean.getSelectedNdomHandShapeImages());
        ArrayList<FingerTipImage> rOImages =  new ArrayList<>(hshFormBean.getSelectedDomOrientationFingerTipImages());
        ArrayList<FingerTipImage> lOImages =  new ArrayList<>(hshFormBean.getSelectedNdomOrientationFingerTipImages());
		ArrayList<PalmImage> rPOImages = new ArrayList<>(hshFormBean.getSelectedDomOrientationPalmImages());
		ArrayList<PalmImage> lPOImages = new ArrayList<>(hshFormBean.getSelectedNdomOrientationPalmImages());

        ndomHPanel.setHandshapes(lhImages);
        ndomHPanel.setFingerTipOrientationsImages(lOImages);
		ndomHPanel.setPalmOrientationsImages(lPOImages);
		ndomHPanel.repaint();
		ndomHPanel.revalidate();

        domHPanel.setHandshapes(rhImages);
        domHPanel.setFingerTipOrientationsImages(rOImages);
		domHPanel.setPalmOrientationsImages(rPOImages);
		domHPanel.repaint();
		domHPanel.revalidate();

		populate();
		removeAll();
		init();

		repaint();
		revalidate();
	}

	@Override
    public void updateParentInfoSelectedEntities(ArrayList<HandShapeImage> domHandshapes, ArrayList<HandShapeImage> ndomHandshapes,
												 ArrayList<FingerTipImage> domOrientations, ArrayList<FingerTipImage> ndomOrientations, ArrayList<PalmImage> domPalmOrientations, ArrayList<PalmImage> ndomPalmOrientations) {

		domHPanel.setHandshapesAndOrientations(domHandshapes, domOrientations, domPalmOrientations);
		domHPanel.repaint();
		domHPanel.revalidate();

		ndomHPanel.setHandshapesAndOrientations(ndomHandshapes, ndomOrientations, ndomPalmOrientations);
		ndomHPanel.repaint();
		ndomHPanel.revalidate();

		hshFormBean.setSelectedDomHandShapeImages(domHandshapes);
		hshFormBean.setSelectedNdomHandShapeImages(ndomHandshapes);
		hshFormBean.setSelectedDomOrientationFingerTipImages(domOrientations);
		hshFormBean.setSelectedNdomOrientationFingerTipImages(ndomOrientations);
		hshFormBean.setSelectedDomPalmOrientationImages(domPalmOrientations);
		hshFormBean.setSelectedNdomPalmOrientationImages(ndomPalmOrientations);
    }

	public void clean() {
		domHPanel.clean();
		ndomHPanel.clean();

		oneHanded.removeItemListener(this);
		twoHanded.removeItemListener(this);
		dominantHand.removeItemListener(this);
		nonDominantHand.removeItemListener(this);

		sameDifDomNdomHandshapes.removeActionListener(sameDifDomNdomHandshapesComboBoxListener);
	}

	public String getSelectedSameDifDomNdomHandshapesStr() {
		return (String)sameDifDomNdomHandshapes.getSelectedItem(); //.getSelectedIndex();
	}

	public GestureDomNdomHandshapesType getSelectedSameDifDomNdomHandshapes() {
		return (GestureDomNdomHandshapesType) sameDifDomNdomHandshapes.getSelectedItem();
	}

	public void setSelectedSameDifDomNdomHandshapes(GestureDomNdomHandshapesType type) {
		sameDifDomNdomHandshapes.setSelectedItem(type);
	}

	public void setSelectedSameDifDomNdomHandshapesVisible(boolean visible) {
		sameDifDomNdomHandshapes.setVisible(visible);
	}

	@Override
	public GestureStartEndHandshapesType getSelectedSameDifStartEndHandshapes() {
		return (GestureStartEndHandshapesType) sameDifStartEndHandshapes.getSelectedItem();
	}

	public void setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType type) {
		sameDifStartEndHandshapes.setSelectedItem(type);
	}

	public void setSelectedSameDifStartEndHandshapesVisible(boolean visible) {
		sameDifStartEndHandshapes.setVisible(visible);
	}

	@Override
	public GestureStartEndHandshapesType getSelectedSameDifStartEndHandshapesNdom() {
		return (GestureStartEndHandshapesType) sameDifStartEndHandshapesNdom.getSelectedItem();
	}

	public void setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType type) {
		sameDifStartEndHandshapesNdom.setSelectedItem(type);
	}

	public void setSelectedSameDifStartEndHandshapesNdomVisible(boolean visible) {
		sameDifStartEndHandshapesNdom.setVisible(visible);
	}

	@Override
	public GestureStartEndHandshapesType getSelectedSameDifStartEndHandshapesDom() {
		return (GestureStartEndHandshapesType) sameDifStartEndHandshapesDom.getSelectedItem();
	}

	public void setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType type) {
		sameDifStartEndHandshapesDom.setSelectedItem(type);
	}

	public void setSelectedSameDifStartEndHandshapesDomVisible(boolean visible) {
		sameDifStartEndHandshapesDom.setVisible(visible);
	}

	public GestureEventsEntity getSelectedEntity() {
		return this.selectedEntity;
	}

	public void setOneFrameGloss(boolean selected) {
		oneFrameGloss.setSelected(selected);
	}

	@Override
	public boolean isOneFrameEntity() {
		return oneFrameGloss.isSelected();
	}

	public void setMarkedNumberOfHands(boolean selected) {
		markedForm.setSelected(selected);
	}

	public boolean isMarkedNumberOfHands() {
		return markedForm.isSelected();
	}

	public void setPassiveBaseArmEnabled(boolean enable) {
		passiveBaseArm.setEnabled(enable);
	}

	public void setPassiveBaseArmSelected(boolean selected) {
		passiveBaseArm.setSelected(selected);
	}

	public boolean isPassiveBaseArm() {
		return passiveBaseArm.isSelected();
	}

	public boolean isTwoHanded() {
		return twoHanded.isSelected();
	}

	public void selectOneHanded() {
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			oneHanded.setSelected(true);
		}

		//oneHanded.setSelected(true);
	}

	public void selectTwoHanded() {
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			twoHanded.setSelected(true);
		}
		//twoHanded.setSelected(true);

		// sameDifHandshapes.setSelectedIndex(0);
		// sameDifStartEndHandshapes.setSelectedIndex(0);
	}

	public HandShapeFormBean getHandShapeFormBean() {
		return hshFormBean;
	}

	public SelectedGestureHandshapesPanel getSelectedNdomHandshapesPane() {
		return ndomHPanel;
	}

	public SelectedGestureHandshapesPanel getSelectedDomHandshapesPane() {
		return domHPanel;
	}

	private JPanel selectedImagesPanel() {
		SS3CodingScheme scheme = hshFormBean.getSS3CodingScheme();

		JLabel domLbl = new JLabel("Dominant Hand");
		JLabel ndomLbl = new JLabel("Non-Dominant Hand");

		ArrayList<HandShapeImage> domhList = bean.getSelectedDomHandShapeImages();
		ArrayList<FingerTipImage> domOList = bean.getSelectedDomOrientationFingerTipImages();
		ArrayList<PalmImage> domPOList = bean.getSelectedDomOrientationPalmImages();
		domHPanel = new SelectedGestureHandshapesPanel(domhList, domOList, domPOList, scheme, this);
		domHPanel.setPreferredSize(new Dimension((int) domHPanel.getPreferredSize().getWidth(), 60));
		domHPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		ArrayList<HandShapeImage> ndomhList = bean.getSelectedNdomHandShapeImages();
		ArrayList<FingerTipImage> ndomOList = bean.getSelectedNdomOrientationFingerTipImages();
		ArrayList<PalmImage> ndomPOList = bean.getSelectedNdomOrientationPalmImages();
		ndomHPanel = new SelectedGestureHandshapesPanel(ndomhList, ndomOList, ndomPOList, scheme, this);
		ndomHPanel.setPreferredSize(new Dimension((int) ndomHPanel.getPreferredSize().getWidth(), 60));
		ndomHPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		JPanel panel = new JPanel();
		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		panel.setLayout(gridBag);

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.ipady = 0;
		c.weightx = 0.5;
		gridBag.setConstraints(domLbl, c);
		panel.add(domLbl, c);
		c.gridx = 1;

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(clearHandshapes);

		c.fill = GridBagConstraints.NORTHEAST;
		panel.add(buttonPanel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 2;
		c.ipady = 49; // make this component tall
		c.ipadx = 300;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(domHPanel, c);
		panel.add(domHPanel);

		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 0; // reset to default
		c.weighty = 1.0; // request any extra vertical space
		c.ipady = 10;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(ndomLbl, c);
		panel.add(ndomLbl);

		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 2;
		c.ipady = 49; // make this component tall
		c.ipadx = 300;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(ndomHPanel, c);
		panel.add(ndomHPanel);

		panel.setBorder(BorderFactory.createTitledBorder("Selected Handshapes"));
		return panel;
	}

	/**
     * Updates and returns HandShapeBean object.
     * @return HandShapeBean object that has selected HandShapeImage objects for both hands
     */
    public HandShapeFormBean getHandShapeBean() {
		ArrayList<HandShapeImage> rhList = domHPanel.getHandshapes();
		ArrayList<HandShapeImage> lhList = ndomHPanel.getHandshapes();

		bean.setSelectedDomHandShapeImages(rhList);
        bean.setSelectedNdomHandShapeImages(lhList);

		// bean.setMarkedForm(markedForm.isSelected());
		return bean;
	}

	private void populate() {
		// this is done so that setSelectedIndex does not trigger a removal of nondominant handshape images
		// that should only occur with a mouse click on the sameDifHandshapes combobox
		// this is undone at the end of this function
		sameDifDomNdomHandshapesComboBoxListener.setIgnoreFlag(true);

		String fieldID = wrapper.getField().getFieldID();

		EventsEntity ent = SS3Singleton.getMediaToolBar(null).selectedEntity();

		GestureEventsEntity entity = (GestureEventsEntity) ent;

		GestureEntityProperties properties = entity.getGestureEntityProperties();
		//String prClassification = properties.getTwoHandedHandshapesClassification();

		String startEndHandshape = properties.getStartEndHandshape();
		String startEndHandshapeNdom = properties.getStartEndHandshapeNdomHand();
		String startEndHandshapeDom = properties.getStartEndHandshapeDomHand();

		twoHanded.setEnabled(true);

		if (fieldID.equals(Util.DOM_GESTURE_FIELD_ID)) {
			dominantHand.setSelected(true);
		} else {
			nonDominantHand.setSelected(true);
		}

		dominantHand.setEnabled(false);
		nonDominantHand.setEnabled(false);

		oneFrameGloss.setSelected(properties.isOneFrameGesture());
		oneFrameGloss.setEnabled(false);

		String twoHandedHandshapesClassification = properties
				.getTwoHandedHandshapesClassification();

		if (properties.isTwoHanded()) {
			twoHanded.setSelected(true);
			if (twoHandedHandshapesClassification == null
					|| twoHandedHandshapesClassification.trim().length() == 0) {
				sameDifDomNdomHandshapes.setSelectedIndex(0);
				sameDifStartEndHandshapes.setSelectedIndex(0);
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GestureEntityProperties.SAME_HANDSHAPES_DOM_NDOM)) {
				sameDifDomNdomHandshapes.setSelectedIndex(0);
				if (startEndHandshape
						.equalsIgnoreCase(GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapes.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapes.setSelectedIndex(0);
				}
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GestureEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM)) {
				sameDifDomNdomHandshapes.setSelectedIndex(1);
				if (startEndHandshape
						.equalsIgnoreCase(GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapes.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapes.setSelectedIndex(0);
				}
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GestureEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM)) {
				sameDifDomNdomHandshapes.setSelectedIndex(2);
				if (startEndHandshapeDom
						.equalsIgnoreCase(GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapesDom.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapesDom.setSelectedIndex(0);
				}

				if (startEndHandshapeNdom
						.equalsIgnoreCase(GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapesNdom.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapesNdom.setSelectedIndex(0);
				}
			}
		} else {
			oneHanded.setSelected(true);
			if (startEndHandshape
					.equalsIgnoreCase(GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
				sameDifStartEndHandshapes.setSelectedIndex(1);
			} else {
				// startEndHandshape
				sameDifStartEndHandshapes.setSelectedIndex(0);
			}
		}

		markedForm.setSelected(properties.isMarkedNumberOfHands());
		passiveBaseArm.setSelected(properties.isPassiveBaseArm());

		// undo the ignore on the sameDifHandshapes listener
		sameDifDomNdomHandshapesComboBoxListener.setIgnoreFlag(false);
	}

	public void init() {
		GridBagLayout gbl = new GridBagLayout();
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(gbl);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = gbc.weighty = 0.5;

		gbc.gridwidth = 4;

//		JPanel signTypePanel = new JPanel(new GridLayout(2, 4));
//		signTypePanel.setBorder(new TitledBorder("Sign Type"));
//		signTypePanel.add(fingerspelled);
//		signTypePanel.add(loan);
//		signTypePanel.add(number);
//		signTypePanel.add(new JLabel());
//		signTypePanel.add(lexical);
//		signTypePanel.add(classifier);
//		signTypePanel.add(gesture);
//		signTypePanel.add(nameSign);
//
//		centerPanel.add(signTypePanel, gbc);

		gbc.gridx = 0;
		gbc.gridy++;
		gbc.gridwidth = 1;
		centerPanel.add(oneHanded, gbc);

		gbc.gridx++;
		centerPanel.add(dominantHand, gbc);

		gbc.gridx++;
		centerPanel.add(nonDominantHand, gbc);


		gbc.gridx = 0;
		gbc.gridy++;
		gbc.gridwidth = 1;
		centerPanel.add(twoHanded, gbc);

		gbc.gridx++;
		gbc.gridwidth = 2;

		if (oneHanded.isSelected()) {
			centerPanel.add(sameDifStartEndHandshapes, gbc);
			sameDifStartEndHandshapes.setVisible(true);
		} else if (twoHanded.isSelected()) {
			centerPanel.add(sameDifDomNdomHandshapes, gbc);
			sameDifDomNdomHandshapes.setVisible(true);

			gbc.gridy++;
			if (sameDifDomNdomHandshapes.getSelectedIndex() == 0 || sameDifDomNdomHandshapes.getSelectedIndex() == 1) {
				centerPanel.add(sameDifStartEndHandshapes, gbc);
				sameDifStartEndHandshapes.setVisible(true);
			} else {
				gbc.gridwidth = 1;
				centerPanel.add(sameDifStartEndHandshapesNdom, gbc);
				sameDifStartEndHandshapesNdom.setVisible(true);
				gbc.gridx++;
				centerPanel.add(sameDifStartEndHandshapesDom, gbc);
				sameDifStartEndHandshapesDom.setVisible(true);
			}
		}

		gbc.gridx = 0;
		gbc.gridy++;
		gbc.gridwidth = 1;
//		centerPanel.add(markedForm, gbc);

		gbc.gridx++;
		centerPanel.add(oneFrameGloss, gbc);


		gbc.gridx = 0;
		gbc.gridy++;
//		centerPanel.add(passiveBaseArm, gbc);

		gbc.gridx += 2;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		centerPanel.add(selectHandshapes, gbc);

		setLayout(new BorderLayout());

		centerPanel.setBorder(new EmptyBorder(2, 2, 2, 2));
		add(centerPanel, BorderLayout.CENTER);

		southPanel = selectedImagesPanel();
		add(southPanel, BorderLayout.SOUTH);

		invalidate();
		revalidate();
		repaint();
		parentDialog.pack();
	}

	public void updateSelectedIconsPanel() {
		remove(southPanel);
		southPanel = selectedImagesPanel();
		add(southPanel, BorderLayout.SOUTH);
		invalidate();
		revalidate();
		repaint();
	}

	private JDialog proceedRemoveDialog = null;
	private JButton proceedRemoveBtn = null;
	private JButton cancelBtn = null;

	private void removeHandshapes() {

		if (proceedRemoveDialog == null) {
			proceedRemoveDialog = new JDialog(parentDialog, "Handshapes ", true);

			cancelBtn = new JButton("Cancel");
			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					twoHanded.setSelected(true);

					passiveBaseArm.setEnabled(true);

					proceedRemoveDialog.dispose();
				}
			});

			proceedRemoveBtn = new JButton("Proceed");
			proceedRemoveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					if (oneHanded.isSelected()) {
						selectOnehandedSign();
					}

					// ensure switch parameters on proceed
					if(isFingerspelled() || isNumber() || isLoan()) {
						sameDifStartEndHandshapes.setVisible(false);
						setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesNdom.setVisible(false);
						setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesDom.setVisible(false);
						setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
						markedForm.setSelected(false);
					}

					// default for all types for proceeding to switch to one-handed is same handshapes on dom/ndom
					sameDifDomNdomHandshapes.setSelectedIndex(0);

					passiveBaseArm.setSelected(false);
					passiveBaseArm.setEnabled(false);

					if (domHPanel != null && ndomHPanel != null) {
//						SelectedHandshapesPanel domSHPanel, ndomSHPanel;
//						if (associatedFieldsPanel.isDominantHandRight()) {
//							domSHPanel = domHPanel;
//							ndomSHPanel = ndomHPanel;
//						} else {
//							domSHPanel = ndomHPanel;
//							ndomSHPanel = domHPanel;
//						}

                        String fieldID = gestureInfoPanel.getCurrentGestureEntity().getField().getFieldID();

						if (fieldID.equals(Util.DOM_GESTURE_FIELD_ID)) {
							ndomHPanel.setHandshapes(new ArrayList<HandShapeImage>());
							ndomHPanel.setFingerTipOrientationsImages(new ArrayList<FingerTipImage>());
							ndomHPanel.setPalmOrientationsImages(new ArrayList<PalmImage>());
							ndomHPanel.repaint();
							ndomHPanel.revalidate();
							hshFormBean.setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
							hshFormBean.setSelectedNdomOrientationFingerTipImages(new ArrayList<FingerTipImage>());
							hshFormBean.setSelectedNdomPalmOrientationImages(new ArrayList<PalmImage>());
						} else {
							domHPanel.setHandshapes(new ArrayList<HandShapeImage>());
							domHPanel.setFingerTipOrientationsImages(new ArrayList<>());
							domHPanel.setPalmOrientationsImages(new ArrayList<>());
							domHPanel.repaint();
							domHPanel.revalidate();
							hshFormBean.setSelectedDomHandShapeImages(new ArrayList<HandShapeImage>());
							hshFormBean.setSelectedDomOrientationFingerTipImages(new ArrayList<>());
							hshFormBean.setSelectedDomPalmOrientationImages(new ArrayList<>());
						}
					}

					proceedRemoveDialog.dispose();
				}
			});

			proceedRemoveDialog.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent we) {
					twoHanded.setSelected(true);
				}
			});

			JLabel l1 = new JLabel(" If you complete this operation,");
			JLabel l2 = new JLabel(
					"previously selected handshapes will be discarded.");
			JLabel l3 = new JLabel("");
			JLabel l4 = new JLabel("(This operation cannot be undone.)");
			JLabel l5 = new JLabel("");

			JPanel warningPanel = new JPanel();

			GridBagConstraints c = new GridBagConstraints();
			warningPanel.setLayout(new GridBagLayout());

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(1, 1, 1, 1);

			c.gridy = 0;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l1, c);

			c.gridy = 1;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l2, c);

			c.gridy = 2;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l3, c);

			c.gridy = 3;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l4, c);

			c.gridy = 4;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l5, c);

			ButtonPanel buttonPanel = new ButtonPanel();
			buttonPanel.add(cancelBtn);
			buttonPanel.add(proceedRemoveBtn);

			c.gridy = 5;
			c.gridx = 2;
			c.gridwidth = 1;
			c.anchor = GridBagConstraints.EAST;
			warningPanel.add(buttonPanel, c);

			c.anchor = GridBagConstraints.WEST;
			c.gridy = 6;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(new JLabel(""), c);

			warningPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10,
					10));

			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(warningPanel, BorderLayout.CENTER);

			Container contentPane = proceedRemoveDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);
			proceedRemoveDialog.pack();
		}

		Point p = parentDialog.getLocation();
		int dx = (parentDialog.getWidth() - proceedRemoveDialog.getWidth()) / 2;
		int dy = (parentDialog.getHeight() - proceedRemoveDialog.getHeight()) / 2;

		int x = (p.x + dx);
		int y = (p.y + dy);

		proceedRemoveDialog.setLocation(x, y);
		proceedRemoveDialog.setVisible(true);
	}

	@Override
	public void itemStateChanged(ItemEvent event) {

		JRadioButton obj = (JRadioButton) event.getSource();
		String str = obj.getActionCommand();

		switch (Integer.parseInt(str)) {

			case ONE_HANDED:
				selectHandshapes.requestFocus();
				selectHandshapes.requestFocus(true);
				selectHandshapes.requestFocusInWindow();

				if (domHPanel != null && ndomHPanel != null) {

					ArrayList<HandShapeImage> rhList1 = domHPanel.getHandshapes();
					ArrayList<HandShapeImage> lhList1 = ndomHPanel.getHandshapes();

					if (oneHanded.isSelected() && !rhList1.isEmpty()
							&& !lhList1.isEmpty()) {
						removeHandshapes();
					} else if (oneHanded.isSelected()) {
						selectOnehandedSign();
					}
				} else if (oneHanded.isSelected()) {
					selectOnehandedSign();

					if(isFingerspelled() || isNumber()
							|| isLoan()) {
						sameDifDomNdomHandshapes.setVisible(false);
						setSelectedSameDifDomNdomHandshapes(GestureDomNdomHandshapesType.SAME);
						sameDifStartEndHandshapes.setVisible(false);
						setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesNdom.setVisible(false);
						setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesDom.setVisible(false);
						setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
					}

					//Setting marked # hands to false as this is the default option in all cases.
					markedForm.setSelected(false);
				}

				break;

			case TWO_HANDED:

				if (twoHanded.isSelected()) {
					// dominantHand.setEnabled(false);
					// nonDominantHand.setEnabled(false);
					selectHandshapes.setEnabled(true);

					if(isFingerspelled() || isNumber() || isLoan()) {
						sameDifDomNdomHandshapes.setVisible(true);
						setSelectedSameDifDomNdomHandshapes(GestureDomNdomHandshapesType.SAME);
						sameDifStartEndHandshapes.setVisible(false);
						setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesNdom.setVisible(false);
						setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesDom.setVisible(false);
						setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
					} else {
						setSelectedSameDifDomNdomHandshapes(GestureDomNdomHandshapesType.SAME);
						sameDifDomNdomHandshapes.setVisible(true);
						setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapes.setVisible(true);
						setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesNdom.setVisible(true);
						setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesDom.setVisible(true);
					}


					//
                // boolean changed = isChanged();
                //
//                associatedFieldsPanel.getEnterButton().setEnabled(true);

                //

					// if same handshapes L&R are selected and L or R handshape is
					// populated, double the selection
					removeAll();
					init();

					// if(sameDifHandshapes.getSelectedIndex() == 0) {
					ArrayList<HandShapeImage> rhList = domHPanel.getHandshapes();
					ArrayList<HandShapeImage> lhList = ndomHPanel.getHandshapes();
					ArrayList<FingerTipImage> rOList = domHPanel.getFingerTipOrientationsImages();
					ArrayList<FingerTipImage> lOList = ndomHPanel.getFingerTipOrientationsImages();
					ArrayList<PalmImage> rPList = domHPanel.getPalmOrientationsImages();
					ArrayList<PalmImage> lPList = ndomHPanel.getPalmOrientationsImages();

					if (rhList.isEmpty() && !lhList.isEmpty()) {
						domHPanel.setHandshapes(lhList);
						domHPanel.repaint();
						domHPanel.revalidate();
                    hshFormBean.setSelectedDomHandShapeImages(lhList);
					}

					if (rOList.isEmpty() && !lOList.isEmpty()) {
						domHPanel.setFingerTipOrientationsImages(lOList);
						domHPanel.repaint();
						domHPanel.revalidate();
                    hshFormBean.setSelectedDomOrientationFingerTipImages(lOList);
					}

					if (lhList.isEmpty() && !rhList.isEmpty()) {
						ndomHPanel.setHandshapes(rhList);
						ndomHPanel.repaint();
						ndomHPanel.revalidate();
                    hshFormBean.setSelectedNdomHandShapeImages(rhList);
					}

					if (lOList.isEmpty() && !rOList.isEmpty()) {
						ndomHPanel.setFingerTipOrientationsImages(rOList);
						ndomHPanel.repaint();
						ndomHPanel.revalidate();
                    hshFormBean.setSelectedNdomOrientationFingerTipImages(rOList);
					}

					if (rPList.isEmpty() && !lPList.isEmpty()) {
						domHPanel.setPalmOrientationsImages(lPList);
						domHPanel.repaint();
						domHPanel.revalidate();
						hshFormBean.setSelectedDomPalmOrientationImages(lPList);
					}

					if (lPList.isEmpty() && !rPList.isEmpty()) {
						ndomHPanel.setPalmOrientationsImages(rPList);
						ndomHPanel.repaint();
						ndomHPanel.revalidate();
						hshFormBean.setSelectedNdomPalmOrientationImages(rPList);
					}

					if(isFingerspelled() || isNumber()
							|| isLoan()) {
						markedForm.setSelected(true);
						sameDifStartEndHandshapes.setVisible(false);
						setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesNdom.setVisible(false);
						setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
						sameDifStartEndHandshapesDom.setVisible(false);
						setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
					} else {
						//For all non finger, number and loan signs, set marked # of hands to false.  This is to preserve the default behavior
						markedForm.setSelected(false);

						sameDifStartEndHandshapes.setVisible(true);
						sameDifStartEndHandshapesNdom.setVisible(true);
						sameDifStartEndHandshapesDom.setVisible(true);
					}

					passiveBaseArm.setEnabled(true);
					passiveBaseArm.setSelected(false);
				}
				break;
		}
	}

	private void selectOnehandedSign() {
		// dominantHand.setEnabled(true);
		// nonDominantHand.setEnabled(true);
		selectHandshapes.setEnabled(true);

		removeAll();
		init();
	}

	private SelectedGestureHandshapesPanel createGestureSelectionPanel(HandShapesPaletteFormBean bean, SS3CodingScheme scheme) {
		if (bean.getValues() == null || bean.getValues().isEmpty()) {
			return new SelectedGestureHandshapesPanel(scheme);
		}
		return new SelectedGestureHandshapesPanel(bean.getSelectedHshImages(), bean.getSelectedFingerTipOrientationImages(), bean.getSelectedPalmOrientationImages(), scheme, this);
	}

	private void configurePaletteFormBean(HandShapesPaletteFormBean domHandFormBean, HandShapesPaletteFormBean nonDomHandFormBean,
										  ArrayList<HandShapeImage> domHandImages, ArrayList<HandShapeImage> nonDomHandImages,
										  ArrayList<FingerTipImage> domFingerTipOrientations, ArrayList<FingerTipImage> nonDomFingerTipOrientations,
										  ArrayList<PalmImage> domPalmOrientations, ArrayList<PalmImage> nonDomPalmOrientations,
										  ArrayList<SS3FieldValue> domHandList, ArrayList<SS3FieldValue> nonDomHandList, boolean isDominantHandRight,
										  String RIGHT_HAND_TITLE, String LEFT_HAND_TITLE, HandShapesPaletteFormBean.PaletteType domHandPaletteType, HandShapesPaletteFormBean.PaletteType nonDomHandPaletteType) {

		domHandFormBean.setSelectedHshImages(domHandImages);
		domHandFormBean.setSelectedFingerTipOrientationImages(domFingerTipOrientations);
		domHandFormBean.setSelectedPalmOrientationImages(domPalmOrientations);
		domHandFormBean.setValues(domHandList);
		domHandFormBean.setRightHandSelected(isDominantHandRight);
		domHandFormBean.setTitle(RIGHT_HAND_TITLE);
		domHandFormBean.setPaletteType(domHandPaletteType);

		nonDomHandFormBean.setSelectedHshImages(nonDomHandImages);
		nonDomHandFormBean.setSelectedFingerTipOrientationImages(nonDomFingerTipOrientations);
		nonDomHandFormBean.setSelectedPalmOrientationImages(nonDomPalmOrientations);
		nonDomHandFormBean.setValues(nonDomHandList);
		nonDomHandFormBean.setRightHandSelected(!isDominantHandRight);
		nonDomHandFormBean.setTitle(LEFT_HAND_TITLE);
		nonDomHandFormBean.setPaletteType(nonDomHandPaletteType);
	}

	// private JDialog d = null;

    private GestureHandshapeDialog sameHandDialog = null;
    private GestureDblHandshapeDialog twoHandedDialog = null;

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		switch (Integer.parseInt(str)) {

			case SELECT_HANDSHAPES:
				HandShapesPaletteFormBean palletFormBean = new HandShapesPaletteFormBean();

				SS3CodingScheme scheme = this.hshFormBean.getSS3CodingScheme();
				palletFormBean.setCodingScheme(scheme);

				CSHandShapeTemplates templates = scheme.getHandShapeTemplates();
				CSHandShapeTemplate handShapeTemplate = templates.getSelectedHandShapeTemplate();

				boolean isDominantHandRight = gestureInfoPanel.isDominantHandRight();

				String rstr = isDominantHandRight? "(Dom)" : "(NonDom)";
				String lstr = isDominantHandRight ? "(NonDom)" : "(Dom)";

				String RIGHT_HAND_TITLE = "Right " + rstr + " Hand";
				String LEFT_HAND_TITLE = "Left " + lstr + " Hand";
				String RIGHT_N_LEFT_HANDS_TITLE = "Right " + rstr + " and Left " + lstr + " Hands";

				if (twoHanded.isSelected()) {
					// 2-handed
					int sameDifHandshapesSelectedIndex = sameDifDomNdomHandshapes.getSelectedIndex();

					if (sameDifHandshapesSelectedIndex == 0 || sameDifHandshapesSelectedIndex == 1) {
						// one palette for both hands
						String hndType;

						if (dominantHand.isSelected()) {
							palletFormBean.setSelectedHshImages(bean.getSelectedDomHandShapeImages());
							palletFormBean.setSelectedFingerTipOrientationImages(bean.getSelectedDomOrientationFingerTipImages());
							palletFormBean.setSelectedPalmOrientationImages(bean.getSelectedDomOrientationPalmImages());
							hndType = Constants.TYPE_DOM_GESTURE_HANDSHAPE;
							if (gestureInfoPanel.isDominantHandRight()) {
								palletFormBean.setRightHandSelected(true);
							} else {
								palletFormBean.setRightHandSelected(false);
							}
						} else {
							palletFormBean.setSelectedHshImages(bean.getSelectedNdomHandShapeImages());
							palletFormBean.setSelectedFingerTipOrientationImages(bean.getSelectedNdomOrientationFingerTipImages());
							palletFormBean.setSelectedPalmOrientationImages(bean.getSelectedNdomOrientationPalmImages());
							hndType = Constants.TYPE_NDOM_GESTURE_HANDSHAPE;
							if (gestureInfoPanel.isDominantHandRight()) {
								palletFormBean.setRightHandSelected(false);
							} else {
								palletFormBean.setRightHandSelected(true);
							}
						}

						ArrayList<CSPaletteItemGroup> groups = handShapeTemplate.getPalettesAsHashMap().get(this.paletteID).getGroups();
						ArrayList<SS3FieldValue> list = scheme.getCorrespondingFieldValues(groups, hndType);

						palletFormBean.setValues(list);

						palletFormBean.setTitle(RIGHT_N_LEFT_HANDS_TITLE);
						palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT);

						loadOrUpdateSameHandshapeDialog(palletFormBean, bean, this,
                            hshFormBean.getSS3CodingScheme(), parentDialog,
                            this.paletteID);

					} else {
						// two palettes
						HandShapesPaletteFormBean palletFormBean2 = new HandShapesPaletteFormBean();

						ArrayList<HandShapeImage> selectedDomHandShapeImages = bean.getSelectedDomHandShapeImages();
						ArrayList<HandShapeImage> selectedNdomHandShapeImages = bean.getSelectedNdomHandShapeImages();
						ArrayList<FingerTipImage> selectedDomOrientationImages = bean.getSelectedDomOrientationFingerTipImages();
						ArrayList<FingerTipImage> selectedNdomOrientationImages = bean.getSelectedNdomOrientationFingerTipImages();
						ArrayList<PalmImage> selectedDomPalmOrientationImages = bean.getSelectedDomOrientationPalmImages();
						ArrayList<PalmImage> selectedNdomPalmOrientationImages = bean.getSelectedNdomOrientationPalmImages();

						ArrayList<HandShapeImage> domHandImages, nonDomHandImages;
						ArrayList<FingerTipImage> domFingerTipOrientations, nonDomFingerTipOrientations;
						ArrayList<PalmImage> domPalmOrientations, nonDomPalmOrientaitons;

						// TODO: Hasnain, we should not have confusing rightHand leftHand names when right is not dominant. That is dumb.

						domHandImages = selectedDomHandShapeImages; // Right hand is always treated as dom-hand, even when it's not... So, technically, rightHand can have leftHand info when leftHand is dominant.
						nonDomHandImages = selectedNdomHandShapeImages;
						domFingerTipOrientations = selectedDomOrientationImages;
						nonDomFingerTipOrientations = selectedNdomOrientationImages;
						domPalmOrientations = selectedDomPalmOrientationImages;
						nonDomPalmOrientaitons = selectedNdomPalmOrientationImages;

						ArrayList<CSPaletteItemGroup> domHandUnmarkedItems = handShapeTemplate.getPalettesAsHashMap().get(this.paletteID).getGroups();
						ArrayList<SS3FieldValue> domHandList = scheme.getCorrespondingFieldValues(domHandUnmarkedItems, Constants.TYPE_DOM_GESTURE_HANDSHAPE);
						ArrayList<CSPaletteItemGroup> nonDomHandUnmarkedItems = handShapeTemplate.getPalettesAsHashMap().get(this.paletteID).getGroups();
						ArrayList<SS3FieldValue> nonDomHandList = scheme.getCorrespondingFieldValues(nonDomHandUnmarkedItems, Constants.TYPE_DOM_GESTURE_HANDSHAPE);

						if (this.isDominantHandRight()) {
							this.configurePaletteFormBean(palletFormBean, palletFormBean2,
									domHandImages, nonDomHandImages,
									domFingerTipOrientations, nonDomFingerTipOrientations,
									domPalmOrientations, nonDomPalmOrientaitons, domHandList, nonDomHandList,
									isDominantHandRight, RIGHT_HAND_TITLE, LEFT_HAND_TITLE, HandShapesPaletteFormBean.PaletteType.RIGHT, HandShapesPaletteFormBean.PaletteType.LEFT);
						} else {
							this.configurePaletteFormBean(palletFormBean2, palletFormBean,
									domHandImages, nonDomHandImages,
									domFingerTipOrientations, nonDomFingerTipOrientations,
									domPalmOrientations, nonDomPalmOrientaitons, domHandList, nonDomHandList,
									isDominantHandRight, RIGHT_HAND_TITLE, LEFT_HAND_TITLE, HandShapesPaletteFormBean.PaletteType.LEFT, HandShapesPaletteFormBean.PaletteType.RIGHT);
						}

						SelectedGestureHandshapesPanel domHandSelectionPanel = this.isDominantHandRight() ? this.createGestureSelectionPanel(palletFormBean, scheme) : this.createGestureSelectionPanel(palletFormBean2, scheme);
						SelectedGestureHandshapesPanel nonDomHandSelectionPanel = this.isDominantHandRight() ? this.createGestureSelectionPanel(palletFormBean2, scheme) : this.createGestureSelectionPanel(palletFormBean, scheme);

						domHandSelectionPanel.setPreferredSize(new Dimension(domHandSelectionPanel.getPreferredSize().width, 110));
						nonDomHandSelectionPanel.setPreferredSize(new Dimension(nonDomHandSelectionPanel.getPreferredSize().width, 110));

						GeneralDoublePaletteDialog<SelectedGestureHandshapesPanel, GestureStartEndHandshapesType> doublePaletteDialog;

                        doublePaletteDialog = new GestureDoublePaletteDialog<>(palletFormBean, palletFormBean2, this.bean,
                                this, domHandSelectionPanel, nonDomHandSelectionPanel,
                                this.hshFormBean.getSS3CodingScheme(), this.parentDialog,
                                Util.DEFAULT_GESTURE_MENU_ID, this.paletteID,
                                Util.DEFAULT_GESTURE_FINGER_TIP_ORIENTATION_GROUP_ID,
                                Util.DEFAULT_GESTURE_PALM_ORIENTATION_PALETTE_GROUP_ID);

                        doublePaletteDialog.setLocationRelativeTo(this.parentDialog);
						doublePaletteDialog.setVisible(true);
						doublePaletteDialog.toFront();
					}
				} else {
					ArrayList<CSPaletteItemGroup> items = new ArrayList<>(handShapeTemplate.getPalettesAsHashMap().get(this.paletteID).getGroups());

					if (dominantHand.isSelected()) {
						palletFormBean.setSelectedHshImages(bean.getSelectedDomHandShapeImages());
						palletFormBean.setSelectedFingerTipOrientationImages(bean.getSelectedDomOrientationFingerTipImages());
						palletFormBean.setSelectedPalmOrientationImages(bean.getSelectedDomOrientationPalmImages());
						palletFormBean.setValues(scheme.getCorrespondingFieldValues(items, Constants.TYPE_DOM_GESTURE_HANDSHAPE));
						if (gestureInfoPanel.isDominantHandRight()) {
							palletFormBean.setRightHandSelected(true);
							palletFormBean.setTitle(RIGHT_HAND_TITLE);
							palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT);
						} else {
							palletFormBean.setRightHandSelected(false);
							palletFormBean.setTitle(LEFT_HAND_TITLE);
							palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.LEFT);
						}
					} else if (nonDominantHand.isSelected()) {
						palletFormBean.setSelectedHshImages(bean.getSelectedNdomHandShapeImages());
						palletFormBean.setSelectedFingerTipOrientationImages(bean.getSelectedNdomOrientationFingerTipImages());
						palletFormBean.setSelectedPalmOrientationImages(bean.getSelectedNdomOrientationPalmImages());
						palletFormBean.setValues(scheme.getCorrespondingFieldValues(items, Constants.TYPE_NDOM_GESTURE_HANDSHAPE));
						if (gestureInfoPanel.isDominantHandRight()) {
							palletFormBean.setRightHandSelected(false);
							palletFormBean.setTitle(LEFT_HAND_TITLE);
							palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.LEFT);
						} else {
							palletFormBean.setRightHandSelected(true);
							palletFormBean.setTitle(RIGHT_HAND_TITLE);
							palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT);
						}
					}

					loadOrUpdateSameHandshapeDialog(palletFormBean, bean, this, hshFormBean.getSS3CodingScheme(), this.parentDialog, this.paletteID);
				}

				break;

			case CLEAR_HANDSHAPES:
				bean.clearDomHandShapeImages();
				bean.clearNdomHandShapeImages();
				bean.clearDomOrientationFingerTipImages();
				bean.clearNdomOrientationFingerTipImages();
				bean.clearDomOrientationPalmImages();
				bean.clearNdomOrientationPalmImages();

				hshFormBean.setSelectedNdomHandShapeImages(new ArrayList<>());
				hshFormBean.setSelectedDomHandShapeImages(new ArrayList<>());
				hshFormBean.setSelectedDomOrientationFingerTipImages(new ArrayList<>());
				hshFormBean.setSelectedNdomOrientationFingerTipImages(new ArrayList<>());
				hshFormBean.setSelectedDomPalmOrientationImages(new ArrayList<>());
				hshFormBean.setSelectedNdomPalmOrientationImages(new ArrayList<>());

				if (sameHandDialog != null && isFingerspelled()) {
					sameHandDialog.getPalette().getPalette().actionPerformed(null);    // this is to clear the text inside the palette window
				}
				updateSelectedIconsPanel();
				break;

		}

		// Enable/Disable Sign Bank buttons based on this sign's type
		gestureInfoPanel.checkSignTypeAndUpdateSignBankFunctionality();
	}

    private void loadOrUpdateSameHandshapeDialog(HandShapesPaletteFormBean formBean,
												 HandShapeFormBean bean,
                                                 GestureHandShapePanel panel,
                                                 SS3CodingScheme scheme,
                                                 JDialog dialog, String paletteID) {
        if(twoHandedDialog != null) {
			twoHandedDialog.setVisible(false);
		}

		SelectedGestureHandshapesPanel selectedGesturePanel;

		if (formBean.getValues().isEmpty() || formBean.getValues() == null) {
			selectedGesturePanel = new SelectedGestureHandshapesPanel(scheme);
		} else {
			selectedGesturePanel = new SelectedGestureHandshapesPanel(formBean.getSelectedHshImages(), formBean.getSelectedFingerTipOrientationImages(), formBean.getSelectedPalmOrientationImages(), scheme, this);
		}

		selectedGesturePanel.setPreferredSize(new Dimension(selectedGesturePanel.getPreferredSize().width, 110));
		// TODO: Hasnain, for my OCD sake, the finger tip-palm ids should be 2 then 3, not 3 then 2... Need to change XML for this fix and update Util.java
		GesturePaletteDialog<SelectedGestureHandshapesPanel, GestureStartEndHandshapesType> gesturePaletteDialog = new GesturePaletteDialog<>(formBean, bean, scheme, dialog, panel, selectedGesturePanel, Util.DEFAULT_GESTURE_MENU_ID, paletteID,
				Util.DEFAULT_GESTURE_FINGER_TIP_ORIENTATION_GROUP_ID, Util.DEFAULT_GESTURE_PALM_ORIENTATION_PALETTE_GROUP_ID);
		gesturePaletteDialog.setLocationRelativeTo(null); //parentDialog);
		gesturePaletteDialog.setLocation(this.parentDialog.getWidth() + 15, 10);
		gesturePaletteDialog.setVisible(true);


/*		GeneralPaletteDialog generalPaletteDialog = new GeneralPaletteDialog(formBean, bean, scheme, dialog, this, Util.DEFAULT_GESTURE_PALETTE_MENU_GROUP_ID);
		generalPaletteDialog.toFront();
		generalPaletteDialog.setVisible(true);*/


    }

	/*public void selectedHandshapesPanelUpdated() {
		ArrayList<HandShapeImage> ndomImages =  new ArrayList<HandShapeImage>(bean.getSelectedNdomHandShapeImages());
		ArrayList<HandShapeImage> domImages =  new ArrayList<HandShapeImage>(bean.getSelectedDomHandShapeImages());


		if (justLoaded) {
			justLoaded = false;
		} else {

//			JButton enterHandshapes = associatedFieldsPanel.getEnterButton();
//
//			if (twoHanded.isSelected()) {
//				if (!ndomImages.isEmpty() && !domImages.isEmpty()) {
//					enterHandshapes.setEnabled(true);
//				} else {
//					enterHandshapes.setEnabled(false);
//				}
//			} else if (oneHanded.isSelected()) {
//				if (ndomImages.isEmpty() && domImages.isEmpty()) {
//					enterHandshapes.setEnabled(false);
//				} else {
//					enterHandshapes.setEnabled(true);
//				}
//			}
		}
	}*/

	public boolean isOneHandedSelected() {
		return oneHanded.isSelected();
	}

	public boolean isTwoHandedSelected() {
		return twoHanded.isSelected();
	}

	/**
	 * @return
	 */
	@Override
	public boolean isDominantHandRight() {
		return this.gestureInfoPanel.isDominantHandRight();
	}

	@Override
	public boolean isDomHandSelected() {
		return dominantHand.isSelected();
	}

	@Override
	public boolean isNondomHandSelected() {
		return nonDominantHand.isSelected();
	}


	public String getSelectedSameDifferentDomNdomHandshapes() {
		int selectedIndex = sameDifDomNdomHandshapes.getSelectedIndex();
		if (selectedIndex == 0) {
			return GestureEntityProperties.SAME_HANDSHAPES_DOM_NDOM;
		} else if (selectedIndex == 1) {
			return GestureEntityProperties.ALTERNATING_SAME_HANDSHAPES_DOM_NDOM;
		} else {
			return GestureEntityProperties.DIFFERENT_HANDSHAPES_DOM_NDOM;
		}
	}

	public String getSelectedSameDifferentStartEndHandshapes() {
		int selectedIndex = sameDifStartEndHandshapes.getSelectedIndex();
		if (selectedIndex == 0) {
			return GestureEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	public String getSelectedSameDifferentNdomHandshape() {
		int selectedIndex = sameDifStartEndHandshapesNdom.getSelectedIndex();
		if (selectedIndex == 0) {
			return GestureEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	public String getSelectedSameDifferentDomHandshape() {
		int selectedIndex = sameDifStartEndHandshapesDom.getSelectedIndex();
		if (selectedIndex == 0) {
			return GestureEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GestureEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	//NOTES:
	//Current approach is just to check if there are any handshapes currently
	//if not, then "defaults should apply"
	//not sure if/how toggling visibility settings of the SameDif stuff should work
	//changing sign type from lexical, to, say, fingerspelled, no longer gets rid of
	//the other info but the display in utterance view seems to disappear.

	void onFingerspelledLoanNumberSelected() {
		//look at whether or not these visibility settings should be toggled by default or not
		setSelectedSameDifStartEndHandshapesVisible(false);
		setSelectedSameDifStartEndHandshapesNdomVisible(false);
		setSelectedSameDifStartEndHandshapesDomVisible(false);
		//if there were no previously selected signs, set defaults
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			selectOneHanded();
			setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
			setPassiveBaseArmEnabled(false);
		}
	}

	void onLexicalClassifierSelected() {
		setSelectedSameDifStartEndHandshapesVisible(true);
		setSelectedSameDifDomNdomHandshapesVisible(true);
		//if there were no previously selected signs, set defaults
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			selectTwoHanded();
			setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
			setSelectedSameDifDomNdomHandshapes(GestureDomNdomHandshapesType.SAME);
			setMarkedNumberOfHands(false);
			setPassiveBaseArmSelected(false);
		}
	}

	void onGestureSelected() {
		setSelectedSameDifStartEndHandshapesVisible(true);
		setSelectedSameDifStartEndHandshapesNdomVisible(true);
		setSelectedSameDifStartEndHandshapesDomVisible(true);
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			selectOneHanded();
			setPassiveBaseArmEnabled(false);
			setMarkedNumberOfHands(false);
			setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
		}
	}

	public boolean isFingerspelled() {
		return fingerspelled.isSelected();
	}

	public void setFingerspelled(boolean selected) {
		fingerspelled.setSelected(selected);
	}

	public boolean isLoan() {
		return loan.isSelected();
	}

	public void setLoan(boolean selected) {
		loan.setSelected(selected);
	}

	public boolean isNumber() {
		return number.isSelected();
	}

	public void setNumber(boolean selected) {
		number.setSelected(selected);
	}

	public boolean isLexical() {
		return lexical.isSelected();
	}

	public void setLexical(boolean selected) {
		lexical.setSelected(selected);
	}

	public boolean isClassifier() {
		return classifier.isSelected();
	}

	public void setClassifier(boolean selected) {
		classifier.setSelected(selected);
	}

	public boolean isGesture() {
		return gesture.isSelected();
	}

	public void setGesture(boolean selected) {
		gesture.setSelected(selected);
	}

	public boolean isNameSign() {
		return nameSign.isSelected();
	}

	public void setNameSign(boolean selected) {
		nameSign.setSelected(selected);
	}

	public GestureInfoPanel getAssociatedFieldsPanel() {
		return gestureInfoPanel;
	}

	/**
	 * @param domHandshapes
	 * @param ndomHandshapes
	 */
}

class CenteredTextCellRenderer extends JLabel implements ListCellRenderer<Object> {
	private static final long serialVersionUID = 1L;

	public CenteredTextCellRenderer() {
	}

	@Override
	public Component getListCellRendererComponent(JList list, Object value,
												  int index, boolean isSelected, boolean cellHasFocus) {

		setHorizontalAlignment(SwingConstants.CENTER);
		setText(value.toString());
		return this;
	}
}

class SameHandshapesDomNdomComboBoxListener implements ActionListener {
	private GestureHandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesDomNdomComboBoxListener(GestureHandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {
			panel.removeAll();
			panel.init();

			GestureDomNdomHandshapesType selectedSameDifDomNdomHandshapesIndex = panel.getSelectedSameDifDomNdomHandshapes();
			HandShapeFormBean bean = panel.getHandShapeBean();

			if (selectedSameDifDomNdomHandshapesIndex == GestureDomNdomHandshapesType.SAME || selectedSameDifDomNdomHandshapesIndex == GestureDomNdomHandshapesType.ALTERNATING) {
				if (panel.isOneHandedSelected()) {
					if (panel.isDomHandSelected()) {
						bean.setSelectedNdomHandShapeImages(new ArrayList<>());
						bean.setSelectedNdomOrientationFingerTipImages(new ArrayList<>());
						bean.setSelectedNdomPalmOrientationImages(new ArrayList<>());
						panel.getHandShapeFormBean().setSelectedNdomHandShapeImages(new ArrayList<>());
						panel.getHandShapeFormBean().setSelectedNdomOrientationFingerTipImages(new ArrayList<>());
						panel.getHandShapeFormBean().setSelectedNdomPalmOrientationImages(new ArrayList<>());
					} else {
						bean.setSelectedDomHandShapeImages(new ArrayList<>());
						bean.setSelectedDomOrientationFingerTipImages(new ArrayList<>());
						bean.setSelectedDomPalmOrientationImages(new ArrayList<>());
						panel.getHandShapeFormBean().setSelectedDomHandShapeImages(new ArrayList<>());
						panel.getHandShapeFormBean().setSelectedDomOrientationFingerTipImages(new ArrayList<>());
						panel.getHandShapeFormBean().setSelectedDomPalmOrientationImages(new ArrayList<>());
					}
				} else {
					if (panel.isDomHandSelected() || bean.getSelectedNdomHandShapeImages().isEmpty()) {
						bean.setSelectedNdomHandShapeImages(bean.getSelectedDomHandShapeImages());
						bean.setSelectedNdomOrientationFingerTipImages(bean.getSelectedDomOrientationFingerTipImages());
						bean.setSelectedNdomPalmOrientationImages(bean.getSelectedDomOrientationPalmImages());
						panel.getHandShapeFormBean().setSelectedNdomHandShapeImages(bean.getSelectedDomHandShapeImages());
						panel.getHandShapeFormBean().setSelectedNdomOrientationFingerTipImages(bean.getSelectedDomOrientationFingerTipImages());
						panel.getHandShapeFormBean().setSelectedNdomPalmOrientationImages(bean.getSelectedDomOrientationPalmImages());
					} else {
						bean.setSelectedDomHandShapeImages(bean.getSelectedNdomHandShapeImages());
						bean.setSelectedDomOrientationFingerTipImages(bean.getSelectedNdomOrientationFingerTipImages());
						bean.setSelectedDomPalmOrientationImages(bean.getSelectedNdomOrientationPalmImages());
						panel.getHandShapeFormBean().setSelectedDomHandShapeImages(bean.getSelectedNdomHandShapeImages());
						panel.getHandShapeFormBean().setSelectedDomOrientationFingerTipImages(bean.getSelectedNdomOrientationFingerTipImages());
						panel.getHandShapeFormBean().setSelectedDomPalmOrientationImages(bean.getSelectedNdomOrientationPalmImages());
					}
				}
			} else if (selectedSameDifDomNdomHandshapesIndex == GestureDomNdomHandshapesType.DIFFERENT) {
				if (panel.isDomHandSelected()) {
					bean.clearNdomHandShapeImages();
					bean.clearNdomOrientationFingerTipImages();
					bean.clearNdomOrientationPalmImages();
					panel.getHandShapeFormBean().setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
					panel.getHandShapeFormBean().setSelectedNdomOrientationFingerTipImages(new ArrayList<>());
					panel.getHandShapeFormBean().setSelectedNdomPalmOrientationImages(new ArrayList<>());
				} else {
					bean.clearDomHandShapeImages();
					bean.clearDomOrientationFingerTipImages();
					bean.clearDomOrientationPalmImages();
					panel.getHandShapeFormBean().setSelectedDomHandShapeImages(new ArrayList<HandShapeImage>());
					panel.getHandShapeFormBean().setSelectedDomOrientationFingerTipImages(new ArrayList<>());
					panel.getHandShapeFormBean().setSelectedDomPalmOrientationImages(new ArrayList<>());
				}
			}
			panel.updateSelectedIconsPanel();
		}
	}
}

class SameHandshapesStartEndComboBoxListener implements ActionListener {
	private GestureHandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndComboBoxListener(GestureHandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {
			GestureStartEndHandshapesType selectedSameDifStartEndHandshapesIndex = panel.getSelectedSameDifStartEndHandshapes();

			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesIndex == GestureStartEndHandshapesType.DIFFERENT && panel.isOneFrameEntity()) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANT_SET_DIF_START_END_ONE_FRAME);
				panel.setSelectedSameDifStartEndHandShapes(GestureStartEndHandshapesType.SAME);
			}
		}
	}
}

class SameHandshapesStartEndDomComboBoxListener implements ActionListener {
	private GestureHandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndDomComboBoxListener(GestureHandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {
			GestureStartEndHandshapesType selectedSameDifStartEndHandshapesDomIndex = panel.getSelectedSameDifStartEndHandshapesDom();

			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesDomIndex == GestureStartEndHandshapesType.DIFFERENT && panel.isOneFrameEntity()) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANT_SET_DIF_START_END_ONE_FRAME);
				panel.setSelectedSameDifStartEndHandshapesDom(GestureStartEndHandshapesType.SAME);
			}
		}
	}
}

class SameHandshapesStartEndNdomComboBoxListener implements ActionListener {
	private GestureHandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndNdomComboBoxListener(GestureHandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {
			GestureStartEndHandshapesType selectedSameDifStartEndHandshapesNdomIndex = panel.getSelectedSameDifStartEndHandshapesNdom();

			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesNdomIndex == GestureStartEndHandshapesType.DIFFERENT && panel.isOneFrameEntity()) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANT_SET_DIF_START_END_ONE_FRAME);
				panel.setSelectedSameDifStartEndHandshapesNdom(GestureStartEndHandshapesType.SAME);
			}
		}
	}
}
