package edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

/**
 * This class stores the IDs for parent regions if ever a reference to them is needed,
 * as well as a list of regions and location types parsed from the coding schema. It
 * also contains utility methods to find regions or location types by their IDs.
 */
public class GestureLocationConfig {
    public static final String REGION_FULL_BODY = "0";
    public static final String REGION_HEAD = "1";
    public static final String REGION_BODY = "2";
    public static final String REGION_ARM = "3";
    public static final String REGION_HAND = "4";

    public static String IMAGE_BASE_PATH;

    private final HashMap<String, GestureLocationImage> images = new HashMap<String, GestureLocationImage>();

    private final HashMap<String, GestureRegion> regions = new HashMap<String, GestureRegion>();

    private final ArrayList<GestureLocationType> locationTypes = new ArrayList<GestureLocationType>();

    public GestureLocationImage getImage(String id) {
        return images.get(id);
    }

    public void addImage(GestureLocationImage image) {
        images.put(image.getId(), image);
    }

    public GestureRegion getRegion(String id) {
        if (id.equals("-1")) {
            return GestureRegion.NONE;
        }
        return regions.get(id);
    }

    public HashMap<String, GestureRegion> getRegions() {
        return regions;
    }

    public void addRegion(GestureRegion rc) {
        regions.put(rc.getId(), rc);
    }

    public void addLocationType(GestureLocationType locType) {
        if (!locationTypes.contains(locType)) {
            locationTypes.add(locType);
        }
        Collections.sort(locationTypes, new Comparator<GestureLocationType>() {
            @Override
            public int compare(GestureLocationType o1, GestureLocationType o2) {
                return o1.getOrder() - o2.getOrder();
            }
        });
    }

    public ArrayList<GestureLocationType> getLocationTypes() {
        return locationTypes;
    }

    public GestureLocationType getLocationType(int id) {
        for (GestureLocationType locationType : locationTypes) {
            if (locationType.getId() == id) {
                return locationType;
            }
        }
        return null;
    }
}
