package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.border.Border;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.LettersHandshapesPalette;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.NumbersHandshapesPallete;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.PaletteUtil;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.SortedHandshapesPalette;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.UnmarkedHandshapesPalette;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.ui.ButtonPanel;

public class GestureGeneralHandshapePalette extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	private final SS3CodingScheme scheme;
	private final SelectedGestureHandshapesPanel panel;
	private final OrientationPalette orientationFingerTipPanel;
	private final OrientationPalette palmOrientationPanel;
	
	private JPanel client = null;
	protected JScrollPane sp = null;

	private final JButton clean = new JButton("Clear");
	
	private JPanel iconPanel = new JPanel();
	
	private final ArrayList<SS3FieldValue> values;
	private final ArrayList<HandShapeImage> selectedHshImages = new ArrayList<>();
	private final ArrayList<FingerTipImage> selectedOrientationFingerTipImages = new ArrayList<>();
	private final ArrayList<PalmImage> selectedPalmOrientationImages = new ArrayList<>();
	
	private PaletteUtil paletteUtil = null;
	
	private final static String GESTURE_PANEL_INDEX = "0";
	private final static String ALL_PANEL_INDEX = "1";
	private final static String LETTERS_PANEL_INDEX = "2";
	private final static String NUMBERS_PANEL_INDEX = "3";
	private final static String UNMARKED_PANEL_INDEX = "4";
	
	private final GestureHandshapesPalette gesturePanel = null;
	private final SortedHandshapesPalette allPanel = null;
	private final LettersHandshapesPalette lettersPanel = null;
	private final NumbersHandshapesPallete numbersPanel = null;
	UnmarkedHandshapesPalette unmarkedPanel = null;
	

	public GestureGeneralHandshapePalette(ArrayList<SS3FieldValue> values, ArrayList<HandShapeImage> selectedHshImages, ArrayList<FingerTipImage> selectedFingerTipOrientationImages, ArrayList<PalmImage> selectedPalmOrientationImages, SS3CodingScheme scheme, int selectedIndex, GestureHandShapePanel hsPanel) {
		this.scheme = scheme;
		this.values = values;
		this.selectedHshImages.addAll(selectedHshImages);
		this.selectedOrientationFingerTipImages.addAll(selectedFingerTipOrientationImages);
		this.selectedPalmOrientationImages.addAll(selectedPalmOrientationImages);
		
		if (values == null) {
            this.panel = new SelectedGestureHandshapesPanel(scheme);
		} else {
            this.panel = new SelectedGestureHandshapesPanel(selectedHshImages, selectedFingerTipOrientationImages, selectedPalmOrientationImages, scheme, hsPanel);
		}

		this.paletteUtil = new PaletteUtil(scheme, this.panel);

        this.iconPanel = new JPanel();
//        iconPanel.setLayout(new CardLayout());
//        
//        gesturePanel = new GestureHandshapesPalette_XYZ(values, panel, scheme);
//        iconPanel.add(gesturePanel, GESTURE_PANEL_INDEX);
//        
//        allPanel = new SortedHandshapesPalette(values, paletteUtil);
//        iconPanel.add(allPanel, ALL_PANEL_INDEX);
//        
//        lettersPanel = new LettersHandshapesPalette(values, paletteUtil,  selectedHshImages);
//        iconPanel.add(lettersPanel, LETTERS_PANEL_INDEX);
//        
//        numbersPanel = new NumbersHandshapesPallet(values, panel, scheme);
//        iconPanel.add(numbersPanel, NUMBERS_PANEL_INDEX);
//        
//        unmarkedPanel = new UnmarkedHandshapesPellet(values, panel, scheme);
//        iconPanel.add(unmarkedPanel, UNMARKED_PANEL_INDEX);

        this.orientationFingerTipPanel = new OrientationPalette(values, this.panel, scheme, Constants.GESTURE_ORIENTATION_FINGER_TIP_ORIENTATION_TILE, hsPanel);
        this.palmOrientationPanel = new OrientationPalette(values, this.panel, scheme, Constants.GESTURE_ORIENTATION_PALM_ORIENTATION_TITLE, hsPanel);

        this.initUI();
        this.setPalette(selectedIndex);
        this.setOrientationFingerTipPalette(0);
        this.setMinimumSize(this.getPreferredSize());
//		setMaximumSize(new Dimension(100, 200));
	}

	// GD 2024-07-17 - as of now not seeing these used, so commenting it out to avoid confusion.
	// actual palette switching happens in GesturePaletteDialog itemStateChanged
	public void setPalette(int selectedIndex) {
//       if (selectedIndex == 0) {
//    	   // all
//           this.iconPanel = new SortedHandshapesPalette(this.values, this.paletteUtil);
//       } else if(selectedIndex == 1) {
//        	// Letters
//           this.iconPanel = new LettersHandshapesPalette(this.values, this.paletteUtil, this.selectedHshImages);
//           this.panel.setLettersHandshapesPallet((LettersHandshapesPalette) this.iconPanel);
//		   this.paletteUtil.setLetter(true);
//        } else if(selectedIndex == 2) {
//        	// numbers
//           this.iconPanel = new NumbersHandshapesPallete(this.values, this.panel, this.scheme);
//        } else if(selectedIndex == 3) {
//        	// unmarked
//           this.iconPanel = new UnmarkedHandshapesPalette(this.values, this.panel, this.scheme);
//	   } else {
//        	// Basic Gestures
//           this.iconPanel = new GestureHandshapesPalette(this.values, this.panel, this.scheme);
//        }
//
//        this.removeAll();
//        this.initUI();
//        this.invalidate();
//        this.revalidate();
	}
	
	 // hardcode index zero, only one for now
	public void setOrientationFingerTipPalette(int selectedIndex) {

		if (selectedIndex == 0) {
//			orientationPanel = new OrientationPalette(values, panel, scheme);    	   
		}

        this.removeAll();

        this.initUI();
        this.invalidate();
        this.revalidate();
	}
	
	private void initUI() {	
		Dimension d = new Dimension(this.getPreferredSize().width, 110);
        this.panel.setPreferredSize(d);

        this.sp = new JScrollPane(this.panel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
//		sp.add(iconPanel);
//		sp.add(orientationPanel);
        this.sp.getViewport().setScrollMode(JViewport.SIMPLE_SCROLL_MODE);
		
//		Dimension spd = new Dimension(getPreferredSize().width - 50, 100);
		//Dimension spd = new Dimension(getPreferredSize().width - 150, 60);
//		sp.setMaximumSize(spd);
//		sp.setPreferredSize(spd);
//		sp.setMinimumSize(spd);

		JPanel p = new JPanel();
		p.setLayout(new BorderLayout());
		JPanel orientationPalettePanel = new JPanel();
		orientationPalettePanel.setLayout(new BoxLayout(orientationPalettePanel, BoxLayout.Y_AXIS));


//		p.add(iconPanel, BorderLayout.NORTH);
//		p.add(orientationPanel, BorderLayout.CENTER);
//		p.add(client, BorderLayout.SOUTH);
		
//		JPanel iconsPanel = new JPanel();
//		iconsPanel.setLayout(new BorderLayout());
//		iconPanel.setMinimumSize(dim);
		
//		iconsPanel.add(iconPanel, BorderLayout.NORTH);
//		iconsPanel.add(orientationPanel, BorderLayout.CENTER);
		
		JScrollPane jsp = new JScrollPane(this.iconPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		Dimension dim = new Dimension(450, 300);
		jsp.setMaximumSize(dim);
		jsp.setPreferredSize(dim);


        this.client = new JPanel();
        this.client.setLayout(new BorderLayout(10, 10));
        this.client.add(this.sp, BorderLayout.SOUTH);

		JScrollPane orientationScrollPane = new JScrollPane(this.orientationFingerTipPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollPane palmOrientationScrollPane = new JScrollPane(this.palmOrientationPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		Border orientationEtched = BorderFactory.createEtchedBorder();
		Border orientationTitle = BorderFactory.createTitledBorder(orientationEtched, this.orientationFingerTipPanel.getTitle());
		orientationScrollPane.setBorder(orientationTitle);

		Border etched = BorderFactory.createEtchedBorder();
		Border titled = BorderFactory.createTitledBorder(etched, this.palmOrientationPanel.getTitle());
		palmOrientationScrollPane.setBorder(titled);

		orientationPalettePanel.add(orientationScrollPane);
		orientationPalettePanel.add(palmOrientationScrollPane);

		JScrollPane clientScrollPane = new JScrollPane(this.client, ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		if (iconPanel instanceof LettersHandshapesPalette) {
			JPanel lettersPanel = new JPanel(new BorderLayout());
			lettersPanel.add(jsp, BorderLayout.NORTH);
			lettersPanel.add(paletteUtil.getLettersTxtfld(), BorderLayout.CENTER);
			p.add(lettersPanel, BorderLayout.NORTH);
		} else {
			p.add(jsp, BorderLayout.NORTH);
		}

		p.add(orientationPalettePanel, BorderLayout.CENTER);
		p.add(clientScrollPane, BorderLayout.SOUTH);

		ButtonPanel bp = new ButtonPanel();
		bp.add(this.clean);
        this.clean.addActionListener(this);

        this.setLayout(new BorderLayout());

        this.add(p, BorderLayout.CENTER);
        this.add(bp, BorderLayout.SOUTH);
	}

	public void setSelectedHshImages(ArrayList<HandShapeImage> selectedHshImages) {
        // panel = new SelectedHandshapesPanel(selectedHshImages);
		this.selectedHshImages.clear();
		this.selectedHshImages.addAll(selectedHshImages);
        this.panel.update(this.selectedHshImages);
    }
	
	public void setSelectedOrientationFingerTipImages(ArrayList<FingerTipImage> selectedOrientationFingerTipImages) {
		this.selectedOrientationFingerTipImages.clear();
		this.selectedOrientationFingerTipImages.addAll(selectedOrientationFingerTipImages);
        this.panel.updateOrientations(this.selectedOrientationFingerTipImages);
    }

	public void setSelectedPalmOrientationImages(ArrayList<PalmImage> selectedPalmOrientationImages) {
		this.selectedPalmOrientationImages.clear();
		this.selectedPalmOrientationImages.addAll(selectedPalmOrientationImages);
        this.panel.updatePalmOrientations(this.selectedPalmOrientationImages);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
        this.panel.removeAll();
		if(this.iconPanel instanceof LettersHandshapesPalette) {
			LettersHandshapesPalette p = (LettersHandshapesPalette) this.iconPanel;
			p.clearEnteredIcons();
		}
	}

	public ArrayList<HandShapeImage> getSelectedHandshapes() {
        this.setSelectedHshImages(this.panel.getHandshapes());	// sets selectedHshImages
		return this.selectedHshImages;
	}
	
	public ArrayList<FingerTipImage> getSelectedFingerTipOrientations() {
		this.setSelectedOrientationFingerTipImages(new ArrayList<>(this.panel.getFingerTipOrientationsImages()));
		return this.selectedOrientationFingerTipImages;
	}


	public ArrayList<PalmImage> getSelectedPalmOrientations() {
        this.setSelectedPalmOrientationImages(new ArrayList<>(this.panel.getPalmOrientationsImages()));
		return this.selectedPalmOrientationImages;
	}
}
