package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;

import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;

import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapePanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.ui.ButtonPanel;

public class GestureHandshapePalette extends JPanel implements ActionListener {

	private HandShapesPaletteFormBean formBean = null;
	private HandShapeFormBean bean = null;
	private GestureGeneralHandshapePalette palette = null;
    private GestureHandShapePanel hsPanel = null;

	private JDialog dialog = null;

	private final int ENTER = 0;
	private final int CANCEL = 1;

    private final JComboBox<String> options = new JComboBox<>(new String[] {"ASL-All", "ASL-Letters", "ASL-Numbers", "ASL-Unmarked", "Basic Gestures"});
    private final JComboBox<String> orientationOptions = new JComboBox<String>(new String[] {"Orientations"});
    
	public GestureHandshapePalette(HandShapesPaletteFormBean formBean,
								   HandShapeFormBean bean, SS3CodingScheme scheme, JDialog dialog, int selectedIndex, GestureHandShapePanel hsPanel) {

		this.formBean = formBean;
		this.bean = bean;
		this.dialog = dialog;
        this.hsPanel = hsPanel;

		palette = new GestureGeneralHandshapePalette(formBean.getValues(),
				formBean.getSelectedHshImages(), formBean.getSelectedFingerTipOrientationImages(), formBean.getSelectedPalmOrientationImages(), scheme, selectedIndex, hsPanel);
		palette.setBorder(new TitledBorder(formBean.getTitle()));

//        options.addItemListener(new WeakItemListener(new ItemListener() {
//            @Override
//            public void itemStateChanged(ItemEvent itemEvent) {
//                int selectedIndex = options.getSelectedIndex();
//                palette.setPalette(selectedIndex);
//            }
//        }, options));

        options.addItemListener(itemEvent -> {
			int selectedIndex1 = options.getSelectedIndex();
			palette.setPalette(selectedIndex1);
		});
        
        orientationOptions.addItemListener(itemEvent -> {

		int selectedIndex1 = orientationOptions.getSelectedIndex();
		palette.setOrientationFingerTipPalette(selectedIndex1);

		});

        options.setSelectedIndex(selectedIndex);
        orientationOptions.setSelectedIndex(0);

        setLayout(new BorderLayout());
        
        JPanel top = new JPanel(new FlowLayout(FlowLayout.CENTER));
		top.add(new JLabel("Handshapes: "));
		top.add(options);
		JPanel bottom = new JPanel(new FlowLayout(FlowLayout.CENTER));
		bottom.add(new JLabel("Orientations: "));
		bottom.add(orientationOptions);

        JPanel languagePanel;
		languagePanel = new JPanel(new BorderLayout());
        languagePanel.add(top, BorderLayout.NORTH);
        languagePanel.add(bottom, BorderLayout.SOUTH);

        JButton enter = new JButton("Enter");
        JButton cancel = new JButton("Cancel");

        ButtonPanel buttonPanel = new ButtonPanel();
        buttonPanel.add(cancel);
        buttonPanel.add(enter);
        add(languagePanel, BorderLayout.NORTH);
        add(palette, BorderLayout.CENTER);
		// Add another palette for orientations under the handshape palette
        add(buttonPanel, BorderLayout.SOUTH);

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));
	}
	
	public void update(HandShapesPaletteFormBean formBean, HandShapeFormBean bean, int selectedIndex) {
		this.bean = bean;
		this.formBean = formBean;
        options.setSelectedIndex(selectedIndex);
        orientationOptions.setSelectedIndex(0);

        palette.setSelectedHshImages(formBean.getSelectedHshImages());
        palette.setSelectedOrientationFingerTipImages(formBean.getSelectedFingerTipOrientationImages());
		palette.setSelectedPalmOrientationImages(formBean.getSelectedPalmOrientationImages());
        palette.setBorder(new TitledBorder(formBean.getTitle()));
        palette.repaint();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		ArrayList<HandShapeImage> selectedHandshapes = palette.getSelectedHandshapes();

		ArrayList<FingerTipImage> selectedOrientations = palette.getSelectedFingerTipOrientations();
		ArrayList<PalmImage> selectedPalmImagesOrientation = palette.getSelectedPalmOrientations();


		switch (Integer.parseInt(str)) {
			case ENTER:

				// first check if this is a one-frame gesture, and if it is then disallow anything more than one handshape in either hand
				if (hsPanel.isOneFrameEntity()) {
					if (selectedHandshapes.size() > 1) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_EXACTLY_ONE);
						break;
					}
				} else if (hsPanel.getSelectedSameDifStartEndHandshapes() == GestureStartEndHandshapesType.DIFFERENT && (selectedHandshapes.size() == 1)) {
					// conversely, if this is not a one-frame gloss and you have dif start/end selected,
					// then disallow a selection of just one handshape in either hand
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_MULTIPLE);
					break;
				} else if (selectedHandshapes.size() == 0 && (selectedOrientations.size() > 0 || selectedPalmImagesOrientation.size() > 0)) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_HANDSHAPES_NOT_SELECTED);
					break;
				} else if (selectedHandshapes.size() != selectedOrientations.size() || selectedHandshapes.size() != selectedPalmImagesOrientation.size()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_SELECT_HANDSHAPES_ORIENTATIONS);
					break;
				}
/*					for (int i = 0; i < selectedHandshapes.size(); i++) {
						// Get the Do Not include field value
						String fingerTipFieldID = formBean.getPaletteType() == GestureHandShapesPaletteFormBean.PaletteType.RIGHT ? Util.DOM_GESTURE_ORIENTATION_FINGER_TIP_FIELD_ID: Util.NDOM_GESTURE_ORIENTATION_FINGER_TIP_FIELD_ID;
						String palmFieldID = formBean.getPaletteType() == GestureHandShapesPaletteFormBean.PaletteType.RIGHT ? Util.DOM_GESTURE_PALM_FIELD_ID: Util.NDOM_GESTURE_PALM_FIELD_ID;
						SS3FieldValue fingerTipDNIFieldValue = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme().getFieldValuesAsMap(fingerTipFieldID).get(Util.DNI_FINGERTIP_ID);
						SS3FieldValue palmDNIFieldValue = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme().getFieldValuesAsMap(palmFieldID).get(Util.DNI_PALM_ID);

						if (selectedHandshapes.size() > selectedOrientations.size()) {
							selectedOrientations.add(new OrientationImage(fingerTipDNIFieldValue, SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme(), "Finger Tip Orientation"));
						}
						if (selectedHandshapes.size() > selectedPalmImagesOrientation.size()) {
							selectedPalmImagesOrientation.add(new OrientationImage(palmDNIFieldValue, SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme(), "Palm Orientation"));
						}
					}*/

					// The code above is if you want to fill in the remaining orientations with the DNI image... So, basically, for any blank orientation, fill it in with the DNI image to match the number
					// of handshapes. I thought we wanted to add this functionality, but we do not at the moment. Leaving it here in case we change our minds later.
				
//	            // if user only selected none-image for one of above lists - disallow
//	            boolean onlyNoneImage = (selectedHandshapes.size() > 0 && selectedHandshapes.stream().allMatch(
//            				img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_HANDSHAPE_IMAGE_ID))) &&
//            		(selectedOrientations.size() > 0 && selectedOrientations.stream().allMatch(
//            				img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_FINGERTIP_IMAGE_ID))) &&
//					(selectedPalmImagesOrientation.size() > 0 && selectedPalmImagesOrientation.stream().allMatch(
//							img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_PALM_IMAGE_ID)));
//	            
//	            if (onlyNoneImage) {
//					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANNOT_SELECT_ONLY_NONE);
//	            	break;
//	            }


				bean.setUpdated(true);

				if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT) {
					if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
						bean.setSelectedDomHandShapeImages(selectedHandshapes);
						bean.setSelectedDomOrientationFingerTipImages(selectedOrientations);
						bean.setSelectedDomPalmOrientationImages(selectedPalmImagesOrientation);
					} else {
						bean.setSelectedNdomHandShapeImages(selectedHandshapes);
						bean.setSelectedNdomOrientationFingerTipImages(selectedOrientations);
						bean.setSelectedNdomPalmOrientationImages(selectedPalmImagesOrientation);
					}
				}

				if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.LEFT) {
					if (hsPanel.getAssociatedFieldsPanel().isDominantHandRight()) {
						bean.setSelectedNdomHandShapeImages(selectedHandshapes);
						bean.setSelectedNdomOrientationFingerTipImages(selectedOrientations);
						bean.setSelectedNdomPalmOrientationImages(selectedPalmImagesOrientation);
					} else {
						bean.setSelectedDomHandShapeImages(selectedHandshapes);
						bean.setSelectedDomOrientationFingerTipImages(selectedOrientations);
						bean.setSelectedDomPalmOrientationImages(selectedPalmImagesOrientation);
					}
				}

				if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT) {
					bean.setSelectedDomHandShapeImages(selectedHandshapes);
					bean.setSelectedNdomHandShapeImages(selectedHandshapes);
					bean.setSelectedDomOrientationFingerTipImages(selectedOrientations);
					bean.setSelectedNdomOrientationFingerTipImages(selectedOrientations);
					bean.setSelectedDomPalmOrientationImages(selectedPalmImagesOrientation);
					bean.setSelectedNdomPalmOrientationImages(selectedPalmImagesOrientation);
				}

				hsPanel.updateParentInfoSelectedEntities(bean.getSelectedDomHandShapeImages(),
													   bean.getSelectedNdomHandShapeImages(),
													   bean.getSelectedDomOrientationFingerTipImages(),
													   bean.getSelectedNdomOrientationFingerTipImages(),
														bean.getSelectedDomOrientationPalmImages(),
														bean.getSelectedNdomOrientationPalmImages());

				dialog.setVisible(false);
				break;

			case CANCEL:
				dialog.setVisible(false);
				break;
		}
	}
	
	public GestureGeneralHandshapePalette getPalette() {
		return palette;
	}
}
