package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.PalettePanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class OrientationPalette extends PalettePanel {

	private static final long serialVersionUID = 1L;

	private SelectedGestureHandshapesPanel hsClient;
	
	private SS3CodingScheme defaultCodingScheme;

	private String title;
	private GestureHandShapePanel hsPanel;

	public OrientationPalette(ArrayList<SS3FieldValue> orientations,
							  HandShapePalletClient hsClient, SS3CodingScheme defaultCodingScheme, String title, GestureHandShapePanel hsPanel) {
		this.hsClient = (SelectedGestureHandshapesPanel) hsClient;
		this.defaultCodingScheme = defaultCodingScheme;
		this.title = title;
		this.hsPanel = hsPanel;
		initUI();
	}

	public String getTitle() {
		return this.title;
	}

	// This method will initialize the UI for the orientation panel
	private void initUI() {
		setLayout(new GridBagLayout());

		this.setLayout(new GridBagLayout());
		this.setPreferredSize(new Dimension(550, 150));

		GridBagConstraints c = new GridBagConstraints();

		populate();

		c.gridwidth = 1;
		c.gridy = 1;
		c.gridx = 1;
		c.weightx = 1;
		c.weighty = 1;
	}

	// This method will populate the orientation panel with the images
	private void populate() {
		this.removeAll();
		this.setBackground(UIPalette.getBackground());

		GridBagConstraints c = new GridBagConstraints();
		// c.insets = new Insets(0, 0, 0, 0);
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weighty = 0;
		c.gridy = 0;
		c.insets = new Insets(10, 0, 5, 0);
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;

		c.anchor = GridBagConstraints.NORTH;

		SS3Field field = null;

		if (this.hsPanel.getSelectedEntity().getField().getFieldID().equals(Util.DOM_GESTURE_FIELD_ID)) {
			if (this.title.equals("Finger Tip Orientation")) {
				field = this.defaultCodingScheme.getField(Util.DOM_GESTURE_ORIENTATION_FINGER_TIP_FIELD_ID);
			} else if (this.title.equals("Palm Orientation")) {
				field = this.defaultCodingScheme.getField(Util.DOM_GESTURE_PALM_FIELD_ID);
			}
		} else if (this.hsPanel.getSelectedEntity().getField().getFieldID().equals(Util.NDOM_GESTURE_FIELD_ID)) {
			if (this.title.equals("Finger Tip Orientation")) {
				field = this.defaultCodingScheme.getField(Util.NDOM_GESTURE_ORIENTATION_FINGER_TIP_FIELD_ID);
			} else if (this.title.equals("Palm Orientation")) {
				field = this.defaultCodingScheme.getField(Util.NDOM_GESTURE_PALM_FIELD_ID);
			}
		}

		int imageCount = field.getValues().size();

		ArrayList<SS3FieldValue> values = new ArrayList<>(field.getValuesAsList());

		values.sort((o1, o2) -> {
			String s1 = o1.getIid();
			String s2 = o2.getIid();
			return s1.compareTo(s2);
		});



		for (int y = 0; y < Math.ceil(imageCount / 5.0); y++) {
			for (int x = 0; x < 5 && ((y * 5) + x) < imageCount; x++) {
				int index = ((y * 5) + x) + 1;

				String id = values.get(index - 1).getId();
				String iid = values.get(index - 1).getIid();

				JLabel label = getGestureImageLabel(id, iid);

				c.gridx = x;
				c.gridy = y;
				this.add(label, c);

				values.remove(iid);
			}
		}
	}
	
	private JLabel getGestureImageLabel(final String id, final String iid) {
		try {
			SS3FieldTypeImg ss3Img = defaultCodingScheme.getImage(iid);
			SS3FieldValue fieldValue = new SS3FieldValue();
			fieldValue.setId(id);
			fieldValue.setIid(iid);

			// String imageType = iid.substring(0, 2).equals("FT") ? "Finger Tip Orientation" : "Palm Orientation";
			final OrientationImage image;
			if (title.equals(("Palm Orientation"))) {
				image = new PalmImage(fieldValue, defaultCodingScheme);
			} else if (title.equals("Finger Tip Orientation")) {
				image = new FingerTipImage(fieldValue, defaultCodingScheme);
			} else {
				image = new OrientationImage(fieldValue, defaultCodingScheme);
			}

			if (image.getImage() == null) {
				return null;
			}

			String lblText = "<html><p style=\"text-align: center;\">" + ss3Img.getLabel().replaceAll("\\\\n", "<br>") + "</p></html>";
			JLabel lbl = new JLabel(lblText, new ImageIcon(image.getImage()), SwingConstants.CENTER);
			lbl.setHorizontalTextPosition(JLabel.CENTER);
			lbl.setVerticalTextPosition(JLabel.BOTTOM);
			lbl.setSize(image.getImage().getWidth(null) + 5, image.getImage().getHeight(null) + 5);
			lbl.setToolTipText(ss3Img.getLabel());
			lbl.setHorizontalTextPosition(JLabel.CENTER);
			lbl.setVerticalTextPosition(JLabel.BOTTOM);
			lbl.addMouseListener(new MouseAdapter() {
				@Override
				public void mousePressed(MouseEvent e) {
					SS3SignStreamApplication.getLogger().info(title + " Orientation selected: " + iid);

					if (image instanceof PalmImage) {
						hsClient.addPalmOrientation((PalmImage) image);
					} else if (image instanceof FingerTipImage) {
						hsClient.addFingerTipOrientation((FingerTipImage) image);
					} else {
						Logger logger = SS3SignStreamApplication.getLogger();
						logger.info(String.format("Unknown Orientation selected: %s %s %s", iid, title, image));
					}

//					SS3Singleton.getApplicationStateController()
//							.setLoadedCollectionUpdated(true);
				}
			});
			return lbl;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
}

