package edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.LettersPalette;
import edu.bu.signstream.grepresentation.listener.WeakFocusListener;
import edu.bu.signstream.grepresentation.listener.WeakKeyListener;
import edu.bu.signstream.grepresentation.listener.WeakMouseListener;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.GestureHandShapePanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.LettersHandshapesPalette;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;

public class SelectedGestureHandshapesPanel extends SelectedPanel {

	private static final long serialVersionUID = 1L;
	private SS3CodingScheme cs;

	private ArrayList<HandShapeImage> handshapes = new ArrayList<HandShapeImage>();
	private ArrayList<FingerTipImage> fingerTipOrientationsImages = new ArrayList<>();
	private ArrayList<PalmImage> palmOrientationsImages = new ArrayList<>();
	private ArrayList<HandShapeImage> formBeanImages = null;
	private ArrayList<FingerTipImage> formBeanFingerTipOrientationImages = new ArrayList<>();
	private ArrayList<PalmImage> formBeanPalmOrientationImages = new ArrayList<>();
	private HashMap<Rectangle, String> handshapePositionMap = new HashMap<Rectangle, String>();
	private final HashMap<Rectangle, String> fingerTipOrientationPositionMap = new HashMap<Rectangle, String>();
	private final HashMap<Rectangle, String> palmOrientationPositionMap = new HashMap<Rectangle, String>();
	private ArrayList<Point> clickedAtPoints = new ArrayList<Point>();
	private GestureHandShapePanel gestureHandShapePanel;

	private LettersPalette lettersPalette = null;

	public SelectedGestureHandshapesPanel(SS3CodingScheme cs) {
        this.init(cs);
	}
	
	public SelectedGestureHandshapesPanel(ArrayList<HandShapeImage> hsImages, ArrayList<FingerTipImage> fingerTipOrientationImages, ArrayList<PalmImage> palmOrientationsImages, SS3CodingScheme cs, GestureHandShapePanel gestureHandShapePanel) {
		this.gestureHandShapePanel = gestureHandShapePanel;
		this.handshapes.addAll(hsImages);
		this.fingerTipOrientationsImages.addAll(fingerTipOrientationImages);
		this.palmOrientationsImages.addAll(palmOrientationsImages);
		this.formBeanImages = hsImages;
		this.formBeanFingerTipOrientationImages = fingerTipOrientationImages;
		this.formBeanPalmOrientationImages = palmOrientationsImages;

		this.init(cs);
	}

	@Override
	public void update(ArrayList<HandShapeImage> hsImages) {
        this.handshapes.clear();
        this.handshapes.addAll(hsImages);
        this.repaint();
	}
	
	public void updateOrientations(ArrayList<FingerTipImage> hsImages) {
        this.fingerTipOrientationsImages.clear();
        this.fingerTipOrientationsImages.addAll(hsImages);
        this.repaint();
	}

	public void updatePalmOrientations(ArrayList<PalmImage> hsImages) {
        this.palmOrientationsImages.clear();
        this.palmOrientationsImages.addAll(hsImages);
        this.repaint();
	}
	
	public void setLettersPalette(LettersPalette palette) {
		this.lettersPalette = palette;
	}
	
	private void init(SS3CodingScheme cs) {
		this.cs = cs;
		this.addMouseListener(new WeakMouseListener(this, this));
		this.setFocusable(true);
		this.addKeyListener(new WeakKeyListener(this, this));
		this.addFocusListener(new WeakFocusListener(this, this));
	}

	@Override
	public void clean() {
		this.removeMouseListener(this);
		this.removeKeyListener(this);
		this.removeFocusListener(this);

        this.handshapes = null;
        this.fingerTipOrientationsImages = null;
        this.palmOrientationsImages = null;
        this.formBeanImages = null;
        this.formBeanFingerTipOrientationImages = null;
        this.formBeanPalmOrientationImages = null;
        this.handshapePositionMap = null;
        this.clickedAtPoints = null;
        this.gestureHandShapePanel = null;
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.isControlDown()) {
            this.clickedAtPoints.add(e.getPoint());
		} else if (e.isShiftDown()) {
            this.clickedAtPoints.add(e.getPoint());
				if (this.clickedAtPoints.size() >= 2) {
					Point point1 = this.clickedAtPoints.get(0);
					Point point2 = this.clickedAtPoints.get(this.clickedAtPoints.size() - 1);

					if (point1.x > point2.x) {
						Point point = point2;
						point2 = point1;
						point1 = point;
					}

					int id1 = -1, id2 = -1;
					Iterator i = this.handshapePositionMap.entrySet().iterator();
					//ArrayList<HandShapeImage> toRemove = new ArrayList<HandShapeImage>();
					while (i.hasNext()) {
						Map.Entry me = (Map.Entry) i.next();
						Rectangle rectangle = (Rectangle) me.getKey();

						if (rectangle == null)
							return;

						if (rectangle.contains(point1)) {
							String value = (String) me.getValue();
							id1 = Integer.parseInt(value);
						}
						if (rectangle.contains(point2)) {
							String value = (String) me.getValue();
							id2 = Integer.parseInt(value);
						}
					}

					if (id1 > -1 && id2 > -1) {
                        this.clickedAtPoints.add(point1);

						Iterator iter = this.handshapePositionMap.entrySet()
								.iterator();
						while (iter.hasNext()) {
							Map.Entry map = (Map.Entry) iter.next();
							String value = (String) map.getValue();
							int id = Integer.parseInt(value);

							if (id1 < id && id < id2) {
								Rectangle rectangle = (Rectangle) map.getKey();
                                this.clickedAtPoints.add(rectangle.getLocation());
							}
						}
                        this.clickedAtPoints.add(point2);
					} else if (id1 > -1) {
                        this.clickedAtPoints.add(point1);
					} else if (id2 > -1) {
                        this.clickedAtPoints.add(point2);
					}
				}
		} else {
            this.clickedAtPoints.clear();
            this.clickedAtPoints.add(e.getPoint());
		}

		this.requestFocus();
        this.repaint();
	}

	@Override
	public void keyTyped(KeyEvent e) {
	}

	@Override
	public void keyPressed(KeyEvent e) {
	}

	@Override
	public void keyReleased(KeyEvent e) {
		int keyCode = e.getKeyCode();
		if (keyCode == KeyEvent.VK_BACK_SPACE || keyCode == KeyEvent.VK_DELETE || keyCode == KeyEvent.VK_DECIMAL) {

			// remove selected icon
			if (this.clickedAtPoints.isEmpty()) {
            } else {
				ArrayList<HandShapeImage> toRemoveHs = new ArrayList<HandShapeImage>();
				ArrayList<OrientationImage> toRemoveO = new ArrayList<OrientationImage>();
				
				for (Map.Entry<Rectangle, String> me : this.handshapePositionMap.entrySet()) {
					Rectangle rectangle = me.getKey();

					if (rectangle == null)
						return;

					for (Point point : this.clickedAtPoints) {
						boolean contains = rectangle.contains(point);

						if (contains) {
							String value = me.getValue();
							HandShapeImage image = this.handshapes.get(Integer
									.parseInt(value));
							toRemoveHs.add(image);
						}
					}
				}

				for (Map.Entry<Rectangle, String> me : this.fingerTipOrientationPositionMap.entrySet()) {
					Rectangle rectangle = me.getKey();

					if (rectangle == null)
						return;

					for (Point point : this.clickedAtPoints) {
						boolean contains = rectangle.contains(point);

						if (contains) {
							String value = me.getValue();
							OrientationImage image = this.fingerTipOrientationsImages.get(Integer
									.parseInt(value));
							toRemoveO.add(image);

						}
					}
				}

				for (Map.Entry<Rectangle, String> me : this.palmOrientationPositionMap.entrySet()) {
					Rectangle rectangle = me.getKey();

					if (rectangle == null)
						return;

					for (Point point : this.clickedAtPoints) {
						boolean contains = rectangle.contains(point);

						if (contains) {
							String value = me.getValue();
							OrientationImage image = this.palmOrientationsImages.get(Integer
									.parseInt(value));
							toRemoveO.add(image);
						}
					}
				}

				for (HandShapeImage remove : toRemoveHs) {
                    this.handshapes.remove(remove);
				}
				
				for (OrientationImage remove : toRemoveO) {
                    this.fingerTipOrientationsImages.remove(remove);
                }

				for (OrientationImage remove : toRemoveO) {
                    this.palmOrientationsImages.remove(remove);
                }

				if (this.lettersPalette != null) {
					this.lettersPalette.synchronizeHandshapeWithTextField(this.handshapes);
				}

                this.synchronizeWithFormBean();
                this.clickedAtPoints.clear();
                this.repaint();
			}
		}
	}

	@Override
	public void setItem(Object obj) {
		SS3FieldValue value = (SS3FieldValue) obj;
        this.handshapes.add(new HandShapeImage(value, this.cs));
        this.synchronizeWithFormBean();

        this.repaint();
        this.revalidate();
	}
	
	public void addFingerTipOrientation(FingerTipImage o) {
        this.fingerTipOrientationsImages.add(o);
        this.synchronizeWithFormBean();
        this.repaint();
        this.revalidate();
	}

	public void addPalmOrientation(PalmImage o) {
        this.palmOrientationsImages.add(o);
        this.synchronizeWithFormBean();
        this.repaint();
        this.revalidate();
	}

	@Override
	public void paint(Graphics g) {
        this.handshapePositionMap.clear();
        this.fingerTipOrientationPositionMap.clear();
        this.palmOrientationPositionMap.clear();

		Graphics2D g2d = (Graphics2D) g;
		int x = 5;
		int y = 5;

		g2d.setColor(Palette.BACKGROUND_COLOR);
		g2d.fillRect(0, 0, this.getWidth() + 50, this.getHeight());

		for (int i = 0; i < this.handshapes.size(); i++) {

			HandShapeImage hsImage = this.handshapes.get(i);
			Image img = hsImage.getImage();
			int width = img.getWidth(null) + 1;
			int height = img.getHeight(null) + 1;

			g2d.setColor(Color.BLACK);
			g2d.drawRect(x - 1, y - 1, width, height);
			g2d.drawImage(img, x, y, Color.WHITE, this);

			Rectangle r = new Rectangle(x - 1, y - 1, width, height);
            this.handshapePositionMap.put(r, "" + i);

			for (Point clickedAtPoint : this.clickedAtPoints) {
				if (clickedAtPoint != null && r.contains(clickedAtPoint)) {
					g2d.setColor(Color.red);
					g2d.drawRect(x - 1, y - 1, width, height);
					g2d.setColor(Color.BLACK);
				}
			}
			x += width + 3;
		}
		
		x = 5;
		y = 40;
		
		for (int i = 0; i < this.fingerTipOrientationsImages.size(); i++) {

			OrientationImage oImg = this.fingerTipOrientationsImages.get(i);
			Image img = oImg.getImage();
			int width = img.getWidth(null) + 1;
			int height = img.getHeight(null) + 1;

			g2d.setColor(Color.BLACK);
			g2d.drawRect(x - 1, y - 1, width, height);
			g2d.drawImage(img, x, y, Color.WHITE, this);

			Rectangle r = new Rectangle(x - 1, y - 1, width, height);
            this.fingerTipOrientationPositionMap.put(r,  "" + i);

			for (Point clickedAtPoint : this.clickedAtPoints) {
				if (clickedAtPoint != null && r.contains(clickedAtPoint)) {
					g2d.setColor(Color.red);
					g2d.drawRect(x - 1, y - 1, width, height);
					g2d.setColor(Color.BLACK);
				}
			}
			x += width + 3;
		}

		x = 5;
		y = 75;

		for (int i = 0; i < this.palmOrientationsImages.size(); i++) {
			OrientationImage oImg = this.palmOrientationsImages.get(i);
			Image img = oImg.getImage();
			int width = img.getWidth(null) + 1;
			int height = img.getHeight(null) + 1;

			g2d.setColor(Color.BLACK);
			g2d.drawRect(x - 1, y - 1, width, height);
			g2d.drawImage(img, x, y, Color.WHITE, this);

			Rectangle r = new Rectangle(x - 1, y - 1, width, height);
            this.palmOrientationPositionMap.put(r,  "" + i);

			for (Point clickedAtPoint : this.clickedAtPoints) {
				if (clickedAtPoint != null && r.contains(clickedAtPoint)) {
					g2d.setColor(Color.red);
					g2d.drawRect(x - 1, y - 1, width, height);
					g2d.setColor(Color.BLACK);
				}
			}
			x += width + 3;
		}

		Dimension d = new Dimension(x, (int) this.getPreferredSize().getHeight());
        this.setPreferredSize(d);
        this.setMinimumSize(d);
        this.setMaximumSize(d);
	}

	@Override
	public void removeAll() {
        this.handshapes.clear();
        this.fingerTipOrientationsImages.clear();
		this.palmOrientationsImages.clear();
        this.synchronizeWithFormBean();

		if (this.lettersPalette != null) {
			this.lettersPalette.synchronizeHandshapeWithTextField(this.handshapes);
		}
		
        this.repaint();
        this.revalidate();
	}
	@Override
	public ArrayList<HandShapeImage> getHandshapes() {
		return this.handshapes;
	}
	
	public ArrayList<FingerTipImage> getFingerTipOrientationsImages() {
		return this.fingerTipOrientationsImages;
	}

	public ArrayList<PalmImage> getPalmOrientationsImages() {
		return this.palmOrientationsImages;
	}


	public void clearHandshapes() {
		this.handshapes.clear();
        this.synchronizeWithFormBean();
	}

	@Override
	public void setHandshapes(ArrayList<HandShapeImage> handshapes) {
		this.handshapes.clear();
		this.handshapes.addAll(handshapes);
        this.synchronizeWithFormBean();
	}
	
	public void setHandshapesAndOrientations(ArrayList<HandShapeImage> handshapes, ArrayList<FingerTipImage> orientations, ArrayList<PalmImage> palmOrientations) {
		this.handshapes.clear();
		this.handshapes.addAll(handshapes);
		this.fingerTipOrientationsImages.clear();
		this.fingerTipOrientationsImages.addAll(orientations);
		this.palmOrientationsImages.clear();
		this.palmOrientationsImages.addAll(palmOrientations);
        this.synchronizeWithFormBean();
	}
	
	public void setFingerTipOrientationsImages(ArrayList<FingerTipImage> fingerTipOrientationsImages) {
		this.fingerTipOrientationsImages.clear();
		this.fingerTipOrientationsImages.addAll(fingerTipOrientationsImages);
        this.synchronizeWithFormBean();
	}

	public void setPalmOrientationsImages(ArrayList<PalmImage> palmOrientationsImages) {
		this.palmOrientationsImages.clear();
		this.palmOrientationsImages.addAll(palmOrientationsImages);
        this.synchronizeWithFormBean();
	}

	public int getNumberOfHandShapes() {
		return this.handshapes.size();
	}

	@Override
	public void synchronizeWithFormBean() {
        this.formBeanImages.clear();
        this.formBeanImages.addAll(this.handshapes);
        this.formBeanFingerTipOrientationImages.clear();
        this.formBeanFingerTipOrientationImages.addAll(this.fingerTipOrientationsImages);
        this.formBeanPalmOrientationImages.clear();
        this.formBeanPalmOrientationImages.addAll(this.palmOrientationsImages);
    }

	@Override
	public void setItem(Object obj, boolean bothHands) {
	}
	
	@Override
	public void focusGained(FocusEvent e) {			
	}
	
	@Override
	public void focusLost(FocusEvent e) {
        this.clickedAtPoints.clear();
        this.repaint();
        this.revalidate();
	}
}
