package edu.bu.signstream.grepresentation.fields.gestureField.domNdomField;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GestureDomNdomHandField extends Field {

	private ArrayList<GestureChainedEventsEntity> entities = new ArrayList<GestureChainedEventsEntity>();

	protected String segmentID = "";

	public GestureDomNdomHandField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.segmentPanel = segmentPanel;
		this.ss3Track = ss3track;
	}

	@Override
	public ArrayList getEntities() {
		return entities;
	}

	@Override
	public EventsEntity getEntityWithRefID(String refEntityID) {
		if (refEntityID != null)
			return null;

		for (GestureChainedEventsEntity entity : entities) {
			if (refEntityID.equalsIgnoreCase(entity.getRefEntityId())) {
				return entity;
			}
		}
		return null;
	}

	@Override
	public EventsEntity getEntity(String entityID) {
		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getID().equalsIgnoreCase(entityID)) {
				return entity;
			}
		}
		return null;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		EventsEntity entity = getEntityWithRefID(refID);
		if (entity != null)
			entity.highlight(highlight);
	}

	@Override
	public Event getEventAt(double x) {
		GestureChainedEventsEntity entity = (GestureChainedEventsEntity) getEntityAt(x);
		if (entity == null)
			return null;

		return entity.getGestureChainedEventsCollection().getEventAt(x);
	}

	@Override
	public EventsEntity getEntityAt(double coordX) {
		for (GestureChainedEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public void deleteEntity(GestureChainedEventsEntity entity) {
		GestureChainedEventsEntity previousEntity = entity.getPreviousEntity();
		GestureChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		entities.remove(entity);
	}

	@Override
	public int getFieldSize() {
		return entities.size();
	}

	@Override
	public void deleteEmptyEntities() {
		ArrayList<GestureChainedEventsEntity> entitiesToDelete = new ArrayList<GestureChainedEventsEntity>();

		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null
					|| entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (GestureChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}

    @Override
    public boolean isIntervalEmpty(Interval interval) {
        return entities.stream()
            .noneMatch(e -> e.overlaps(interval));
    }

	@Override
	public void addEventsEntity(EventsEntity genEentity) {
		GestureChainedEventsEntity entity = (GestureChainedEventsEntity) genEentity;
		double startMovieTime = entity.getStartTimeInfo().doubleValue();

		GestureChainedEventsEntity previousEntity = null;
		GestureChainedEventsEntity nextEntity = null;

		for (GestureChainedEventsEntity ent : entities) {
			double entMovieStartTime = ent.getStartTimeInfo().doubleValue();

			if (entMovieStartTime <= startMovieTime) {
				if (previousEntity == null) {
					previousEntity = ent;
				} else if (entMovieStartTime > previousEntity
						.getStartTimeInfo().doubleValue()) {
					previousEntity = ent;
				}
			} else {
				if (nextEntity == null) {
					nextEntity = ent;
				} else if (entMovieStartTime < nextEntity.getStartTimeInfo()
						.doubleValue()) {
					nextEntity = ent;
				}
			}
		}

		if (previousEntity != null) {
			GestureChainedEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		entities.add(entity);
	}

	@Override
	public double getStartTime() {
		removeEmptyEntities();
		double startTime = 0;
		for (GestureChainedEventsEntity entity : entities) {
			startTime = startTime < entity.getStartTimeInfo().doubleValue() ? startTime
					: entity.getStartTimeInfo().doubleValue();
		}
		return startTime;
	}

	@Override
	public double getEndTime() {
		removeEmptyEntities();
		double endTime = 0;
		for (GestureChainedEventsEntity entity : entities) {
			endTime = endTime > entity.getEndTimeInfo().doubleValue() ? endTime
					: entity.getEndTimeInfo().doubleValue();
		}
		return endTime;
	}

	public Time getStartTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty()) 
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time earliestStartTime = entities.get(0).getStartTimeInfo();
				
		for (GestureChainedEventsEntity entity : entities) {
			earliestStartTime = (earliestStartTime.doubleValue() < entity.getStartTimeInfo().doubleValue()
					? earliestStartTime
					: entity.getStartTimeInfo());
		}
		
		return earliestStartTime;
	}

	public Time getEndTimeInfo() {
		removeEmptyEntities();
		
		if (entities.isEmpty()) 
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time latestEndTime = entities.get(0).getEndTimeInfo();
				
		for (GestureChainedEventsEntity entity : entities) {
			latestEndTime = (latestEndTime.doubleValue() > entity.getEndTimeInfo().doubleValue()
					? latestEndTime
					: entity.getEndTimeInfo());
		}
		
		return latestEndTime;
	}

	public void removeEmptyEntities() {
		ArrayList<GestureChainedEventsEntity> entitiesToDelete = new ArrayList<GestureChainedEventsEntity>();

		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null
					|| entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (GestureChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}

	@Override
	public void unselect() {
		for (GestureChainedEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	@Override
	public double getStartTimeCoordinate() {
		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(getStartTimeInfo());
	}

	@Override
	public double getEndTimeCoordinate() {
		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(getEndTimeInfo());
	}

	@Override
	public Event getSelectedEvent() {
		GestureChainedEventsEntity selectedEntity = (GestureChainedEventsEntity) getSelectedEntity();
		if (selectedEntity != null) {
			return selectedEntity.getSelectedEvent();
		}
		return null;
	}

	@Override
	public EventsEntity getSelectedEntity() {
		for (GestureChainedEventsEntity entity : entities) {
			if (entity.getSelectedEvent() != null)
				return entity;
		}
		return null;
	}

	@Override
	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	@Override
	public void setSegmentID(String segmentID) {
		this.segmentID = segmentID;
	}

	@Override
	public String getSegmentID() {
		return segmentID;
	}

	@Override
	public String getFieldID() {
		if (track != null)
			return track.getFieldId(); // fieldID;
		if (ss3Track != null)
			return ss3Track.getId();
		return null;
	}

	// update creation && update time here
	@Override
	public SS3Track getSS3Track() {
		if (ss3Track != null)
			ss3Track.removeEntities();

		for (GestureChainedEventsEntity entity : entities) {
			ss3Track.addEntity(entity.getSS3Entity());
		}

		return ss3Track;
	}

	@Override
	public int getY() {
		return (int) Y;
	}

	@Override
	public Event selectEvent(double x) {
		Event event = getEventAt(x);
		if (event != null)
			event.select();
		return event;
	}

	@Override
	public boolean contains(double y) {
		int y1 = (int) (Y - SegmentGraphReprUtil.cellHeight);
		int y2 = (int) (Y + SegmentGraphReprUtil.valueHeight);
		return y1 <= y && y <= y2;
	}

	@Override
	public Color getColor() {
		return color;
	}

	@Override
	public void setColor(Color color) {
		this.color = color;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		super.paint(g2d, Y, font);
		for (GestureChainedEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}
	}

	@Override
	public void deleteEntity(EventsEntity entityToDelete) {
		GestureChainedEventsEntity entity = (GestureChainedEventsEntity) entityToDelete;

		GestureChainedEventsEntity previousEntity = entity.getPreviousEntity();
		GestureChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		entities.remove(entity);
	}
}
