package edu.bu.signstream.grepresentation.fields.gestureField.locationField;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations.GestureRegion;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;

public class GestureLocationField extends Field {
    private GestureLocationEntityCollection entities = null;

    public GestureLocationField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
        super(segmentPanel, parentField);
        this.ss3Track = ss3track;
        createTrackEntity(ss3track);
    }

    @Override
    public void deleteEmptyEntities() {
    }

    @Override
    public boolean isIntervalEmpty(Interval interval) {
        return true;
    }

    @Override
    public void addEventsEntity(EventsEntity entity) {
        GestureLocationEntity lEntity = (GestureLocationEntity) entity;
        entities.addEntity(lEntity);
    }

    @Override
    public EventsEntity getEntity(String entityID) {
        return entities.getEntity(entityID);
    }

    @Override
    public ArrayList<GestureLocationEntity> getEntities() {
    	return entities.getEntities();
    }

    /**
     * Sometimes we want to discard the Region.NONE location values
     */
    public ArrayList<EventsEntity> getRegionNoneEntities() {
    	ArrayList<EventsEntity> onlyNoneEntities = new ArrayList<EventsEntity>();

    	for (GestureLocationEntity entity: getEntities()) {
    		boolean entityOnlyNone = true;

    		for (GestureLocationEvent event: entity.getGestureLocationEventsCollection().getEvents()) {
    			if ( !event.getGestureLocation().equals(GestureRegion.NONE)) {
    				entityOnlyNone = false;
    				break;
    			}
    		}

    		if (entityOnlyNone)
    			onlyNoneEntities.add(entity);
    	}

    	return onlyNoneEntities;
    }

    @Override
    public EventsEntity getEntityWithRefID(String refEntityId) {
    	return entities.getEntityWithRefID(refEntityId);
    }

    @Override
    public void highlight(boolean highlight, String refID) {
        if (!highlight) {
            ArrayList ents = entities.getEntities();
            for (int i = 0; i < ents.size(); i++) {
                EventsEntity entity = (EventsEntity) ents.get(i);
                entity.highlight(false);
            }
        } else {
            EventsEntity ent = entities.getEntityWithRefID(refID);

            if (ent != null)
                ent.highlight(true);
        }
    }

    private void createTrackEntity(SS3Track track) {
        ArrayList<GestureLocationEntity> lEntities = new ArrayList<GestureLocationEntity>();
        ArrayList<SS3Entity> ent = track.getEntities();
        for (int j = 0; j < ent.size(); j++) {
            SS3Entity entity = ent.get(j);
            GestureLocationEventsCollection collection = new GestureLocationEventsCollection();

            ArrayList<SS3Item> items = entity.getItemsAsList();
            for (int i = 0; i < items.size(); i++) {
                SS3Item item = items.get(i);
                GestureLocationEvent event = new GestureLocationEvent(segmentPanel);
                event.setSS3Item(item);

                collection.addEvent(event);
            }

            GestureLocationEntity lEntity = new GestureLocationEntity(collection, segmentPanel, this);

            lEntity.setID(entity.getID());
            lEntity.setRefEntityId(entity.getRefEntityId());
            if (entity.getParentEntityIDs() == null || entity.getParentEntityIDs().isEmpty()) {
            	lEntity.updateParentEntity();
            } else {
            	lEntity.setParentEntities(getParentField().getEntities(entity.getParentEntityIDs()));
            }
            // lEntity.setReference(entity.getReference());

            lEntities.add(lEntity);
        }
        entities = new GestureLocationEntityCollection(lEntities);
    }

    @Override
    public EventsEntity getEntityAt(double entityAt) {
        ArrayList<GestureLocationEntity> list = entities.getEntities();
        for (int i = 0; i < list.size(); i++) {
            GestureLocationEntity entity = list.get(i);
            double startTimeCoord = entity.getFirstEvent().getStartTimeCoordinate();
            double endTimeCoord = entity.getLastEvent().getStartTimeCoordinate();

            if (startTimeCoord <= entityAt && endTimeCoord >= entityAt)
                return entity;
        }
        return null;
    }

    @Override
    public Event getEventAt(double x) {
        return entities.getEventAt(x);
    }

    @Override
    public void deleteEntity(EventsEntity entity) {
        entities.deleteEntity((GestureLocationEntity) entity);
    }

    @Override
    public int getFieldSize() {
        return entities.size();
    }

    @Override
    public Event selectEvent(double x) {
        Event event = getEventAt(x);
        if (event != null)
            event.select();
        return event;
    }

    @Override
    public boolean contains(double y) {
        return y > Y - 14 && y < Y + 5;
    }

    @Override
    public double getStartTime() {
        return ss3Track.getStartTime();
    }

    @Override
    public double getEndTime() {
        return ss3Track.getEndTime();
    }

    @Override
    public double getStartTimeCoordinate() {
        return 0;
    }

    @Override
    public double getEndTimeCoordinate() {
        return 0;
    }

    @Override
    public void unselect() {
        entities.unselect();
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public EventsEntity getSelectedEntity() {
        return entities.getSelectedEntity();
    }

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        super.paint(g2d, Y, font);
        entities.paintEntities(g2d, Y, font);
    }
}
