package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations;

import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import java.util.*;

/**
 * This class is used to by GlossEntityProperties to store the list of locations
 * for a gloss, as well as by LocationPanel to represent the list of locations that
 * the user has selected with the mouse
 */
public class LocationInfo {

    private HashMap<LocationType, Region> locations;

    public LocationInfo() {
        locations = new HashMap<LocationType, Region>();
        for (LocationType locationType : SS3Singleton.getLocationConfig().getLocationTypes()) {
            locations.put(locationType, Region.NONE);
        }
    }

    public LocationInfo(HashMap<LocationType, Region> regions) {
        this.locations = new HashMap<LocationType, Region>(regions);
    }

    public Region getLocation(LocationType type) {
        return locations.get(type);
    }

    public HashMap<LocationType, Region> getLocations() {
        return new HashMap<LocationType, Region>(locations);
    }

    public void clear() {
        locations.clear();
    }

    public boolean isEmpty() {
        if (locations.isEmpty()) {
            return true;
        }
        Collection<Region> values = locations.values();
        for (Region value : values) {
            if (!value.equals(Region.NONE)) {
                return false;
            }
        }
        return true;
    }

    public int size() {
        return locations.size();
    }

    public LocationInfo cloneInfo() {
        return new LocationInfo(locations);
    }

    @Override
    public String toString() {
        StringBuilder str = new StringBuilder("[");
        for (Iterator<Map.Entry<LocationType, Region>> iterator = locations.entrySet().iterator(); iterator.hasNext(); ) {
            Map.Entry<LocationType, Region> entry = iterator.next();
            str.append(entry.getKey().getName()).append(":").append(entry.getValue().getName());
            if (iterator.hasNext()) {
                str.append(" ->");
            }
        }
        str.append("]");
        return str.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof LocationInfo)) {
            return false;
        }
        LocationInfo other = (LocationInfo) obj;
        for (LocationType locationType : SS3Singleton.getLocationConfig().getLocationTypes()) {
            if (!other.getLocation(locationType).equals(getLocation(locationType))) {
                return false;
            }
        }
        return true;
    }
}
