package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.stream.Collectors;

import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.time.Interval;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedEventsEntityCollection {
	private ArrayList<ChainedEventsEntity> entities = new ArrayList<ChainedEventsEntity>();
	private SignStreamSegmentPanel segmentPanel;

	private NewChainedEvent newEvent;
	private EnteredText enteredText;

	public ChainedEventsEntityCollection(
			ArrayList<ChainedEventsEntity> entities,
			SignStreamSegmentPanel segmentPanel) {
		this.entities = entities;
		this.segmentPanel = segmentPanel;

        sortEntities();
	}

	public EnteredText getEnteredText() {
		return enteredText;
	}

	public void setEnteredText(EnteredText enteredText) {
		this.enteredText = enteredText;
	}

	public ChainedEventsEntity getNextEntity(Time time) {
		ChainedEventsEntity nextEntity = null;

		for (ChainedEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();

				if (entStartTimeInfo.greaterThanOrEq(time)) {
					if (nextEntity == null) {
						nextEntity = ent;
					} else if (entStartTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())
							&& entEndTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())) {
						nextEntity = ent;
					}
				}
			}
		}

		return nextEntity;
	}
	
	public ChainedEventsEntity getPreviousEntity(Time time) {
		ChainedEventsEntity previousEntity = null;

		for (ChainedEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();

				if (entStartTimeInfo.lessThanOrEq(time)) {
					if (previousEntity == null) {
						previousEntity = ent;
					} else if (entStartTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo())
							&& (entEndTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo()))) {
						previousEntity = ent;
					}
				}
			}
		}
		
		return previousEntity;
	}

    public ArrayList<ChainedEventsEntity> getEntitiesInInterval(Interval interval) {
        return entities.stream()
            .filter(e -> e.overlaps(interval))
            .collect(Collectors.toCollection(ArrayList::new));
    }

	public void removeEmptyEntities() {
		ArrayList<ChainedEventsEntity> entitiesToDelete = new ArrayList<ChainedEventsEntity>();

		for (ChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}
	
	/**
	 * Cleanup method: if there are hanging references to a deleted field value, which is residing in this 
	 * ChainedEventsEntityCollection with an invalid field value ID, then it has since been deleted from the
	 * coding schema, but not yet form this collection.
	 * @param fieldID The Field ID of the field to get from the coding scheme
	 * @return true if succesful, false otherwise
	 */
	boolean deleteInvalidEntities(String fieldID) {
		ArrayList<ChainedEventsEntity> entitiesToDelete = new ArrayList<ChainedEventsEntity>();
		
		SS3CodingScheme dbScheme = segmentPanel.getSS3Database().getSS3CodingScheme();
		SS3Field ss3Field = dbScheme.getField(fieldID);
		ArrayList<SS3FieldValue> ss3FieldValues = ss3Field.getValuesAsList();
		ArrayList<String> ss3FieldValueIDs = new ArrayList<String>();
		
		for (SS3FieldValue ss3FieldValue: ss3FieldValues) {
			String valueID = ss3FieldValue.getId();
			if (valueID == null || valueID.isEmpty())
				continue;
			
			ss3FieldValueIDs.add(valueID);
		}
		
		if (ss3FieldValueIDs.isEmpty()) {
			return false;
		}
		
		for (ChainedEventsEntity entity: entities) {
			String valueID = entity.getValueOrTextValueEvent().getID();
			if ( !ss3FieldValueIDs.contains(valueID)) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}		
		
		return true;
	}

	public ChainedEventsEntity getEntityWithRefID(String refEntityId) {
		for (ChainedEventsEntity entity : entities) {
			if (refEntityId != null
					&& refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	public ChainedEventsEntity getEntity(String entityID) {
		for (ChainedEventsEntity entity : entities) {
			if (entity.getID().equalsIgnoreCase(entityID))
				return entity;
		}
		return null;
	}

	public ArrayList<ChainedEventsEntity> getEntities() {
		return entities;
	}

	public int size() {
		return entities.size();
	}

	public void deleteEntity(ChainedEventsEntity entity) {
		ChainedEventsEntity previousEntity = entity.getPreviousEntity();
		ChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		boolean removed = entities.remove(entity);

        if(!removed) {
            Time startMovieTimeInfo = entity.getStartTimeInfo();
            Time endMovieTimeInfo = entity.getEndTimeInfo();

            ChainedEventsEntity entityToRemove = null;

            for(ChainedEventsEntity entity1 : entities) {
                Time startMovieTime1Info = entity1.getStartTimeInfo();
                Time endMovieTime1Info = entity1.getEndTimeInfo();

                if(startMovieTimeInfo.equals(startMovieTime1Info) && endMovieTimeInfo.equals(endMovieTime1Info)) {
                    entityToRemove = entity1;
                }
            }

            entities.remove(entityToRemove);
        }
	}

	public double getStartTime() {
		removeEmptyEntities();
		double startTime = 0;
		for (ChainedEventsEntity entity : entities) {
			startTime = startTime < entity.getStartTimeInfo().doubleValue() ? startTime
					: entity.getStartTimeInfo().doubleValue();
		}
		return startTime;
	}

	public double getEndTime() {
		removeEmptyEntities();
		double endTime = 0;
		for (ChainedEventsEntity entity : entities) {
			endTime = endTime > entity.getEndTimeInfo().doubleValue() ? endTime
					: entity.getEndTimeInfo().doubleValue();
		}
		return endTime;
	}
	
	public Time getStartTimeInfo() {
		removeEmptyEntities();
		
		if (entities.size() == 0)
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time earliestStartTime = entities.get(0).getStartTimeInfo();
		
		for (ChainedEventsEntity entity : entities) {
			earliestStartTime = earliestStartTime.doubleValue() < entity.getStartTimeInfo().doubleValue() ? earliestStartTime
					: entity.getStartTimeInfo();
		}
		
		return earliestStartTime;
	}
	
	public Time getEndTimeInfo() {
		removeEmptyEntities();
		
		if (entities.size() == 0)
			return SS3Singleton.getVideoTimeRange().zero();
		
		Time endTimeInfo = entities.get(0).getStartTimeInfo();
		
		for (ChainedEventsEntity entity : entities) {
			endTimeInfo = endTimeInfo.doubleValue() > entity.getStartTimeInfo().doubleValue() ? endTimeInfo
					: entity.getStartTimeInfo();
		}
		
		return endTimeInfo;
	}

	public double getStartTimeCoordinate() {
		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(getStartTimeInfo());
	}

	public double getEndTimeCoordinate() {
		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeInfoToCoordinate(getEndTimeInfo());
	}

	public void unselect() {
		for (ChainedEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	public void reset() {
		for (ChainedEventsEntity entity : entities) {
			entity.reset();
		}
	}

	public void addEntity(ChainedEventsEntity entity) {

		Time startTimeInfo = entity.getStartTimeInfo();

		ChainedEventsEntity previousEntity = null;
		ChainedEventsEntity nextEntity = null;

		for (ChainedEventsEntity ent : entities) {
			if(ent.getStartTimeInfo() != null) {

				Time entStartTimeInfo = ent.getStartTimeInfo();
				Time entEndTimeInfo = ent.getEndTimeInfo();

				if (entStartTimeInfo.lessThanOrEq(startTimeInfo)) {
					if (previousEntity == null) {
						previousEntity = ent;
					} else if (entStartTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo())
							&& (entEndTimeInfo.greaterThanOrEq(previousEntity.getStartTimeInfo()))) {
						previousEntity = ent;
					}
				} else {
					if (nextEntity == null) {
						nextEntity = ent;
					} else if (entStartTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())
							&& entEndTimeInfo.lessThanOrEq(nextEntity.getStartTimeInfo())) {
						nextEntity = ent;
					}
				}
			}
		}

		if (previousEntity != null) {
			ChainedEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		this.entities.add(entity);
	}

	public void selectEntity(double x) {
		EventsEntity entity = getChainedEventsEntityAt(x);
		if (entity != null)
			entity.select();
	}

	public ChainedEventsEntity getChainedEventsEntityAt(double coordX) {
		for (ChainedEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public Event getSelectedEvent() {
		for (ChainedEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) { 
				return event;
			}
		}
		return null;
	}

	public ChainedEventsEntity getSelectedEntity() {
		for (ChainedEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) {
				return entity;
			}
		}
		return null;
	}

	public ChainedEvent getEventAt(double x) {
		ChainedEventsEntity entity = getChainedEventsEntityAt(x);
		if (entity == null)
			return null;

		return entity.getChainedEventsCollection().getEventAt(x);
	}

	public void setSelectedEvementValueID(String valueID) {
		Event event = getSelectedEvent();

		if (event instanceof ChainedEvent) {
			ChainedEvent chainedEvent = (ChainedEvent) event;
			chainedEvent.setID(valueID);
		}
	}

	public NewChainedEvent getNewChainedEvent() {
		return newEvent;
	}

	public void setNewChainedEvent(NewChainedEvent newEvent) {
		this.newEvent = newEvent;
	}

	public ChainedEventsCollection createNewChainedEventCollection(
			String valueID) {
		if (valueID != null && newEvent != null) {
			TrackValue trackValue = new TrackValue();
			trackValue.setValueId(valueID);

			trackValue.setStartTimeInfo(newEvent.getStartTimeInfo());
			trackValue.setEndTimeInfo(newEvent.getEndTimeInfo());

			ChainedEvent event = new ChainedEvent(trackValue, segmentPanel);

			ChainedEventsCollection collection = new ChainedEventsCollection();
			collection.setValueEvent(event);
			newEvent = null;

			return collection;
		}
		return null;
	}

    public void sortEntities() {
        int entitiesSize = entities.size();

        if(entitiesSize == 1)
            return;

        double[] startTimes = new double[entitiesSize];
        HashMap<String, ChainedEventsEntity> map = new HashMap<String, ChainedEventsEntity>();

        int i = 0;
        for(ChainedEventsEntity entity : entities) {
            double movieTime = entity.getStartTimeInfo().doubleValue();
            map.put(""+movieTime, entity);
            startTimes[i] = movieTime;
            i++;
        }

        Sorter.sort(startTimes);

        i = 0;
        while(i < entitiesSize-1) {
            ChainedEventsEntity glsEntity = map.get(""+startTimes[i]);
            i++;
            ChainedEventsEntity nextGlsEntity = map.get(""+startTimes[i]);
            glsEntity.setNextEntity(nextGlsEntity);
            nextGlsEntity.setPreviousEntity(glsEntity);
        }
    }

	public void sort(ArrayList<ChainedEventsEntity> entitiesToSort) {
		removeEmptyEntities();

		HashMap<String, ChainedEventsEntity> map = new HashMap<String, ChainedEventsEntity>();

		double[] startTimes = new double[entitiesToSort.size()];

		ArrayList<ChainedEventsEntity> toRemove = new ArrayList<ChainedEventsEntity>();

		for (int i = 0; i < entitiesToSort.size(); i++) {
			ChainedEventsEntity entity = entitiesToSort.get(i);
			if (entity != null && entity.getStartTimeInfo() != null) {
				double startTime = entity.getStartTimeInfo().doubleValue();
				map.put("" + startTime, entity);
				startTimes[i] = startTime;
			} else {
				toRemove.add(entity);
			}
		}

		for (ChainedEventsEntity cEnt : toRemove) {
			entitiesToSort.remove(cEnt);
		}

		Sorter.sort(startTimes);
		entitiesToSort.clear();

		for (int j = 0; j < startTimes.length; j++) {
			ChainedEventsEntity obj = map.get("" + startTimes[j]);
			entitiesToSort.add(obj);
		}
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		for (ChainedEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}

		if (newEvent != null) {
			Painter painter = segmentPanel.getPainter();
			painter.drawNewEvent(newEvent, g2d, Y);
		}
	}
}
