package edu.bu.signstream.grepresentation.fields.nonManualField;

import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignType;

public class GlossEntityProperties {
	
	/**
 	<ENTITY>
		<PROPERTIES SIGN="FINGERSPELLED/LOAN/LEXICAL/CLASSIFIER/GESTURE/NUMBER" 
	     	NAME_SIGN="T/F" 	     
	     	PASSIVE_BASE_ARM="T/F"
	     	NO_OF_HANDS="1/2"
	     	
	     	TWOHANDED_HADSHAPES="SAME/ALTERNATING_SAME/DIFFERENT" 
	     	START_END_HANDSHAPES="SAME/DIFFERENT"
	     	NDOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
	     	DOM_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
		/>
		<DOMINANCE="L/R"/>
		<MARKED_NUMBER_OF_HANDS="T/F"/> 
	</ENTITY>
	 */
	
	// "FINGERSPELLED", "LOAN", "LEXICAL", "CLASSIFIER", "GESTURE", "NUMBER"
	private SignType signType;
	
	// sign bank information for the gloss associated with this properties object
	// GD: 2022-03-39: all of the below are not used anymore, removed from the easy places, but deprecated for the complicated usages
	@Deprecated
	private boolean verifiedFromSignBank = false;
	@Deprecated
	private boolean differingFromOccurrence = false;
	@Deprecated
	private String sourceResource = "";
	@Deprecated
	private String mainGlossID = "";
	@Deprecated
	private String variantID = "";
	@Deprecated
	private String occurrenceID = "";
	
	private boolean nameSign = false;
	private boolean oneFrameGloss = false;
	private boolean markedNumberOfHands = false;
	private boolean passiveBaseArm = false;
	
	private boolean twoHanded = false;

	private boolean twoHandedEnabled = false;

	// "SAME HANDSHAPES DOM NDOM" "ALTERNATING SAME HANDSHAPES DOM NDOM", "DIFFERENT HANDSHAPES DOM NDOM", "" if not applicable
	private String twoHandedHandshapesClassification = GlossEntityProperties.SAME_HANDSHAPES_DOM_NDOM;
	
	// "SAME START/END HANDSHAPE", "DIFFERENT START/END HANDSHAPE", "" if not applicable 
	private String startEndHandshape = "";
	
	// "SAME START/END HANDSHAPE", "DIFFERENT START/END HANDSHAPE", "" if not applicable
	private String startEndHandshapeNdomHand = "";
	
	// "SAME START/END HANDSHAPE", "DIFFERENT START/END HANDSHAPE", "" if not applicable
	private String startEndHandshapeDomHand = "";

	// "R", "L"
	@Deprecated
	private String dominamtHand = "R";
	private String onehandedSignHand = "R";

	public static String SAME_HANDSHAPES_DOM_NDOM = "SAME HANDSHAPES DOM NDOM";
	public static String ALTERNATING_SAME_HANDSHAPES_DOM_NDOM = "ALTERNATING SAME HANDSHAPES DOM NDOM";
	public static String DIFFERENT_HANDSHAPES_DOM_NDOM = "DIFFERENT HANDSHAPES DOM NDOM";
	
	public static String SAME_START_END_HANDSHAPE = "SAME START/END HANDSHAPE";
	public static String DIFFERENT_START_END_HANDSHAPE = "DIFFERENT START/END HANDSHAPE";

	private LocationInfo locationInfo = new LocationInfo();
	
	public GlossEntityProperties() {}
	
	public boolean isVerifiedFromSignBank() {
		return verifiedFromSignBank;
	}
	
	public void setVerifiedFromSignBank(boolean verifiedFromSignBank) {
		this.verifiedFromSignBank = verifiedFromSignBank;
	}
	
	public boolean isDifferingFromOccurrence() {
		return differingFromOccurrence;
	}
	
	public void setDifferingFromOccurrence(boolean differingFromOccurrence) {
		this.differingFromOccurrence = differingFromOccurrence;
	}
	
	public String getSourceResource() {
		return sourceResource;
	}
	
	public void setSourceResource(String sourceResource) {
		this.sourceResource = sourceResource;
	}
	
	public String getMainGlossID() {
		return mainGlossID;
	}
	
	public void setMainGlossID(String mainGlossID) {
		this.mainGlossID = mainGlossID;
	}
	
	public String getVariantID() {
		return variantID;
	}
	
	public void setVariantID(String variantID) {
		this.variantID = variantID;
	}
	
	public String getOccurrenceID() {
		return occurrenceID;
	}
	
	public void setOccurrenceID(String occurrenceID) {
		this.occurrenceID = occurrenceID;
	}
		
	public boolean isTwoHandedEnabled() {
		return twoHandedEnabled;
	}

	public void setTwoHandedEnabled(boolean twoHandedEnabled) {
		this.twoHandedEnabled = twoHandedEnabled;
	}
	
	public boolean isNameSign() {
		return nameSign;
	}

	public void setNameSign(boolean nameSign) {
		this.nameSign = nameSign;
	}
	
	public boolean isOneFrameGloss() {
		return oneFrameGloss;
	}
	
	public void setOneFrameGloss(boolean oneFrameGloss) {
		this.oneFrameGloss = oneFrameGloss;
	}

	public boolean isMarkedNumberOfHands() {
		return markedNumberOfHands;
	}

	public void setMarkedNumberOfHands(boolean markedNumberOfHands) {
		this.markedNumberOfHands = markedNumberOfHands;
	}
	
	public void setMarkedNumberOfHandsString(String markedNumberOfHandsStr) {
		this.markedNumberOfHands = markedNumberOfHandsStr.trim().equalsIgnoreCase("T");
	}

	public boolean isPassiveBaseArm() {
		return passiveBaseArm;
	}

	public void setPassiveBaseArm(boolean passiveBaseArm) {
		this.passiveBaseArm = passiveBaseArm;
	}

	public boolean isTwoHanded() {
		return twoHanded;
	}

	public void setTwoHanded(boolean twoHanded) {
		//if(!twoHanded) {
		//	this.twoHandedHandshapesClassification = "";
		//	this.startEndHandshape = "";
		//	this.startEndHandshapeNdomHand = "";
		//	this.startEndHandshapeDomHand = "";
		//}
		this.twoHanded = twoHanded;
	}

	public String getTwoHandedHandshapesClassification() {
		return twoHandedHandshapesClassification;
	}

	public void setTwoHandedHandshapesClassification(String twoHandedHandshapesClassification) {
		this.twoHandedHandshapesClassification = twoHandedHandshapesClassification;
	}

	public String getStartEndHandshape() {
		return startEndHandshape;
	}

	public void setStartEndHandshape(String startEndHandshape) {
		this.startEndHandshape = startEndHandshape;
	}

	public String getStartEndHandshapeNdomHand() {
		return startEndHandshapeNdomHand;
	}

	public void setStartEndHandshapeNdomHand(String startEndHandshapeNdomHand) {
		this.startEndHandshapeNdomHand = startEndHandshapeNdomHand;
	}

	public String getStartEndHandshapeDomHand() {
		return startEndHandshapeDomHand;
	}

	public void setStartEndHandshapeDomHand(String startEndHandshapeDomHand) {
		this.startEndHandshapeDomHand = startEndHandshapeDomHand;
	}

	@Deprecated
	public String getDominamtHand() {
		return dominamtHand;
	}

	@Deprecated
	public void setDominamtHand(String dominamtHand) {
		this.dominamtHand = dominamtHand;
	}

	public void setFingerspelled() {
		this.signType = SignType.FINGERSPELLED;
	}
	
	public void setLoan() {
		this.signType = SignType.LOAN;
	}
	
	public void setLexical() {
		this.signType = SignType.LEXICAL;
	}
	
	public void setClassifier() {
		this.signType = SignType.CLASSIFIER;
	}
	
	public void setSignTypeAsString(String signType) {
		if (signType.isEmpty()) {
			this.signType = null;
		} else {
			this.signType = SignType.valueOf(signType);
		}
	}
	
	public void setGesture() {
		this.signType = SignType.GESTURE;
	}
	
	public void setNumber() {
		this.signType = SignType.NUMBER;
	}
	
	public SignType getSignType() {
		return this.signType;
	}
	
	public boolean isFingerspelled() {
		return SignType.FINGERSPELLED.equals(this.signType);
	}
	
	public boolean isLoan() {
		return SignType.LOAN.equals(this.signType);
	}
	
	public boolean isLexical() {
		return SignType.LEXICAL.equals(this.signType);
	}
	
	public boolean isClassifier() {
		return SignType.CLASSIFIER.equals(this.signType);
	}
	
	public boolean isGesture() {
		return SignType.GESTURE.equals(this.signType);
	}
	
	public boolean isNumber() {
		return SignType.NUMBER.equals(this.signType);
	}
	
	public String getOnehandedSignHand() {
		return onehandedSignHand;
	}

	public void setOnehandedSignHand(String onehandedSignHand) {
		this.onehandedSignHand = onehandedSignHand;
	}
	
	public String getNameSign() {
		if(nameSign)
			return "T";
		return "F";
	}
	
	public String getPassiveBaseArm() {
		if(passiveBaseArm) 
			return "T";
		return "F";
	}
	 
	public String getMarkedNumberOfHands() {
		if(markedNumberOfHands) 
			return "T";
		return "F";
	}
	
	public String getNumberOfHands() {
		if(twoHanded) 
			return "2";
		return "1";
	}

	public LocationInfo getLocationInfo() {
		return locationInfo;
	}

	public void setLocationInfo(LocationInfo locationInfo) {
		this.locationInfo = locationInfo;
	}

	public void clearHandshapeInformation() {
		twoHandedHandshapesClassification = "";
		startEndHandshape = "";
		startEndHandshapeNdomHand = "";
		startEndHandshapeDomHand = "";
	}
	
	@Override
	public String toString() {		
		String stringToPrint = "SignType = " + signType + "\n, nameSign = " + nameSign + ", \n markedNumberOfHands = " + markedNumberOfHands + ", \n passiveBaseArm = " + passiveBaseArm;
		stringToPrint = stringToPrint + ", \n twoHanded = " + twoHanded + ", \n twoHandedEnabled = " + twoHandedEnabled + ", \n twoHandedHandshapesClassification = " + twoHandedHandshapesClassification;
		stringToPrint = stringToPrint + ", \n startEndHandshape = " + startEndHandshape + ", \n startEndHandshapeNdomHand = " + startEndHandshapeNdomHand + ", \n startEndHandshapeDomHand = " + startEndHandshapeDomHand;
		stringToPrint = stringToPrint + ", \n onehandedSignHand = " + onehandedSignHand;
		return stringToPrint; 
	}
}
