package edu.bu.signstream.grepresentation.fields.palettes;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

public class GestureOrientationImagesPanel<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends GeneralHandShapePalettePanel<T, K> {
    private ArrayList<FingerTipImage> selectedFingerTipImages;
    private ArrayList<PalmImage> selectedPalmImages;

    public GestureOrientationImagesPanel(HandShapesPaletteFormBean formBean, SS3CodingScheme scheme, ParentPaletteInformationHelper<K> parentPaletteInformationPanel, T selectionPanel, String templateID, String currentPaletteID) {
        super(formBean, scheme, parentPaletteInformationPanel, selectionPanel, templateID, currentPaletteID);
        this.selectedFingerTipImages = new ArrayList<>(formBean.getSelectedFingerTipOrientationImages());
        this.selectedPalmImages = new ArrayList<>(formBean.getSelectedPalmOrientationImages());
    }

    @Override
    public JLabel getImage(SS3FieldValue value) {
        // Determine if string matches regex
        OrientationImage image;
        if (value.getIid().matches("FTORN_\\d{3}")) {
            image = new FingerTipImage(value, super.getScheme());
        } else if (value.getIid().matches("PLMORN_\\d{3}")) {
            image = new PalmImage(value, super.getScheme());
        } else {
            image = new OrientationImage(value, super.getScheme());
        }

        Image orientationImage = image.getImage();
        JLabel imageLabel = new JLabel();
        Font imageLabelFont = new Font(imageLabel.getFont().getName(), Font.PLAIN, 10);
        imageLabel.setFont(imageLabelFont);
        imageLabel.setBorder(BorderFactory.createLineBorder(UIPalette.getBackground()));
        imageLabel.setOpaque(true);
        imageLabel.setIcon(new ImageIcon(orientationImage));
        imageLabel.setText(value.getLabel());

        imageLabel.addMouseListener(new MouseListener() { // TODO: Since we need to override the mousePressed method, we need to implement the MouseListener interface
            @Override
            public void mouseClicked(MouseEvent e) {}

            /**
             * Invoked when a mouse button has been pressed on a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mousePressed(MouseEvent e) {
                // Determine what panel the image is in
                // GestureOrientationPalettePanel.super.getSelectionPanel().setItem(image);

                SelectedGestureHandshapesPanel selectionPanel = (SelectedGestureHandshapesPanel) GestureOrientationImagesPanel.super.getSelectionPanel();

                if (image instanceof FingerTipImage) {
                    GestureOrientationImagesPanel.this.selectedFingerTipImages.add((FingerTipImage) image);
                    selectionPanel.addFingerTipOrientation((FingerTipImage) image);
                } else if (image instanceof PalmImage) {
                    GestureOrientationImagesPanel.this.selectedPalmImages.add((PalmImage) image);
                    selectionPanel.addPalmOrientation((PalmImage) image);
                }
            }

            /**
             * Invoked when a mouse button has been released on a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseReleased(MouseEvent e) {}

            /**
             * Invoked when the mouse enters a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseEntered(MouseEvent e) {}

            /**
             * Invoked when the mouse exits a component.
             *
             * @param e the event to be processed
             */
            @Override
            public void mouseExited(MouseEvent e) {}
        });

        return imageLabel;
    }

    public ArrayList<FingerTipImage> getSelectedFingerTipImages() { // TODO: Hasnain, only one of these, not both.
        return this.selectedFingerTipImages;
    }

    public ArrayList<PalmImage> getSelectedPalmImages() {
        return this.selectedPalmImages;
    }
}
