package edu.bu.signstream.grepresentation.fields.palettes;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteMenu;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.PaletteReference;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GesturePaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Objects;

// GestureHandShapePanel extends JPanel implements ItemListener, ActionListener, ParentPaletteInformationHelper

public class GesturePaletteDialog<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends GlossPaletteDialog<T, K> {
    private final String palmPaletteGroupID;
    private final String fingerTipPaletteGroupID;
    private GestureOrientationPalettePanel<T, K> orientationPalettePanel;

    public GesturePaletteDialog(HandShapesPaletteFormBean formBean, HandShapeFormBean bean, SS3CodingScheme scheme, JDialog parentDialog, ParentPaletteInformationHelper<K> parentHandShapePanel,
                                T selectionPanel, String defaultMenuID, String paletteID, String fingerTipPaletteGroupID, String palmPaletteGroupID) {
        super(formBean, bean, scheme, parentDialog, parentHandShapePanel, selectionPanel, defaultMenuID, paletteID);
        this.fingerTipPaletteGroupID = fingerTipPaletteGroupID;
        this.palmPaletteGroupID = palmPaletteGroupID;
        this.initializeGesturePaletteDialog();
    }


    private void initializeGesturePaletteDialog() {
        GeneralHandShapePalettePanel<T, K> centerPalette = super.getCenterPalettePanel();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        // Create Orientation Palette for Finger Tips and Palm

        this.orientationPalettePanel = new GestureOrientationPalettePanel<>(super.getFormBean(),
                super.getScheme(), super.getParentHandShapePanel(), super.getSelectionPanel(),
                super.getPaletteInformationHelper(), super.getDefaultMenuID(), this.fingerTipPaletteGroupID, this.palmPaletteGroupID);

        gridBagConstraints.gridy = GridBagConstraints.RELATIVE;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;

        this.orientationPalettePanel.setPreferredSize(new Dimension(centerPalette.getPreferredSize().width, this.orientationPalettePanel.getPreferredSize().height));
        centerPalette.add(this.orientationPalettePanel, gridBagConstraints);

        centerPalette.revalidate();
        centerPalette.setSize(centerPalette.getPreferredSize());
        centerPalette.repaint();

        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();
    }

    private void onChangeListener() {
        // super.getTopPanel()

    }


    @Override
    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
            super.itemStateChanged(e);
            this.orientationPalettePanel.removeAll();
            this.initializeGesturePaletteDialog();
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();

        if (actionCommand.equals(super.getCANCEL())) {
            super.dispose();
        } else if (actionCommand.equals(super.getENTER())) {
            ArrayList<HandShapeImage> selectedHandshapes = super.getCenterPalettePanel().getSelectionPanel().getHandshapes();
            ArrayList<FingerTipImage> fingerTipOrientations = ((SelectedGestureHandshapesPanel) (super.getCenterPalettePanel().getSelectionPanel())).getFingerTipOrientationsImages();
            ArrayList<PalmImage> palmOrientations = ((SelectedGestureHandshapesPanel) (super.getCenterPalettePanel().getSelectionPanel())).getPalmOrientationsImages();

            GesturePaletteInformationHelper hsPanel = (GesturePaletteInformationHelper) super.getParentHandShapePanel();
            HandShapeFormBean bean = super.getBean();
            HandShapesPaletteFormBean formBean = super.getFormBean();
            // first check if this is a one-frame gesture, and if it is then disallow anything more than one handshape in either hand
            if (hsPanel.isOneFrameEntity()) {
                if (selectedHandshapes.size() > 1) {
                    SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_EXACTLY_ONE);
                    return;
                }
            } else if (hsPanel.getSelectedSameDifStartEndHandshapes() == GestureStartEndHandshapesType.DIFFERENT && (selectedHandshapes.size() == 1)) {
                // conversely, if this is not a one-frame gloss and you have dif start/end selected,
                // then disallow a selection of just one handshape in either hand
                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_MULTIPLE);
                return;
            } else if (selectedHandshapes.size() != fingerTipOrientations.size() || selectedHandshapes.size() != palmOrientations.size()) {
                // user must select the same number of handshape and orientation images
                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_SELECT_HANDSHAPES_ORIENTATIONS);
                return;
            }

//            // if user only selected none-image for one of above lists - disallow
//            boolean onlyNoneImage = false;
//            onlyNoneImage = !selectedHandshapes.isEmpty() && selectedHandshapes.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_HANDSHAPE_IMAGE_ID)) || !fingerTipOrientations.isEmpty() && fingerTipOrientations.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_FINGERTIP_IMAGE_ID)) || !palmOrientations.isEmpty() && palmOrientations.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_PALM_IMAGE_ID));
//
//            if (onlyNoneImage) {
//                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANNOT_SELECT_ONLY_NONE);
//                return;
//            }

            bean.setUpdated(true);

            if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT) {
                if (super.getParentHandShapePanel().isDominantHandRight()) {
                    bean.setSelectedDomHandShapeImages(selectedHandshapes);
                    bean.setSelectedDomOrientationFingerTipImages(fingerTipOrientations);
                    bean.setSelectedDomPalmOrientationImages(palmOrientations);
                } else {
                    bean.setSelectedNdomHandShapeImages(selectedHandshapes);
                    bean.setSelectedNdomOrientationFingerTipImages(fingerTipOrientations);
                    bean.setSelectedNdomPalmOrientationImages(palmOrientations);
                }
            }

            if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.LEFT) {
                if (super.getParentHandShapePanel().isDominantHandRight()) {
                    bean.setSelectedNdomHandShapeImages(selectedHandshapes);
                    bean.setSelectedNdomOrientationFingerTipImages(fingerTipOrientations);
                    bean.setSelectedNdomPalmOrientationImages(palmOrientations);
                } else {
                    bean.setSelectedDomHandShapeImages(selectedHandshapes);
                    bean.setSelectedDomOrientationFingerTipImages(fingerTipOrientations);
                    bean.setSelectedDomPalmOrientationImages(palmOrientations);
                }
            }

            if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT) {
                bean.setSelectedDomHandShapeImages(selectedHandshapes);
                bean.setSelectedNdomHandShapeImages(selectedHandshapes);
                bean.setSelectedDomOrientationFingerTipImages(fingerTipOrientations);
                bean.setSelectedNdomOrientationFingerTipImages(fingerTipOrientations);
                bean.setSelectedDomPalmOrientationImages(palmOrientations);
                bean.setSelectedNdomPalmOrientationImages(palmOrientations);
            }

            hsPanel.updateParentInfoSelectedEntities(bean.getSelectedDomHandShapeImages(),
                    bean.getSelectedNdomHandShapeImages(),
                    bean.getSelectedDomOrientationFingerTipImages(),
                    bean.getSelectedNdomOrientationFingerTipImages(),
                    bean.getSelectedDomOrientationPalmImages(),
                    bean.getSelectedNdomOrientationPalmImages());

            super.dispose();
        }
    }
}
