package edu.bu.signstream.grepresentation.fields.palettes;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapePalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.templates.CSHandShapeTemplate;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GenericStartEndHandShapeTypes;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.GestureStartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.SelectedGestureHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GlossPaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

public class GlossPaletteDialog<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends GeneralPaletteDialog<T, K> {
    private GeneralHandShapePalettePanel<T, K> centerPalettePanel;
    private final JPanel centerPalettePanelContainer;


    public GlossPaletteDialog(HandShapesPaletteFormBean formBean, HandShapeFormBean bean, SS3CodingScheme scheme, JDialog parentDialog, ParentPaletteInformationHelper<K> parentHandShapePanel, T selectionPanel, String defaultMenuID, String paletteID) {
        super(formBean, bean, scheme, parentDialog, parentHandShapePanel, selectionPanel, defaultMenuID, Util.DEFAULT_GLOSS_HANDSHAPE_PALETTE_GROUP_ID, paletteID);

        String selectedPaletteID = super.getParentHandShapePanel().isDomHandSelected() ? super.getPaletteInformationHelper().getCurrentDomPaletteID() : super.getPaletteInformationHelper().getCurrentNDomPaletteID();

        if (selectedPaletteID.equals(Util.LETTERS_PALETTE_ID)) {
            this.centerPalettePanel = new LettersPalette<T, K>(formBean, scheme, super.getParentHandShapePanel(), selectionPanel, super.getPaletteInformationHelper().getCurrentTemplateID(), selectedPaletteID);
            if (selectionPanel instanceof SelectedHandshapesPanel) {
            	SelectedHandshapesPanel shp = (SelectedHandshapesPanel) selectionPanel;
            	shp.setLettersPalette((LettersPalette)this.centerPalettePanel);
            } else if (selectionPanel instanceof SelectedGestureHandshapesPanel) {
            	SelectedGestureHandshapesPanel sghp = (SelectedGestureHandshapesPanel) selectionPanel;
            	sghp.setLettersPalette((LettersPalette)this.centerPalettePanel);
            }
        } else {
            this.centerPalettePanel = new GeneralHandShapePalettePanel<>(formBean, scheme, super.getParentHandShapePanel(), selectionPanel, super.getPaletteInformationHelper().getCurrentTemplateID(), selectedPaletteID);
        }

        this.centerPalettePanelContainer = new JPanel();
        this.centerPalettePanelContainer.setLayout(new GridLayout());
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        this.centerPalettePanelContainer.add(this.centerPalettePanel, gridBagConstraints);

        super.add(this.centerPalettePanelContainer, BorderLayout.CENTER);
        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();

    }

    public GeneralHandShapePalettePanel<T, K> getCenterPalettePanel() {
        return this.centerPalettePanel;
    }

    public JPanel getCenterPalettePanelContainer() {
        return this.centerPalettePanelContainer;
    }


    @Override
    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
            String currentItem = (String) e.getItem();
            CSHandShapeTemplate defaultTemplate = super.getScheme().getHandShapeTemplates().getTemplates().get(super.getPaletteInformationHelper().getCurrentTemplateID()); // Get the default template based on the template ID
            String palettePrefix = defaultTemplate.getLabel(); // For example: "ASL" for "American Sign Language" ~ Straight from the defCodingScheme.xml

            for (CSHandShapePalette palette : super.getPaletteInformationHelper().getHandShapePalettes()) {
                String paletteName = String.format("%s - %s", palettePrefix, palette.getName());
                if (currentItem.equals(paletteName)) {
                    super.getPaletteInformationHelper().setCurrentDomPaletteID(palette.getId());
                    break;
                }
            }

            this.centerPalettePanelContainer.remove(this.centerPalettePanel);

            if (super.getPaletteInformationHelper().getCurrentDomPaletteID().equals(Util.LETTERS_PALETTE_ID)) {
                this.centerPalettePanel = new LettersPalette<>(super.getFormBean(), super.getScheme(), super.getParentHandShapePanel(), super.getSelectionPanel(), super.getPaletteInformationHelper().getCurrentTemplateID(), super.getPaletteInformationHelper().getCurrentDomPaletteID());
                var selectionPanel = super.getSelectionPanel();
                if (selectionPanel instanceof SelectedHandshapesPanel) {
                	SelectedHandshapesPanel shp = (SelectedHandshapesPanel) selectionPanel;
                	shp.setLettersPalette((LettersPalette)this.centerPalettePanel);
                } else if (selectionPanel instanceof SelectedGestureHandshapesPanel) {
                	SelectedGestureHandshapesPanel sghp = (SelectedGestureHandshapesPanel) selectionPanel;
                	sghp.setLettersPalette((LettersPalette)this.centerPalettePanel);
                }
            } else {
                this.centerPalettePanel = new GeneralHandShapePalettePanel<>(super.getFormBean(), super.getScheme(), super.getParentHandShapePanel(), super.getSelectionPanel(), super.getPaletteInformationHelper().getCurrentTemplateID(),  super.getPaletteInformationHelper().getCurrentDomPaletteID());
            }

            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.fill = GridBagConstraints.BOTH;
            gridBagConstraints.weightx = 1.0;
            gridBagConstraints.weighty = 1.0;
            gridBagConstraints.anchor = GridBagConstraints.NORTH;

            this.centerPalettePanelContainer.add(this.centerPalettePanel, gridBagConstraints);
            this.centerPalettePanelContainer.revalidate();
            super.setSize(super.getPreferredSize());
            this.centerPalettePanelContainer.repaint();
            super.revalidate();
            super.repaint();

            System.gc();
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();

        HandShapeFormBean bean =  super.getBean();
        HandShapesPaletteFormBean formBean = super.getFormBean();
        GlossPaletteInformationHelper parentHandShapePanel = (GlossPaletteInformationHelper) super.getParentHandShapePanel();

        if (actionCommand.equals(super.getCANCEL())) {
            super.dispose();
        } else if (actionCommand.equals(super.getENTER())) {
            ArrayList<HandShapeImage> selectedHandShapeImages = super.getSelectionPanel().getHandshapes();
            if (parentHandShapePanel.isOneFrameEntity()) {
                if (selectedHandShapeImages.size() > 1) {
                    SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_HANDSHAPES_EXACTLY_ONE);
                    return;
                } else if (parentHandShapePanel.getSelectedSameDifStartEndHandshapes() == StartEndHandshapesType.DIFFERENT) {
                	SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
                	return;
                }
            }

            bean.setUpdated(true);

            if (super.getFormBean().getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT) {
                if (parentHandShapePanel.isDominantHandRight()) {
                    bean.setSelectedDomHandShapeImages(selectedHandShapeImages);
                } else {
                    bean.setSelectedNdomHandShapeImages(selectedHandShapeImages);
                }
            }

            if (super.getFormBean().getPaletteType() == HandShapesPaletteFormBean.PaletteType.LEFT) {
                if (parentHandShapePanel.isDominantHandRight()) {
                    bean.setSelectedNdomHandShapeImages(selectedHandShapeImages);
                } else {
                    bean.setSelectedDomHandShapeImages(selectedHandShapeImages);
                }
            }

            if (formBean.getPaletteType() == HandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT) {
                bean.setSelectedDomHandShapeImages(selectedHandShapeImages);
                bean.setSelectedNdomHandShapeImages(selectedHandShapeImages);
            }


            parentHandShapePanel.updateParentInfoSelectedHandshapes(bean.getSelectedDomHandShapeImages(),
                    bean.getSelectedNdomHandShapeImages());

            super.getParentDialog().setVisible(true);

            super.dispose();
        }
    }
}
