package edu.bu.signstream.grepresentation.fields.palettes.doublePalettes;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.handShapes.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.HandShapeFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedPanel;
import edu.bu.signstream.grepresentation.fields.palettes.GeneralHandShapePalettePanel;
import edu.bu.signstream.grepresentation.fields.palettes.GestureOrientationPalettePanel;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.GesturePaletteInformationHelper;
import edu.bu.signstream.grepresentation.fields.palettes.helpers.interfaces.ParentPaletteInformationHelper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.util.ArrayList;

public class GestureDoublePaletteDialog<T extends SelectedPanel, K extends Enum<?> & GenericStartEndHandShapeTypes> extends GlossDoublePaletteDialog<T, K> {
    private GestureOrientationPalettePanel<T, K> rightHandOrientationPanel;
    private GestureOrientationPalettePanel<T, K> leftHandOrientationPanel;

    private JPanel doubleHandShapePanel;
    private JPanel orientationPanelContainer;

    private final String fingerTipPaletteID;
    private final String palmPaletteID;

    public GestureDoublePaletteDialog(HandShapesPaletteFormBean domHandFormBean,
                                      HandShapesPaletteFormBean nonDomHandFormBean, HandShapeFormBean bean,
                                      ParentPaletteInformationHelper<K> parentInformationPanel, T domHandSelectionPanel,
                                      T nonDomHandSelectionPanel, SS3CodingScheme scheme, JDialog parentDialog,
                                      String defaultPaletteMenuID, String selectedPaletteID, String fingerTipPaletteID,
                                      String palmPaletteID) {
        super(domHandFormBean, nonDomHandFormBean, bean, parentInformationPanel, domHandSelectionPanel, nonDomHandSelectionPanel, scheme, parentDialog, defaultPaletteMenuID, selectedPaletteID);
        this.fingerTipPaletteID = fingerTipPaletteID;
        this.palmPaletteID = palmPaletteID;
        this.addOrientationPanels();

        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();
    }

    @Override
    protected void updateCenterPanel(GeneralHandShapePalettePanel<T, K> panel, int indexToAdd) {
        this.doubleHandShapePanel.remove(indexToAdd);
        this.doubleHandShapePanel.add(panel, indexToAdd);
        this.doubleHandShapePanel.revalidate();
        this.doubleHandShapePanel.setSize(this.doubleHandShapePanel.getPreferredSize());
        this.doubleHandShapePanel.repaint();

        super.getCenterPanel().revalidate();
        super.getCenterPanel().setSize(super.getCenterPanel().getPreferredSize());
        super.getCenterPanel().repaint();

        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();
    }

    private void updateOrientationPanelSize() {
        Dimension rightHandOrientationPanelSize = new Dimension(super.getRightHandPanel().getWidth(), this.rightHandOrientationPanel.getPreferredSize().height);
        Dimension leftHandOrientationPanelSize = new Dimension(super.getLeftHandPanel().getWidth(), this.leftHandOrientationPanel.getPreferredSize().height);


        this.rightHandOrientationPanel.setPreferredSize(rightHandOrientationPanelSize);
        this.leftHandOrientationPanel.setPreferredSize(leftHandOrientationPanelSize);
        super.getRightHandSelectionClearPanel().setPreferredSize(new Dimension(super.getRightHandPanel().getWidth(), super.getRightHandSelectionClearPanel().getPreferredSize().height));
        super.getLeftHandSelectionClearPanel().setPreferredSize(new Dimension(super.getLeftHandPanel().getWidth(), super.getLeftHandSelectionClearPanel().getPreferredSize().height));
    }

    private void revalidateCenterPanelComponents() {
        this.doubleHandShapePanel.revalidate();
        this.doubleHandShapePanel.setSize(this.doubleHandShapePanel.getPreferredSize());
        this.doubleHandShapePanel.repaint();

        this.orientationPanelContainer.revalidate();
        this.orientationPanelContainer.setSize(this.orientationPanelContainer.getPreferredSize());
        this.orientationPanelContainer.repaint();

        super.getCenterPanel().revalidate();
        super.getCenterPanel().setSize(super.getCenterPanel().getPreferredSize());
        super.getCenterPanel().repaint();

        super.revalidate();
        super.setSize(super.getPreferredSize());
        super.repaint();
    }

    @Override
    protected void addComboBoxListeners() {
        super.getRightHandComboBox().addItemListener(e -> {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                String currentItem = (String) e.getItem();
                super.getPaletteInformationHelper().setCurrentDomPaletteID(super.getPaletteID(currentItem));
                int rightHandPanelIndex = this.doubleHandShapePanel.getComponentZOrder(super.getRightHandPanel());
                super.setRightHandPanel(super.getNewPanel(true));
                this.updateCenterPanel(super.getRightHandPanel(), rightHandPanelIndex);

                this.updateOrientationPanelSize();
                this.revalidateCenterPanelComponents();
            }
        });

        super.getLeftHandComboBox().addItemListener(e -> {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                String currentItem = (String) e.getItem();
                super.getPaletteInformationHelper().setCurrentNDomPaletteID(super.getPaletteID(currentItem));
                int leftHandPanelIndex = this.doubleHandShapePanel.getComponentZOrder(super.getLeftHandPanel());
                super.setLeftHandPanel(super.getNewPanel(false));
                this.updateCenterPanel(super.getLeftHandPanel(), leftHandPanelIndex);

                this.updateOrientationPanelSize();
                this.revalidateCenterPanelComponents();
            }
        });
    }

    @Override
    protected void addBottomPanelListeners() {
        JButton cancelButton = super.getCancelButton();
        JButton enterButton = super.getEnterButton();

        cancelButton.addActionListener(e -> super.dispose());

        enterButton.addActionListener(e -> {
            super.getBean().setUpdated(true);

            ArrayList<HandShapeImage> rhImages = new ArrayList<>(super.getRightHandPanel().getSelectionPanel().getHandshapes());
            ArrayList<HandShapeImage> lhImages =  new ArrayList<>(super.getLeftHandPanel().getSelectionPanel().getHandshapes());
            ArrayList<FingerTipImage> rOImages = new ArrayList<>(((SelectedGestureHandshapesPanel) super.getRightHandPanel().getSelectionPanel()).getFingerTipOrientationsImages());
            ArrayList<FingerTipImage> lOImages =  new ArrayList<>(((SelectedGestureHandshapesPanel) super.getLeftHandPanel().getSelectionPanel()).getFingerTipOrientationsImages());
            ArrayList<PalmImage> rPOImages = new ArrayList<>(((SelectedGestureHandshapesPanel) super.getRightHandPanel().getSelectionPanel()).getPalmOrientationsImages());
            ArrayList<PalmImage> lPOImages =  new ArrayList<>(((SelectedGestureHandshapesPanel) super.getLeftHandPanel().getSelectionPanel()).getPalmOrientationsImages());

            GesturePaletteInformationHelper parentInfoPanel = (GesturePaletteInformationHelper) super.getParentPaletteInformationPanel();

            // first check if this is a one-frame gloss, and if it is then disallow anything more than one handshape in either hand
            if (parentInfoPanel.isOneFrameEntity()) {
                if (rhImages.size() > 1 || lhImages.size() > 1) {
                    SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_EXACTLY_ONE);
                    return;
                }
            } else if ( (parentInfoPanel.getSelectedSameDifStartEndHandshapesDom() == GestureStartEndHandshapesType.DIFFERENT && rhImages.size() == 1)
                    || (parentInfoPanel.getSelectedSameDifStartEndHandshapesNdom() == GestureStartEndHandshapesType.DIFFERENT && lhImages.size() == 1) ) {
                // conversely, if this is not a one-frame gloss and you have dif start/end selected,
                // then disallow a selection of just one handshape in either hand
                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.SELECT_GESTURE_HANDSHAPES_MULTIPLE);
                return;
            } else if ( (rhImages.size() != rOImages.size()) || (lhImages.size() != lOImages.size()) || (rhImages.size() != rPOImages.size()) || (lhImages.size() != lPOImages.size())) {
                // user must select the same number of handshape and orientation images
                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_SELECT_HANDSHAPES_ORIENTATIONS);
                return;
            }

//            // if user only selected none-image for one of above lists - disallow
//            boolean onlyNoneImage = false;
//            onlyNoneImage = !rhImages.isEmpty() && rhImages.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_HANDSHAPE_IMAGE_ID)) || !lhImages.isEmpty() && lhImages.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_HANDSHAPE_IMAGE_ID)) || !rOImages.isEmpty() && rOImages.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_FINGERTIP_IMAGE_ID)) || !lOImages.isEmpty() && lOImages.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_FINGERTIP_IMAGE_ID)) || !rPOImages.isEmpty() && rPOImages.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_PALM_IMAGE_ID)) || !lPOImages.isEmpty() && lPOImages.stream().allMatch(
//                    img -> img.getSS3FieldValue().getIid().equalsIgnoreCase(Util.DNI_PALM_IMAGE_ID));
//
//            if (onlyNoneImage) {
//                SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.GESTURE_CANNOT_SELECT_ONLY_NONE);
//                return;
//            }

            parentInfoPanel.updateParentInfoSelectedEntities(rhImages, lhImages, rOImages, lOImages, rPOImages, lPOImages);

            super.dispose();
        });
    }



    private void addOrientationPanels() {
        this.orientationPanelContainer = new JPanel();
        this.orientationPanelContainer.setLayout(new BoxLayout(this.orientationPanelContainer, BoxLayout.X_AXIS));

        this.rightHandOrientationPanel = new GestureOrientationPalettePanel<>(super.getRightHandFormBean(), super.getScheme(), super.getParentPaletteInformationPanel(),
                super.getDomHandSelectionPanel(), super.getPaletteInformationHelper(), super.getDefaultPaletteMenuID(), this.fingerTipPaletteID, this.palmPaletteID);

        this.leftHandOrientationPanel = new GestureOrientationPalettePanel<>(super.getLeftHandFormBean(), super.getScheme(), super.getParentPaletteInformationPanel(),
                super.getNonDomHandSelectionPanel(), super.getPaletteInformationHelper(), super.getDefaultPaletteMenuID(), this.fingerTipPaletteID, this.palmPaletteID);


        this.rightHandOrientationPanel.revalidate();
        this.leftHandOrientationPanel.revalidate();

        this.updateOrientationPanelSize();

        this.orientationPanelContainer.add(this.rightHandOrientationPanel);
        this.orientationPanelContainer.add(this.leftHandOrientationPanel);

        this.doubleHandShapePanel = super.getCenterPanel();

        JPanel newCenterPanel = new JPanel();
        newCenterPanel.setLayout(new BoxLayout(newCenterPanel, BoxLayout.Y_AXIS));
        newCenterPanel.add(this.doubleHandShapePanel);
        newCenterPanel.add(this.orientationPanelContainer);

        super.setCenterPanel(newCenterPanel);
    }

    public GestureOrientationPalettePanel<T, K> getRightHandOrientationPanel() {
        return this.rightHandOrientationPanel;
    }

    public GestureOrientationPalettePanel<T, K> getLeftHandOrientationPanel() {
        return this.leftHandOrientationPanel;
    }

}
