package edu.bu.signstream.grepresentation.listener;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Level;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureField;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.CustomFieldsLabelView;
import edu.bu.signstream.grepresentation.view.GestureFieldsLabelView;
import edu.bu.signstream.grepresentation.view.GlossFieldsLabelView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.NonManualFieldsLabelView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class GestureNRelatedFieldsListener extends SignStreamMouseListener {

	public GestureNRelatedFieldsListener(SignStreamSegmentPanel segmentPanel,
			SegmentGraphicRepresentation nonManualRep,
			SegmentGraphicRepresentation glossRep,
			SegmentGraphicRepresentation gestureRep,
			SegmentGraphicRepresentation customRep,
			NonManualFieldsLabelView nmLabelView,
			GlossFieldsLabelView glsLabelView,
			GestureFieldsLabelView gesLabelView,
			CustomFieldsLabelView cstLabelView) {

		super(segmentPanel, nonManualRep, glossRep, gestureRep, customRep, nmLabelView, glsLabelView, gesLabelView, cstLabelView);
	}

    @Override
    public SegmentGraphicRepresentation view() { return gestureRep; }

    private boolean mouseDragged = false;
	private boolean mouseDraggedAlt = false;

	/**
	 * set re sizable cursor (chained events, chained event entity, and slider)
	 * and show chained event duration
	 */
	@Override
	public void mouseMoved(MouseEvent e) {
	    super.mouseMoved(e);

		Point point = e.getPoint();

		if (slider != null
				&& point.x - 3 < slider.getSliderPosition()
				&& point.x + 3 > slider.getSliderPosition()) {
			gestureRep.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		} else {
			gestureRep.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		// change magnified events to normal
		FieldWrappersCollection wrappers = gestureRep
				.getFieldWrappersCollection();

		FieldWrapper rhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_DOM_HAND_SHAPE);
		FieldWrapper lhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_NDOM_HAND_SHAPE);

		boolean minified = false;

		if (minifyAllHandShapeIcons(rhWrapper))
			minified = true;

		if (minifyAllHandShapeIcons(lhWrapper))
			minified = true;

		if (minified) {
			gestureRep.repaint();
		}

		FieldWrapper wrapper = gestureRep.getFieldWrappersCollection()
				.getFieldWrapperAt(point.y);

		if (wrapper != null) {
			wrapper.mouseMoved(point, gestureRep);
			repaintAll();
		}
	}

	@Override
	public void mouseDragged(MouseEvent e) {
        var point = e.getPoint();
        FieldWrapper wrapper = gestureRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

        var player = SS3Singleton.getMediaPlayer();
        if (player.playing()) player.pause();

        if (wrapper == null) {
            var time = SS3Singleton.getVideoTimeRange().time(e.getX(), TimeUnit.COORDINATE);
            player.seek(time);
            return;
        }

        if (wrapper != null) {
            if (wrapper instanceof NonManualFieldWrapper) {
                NonManualFieldWrapper nmWrapper = (NonManualFieldWrapper) wrapper;
                gestureRep.setCursor(new Cursor(nmWrapper.getCursorType(point)));

                if (isShiftPressed(e) || isAltPressed(e) || isCtrlPressed(e))
                    nmWrapper.mouseDraggedAlt(point);
                else
                    wrapper.mouseDragged(point, gestureRep);
            } else {
                wrapper.mouseDragged(point, gestureRep);
            }
        }

        repaintAll();
	}

	
	public FieldWrapper getFieldWrapperWithSelectedEvent() {
		return gestureRep.getFieldWrappersCollection().getSelectedFieldWrapper();
	}

	private int ex = -10;

	@Override
	public void mousePressed(MouseEvent e) {
        segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut().requestFocus();

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Pressed");
		Util.dragging(true);

		if (ex == e.getPoint().x)
			return;
		
		ex = e.getPoint().x;

		boolean isShiftPressed = isShiftPressed(e);
		boolean isAltPressed = isAltPressed(e);
		boolean isCtrlPressed = isCtrlPressed(e);

		if (!isShiftPressed && !isAltPressed && !isCtrlPressed)
			return;

		if (isAltPressed && isCtrlPressed)
			return;

		FieldWrapper wrapper = gestureRep.getFieldWrappersCollection()
				.getFieldWrapperAt(e.getPoint().y);

		if (wrapper == null) {
            var player = SS3Singleton.getMediaPlayer();
            var time = SS3Singleton.getVideoTimeRange().time(e.getX(), TimeUnit.COORDINATE);
            if (player.playing()) player.pause();
            player.seek(time);
            return;
        }

		if (wrapper instanceof GestureFieldWrapper) {
			GestureFieldWrapper gesWrapper = (GestureFieldWrapper) wrapper;
			GestureField gesField = (GestureField) gesWrapper.getField();

			if (isShiftPressed) {
				gesField.setSelectCompoundStartCoord(e.getPoint().x - 3);
				repaintAll();
			} else if (isAltPressed || isCtrlPressed) {
				gesWrapper.mousePressedAlt(e.getPoint(), gestureRep);

				Event selectedEvent = wrapper.getSelectedEvent();
				if (selectedEvent != null) {
					segmentPanel.setSelectedEvent(wrapper.getSelectedEvent());
					segmentPanel.setSelectedFieldWrapper(wrapper);
				}

				repaintAll();
			}
		} else {
			if (isAltPressed || isCtrlPressed) {
				wrapper.mousePressedAlt(e.getPoint(), gestureRep);
				repaintAll();
				if (e.getPoint().y > SegmentGraphReprUtil.axesBottomHeight) {
					unselectAll();
				}
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Released");
		Util.dragging(false);

		Point point = e.getPoint();
		FieldWrapper wrapper = gestureRep.getFieldWrappersCollection().getFieldWrapperAt(e.getPoint().y);

        mouseDragged = false;

		if (wrapper != null) {
			Event selectedEvent = wrapper.getSelectedEvent();
			if (selectedEvent != null) {
				segmentPanel.setSelectedFieldWrapper(wrapper);
				segmentPanel.setSelectedEvent(selectedEvent);
			}

			wrapper.mouseReleased(point, gestureRep);

			repaintAll();
		}

		// unused block
		if (wrapper instanceof HandShapeFieldWrapper && mouseDraggedAlt) {
			HandShapeFieldWrapper hsWrapper = (HandShapeFieldWrapper) wrapper;
			hsWrapper.mouseReleasedAlt(point);
			mouseDraggedAlt = false;
			repaintAll();
		}
	}

	// to prevent multiple call of dblClick method
	private Point lastMouseClickedPoint = null;

	// select chained event or chained events entity
	@Override
	public void mouseClicked(final MouseEvent e) {

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Clicked");

		if (mouseDragged) {
			// do not select
			mouseDragged = false;
			return;
		}
		
		SS3Singleton.getMediaToolBar().deselect();
		segmentPanel.repaint();

		Point point = e.getPoint();

		FieldWrapper wrapper = gestureRep.getFieldWrappersCollection()
				.getFieldWrapperAt(point.y);
		
		EventsEntity entityAt = null;
		if (wrapper != null) {
			if (wrapper instanceof GestureFieldWrapper && e.getClickCount() == 2 && !isAltPressed(e) && !isCtrlPressed(e)) {
				GestureFieldWrapper gesFldWrapper = (GestureFieldWrapper) wrapper;
				gesFldWrapper.mouseCtrlAltClicked(point);
				return;
			}

			wrapper.mouseClicked(point, gestureRep);

			if (wrapper.getSelectedEvent() != null) {
				entityAt = wrapper.getEntityAt(point.x);

				if (entityAt != null) {
					unselectAll();
					wrapper.mouseClicked(point, gestureRep);
				}
			}

			// right btn
			if (e.getButton() == MouseEvent.BUTTON3 || isCtrlPressed(e) || isAltPressed(e)) {
				wrapper.mouseDblRightClicked(point, gestureRep);
				highlightLabel(wrapper);
				return;
			}

			if (e.getClickCount() == 2) {

				if (lastMouseClickedPoint != null
						&& lastMouseClickedPoint.x == e.getX()
						&& lastMouseClickedPoint.y == e.getY()) {
					lastMouseClickedPoint = e.getPoint();
					return;
				}

				lastMouseClickedPoint = e.getPoint();

				// left btn
				if (e.getButton() == MouseEvent.BUTTON1) {
					wrapper.mouseDblLeftClicked(point, gestureRep);
					highlightLabel(wrapper);
					return;
				}
			}
		}

		if (wrapper != null) {
			Event event = wrapper.getSelectedEvent();
			if (event != null) {
				segmentPanel.setSelectedEvent(wrapper.getSelectedEvent());
				segmentPanel.setSelectedFieldWrapper(wrapper);
				
				
				
				SS3Singleton.getMediaToolBar(null).selectEvent(event, entityAt,
						wrapper.getField());
				event.select();
			}
		}
		repaintAll();
	}

	@Override
	protected void highlightLabel(FieldWrapper wrapper) {
		String fieldID = wrapper.getField().getFieldID();

		LabelObject parentObject = gesLabelView.getParentLabelObject();

		ArrayList<LabelObject> gesLabels = parentObject.getAllChildren();
		// highlight only selected field label
		for (LabelObject object : gesLabels) {
			String id = object.getFieldID();
			if (fieldID.equalsIgnoreCase(id)) {
				object.setUpdatable(true);
			} else {
				object.setUpdatable(false);
			}
		}

		ArrayList<LabelObject> labels = nmLabelView.getLabels();
		// unhighlight all labels
		for (LabelObject object : labels) {
			object.setUpdatable(false);
		}

		nmLabelView.repaint();
		glsLabelView.repaint();
		gesLabelView.repaint();
	}
}
