package edu.bu.signstream.grepresentation.requestsHandler;

import edu.bu.signstream.common.graph.Graph;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations.GestureLocationObject;
import edu.bu.signstream.time.Time;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.apexField.ApexFieldWrapper;
import edu.bu.signstream.grepresentation.fields.apexField.ChainedApexEvent;
import edu.bu.signstream.grepresentation.fields.apexField.ChainedApexEventsCollection;
import edu.bu.signstream.grepresentation.fields.domNdomField.DomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEvent;
import edu.bu.signstream.grepresentation.fields.apexField.ApexChainedApexEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEvent;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureEventsEntityCollection;
import edu.bu.signstream.grepresentation.fields.gestureField.GestureFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations.GestureLocationInfo;
import edu.bu.signstream.grepresentation.fields.gestureField.dialog.associatedFields.locations.GestureLocationType;
import edu.bu.signstream.grepresentation.fields.gestureField.domNdomField.GestureDomNdomHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.handshapes.GestureHandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.locationField.GestureLocationEvent;

import edu.bu.signstream.grepresentation.fields.gestureField.orientationField.GestureOrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.gestureField.gesturePropertyField.GesturePropertyEntity;
import edu.bu.signstream.grepresentation.fields.gestureField.gesturePropertyField.GesturePropertyFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.locationField.LocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.signTypeField.*;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.ui.SS3SignStreamApplication;

import java.awt.geom.Line2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.*;
import java.util.logging.Level;
import java.util.stream.Collectors;

public class RequestHandler {

	private SS3Database database;
	private SignStreamSegmentPanel segmentPanel;
	private FieldsFactory factory;

	public void init(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		this.factory = new FieldsFactory();
		this.database = segmentPanel.getSS3Database();

	}

	// take a panel object with the Segment objects already loaded and load
	// utterance
//	public void loadPredefinedPanelSelection() {
//		segmentPanel.loadUtterance(segmentPanel.getSegments());
//	}

	public Segment convertMultipleItemsAndAddToSegmentPanel(SS3GlossWindowSegment segmentTier,
			PresentationField parentField) {

		Segment segment = convertItemsToSegment(segmentTier, parentField);
		segmentPanel.getSegments().add(segment);

		return segment;
	}

	public Segment convertMultipleItemsAndAddToSegmentPanelAtIndex(SS3GlossWindowSegment segmentTier,
			PresentationField parentField, int indexToInsert) {

		Segment segment = convertItemsToSegment(segmentTier, parentField);

		if (indexToInsert > -1 && indexToInsert <= segmentPanel.getSegments().size())
			segmentPanel.getSegments().add(indexToInsert, segment);

		return segment;
	}

	public Segment convertItemsToSegment(SS3GlossWindowSegment segmentTier, PresentationField parentField) {
		// check if we are adding the same segment tier (parentField) as an existing one
		// because, if so, just make a duplicate reference to it, so any changes made in
		// one reflect in the other
		var segmentsList = segmentPanel.getSegments();
		if (segmentPanel.getUtteranceView() != null) {	// while we are in utterance view and things are changing, defer to utterance view
			segmentsList = segmentPanel.getUtteranceView().getSegments();
		}
		
		if ( ! segmentsList.isEmpty()) {
			for (Segment seg : segmentPanel.getSegments()) {
				if (seg.getParentField().equals(parentField)) {
					return seg;
				}
			}
		}

		RootLabelObject root = getCodingSchemaAsTreeNode(segmentTier, parentField);
		Segment segment = new Segment(segmentPanel, parentField, segmentTier.getID());
		
		segment.setViewDataManager(parentField.getSS3GlossWindowSegment().getViewDataManager());
		segment.setViewMode(parentField.getSS3GlossWindowSegment().getViewMode());
		
		// GD 2024-07-19: unsure if there is an effective difference between the above and below, but leaving for debugging purposes
//		segment.setViewMode(segmentTier.getViewMode());
//		segment.setViewDataManager(segmentTier.getViewDataManager());
		
		
		segment.setSS3GlossWindowItems(segmentTier.getSS3GlossWindowItems()); // GREG TODO

		segment.setRootLabelObject(root);
		for (LabelObject obj : root.getAllLabelObjects())
		    segment.addFieldWrapper(obj.getWrapper());

		return segment;
	}

	private RootLabelObject getCodingSchemaAsTreeNode(SS3GlossWindowSegment segmentTier,
			PresentationField parentField) {

		RootLabelObject root = new RootLabelObject();
		LabelObject nonManObject = new LabelObject("", "");
		LabelObject glossNRelatedObject = new LabelObject("", "");
		LabelObject graphObject = new LabelObject("", "");
		LabelObject gestureObject = new LabelObject("", "");
		LabelObject customObject = new LabelObject("", "");
		root.setNonManuelLabelObject(nonManObject);
		root.setGlossLabelObject(glossNRelatedObject);
		root.setGraphLabelObject(graphObject);
		root.setGestureLabelObject(gestureObject);
		root.setCustomLabelObject(customObject);
		SS3CodingScheme dbSchema = segmentPanel.getSS3Database().getSS3CodingScheme();

		var statementFields = segmentTier.getStatementFields().stream()
            .collect(Collectors.toMap(f -> f.getId(), f -> f));

		List<SS3Field> graphFields = dbSchema.getFields(Constants.TYPE_GRAPH);

		for (var graph : database.getGraphs()) {
		    // Don't add if the file does not exist (was not located)
            if (!new File(graph.uri()).exists())
                continue;

			// Don't add if there are no selected participants associated with this graph
			if (database.getSelectedParticipants().stream().noneMatch(p -> graph.id().equals(p.getGraphFileID())))
				continue;

			var fields = parseGraphFileToRawFormat(graph);

			var max = fields.values().stream().flatMap(Collection::stream).map(r -> r.time).max(Time::compareTo)
					.orElse(graph.range().start());

			graph.range().duration(max.subtract(graph.range().start()));

			for (var entry : fields.entrySet()) {
				var id = entry.getKey();
				var rows = entry.getValue();

				for (var gf : graphFields) {
					if (gf.getId().trim().equalsIgnoreCase(id.trim())) {
						var label = addField(gf, graphObject, false, segmentTier, parentField, graph.range());
						label.setVisible(true);
						var wrapper = (GraphFieldWrapper) label.getWrapper();
						wrapper.setRawGraphList(rows);
						wrapper.setGraphFileID(graph.id());
						break;
					}
				}
			}
		}

		// add non-manual fields
		ArrayList<SS3Field> nmFields = dbSchema.getFields(Constants.TYPE_NON_MANUAL, Constants.TYPE_NON_MANUAL_FREE_TEXT);

		for (SS3Field nmField : nmFields) {
		    var range = SS3Singleton.getVideoTimeRange();

            var stmt = statementFields.get(nmField.getId());
		    if (stmt != null && stmt.getTimeRange() != null)
		        range = stmt.getTimeRange();

			addField(nmField, nonManObject, false, segmentTier, parentField, range);
		}

		// custom fields
		ArrayList<SS3Field> customFields = dbSchema.getFields(Constants.TYPE_CUSTOM_INTERVAL, Constants.TYPE_CUSTOM_POINT,
				Constants.TYPE_CUSTOM_SECTION);
		for (SS3Field customField : customFields) {
            var range = SS3Singleton.getAudioTimeRange() != null
                ? SS3Singleton.getAudioTimeRange()
                : SS3Singleton.getVideoTimeRange();

            var stmt = statementFields.get(customField.getId());
            if (stmt != null && stmt.getTimeRange() != null)
                range = stmt.getTimeRange();

			addField(customField, customObject, false, segmentTier, parentField, range);
		}

		// add manual fields in the right order
		// dom gloss, ndom gloss, dom hand/dom hshape, ndom hand/ndom hshape,
		// translation
		GlossFieldWrapper domGlsWrapper, ndomGlsWrapper;
		DomNdomHandFieldWrapper domHWrapper, ndomHWrapper;
		HandShapeFieldWrapper domHsWrapper, ndomHsWrapper;
		LocationFieldWrapper domLocationWrp, ndomLocationWrp;
		SignTypeFieldWrapper domSignTypeWrp, ndomSignTypeWrp;
		//
		// GestureFieldWrapper gestWrp;

		SS3Field domGlsField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GLOSS));
		SS3Field ndomGlsField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GLOSS));
		LabelObject domGlsObj, ndomGlsObj;

		domGlsObj = addField(domGlsField, glossNRelatedObject, false, segmentTier, parentField);
		ndomGlsObj = addField(ndomGlsField, glossNRelatedObject, false, segmentTier, parentField);

		domGlsWrapper = (GlossFieldWrapper) domGlsObj.getWrapper();
		ndomGlsWrapper = (GlossFieldWrapper) ndomGlsObj.getWrapper();

		// set dependency, gloss entities
		if (domGlsWrapper != null && ndomGlsWrapper != null) {
			GlossField glsField = (GlossField) domGlsWrapper.getField();

			GlossChainedEventsEntityCollection collection = glsField.getGlossChainedEventsEntityCollection();
			ArrayList<GlossChainedEventsEntity> glsEntities = collection.getEntities();

			for (GlossChainedEventsEntity cEntity : glsEntities) {
				String entID = cEntity.getDependentGlossEntityID();

				GlossField glsFld = ndomGlsWrapper.getGlossField();

				GlossChainedEventsEntityCollection glsCollection = glsFld.getGlossChainedEventsEntityCollection();
				GlossChainedEventsEntity ndEntity = glsCollection.getEntity(entID);

				if (ndEntity != null) {
					cEntity.setDependentGlossEntity(ndEntity);
					ndEntity.setDependentGlossEntity(cEntity);
				}
			}
		}

		// add right hand
		SS3Field domHField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_HAND));
		SS3Field domHShapeField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_HAND_SHAPE));
		SS3Field domLocField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_LOCATION));
		SS3Field domSignTypeField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_SIGN_TYPE));
		SS3Field ndomHField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_HAND));
		SS3Field ndomHShapeField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_HAND_SHAPE));
		SS3Field ndomLocField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_LOCATION));
		SS3Field ndomSignTypeField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_SIGN_TYPE));
		//
		// SS3Field domGestField =
		// schema.getField(schema.getFieldID(Constants.TYPE_DOM_GESTURE));
		// SS3Field gTypeField = new SS3Field();
		// gTypeField.setName(schema.getGesturePropertys().get(0).getAbbr());

		/*
		 * //gesture properties ArrayList<SS3Field> gesturePropertyFields = new
		 * ArrayList<SS3Field>(); for (GestureProperty gt : schema.getGesturePropertys()) {
		 * SS3Field gtField = new SS3Field(); gtField.setName(gt.getAbbr());
		 * gesturePropertyFields.add(gtField); }
		 */

		LabelObject domHObj, domHShapeObj, domLocLabelObj, domSignTypeLabelObj, ndomHObj, ndomHShapeObj,
				ndomLocLabelObj, ndomSignTypeLabelObj;
		//
		// LabelObject gestObj, gTypeObj;

		domHObj = addField(domHField, glossNRelatedObject, false, segmentTier, parentField);
		domHShapeObj = addField(domHShapeField, domHObj, true, segmentTier, parentField, TimeRange.ABSOLUTE);
		ndomHObj = addField(ndomHField, glossNRelatedObject, false, segmentTier, parentField);
		ndomHShapeObj = addField(ndomHShapeField, ndomHObj, true, segmentTier, parentField);
		//
		// gestObj = addField(domGestField, gestureObject, false, items, parentField);
		// gTypeObj = addField(gTypeField, gestObj, true, items, parentField);

		domLocLabelObj = addField(domLocField, glossNRelatedObject, false, segmentTier, parentField);
		ndomLocLabelObj = addField(ndomLocField, glossNRelatedObject, false, segmentTier, parentField);
		domSignTypeLabelObj = addField(domSignTypeField, glossNRelatedObject, false, segmentTier, parentField);
		ndomSignTypeLabelObj = addField(ndomSignTypeField, glossNRelatedObject, false, segmentTier, parentField);

		domHObj.setLabel("dom hand");
		domHShapeObj.setLabel("dom hshape");
		ndomHObj.setLabel("ndom hand");
		ndomHShapeObj.setLabel("ndom hshape");
		domLocLabelObj.setLabel("dom loc");
		ndomLocLabelObj.setLabel("ndom loc");
		domSignTypeLabelObj.setLabel("dom sign type");
		ndomSignTypeLabelObj.setLabel("ndom sign type");
		//
		// gTypeObj.setLabel("test gesture type");

		domHWrapper = (DomNdomHandFieldWrapper) domHObj.getWrapper();
		domHsWrapper = (HandShapeFieldWrapper) domHShapeObj.getWrapper();
		ndomHWrapper = (DomNdomHandFieldWrapper) ndomHObj.getWrapper();
		ndomHsWrapper = (HandShapeFieldWrapper) ndomHShapeObj.getWrapper();

		domLocationWrp = (LocationFieldWrapper) domLocLabelObj.getWrapper();
		ndomLocationWrp = (LocationFieldWrapper) ndomLocLabelObj.getWrapper();
		domSignTypeWrp = (SignTypeFieldWrapper) domSignTypeLabelObj.getWrapper();
		ndomSignTypeWrp = (SignTypeFieldWrapper) ndomSignTypeLabelObj.getWrapper();
		//
		// gestWrp = (GestureFieldWrapper) gTypeObj.getWrapper();

		setGlossAndRelated(domGlsWrapper, ndomGlsWrapper, domHWrapper, domHsWrapper, domLocationWrp, domSignTypeWrp);
		setGlossAndRelated(ndomGlsWrapper, domGlsWrapper, ndomHWrapper, ndomHsWrapper, ndomLocationWrp,
				ndomSignTypeWrp);

		SS3Field etField = dbSchema.getField(Util.FREE_TEXT_FIELD_ID);
		addField(etField, glossNRelatedObject, false, segmentTier, parentField);

		setDependency(domGlsWrapper, ndomGlsWrapper, domHWrapper, ndomHWrapper, domHsWrapper, ndomHsWrapper);

//		addSignTypeDependencies(domGlsWrapper, ndomGlsWrapper, domSignTypeWrp, ndomSignTypeWrp);

		GestureFieldWrapper domGesWrapper, ndomGesWrapper;
		ApexFieldWrapper domApexWrapper, ndomApexWrapper;
		GestureDomNdomHandFieldWrapper domGestureHWrapper, ndomGestureHWrapper;
		GestureHandShapeFieldWrapper domGestureHsWrapper, ndomGestureHsWrapper;
		GestureOrientationFieldWrapper domGestureOrientFingerTipWrapper, ndomGestureOrientFingerTipWrapper, domGesturePalmOrientWrapper, ndomGesturePalmOrientWrapper;
		GestureLocationFieldWrapper domGestureLocationWrapper, ndomGestureLocationWrapper;
		GesturePropertyFieldWrapper domGesturePropertyWrp, ndomGesturePropertyWrp;

		SS3Field domGesField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE));
		SS3Field ndomGesField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE));
		LabelObject domGesObj, ndomGesObj;
		domGesObj = addField(domGesField, gestureObject, false, segmentTier, parentField);
		ndomGesObj = addField(ndomGesField, gestureObject, false, segmentTier, parentField);

		domGesWrapper = (GestureFieldWrapper) domGesObj.getWrapper();
		ndomGesWrapper = (GestureFieldWrapper) ndomGesObj.getWrapper();

		SS3Field domGestureApexField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_APEX));
		SS3Field domGestureHField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_HAND));
		SS3Field domGestureHShapeField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_HANDSHAPE));
		SS3Field domGestureFingerTipOrientField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_FINGER_TIP_ORIENTATION));
		SS3Field domGesturePropertyField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_PROPERTY));
		SS3Field ndomGestureApexField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_APEX));
		SS3Field domGestureLocField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_LOCATION));
		SS3Field ndomGestureHField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_HAND));
		SS3Field ndomGestureHShapeField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_HANDSHAPE));
		SS3Field ndomGestureFingerTipOrientField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_FINGER_TIP_ORIENTATION));
		SS3Field ndomGesturePropertyField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_PROPERTY));
		SS3Field ndomGestureLocField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_LOCATION));
		SS3Field domGesturePalmOrientationField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_DOM_GESTURE_PALM_ORIENTATION));
		SS3Field ndomGesturePalmOrientationField = dbSchema.getField(dbSchema.getFieldID(Constants.TYPE_NDOM_GESTURE_PALM_ORIENTATION));

		LabelObject domGestureApexObj, domGestureHObj, domGestureHShapeObj, domGestureFingerTipOrientObj, domGestureLocLabelObj,
				domGesturePropertyLabelObj, ndomGestureApexObj, ndomGestureHObj, ndomGestureHShapeObj, ndomGestureFingerTipOrientObj,
				ndomGestureLocLabelObj, ndomGesturePropertyLabelObj, domGesturePalmOrientationObj, ndomGesturePalmOrientationObj;

		domGestureApexObj = addField(domGestureApexField, gestureObject, false, segmentTier, parentField);
		ndomGestureApexObj = addField(ndomGestureApexField, gestureObject, false, segmentTier, parentField);

		domGestureHObj = addField(domGestureHField, gestureObject, false, segmentTier, parentField);
		domGestureHShapeObj = addField(domGestureHShapeField, domGestureHObj, true, segmentTier, parentField,
				TimeRange.ABSOLUTE);
		domGestureFingerTipOrientObj = addField(domGestureFingerTipOrientField, domGestureHObj, true, segmentTier, parentField,
				TimeRange.ABSOLUTE);
		domGesturePalmOrientationObj = addField(domGesturePalmOrientationField, domGestureHObj, true, segmentTier, parentField, TimeRange.ABSOLUTE);

		ndomGestureHObj = addField(ndomGestureHField, gestureObject, false, segmentTier, parentField);
		ndomGestureHShapeObj = addField(ndomGestureHShapeField, ndomGestureHObj, true, segmentTier, parentField);
		ndomGestureFingerTipOrientObj = addField(ndomGestureFingerTipOrientField, ndomGestureHObj, true, segmentTier, parentField);
		ndomGesturePalmOrientationObj = addField(ndomGesturePalmOrientationField, ndomGestureHObj, true, segmentTier, parentField);

	    domGestureLocLabelObj = addField(domGestureLocField, gestureObject, false, segmentTier, parentField);
	    ndomGestureLocLabelObj = addField(ndomGestureLocField, gestureObject, false, segmentTier, parentField);
		
		domGesturePropertyLabelObj = addField(domGesturePropertyField, gestureObject, false, segmentTier, parentField);
		ndomGesturePropertyLabelObj = addField(ndomGesturePropertyField, gestureObject, false, segmentTier, parentField);

		ArrayList<SS3Field> domGesturePropertyCategoryFields = dbSchema.getFields(Constants.TYPE_DOM_GESTURE_CATEGORY);
		domGesturePropertyCategoryFields.sort(new Comparator<SS3Field>() {
			@Override
			public int compare(SS3Field f1, SS3Field f2) {
				Integer aid = f1.getOrder();
				Integer bid = f2.getOrder();
				return aid.compareTo(bid);
			}
		});

		ArrayList<LabelObject> domGesturePropertyCategoryLabelObjects = new ArrayList<LabelObject>();
		for (SS3Field gesturePropertyCategoryField : domGesturePropertyCategoryFields) {
			LabelObject obj = addField(gesturePropertyCategoryField, domGesturePropertyLabelObj, true, segmentTier,
					parentField);
			domGesturePropertyCategoryLabelObjects.add(obj);
		}

		ArrayList<SS3Field> ndomGesturePropertyCategoryFields = dbSchema.getFields(Constants.TYPE_NDOM_GESTURE_CATEGORY);
		ndomGesturePropertyCategoryFields.sort(new Comparator<SS3Field>() {
			@Override
			public int compare(SS3Field f1, SS3Field f2) {
				Integer aid = f1.getOrder();
				Integer bid = f2.getOrder();
				return aid.compareTo(bid);
			}
		});

		ArrayList<LabelObject> ndomGesturePropertyCategoryLabelObjects = new ArrayList<LabelObject>();
		for (SS3Field gesturePropertyCategoryField : ndomGesturePropertyCategoryFields) {
			LabelObject obj = addField(gesturePropertyCategoryField, ndomGesturePropertyLabelObj, true, segmentTier,
					parentField);
			ndomGesturePropertyCategoryLabelObjects.add(obj);
		}

		domApexWrapper = (ApexFieldWrapper) domGestureApexObj.getWrapper();
		ndomApexWrapper = (ApexFieldWrapper) ndomGestureApexObj.getWrapper();

		domGestureHWrapper = (GestureDomNdomHandFieldWrapper) domGestureHObj.getWrapper();
		domGestureHsWrapper = (GestureHandShapeFieldWrapper) domGestureHShapeObj.getWrapper();
		domGestureOrientFingerTipWrapper = (GestureOrientationFieldWrapper) domGestureFingerTipOrientObj.getWrapper();
		domGesturePalmOrientWrapper = (GestureOrientationFieldWrapper) domGesturePalmOrientationObj.getWrapper();

		ndomGestureHWrapper = (GestureDomNdomHandFieldWrapper) ndomGestureHObj.getWrapper();
		ndomGestureHsWrapper = (GestureHandShapeFieldWrapper) ndomGestureHShapeObj.getWrapper();
		ndomGestureOrientFingerTipWrapper = (GestureOrientationFieldWrapper) ndomGestureFingerTipOrientObj.getWrapper();
		ndomGesturePalmOrientWrapper = (GestureOrientationFieldWrapper) ndomGesturePalmOrientationObj.getWrapper();

		domGestureLocationWrapper = (GestureLocationFieldWrapper) domGestureLocLabelObj.getWrapper();
		ndomGestureLocationWrapper = (GestureLocationFieldWrapper) ndomGestureLocLabelObj.getWrapper();
		
		domGesturePropertyWrp = (GesturePropertyFieldWrapper) domGesturePropertyLabelObj.getWrapper();
		ndomGesturePropertyWrp = (GesturePropertyFieldWrapper) ndomGesturePropertyLabelObj.getWrapper();

		setGestureAndRelated(domGesWrapper, ndomGesWrapper, domApexWrapper, domGestureHWrapper, domGestureHsWrapper,
				domGestureOrientFingerTipWrapper, domGesturePalmOrientWrapper, domGestureLocationWrapper, domGesturePropertyWrp, domGesturePropertyCategoryLabelObjects);
		setGestureAndRelated(ndomGesWrapper, domGesWrapper, ndomApexWrapper, ndomGestureHWrapper, ndomGestureHsWrapper,
				ndomGestureOrientFingerTipWrapper, ndomGesturePalmOrientWrapper, ndomGestureLocationWrapper, ndomGesturePropertyWrp, ndomGesturePropertyCategoryLabelObjects);

		setDependency(domGesWrapper, ndomGesWrapper, domGestureHWrapper, ndomGestureHWrapper, domGestureHsWrapper,
				ndomGestureHsWrapper, domGestureOrientFingerTipWrapper, ndomGestureOrientFingerTipWrapper, domGesturePalmOrientWrapper, ndomGesturePalmOrientWrapper);

		return root;
	}

	private void setGlossAndRelated(GlossFieldWrapper glsWrp1, GlossFieldWrapper glsWrp2,
			DomNdomHandFieldWrapper handWrp, HandShapeFieldWrapper handShapeWrp, LocationFieldWrapper locationWrp,
			SignTypeFieldWrapper signTypeWrp) {

		GlossChainedEventsEntityCollection glsCollection = glsWrp1.getGlossField()
				.getGlossChainedEventsEntityCollection();

		if (glsCollection != null) {
			ArrayList<GlossChainedEventsEntity> handGlosses = glsCollection.getEntities();

			for (GlossChainedEventsEntity gloss : handGlosses) {

				setDependentGlossEntity(gloss, glsWrp2);

				setDependencies(gloss, glsWrp1.getGlossField().getFieldID(), handWrp, handShapeWrp, locationWrp,
						signTypeWrp);
			}
		}
	}

	private void setGestureAndRelated(GestureFieldWrapper gestWrp1, GestureFieldWrapper gestWrp2,
			ApexFieldWrapper apexFieldWrp, GestureDomNdomHandFieldWrapper handWrp,
			GestureHandShapeFieldWrapper handShapeWrp, GestureOrientationFieldWrapper orientationFingerTipWrp, GestureOrientationFieldWrapper palmOrientationWrp,
			GestureLocationFieldWrapper locationWrp, GesturePropertyFieldWrapper signTypeWrp,
			ArrayList<LabelObject> gesturePropertyCategoryLabelObjects) {

		GestureEventsEntityCollection gestCollection = gestWrp1.getGestureField().getGestureEventsEntityCollection();

		if (gestCollection != null) {
			ArrayList<GestureEventsEntity> gestureEvents = gestCollection.getEntities();

			for (GestureEventsEntity gestureEvent : gestureEvents) {

				setDependentGestureEntity(gestureEvent, gestWrp2);

				setDependencies(gestureEvent, gestWrp1.getGestureField().getFieldID(), apexFieldWrp, handWrp, handShapeWrp,
						orientationFingerTipWrp, palmOrientationWrp, locationWrp, signTypeWrp, gesturePropertyCategoryLabelObjects);
			}
		}
	}

	private void setDependencies(GlossChainedEventsEntity gloss, String glossFieldID, DomNdomHandFieldWrapper handWrp,
			HandShapeFieldWrapper handShapeWrp, LocationFieldWrapper locationWrp, SignTypeFieldWrapper signTypeWrp) {
		String depHandID = gloss.getDependentHandEntityID();
		String depHandShapeID = gloss.getDependentHandShapeEntityID();
		String depLocationID = gloss.getDependentLocationEntityID();
		String depSignTypeID = gloss.getDependentSignTypeEntityID();

		// block concerning dependencies for hand ID (depHandID) and hand wrapper
		// (handWrp)
		if (depHandID != null && !depHandID.isEmpty()) {
			EventsEntity hndEnt = handWrp.getField().getEntity(depHandID);

			// if hand entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (hndEnt != null) {
				ChainedEventsEntity hndEntity = (ChainedEventsEntity) hndEnt;
				hndEnt.setRefEntityId(gloss.getID());
				gloss.setDependentHandEntity(hndEntity);
			} else {
				// else: gloss has hand entity ID saved, but there is not a corresponding hand
				// entity object in database/track (and thus the wrapper)
				String fieldID = handWrp.getField().getFieldID();
				SS3Track hndTrack = database.getTrack(fieldID);

				ChainedEventsEntity handEntity = null;

				// try to re-create hand entity strictly from db track
				SS3Entity handSS3Entity = hndTrack.getSS3Entity(depHandID);
				if (handSS3Entity != null) {
					handEntity = createTrackEntity(handSS3Entity, handWrp.getField());
				}

				// if db recreation failed, make a new hand entity object and give it ID of
				// depHandID
				if (handEntity == null) {
					ChainedEventsCollection collection = new ChainedEventsCollection();
					GlossChainedEvent glossTextValueEvent = (GlossChainedEvent) gloss.getTextValue();
					collection.setTextValueEvent(glossTextValueEvent.cloneGlossChainedEvent());
					handEntity = new ChainedEventsEntity(collection, segmentPanel, handWrp.getField());

					handEntity.setID(depHandID);
					handEntity.setField(handWrp.getField());
					handEntity.updateParentEntity();

					boolean isLockedGlossPhonAlignments = gloss.isLockedGlossPhonAlignments();
					handEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);

					handEntity.setStartTimeInfo(gloss.getStartTimeInfo());
					handEntity.setEndTimeInfo(gloss.getEndTimeInfo());
				}

				// add hand entity into the already created hand wrapper
				handWrp.addEventsEntity(handEntity);
				// handWrp.getField().addEventsEntity(handEntity); // commenting this - it does
				// the same thing as the above line

				// set reference id's in both directions (gloss entity <--> hand entity)
				handEntity.setRefEntityId(gloss.getID());
				gloss.setDependentHandEntity(handEntity);
			}
		}

		// block concerning dependencies for hs ID (depHandShapeID) and hs wrapper
		// (handShapeWrp)
		if (depHandShapeID != null && !depHandShapeID.isEmpty()) {
			EventsEntity hndShapeEnt = handShapeWrp.getHandShapeField().getEntity(depHandShapeID);

			// if hs entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (hndShapeEnt != null) {
				HandShapeEntity hndShapeEntity = (HandShapeEntity) hndShapeEnt;
				hndShapeEntity.setRefEntityId(
						gloss.getDependentHandEntity() != null ? gloss.getDependentHandEntityID() : null);
				gloss.setDependentHandShapeEntity(hndShapeEntity);
			} else {
				// else: gloss has hs entity ID saved, but there is not a corresponding hs
				// entity object in database/track (and thus the wrapper)
				String fieldID = handShapeWrp.getField().getFieldID();
				SS3Track handShapeTrack = database.getTrack(fieldID);

				HandShapeEntity handShapeEntity = null;

				SS3Entity handShapeSS3Entity = handShapeTrack.getSS3Entity(depHandShapeID);

				// try to re-create hs entity strictly from db track
				if (handShapeSS3Entity != null) {
					handShapeEntity = new HandShapeEntity(fieldID, handShapeSS3Entity, segmentPanel,
							handShapeWrp.getHandShapeField());
					handShapeEntity.setField(handShapeWrp.getField());

					handShapeEntity.setID(handShapeSS3Entity.getID());
					handShapeEntity.setRefEntityId(handShapeSS3Entity.getRefEntityId());
					handShapeEntity.updateParentEntity();
				}

				if (handShapeEntity != null) {
					handShapeWrp.addEventsEntity(handShapeEntity);

					handShapeEntity.setRefEntityId(
							gloss.getDependentHandEntity() != null ? gloss.getDependentHandEntityID() : null);
					gloss.setDependentHandShapeEntity(handShapeEntity);
				}
			}
		}

		if (depLocationID != null && !depLocationID.isEmpty()) {
			LocationEntity locEntity = (LocationEntity) locationWrp.getField().getEntity(depLocationID);
			if (locEntity != null) {
				HashMap<LocationType, Region> locations = new HashMap<LocationType, Region>();
				for (LocationType locationType : SS3Singleton.getLocationConfig().getLocationTypes()) {
					for (LocationEvent event : locEntity.getEvents()) {
						if (event.getLocationType().equals(locationType)) {
							locations.put(locationType, event.getLocation());
							break;
						}
					}
				}
				LocationInfo parsedInfo = new LocationInfo(locations);
				if (parsedInfo.isEmpty()) {
					locationWrp.getField().deleteEntity(locEntity);
				} else {
					gloss.getGlossEntityProperties().setLocationInfo(parsedInfo);

					gloss.setDependentLocationEntity(locEntity);

					GlossChainedEventsEntity depGlsEntity = gloss.getDependentGlossEntity();

					if (glossFieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
						locEntity.setRefGlossEntity(gloss);
					} else {
						if (depGlsEntity != null) {
							locEntity.setRefGlossEntity(depGlsEntity);
						} else {
							locEntity.setRefGlossEntity(gloss);
						}
					}

					locEntity.setStartTimeInfo(gloss.getStartTimeInfo());
					locEntity.setEndTimeInfo(gloss.getEndTimeInfo());
				}
			} else {
				gloss.setDependentLocationEntityID(null);
			}
		}

		if (depSignTypeID != null && !depSignTypeID.isEmpty()) {
			SignTypeEntity signTypeEntity = (SignTypeEntity) signTypeWrp.getField().getEntity(depSignTypeID);
			if (signTypeEntity != null) {
				gloss.setDependentSignTypeEntity(signTypeEntity);

				GlossChainedEventsEntity depGlsEntity = gloss.getDependentGlossEntity();

				if (glossFieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
					signTypeEntity.setRefGlossEntity(gloss);
				} else {
					if (depGlsEntity != null) {
						signTypeEntity.setRefGlossEntity(depGlsEntity);
					} else {
						signTypeEntity.setRefGlossEntity(gloss);
					}
				}
				signTypeEntity.getEvent().setSignType(SignType.getSignTypeFromGloss(gloss));
				signTypeEntity.getEvent().setNameSign(gloss.getGlossEntityProperties().isNameSign());
			} else {
				gloss.setDependentSignTypeEntity(null);
			}
		}
	}

	private void setDependencies(GestureEventsEntity gest, String glossFieldID, ApexFieldWrapper apexFieldWrp,
								 GestureDomNdomHandFieldWrapper handWrp, GestureHandShapeFieldWrapper handShapeWrp,
								 GestureOrientationFieldWrapper orientationFingerTipWrp, GestureOrientationFieldWrapper palmOrientationWrp, GestureLocationFieldWrapper locationWrp,
								 GesturePropertyFieldWrapper signTypeWrp, ArrayList<LabelObject> gesturePropertyCategoryLabelObjects) {
		String depApexID = gest.getDependentApexEntityID();
		String depHandID = gest.getDependentHandEntityID();
		String depHandShapeID = gest.getDependentHandShapeEntityID();
		String depOrientationFingerTipID = gest.getDependentOrientationFingerTipEntityID();
		String depLocationID = gest.getDependentLocationEntityID();
		String depSignTypeID = gest.getDependentPropertyEntityID();
		// String depPalmOrientationID = gest.getDependentPalmOrientationEntity().getID();


		if (depApexID != null && !depApexID.isEmpty()) {
			EventsEntity apexEnt = apexFieldWrp.getField().getEntity(depApexID);

			// if hand entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (apexEnt != null) {
				ApexChainedApexEventsEntity apexEntity = (ApexChainedApexEventsEntity) apexEnt;
				apexEnt.setRefEntityId(gest.getID());
				gest.setDependentApexEntity(apexEntity);
			} else {
				String fieldID = apexFieldWrp.getField().getFieldID();
				SS3Track apexTrack = database.getTrack(fieldID);

				ApexChainedApexEventsEntity apexEntity = null;

				// try to re-create hand entity strictly from db track
				SS3Entity apexSS3Entity = apexTrack.getSS3Entity(depApexID);
				if (apexSS3Entity != null) {
					apexEntity = createTrackApexEntity(apexSS3Entity, apexFieldWrp.getField());
				}

				// if db recreation failed, make a new hand entity object and give it ID of
				// depHandID
				if (apexEntity == null) {
					ChainedApexEventsCollection collection = new ChainedApexEventsCollection();
					ApexChainedApexEvent textValueEvent = new ApexChainedApexEvent(segmentPanel);
					textValueEvent.setText("apex");
					collection.setTextValueEvent(textValueEvent);
					apexEntity = new ApexChainedApexEventsEntity(collection, segmentPanel, handWrp.getField());

					apexEntity.setID(depApexID);
					apexEntity.setField(apexFieldWrp.getField());
					apexEntity.updateParentEntity();

					boolean isLockedGesturePhonAlignments = gest.isLockedGesturePhonAlignments();
					apexEntity.setLockedApexPhonAlignments(isLockedGesturePhonAlignments);

					var start = gest.getValueOrTextValueEvent().getStartTimeInfo();
					var end = gest.getValueOrTextValueEvent().getEndTimeInfo();

					var quarter = end.distance(start).divide(4);

					apexEntity.setStartTimeInfo(start.add(quarter));
					apexEntity.setEndTimeInfo(end.subtract(quarter));
				}

				// add hand entity into the already created hand wrapper
				apexFieldWrp.addEventsEntity(apexEntity);
				// handWrp.getField().addEventsEntity(handEntity); // commenting this - it does
				// the same thing as the above line

				// set reference id's in both directions (gest entity <--> hand entity)
				apexEntity.setRefEntityId(gest.getID());
				gest.setDependentApexEntity(apexEntity);
			}
		}

		// block concerning dependencies for hand ID (depHandID) and hand wrapper
		// (handWrp)
		if (depHandID != null && !depHandID.isEmpty()) {
			EventsEntity hndEnt = handWrp.getField().getEntity(depHandID);

			// if hand entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (hndEnt != null) {
				GestureChainedEventsEntity hndEntity = (GestureChainedEventsEntity) hndEnt;
				hndEnt.setRefEntityId(gest.getID());
				gest.setDependentHandEntity(hndEntity);
			} else {
				// else: gest has hand entity ID saved, but there is not a corresponding hand
				// entity object in database/track (and thus the wrapper)
				String fieldID = handWrp.getField().getFieldID();
				SS3Track hndTrack = database.getTrack(fieldID);

				GestureChainedEventsEntity handEntity = null;

				// try to re-create hand entity strictly from db track
				SS3Entity handSS3Entity = hndTrack.getSS3Entity(depHandID);
				if (handSS3Entity != null) {
					handEntity = createGestureHandTrackEntity(handSS3Entity, handWrp.getField());
				}

				// if db recreation failed, make a new hand entity object and give it ID of
				// depHandID
				if (handEntity == null) {
					GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
					GestureEvent gestTextValueEvent = (GestureEvent) gest.getTextValue();
					collection.setTextValueEvent(gestTextValueEvent.cloneGestureEvent());
					handEntity = new GestureChainedEventsEntity(collection, segmentPanel, handWrp.getField());

					handEntity.setID(depHandID);
					handEntity.setField(handWrp.getField());
					handEntity.updateParentEntity();

					boolean isLockedGesturePhonAlignments = gest.isLockedGesturePhonAlignments();
					handEntity.setLockedGesturePhonAlignments(isLockedGesturePhonAlignments);

					handEntity.setStartTimeInfo(gest.getStartTimeInfo());
					handEntity.setEndTimeInfo(gest.getEndTimeInfo());
				}

				// add hand entity into the already created hand wrapper
				handWrp.addEventsEntity(handEntity);
				// handWrp.getField().addEventsEntity(handEntity); // commenting this - it does
				// the same thing as the above line

				// set reference id's in both directions (gest entity <--> hand entity)
				handEntity.setRefEntityId(gest.getID());
				gest.setDependentHandEntity(handEntity);
			}
		}

		// block concerning dependencies for hs ID (depHandShapeID) and hs wrapper
		// (handShapeWrp)
		if (depHandShapeID != null && !depHandShapeID.isEmpty()) {
			EventsEntity hndShapeEnt = handShapeWrp.getGestureHandShapeField().getEntity(depHandShapeID);

			// if hs entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (hndShapeEnt != null) {
				GestureHandShapeEntity hndShapeEntity = (GestureHandShapeEntity) hndShapeEnt;
				hndShapeEntity.setRefEntityId(
						gest.getDependentHandEntity() != null ? gest.getDependentHandEntityID() : null);
				gest.setDependentHandShapeEntity(hndShapeEntity);
				hndShapeEntity.setRefGestureEntity(gest);
			} else {
				// else: gest has hs entity ID saved, but there is not a corresponding hs
				// entity object in database/track (and thus the wrapper)
				String fieldID = handShapeWrp.getField().getFieldID();
				SS3Track handShapeTrack = database.getTrack(fieldID);

				GestureHandShapeEntity handShapeEntity = null;

				SS3Entity handShapeSS3Entity = handShapeTrack.getSS3Entity(depHandShapeID);

				// try to re-create hs entity strictly from db track
				if (handShapeSS3Entity != null) {
					handShapeEntity = new GestureHandShapeEntity(fieldID, handShapeSS3Entity, segmentPanel,
							handShapeWrp.getGestureHandShapeField());
					handShapeEntity.setField(handShapeWrp.getField());

					handShapeEntity.setID(handShapeSS3Entity.getID());
					handShapeEntity.setRefEntityId(handShapeSS3Entity.getRefEntityId());
					handShapeEntity.updateParentEntity();
				}

				if (handShapeEntity != null) {
					handShapeWrp.addEventsEntity(handShapeEntity);

					handShapeEntity.setRefEntityId(
							gest.getDependentHandEntity() != null ? gest.getDependentHandEntityID() : null);
					gest.setDependentHandShapeEntity(handShapeEntity);
					handShapeEntity.setRefGestureEntity(gest);
				}
			}
		}

		// block concerning dependencies for orientation ID (depOrientationFingerTipID) and orn
		// wrapper
		// (orientationFingerTipWrp)
		if (depOrientationFingerTipID != null && !depOrientationFingerTipID.isEmpty()) {
			EventsEntity orientationFingerTipEnt = orientationFingerTipWrp.getGestureOrientationField().getEntity(depOrientationFingerTipID);
			// EventsEntity palmOrientationEntity = palmOrientationWrp.getGestureOrientationField();

			// if orn entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (orientationFingerTipEnt != null) {
				GestureOrientationEntity orientationFingerTipEntity = (GestureOrientationEntity) orientationFingerTipEnt;
				orientationFingerTipEntity.setRefEntityId(
						gest.getDependentHandEntity() != null ? gest.getDependentOrientationFingerTipEntityID() : null);
				gest.setDependentOrientationFingerTipEntity(orientationFingerTipEntity);
				orientationFingerTipEntity.setRefGestureEntity(gest);

				GestureOrientationEntity palmOrientationEntity = (GestureOrientationEntity) orientationFingerTipEnt;

			} else {
				// else: gest has orn entity ID saved, but there is not a corresponding orn
				// entity object in database/track (and thus the wrapper)
				String fieldID = orientationFingerTipWrp.getField().getFieldID();
				SS3Track orientationTrack = database.getTrack(fieldID);

				GestureOrientationEntity orientationEntity = null;

				SS3Entity orientationSS3Entity = orientationTrack.getSS3Entity(depOrientationFingerTipID);

				// try to re-create orn entity strictly from db track
				if (orientationSS3Entity != null) {
					orientationEntity = new GestureOrientationEntity(fieldID, orientationSS3Entity, segmentPanel,
							orientationFingerTipWrp.getGestureOrientationField() /*Hard Coded For now... I don't think this code is used.*/);
					orientationEntity.setField(orientationFingerTipWrp.getField());

					orientationEntity.setID(orientationSS3Entity.getID());
					orientationEntity.setRefEntityId(orientationSS3Entity.getRefEntityId());
					orientationEntity.updateParentEntity();
				}

				if (orientationEntity != null) {
					orientationFingerTipWrp.addEventsEntity(orientationEntity);

					orientationEntity.setRefEntityId(
							gest.getDependentHandEntity() != null ? gest.getDependentHandEntityID() : null);
					gest.setDependentOrientationFingerTipEntity(orientationEntity);
					orientationEntity.setRefGestureEntity(gest);
				}
			}
		}
		
		if (depLocationID != null && !depLocationID.isEmpty()) {
			GestureLocationEntity locEntity = (GestureLocationEntity) locationWrp.getField().getEntity(depLocationID);
			if (locEntity != null) {
				HashMap <GestureLocationType, GestureLocationObject> locations= new HashMap<GestureLocationType, GestureLocationObject>();
				for (GestureLocationType locationType : SS3Singleton.getGestureLocationConfig().getLocationTypes()) {
					for (GestureLocationEvent event : locEntity.getEvents()) {
						if (event.getLocationType().equals(locationType)) {
							locations.put(locationType, new GestureLocationObject(event.getGestureLocation(), event.getIsContralateral()));
							break;
						}
					}
				}
				GestureLocationInfo parsedInfo = new GestureLocationInfo(locations);
				if (parsedInfo.isEmpty()) {
					locationWrp.getField().deleteEntity(locEntity);
				} else {
					gest.getGestureEntityProperties().setLocationInfo(parsedInfo);
					
					gest.setDependentLocationEntity(locEntity);

					locEntity.setRefGestureEntity(gest);
	
					locEntity.setStartTimeInfo(gest.getStartTimeInfo());
					locEntity.setEndTimeInfo(gest.getEndTimeInfo());
				}
			} else {
				gest.setDependentLocationEntityID(null);
			}
//			if (locEntity != null) {
//				locationWrp.addEventsEntity(locEntity);
//			}
		}

		if (depSignTypeID != null && !depSignTypeID.isEmpty()) {
			GesturePropertyEntity signTypeEntity = (GesturePropertyEntity) signTypeWrp.getField().getEntity(depSignTypeID);

			if (signTypeEntity != null) {
				gest.setDependentPropertyEntity(signTypeEntity);

				signTypeEntity.setRefGestureEntity(gest);

				// associate the dependent gesture property entities
				if (gesturePropertyCategoryLabelObjects != null && !gesturePropertyCategoryLabelObjects.isEmpty()) {
					String gesturePropertyID = signTypeEntity.getID();
					for (LabelObject labelObject : gesturePropertyCategoryLabelObjects) {
						ArrayList<EventsEntity> entities = labelObject.getWrapper().getField().getEntities();
						for (EventsEntity entity : entities) {
							String entityRefID = entity.getRefEntityId();
							if (gesturePropertyID.equalsIgnoreCase(entityRefID)) {
								signTypeEntity.addDependentGesturePropertyEntity((GesturePropertyEntity) entity);
							}
						}
					}
				}

			} else {
				gest.setDependentPropertyEntity(null);
			}
		}
	}

	// GD: 2021-06-01 as of now, this is only used in context of creating a hand entity object
	// if you are looking for field creation from database tracks, see class FieldsFactory
	private ChainedEventsEntity createTrackEntity(SS3Entity entity, Field field) {

		String fieldID = field.getFieldID();

		ChainedEventsCollection collection = new ChainedEventsCollection();
		ArrayList<SS3Item> items = entity.getItemsAsList();

        for (SS3Item item : items) {
            String id = item.getId();

            if (id.equalsIgnoreCase("" + Constants.NONMANUAL_ONSET_CODE)) {
                collection.setOnsetEvent(new ChainedEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.NONMANUAL_OFFSET_CODE)) {
                collection.setOffsetEvent(new ChainedEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.NONMANUAL_INITIAL_HOLD_CODE)) {
            	collection.setInitialHoldEvent(new ChainedEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.NONMANUAL_FINAL_HOLD_CODE)) {
            	collection.setFinalHoldEvent(new ChainedEvent(item, segmentPanel));
            }
            else {
                if (fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID) || fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
                    collection.setTextValueEvent(new GlossChainedEvent(item, segmentPanel));
                } else {
                    collection.setValueEvent(new ChainedEvent(item, segmentPanel));
                }
            }
        }

		ChainedEventsEntity cEntity = new ChainedEventsEntity(collection, segmentPanel, field);
		cEntity.setID(entity.getID());
		cEntity.setField(field);
		cEntity.setRefEntityId(entity.getRefEntityId());
		cEntity.updateParentEntity();

		boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
		cEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);

		return cEntity;
	}

	private GestureChainedEventsEntity createGestureHandTrackEntity(SS3Entity entity, Field field) {

		String fieldID = field.getFieldID();

		GestureChainedEventsCollection collection = new GestureChainedEventsCollection();
		ArrayList<SS3Item> items = entity.getItemsAsList();
		
		for (SS3Item item : items) {
			String id = item.getId();

			if (id.equalsIgnoreCase("" + Constants.GESTURE_PREPARATION_CODE)) {
				collection.setPreparationEvent(new GestureChainedEvent(item, segmentPanel));
			} else if (id.equalsIgnoreCase("" + Constants.GESTURE_RETRACTION_CODE)) {
				collection.setRetractionEvent(new GestureChainedEvent(item, segmentPanel));
			} else if (id.equalsIgnoreCase("" + Constants.GESTURE_PRESTROKE_HOLD_CODE)) {
				collection.setPrestrokeHoldEvent(new GestureChainedEvent(item, segmentPanel));
			} else if (id.equalsIgnoreCase("" + Constants.GESTURE_POSTSTROKE_HOLD_CODE)) {
				collection.setPoststrokeHoldEvent(new GestureChainedEvent(item, segmentPanel));
			} else if (id.equalsIgnoreCase("" + Constants.GESTURE_RELAXED_CODE)) {
				collection.setRelaxedEvent(new GestureChainedEvent(item, segmentPanel));
			} else {
				collection.setTextValueEvent(new GestureEvent(item, segmentPanel));
			}
		}

		GestureChainedEventsEntity cEntity = new GestureChainedEventsEntity(collection, segmentPanel, field);
		cEntity.setID(entity.getID());
		cEntity.setField(field);
		cEntity.setRefEntityId(entity.getRefEntityId());
		cEntity.updateParentEntity();

		boolean isLockedGesturePhonAlignments = entity.isLockedGesturePhoneAlignments();
		cEntity.setLockedGesturePhonAlignments(isLockedGesturePhonAlignments);

		return cEntity;
	}

	private ApexChainedApexEventsEntity createTrackApexEntity(SS3Entity entity, Field field) {

		String fieldID = field.getFieldID();

		ChainedApexEventsCollection collection = new ChainedApexEventsCollection();
		ArrayList<SS3Item> items = entity.getItemsAsList();

        for (SS3Item item : items) {
            String id = item.getId();

            if (id.equalsIgnoreCase("" + Constants.APEX_ONSET_CODE)) {
                collection.setOnsetEvent(new ChainedApexEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.APEX_OFFSET_CODE)) {
                collection.setOffsetEvent(new ChainedApexEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.APEX_INITIAL_HOLD_CODE)) {
            	collection.setInitialHoldEvent(new ChainedApexEvent(item, segmentPanel));
            } else if (id.equalsIgnoreCase("" + Constants.APEX_FINAL_HOLD_CODE)) {
            	collection.setFinalHoldEvent(new ChainedApexEvent(item, segmentPanel));
            }
            else {
				if (fieldID.equalsIgnoreCase(Util.DOM_GESTURE_APEX_FIELD_ID)
						|| fieldID.equalsIgnoreCase(Util.NDOM_GESTURE_APEX_FIELD_ID)) {
					collection.setTextValueEvent(new ApexChainedApexEvent(item, segmentPanel));
				} else {
					collection.setValueEvent(new ChainedApexEvent(item, segmentPanel));
				}
            }
        }

		ApexChainedApexEventsEntity cEntity = new ApexChainedApexEventsEntity(collection, segmentPanel, field);
		cEntity.setID(entity.getID());
		cEntity.setField(field);
		cEntity.setRefEntityId(entity.getRefEntityId());
		cEntity.updateParentEntity();

		boolean isLockedGesturePhonAlignments = entity.isLockedGesturePhoneAlignments();
		cEntity.setLockedApexPhonAlignments(isLockedGesturePhonAlignments);

		return cEntity;
	}

	private void setDependentGlossEntity(GlossChainedEventsEntity entity, GlossFieldWrapper glsWrp) {
		String id = entity.getDependentGlossEntityID();
		if (id != null && !id.isEmpty()) {
			EventsEntity ndEntity = glsWrp.getGlossField().getEntity(id);
			if (ndEntity != null) {
				GlossChainedEventsEntity dependentGlossEntity = (GlossChainedEventsEntity) ndEntity;
				entity.setDependentGlossEntity(dependentGlossEntity);
			}
		}
	}

	private void setDependentGestureEntity(GestureEventsEntity entity, GestureFieldWrapper gestWrp) {
		String id = entity.getDependentGestureEntityID();
		if (id != null && !id.isEmpty()) {
			EventsEntity ndEntity = gestWrp.getGestureField().getEntity(id);
			if (ndEntity != null) {
				GestureEventsEntity dependentGestureEntity = (GestureEventsEntity) ndEntity;
				entity.setDependentGestureEntity(dependentGestureEntity);
			}
		}
	}

	private LabelObject addField(SS3Field field, LabelObject parent, boolean addDependentWrapper,
			SS3GlossWindowSegment segmentTier, PresentationField parentField, TimeRange timeRange) {

		if (field != null) {

			LabelObject obj = new LabelObject(parent, field.getId(), field.getLabelOrName());

			SS3Track track = database.getTrack(field.getId(), segmentTier);

//			FieldWrapper wrapper = factory.createFieldWrapper(track, segmentPanel, parentField, timeRange);
			FieldWrapper wrapper = factory.createFieldWrapper(track, segmentPanel, parentField, SS3Singleton.getVideoTimeRange());

			boolean isVisible = !track.getEntities().isEmpty();
			obj.setVisible(isVisible);
			wrapper.setVisible(isVisible);

			for (var u : segmentTier.getSS3GlossWindowItems())
			    if (u.getPermanentVisibleFieldIDs().contains(field.getId()))
                    obj.setUtterancePermanentlyVisible(u);

            if (segmentTier.getPermanentFieldIDs().contains(field.getId()))
                obj.setPermanentlyVisibleAcrossSegmentTier(true);

            if (segmentPanel.getSS3Database().getSS3GlossWindow().getPermanentFieldIDs().contains(field.getId()))
                obj.setPermanentlyVisibleGlobally(true);

			wrapper.getField().setColor(field.getColor());

			if (addDependentWrapper) {
				parent.getWrapper().addDependentWrapper(wrapper);
			}

			obj.setWrapper(wrapper);
			parent.addChild(obj);
			return obj;
		}

		return null;
	}

	@Deprecated
	// JACK TODO: Callers should probably specify the desired default TimeRange
	// explicitly.
	private LabelObject addField(SS3Field field, LabelObject parent, boolean addDependentWrapper,
			SS3GlossWindowSegment segmentTier, PresentationField parentField) {
		return addField(field, parent, addDependentWrapper, segmentTier, parentField, SS3Singleton.getVideoTimeRange());
	}

	// parse the input graph file to multiple maps of a raw format - i.e.
	// non-normalized
	private Map<String, List<GraphFileRow>> parseGraphFileToRawFormat(Graph graph) {

		Map<String, List<GraphFileRow>> graphMap = new HashMap<String, List<GraphFileRow>>();
		String rowFieldID = null;
		String rowFieldName = null;
		double rowTimeValue = 0;
		double rowFieldValue = 0.0d;

		List<String[]> lines = new ArrayList<String[]>();
		BufferedReader in = null;

		try {
			in = new BufferedReader(new FileReader(Paths.get(graph.uri()).toAbsolutePath().toString()));
			String str = "";

			while ((str = in.readLine()) != null) {
				String[] splitLine = str.split(Util.CSV_REGEX_PATTERN, -1);
				lines.add(splitLine);
			}
		} catch (IOException e) {
			SS3SignStreamApplication.logger.log(Level.INFO, "File Read Error");
		}

		for (String[] line : lines) {

			// parts per line
			if (line.length != Util.GRAPH_FILE_LINE_ITEM_COUNT) {
				continue;
			}

			// time attribute
			String[] timeAttr = line[0].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (timeAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			String timeValue = timeAttr[1] != null ? timeAttr[1].trim() : "";

			// field id attribute
			String[] fieldIDAttr = line[1].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (fieldIDAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			// field name attribute
			String[] fieldNameAttr = line[2].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (fieldNameAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			// field value attribute
			String[] fieldValueAttr = line[3].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (fieldValueAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			try {
				rowTimeValue = Double.parseDouble(timeValue);
				rowFieldID = fieldIDAttr[1].trim();
				rowFieldName = fieldNameAttr[1].trim();
				rowFieldValue = Double.parseDouble(fieldValueAttr[1].trim());

				// Times are in milliseconds.
				var time = graph.range().time(Math.round(rowTimeValue * 1_000_000), TimeUnit.NANOSECOND);
				GraphFileRow row = new GraphFileRow(time, rowFieldValue, rowFieldID, rowFieldName);

				if (graphMap.get(rowFieldID) == null) {
					List<GraphFileRow> rows = new ArrayList<GraphFileRow>();
					rows.add(row);
					graphMap.put(rowFieldID, rows);
				}
				List<GraphFileRow> rows = graphMap.get(rowFieldID);
				rows.add(row);

			} catch (NumberFormatException e) {
				SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());
				continue;
			}

		}
		try {
			in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return graphMap;
	}

	private void setOneHandDependency(GlossFieldWrapper glsWrp, DomNdomHandFieldWrapper hndWrp,
			HandShapeFieldWrapper hsWrp) {

		ArrayList hsEntities = hsWrp.getHandShapeField().getEntities();

		ArrayList hnEntities = hndWrp.getField().getEntities();

		ArrayList<HandShapeEntity> validHsEntities = new ArrayList<HandShapeEntity>();
		ArrayList<ChainedEventsEntity> validHndEntities = new ArrayList<ChainedEventsEntity>();

		for (Object hsObject : hsEntities) {
			HandShapeEntity hsEntity = (HandShapeEntity) hsObject;

			ChainedEventsEntity hndEntity = getHandEntity(hsEntity, hndWrp);
			GlossChainedEventsEntity glsEntity = getGlossEntity(hndEntity, glsWrp);

			if (hndEntity != null && glsEntity != null) {
				validHsEntities.add(hsEntity);
				validHndEntities.add(hndEntity);

				setDependencies(glsEntity, hndEntity, hsEntity);
			} else {
				glsEntity = getGlossEntity(hsEntity, glsWrp.getGlossField().getGlossChainedEventsEntityCollection());

				if (glsEntity != null) {
					hndEntity = glsEntity.clone(hndWrp.getField().getFieldID());
					hndWrp.getField().addEventsEntity(hndEntity);

					setDependencies(glsEntity, hndEntity, hsEntity);

					validHsEntities.add(hsEntity);
					validHndEntities.add(hndEntity);
				}
			}
		}

		// clean up hs entities
		if (validHsEntities.size() < hsEntities.size()) {
			ArrayList<HandShapeEntity> delHsEntities = new ArrayList<HandShapeEntity>();

			for (Object hsEntity1 : hsEntities) {
				HandShapeEntity hsEntity = (HandShapeEntity) hsEntity1;
				if (!validHsEntities.contains(hsEntity)) {
					delHsEntities.add(hsEntity);
				}
			}

			for (HandShapeEntity del : delHsEntities) {
				hsEntities.remove(del);
			}
		}

		// clean up hand entities
		if (validHndEntities.size() < hnEntities.size()) {
			ArrayList<ChainedEventsEntity> delHndEntities = new ArrayList<ChainedEventsEntity>();

			for (Object hnEntity : hnEntities) {
				ChainedEventsEntity hEntity = (ChainedEventsEntity) hnEntity;
				if (!validHndEntities.contains(hEntity)) {
					delHndEntities.add(hEntity);
				}
			}

			for (ChainedEventsEntity del : delHndEntities) {
				hnEntities.remove(del);
			}
		}
	}

	private void setOneHandDependency(GestureFieldWrapper gestWrp, GestureDomNdomHandFieldWrapper hndWrp,
			GestureHandShapeFieldWrapper hsWrp, GestureOrientationFieldWrapper ornFingerTipWrp, GestureOrientationFieldWrapper palmWrp) {
// Below is the tree hierarchy of the entities
// 1. HandShapeEntity -> ChainedEventsEntity -> GestureChainedEventsEntity
// 2. HandShapeEntity -> ChainedEventsEntity -> GestureChainedEventsEntity -> GestureEntity
// 3. HandShapeEntity -> ChainedEventsEntity -> GestureChainedEventsEntity -> GestureEntity -> GlossChainedEventsEntity
// 4. HandShapeEntity -> ChainedEventsEntity -> GestureChainedEventsEntity -> GestureEntity -> GlossChainedEventsEntity -> GlossEntity

		ArrayList<GestureHandShapeEntity> hsEntities = hsWrp.getGestureHandShapeField().getEntities();
		ArrayList<GestureOrientationEntity> ornFingerTipEntities = ornFingerTipWrp.getGestureOrientationField().getEntities();
		ArrayList<GestureOrientationEntity> palmEntities = palmWrp.getGestureOrientationField().getEntities();

		ArrayList hnEntities = hndWrp.getField().getEntities();

		ArrayList<GestureHandShapeEntity> validHsEntities = new ArrayList<GestureHandShapeEntity>();
		ArrayList<GestureOrientationEntity> validOrnFingerTipEntities = new ArrayList<GestureOrientationEntity>();
		ArrayList<GestureChainedEventsEntity> validHndEntities = new ArrayList<GestureChainedEventsEntity>();
		ArrayList<GestureOrientationEntity> validOrnPalmEntities = new ArrayList<GestureOrientationEntity>();

		ArrayList<GestureEventsEntity> gestureEventsEntities = new ArrayList<>();
		ArrayList<GestureChainedEventsEntity> gestureChainedEventsEntities = new ArrayList<>();
		ArrayList<GestureOrientationEntity> gestureOrientationEntities = new ArrayList<>();

		for (Object hsObject : hsEntities) {
			GestureHandShapeEntity hsEntity = (GestureHandShapeEntity) hsObject;

			GestureChainedEventsEntity hndEntity = getHandEntity(hsEntity, hndWrp);
			GestureEventsEntity gestEntity = getGestureEntity(hndEntity, gestWrp);

			if (hndEntity != null && gestEntity != null) {
				validHsEntities.add(hsEntity);
				validHndEntities.add(hndEntity);

				setDependencies(gestEntity, hndEntity, hsEntity);
			} else {
				gestEntity = getGestureEntity(hsEntity, gestWrp.getGestureField().getGestureEventsEntityCollection());

				if (gestEntity != null) {
					hndEntity = gestEntity.clone(hndWrp.getField().getFieldID());
					hndWrp.getField().addEventsEntity(hndEntity);

					setDependencies(gestEntity, hndEntity, hsEntity);

					validHsEntities.add(hsEntity);
					validHndEntities.add(hndEntity);
				}
			}
		}

		for (GestureOrientationEntity ornFingerTipEntity : ornFingerTipEntities) {
			GestureChainedEventsEntity hndEntity = getHandEntity(ornFingerTipEntity, hndWrp);
			GestureEventsEntity gestEntity = getGestureEntity(hndEntity, gestWrp);

			if (hndEntity != null && gestEntity != null) {
				validOrnFingerTipEntities.add(ornFingerTipEntity);
				validHndEntities.add(hndEntity);

				// setDependencies(gestEntity, hndEntity, ornEntity);
				gestureEventsEntities.add(gestEntity);
				gestureChainedEventsEntities.add(hndEntity);
				gestureOrientationEntities.add(ornFingerTipEntity);
			} else {
				gestEntity = getGestureEntity(ornFingerTipEntity, gestWrp.getGestureField().getGestureEventsEntityCollection());

				if (gestEntity != null) {
					hndEntity = gestEntity.clone(hndWrp.getField().getFieldID());
					hndWrp.getField().addEventsEntity(hndEntity);

					// setDependencies(gestEntity, hndEntity, ornEntity);
					gestureEventsEntities.add(gestEntity);
					gestureChainedEventsEntities.add(hndEntity);
					gestureOrientationEntities.add(ornFingerTipEntity);

					validOrnFingerTipEntities.add(ornFingerTipEntity);
					validHndEntities.add(hndEntity);
				}
			}
		}

		setDependencies(gestureEventsEntities, gestureChainedEventsEntities, gestureOrientationEntities, false);

		gestureEventsEntities.clear();
		gestureChainedEventsEntities.clear();
		gestureOrientationEntities.clear();

		for (GestureOrientationEntity palmEntity : palmEntities) {
			GestureChainedEventsEntity hndEntity = getHandEntity(palmEntity, hndWrp);
			GestureEventsEntity gestEntity = getGestureEntity(hndEntity, gestWrp);

			if (hndEntity != null && gestEntity != null) {
				validOrnPalmEntities.add(palmEntity);
				validHndEntities.add(hndEntity);

				// setDependencies(gestEntity, hndEntity, palmEntity);
				gestureEventsEntities.add(gestEntity);
				gestureChainedEventsEntities.add(hndEntity);
				gestureOrientationEntities.add(palmEntity);
			} else {
				gestEntity = getGestureEntity(palmEntity, gestWrp.getGestureField().getGestureEventsEntityCollection());

				if (gestEntity != null) {
					hndEntity = gestEntity.clone(hndWrp.getField().getFieldID());
					hndWrp.getField().addEventsEntity(hndEntity);

					// setDependencies(gestEntity, hndEntity, palmEntity);
					gestureEventsEntities.add(gestEntity);
					gestureChainedEventsEntities.add(hndEntity);
					gestureOrientationEntities.add(palmEntity);

					validOrnPalmEntities.add(palmEntity);
					validHndEntities.add(hndEntity);
				}
			}
		}

		setDependencies(gestureEventsEntities, gestureChainedEventsEntities, gestureOrientationEntities, true);

		// clean up hs entities
		if (validHsEntities.size() < hsEntities.size()) {
			ArrayList<GestureHandShapeEntity> delHsEntities = new ArrayList<GestureHandShapeEntity>();

			for (Object hsEntity1 : hsEntities) {
				GestureHandShapeEntity hsEntity = (GestureHandShapeEntity) hsEntity1;
				if (!validHsEntities.contains(hsEntity)) {
					delHsEntities.add(hsEntity);
				}
			}

			for (GestureHandShapeEntity del : delHsEntities) {
				hsEntities.remove(del);
			}
		}

		// clean up orn entities
		if (validOrnFingerTipEntities.size() < ornFingerTipEntities.size()) {
			ArrayList<GestureOrientationEntity> delOrnFingerTipEntities = new ArrayList<GestureOrientationEntity>();

			for (Object ornFingerTipEntity1 : ornFingerTipEntities) {
				GestureOrientationEntity ornFingerTipEntity = (GestureOrientationEntity) ornFingerTipEntity1;
				if (!validOrnFingerTipEntities.contains(ornFingerTipEntity)) {
					delOrnFingerTipEntities.add(ornFingerTipEntity);
				}
			}

			for (GestureOrientationEntity del : delOrnFingerTipEntities) {
				ornFingerTipEntities.remove(del);
			}
		}

		// clean up hand entities
		if (validHndEntities.size() < hnEntities.size()) {
			ArrayList<GestureChainedEventsEntity> delHndEntities = new ArrayList<GestureChainedEventsEntity>();

			for (Object hnEntity : hnEntities) {
				GestureChainedEventsEntity hEntity = (GestureChainedEventsEntity) hnEntity;
				if (!validHndEntities.contains(hEntity)) {
					delHndEntities.add(hEntity);
				}
			}

			for (GestureChainedEventsEntity del : delHndEntities) {
				hnEntities.remove(del);
			}
		}

		// clean up palm entities
		if (validOrnPalmEntities.size() < palmEntities.size()) {
			ArrayList<GestureOrientationEntity> delOrnPalmEntities = new ArrayList<>();

			for (GestureOrientationEntity palmEntity : palmEntities) {
				if (!validOrnPalmEntities.contains(palmEntity)) {
					delOrnPalmEntities.add(palmEntity);
				}
			}

			for (GestureOrientationEntity del : delOrnPalmEntities) {
				palmEntities.remove(del);
			}
		}
	}

	private void setDependency(GlossFieldWrapper glsWrp, GlossFieldWrapper ndGlsWrp,
			DomNdomHandFieldWrapper rightHandWrp, DomNdomHandFieldWrapper leftHandWrp,
			HandShapeFieldWrapper rightHandShapeWrp, HandShapeFieldWrapper leftHandShapeWrp) {

		setOneHandDependency(glsWrp, rightHandWrp, rightHandShapeWrp);
		setOneHandDependency(ndGlsWrp, leftHandWrp, leftHandShapeWrp);

		GlossChainedEventsEntityCollection glsColl = glsWrp.getGlossField().getGlossChainedEventsEntityCollection();
		GlossChainedEventsEntityCollection ndGlsColl = ndGlsWrp.getGlossField().getGlossChainedEventsEntityCollection();

		ArrayList<GlossChainedEventsEntity> ndGlss = ndGlsColl.getEntities();
		for (GlossChainedEventsEntity gls : ndGlss) {
			String refID = gls.getRefEntityId();
			GlossChainedEventsEntity nGls = glsColl.getEntity(refID);
			if (nGls != null) {
				gls.setDependentGlossEntity(nGls);
				nGls.setDependentGlossEntity(gls);
			} else {
				String txt = gls.getText();
			}
		}
	}

	private void setDependency(GestureFieldWrapper gestWrp, GestureFieldWrapper ndGestWrp,
			GestureDomNdomHandFieldWrapper rightHandWrp, GestureDomNdomHandFieldWrapper leftHandWrp,
			GestureHandShapeFieldWrapper rightHandShapeWrp, GestureHandShapeFieldWrapper leftHandShapeWrp,
			GestureOrientationFieldWrapper rightHandOrientationFingerTipWrp, GestureOrientationFieldWrapper leftHandOrientationFingerTipWrp,
			GestureOrientationFieldWrapper rightPalmOrientationWrp, GestureOrientationFieldWrapper leftPalmOrientationWrp) {

		setOneHandDependency(gestWrp, rightHandWrp, rightHandShapeWrp, rightHandOrientationFingerTipWrp, rightPalmOrientationWrp);
		setOneHandDependency(ndGestWrp, leftHandWrp, leftHandShapeWrp, leftHandOrientationFingerTipWrp, leftPalmOrientationWrp);

		GestureEventsEntityCollection gestColl = gestWrp.getGestureField().getGestureEventsEntityCollection();
		GestureEventsEntityCollection ndGestColl = ndGestWrp.getGestureField().getGestureEventsEntityCollection();

		ArrayList<GestureEventsEntity> ndGests = ndGestColl.getEntities();
		for (GestureEventsEntity gest : ndGests) {
			String refID = gest.getRefEntityId();
			GestureEventsEntity nGest = gestColl.getEntity(refID);
			if (nGest != null) {
				gest.setDependentGestureEntity(nGest);
				nGest.setDependentGestureEntity(gest);
			} else {
				String txt = gest.getText();
			}
		}
	}

	private void addSignTypeDependencies(GlossFieldWrapper dGlsWrapper, GlossFieldWrapper ndGlsWrapper,
			SignTypeFieldWrapper dSignTypeFieldWrapper, SignTypeFieldWrapper ndSignTypeFieldWrapper) {
		SignTypeField dSignTypeField = (SignTypeField) dSignTypeFieldWrapper.getField();
		SignTypeField ndSignTypeField = (SignTypeField) ndSignTypeFieldWrapper.getField();

		ArrayList glossEntities = dGlsWrapper.getGlossField().getEntities();
		for (Object obj : glossEntities) {
			GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) obj;
			if (glsEntity.getDependentSignTypeEntity() == null) {
				SignTypeEntity newEntity = getSignTypeEntity(glsEntity, dSignTypeField);
				glsEntity.setDependentSignTypeEntity(newEntity);
				newEntity.setRefGlossEntity(glsEntity);
				GlossChainedEventsEntity depGlsEntity = glsEntity.getDependentGlossEntity();
				if (depGlsEntity != null) {
					depGlsEntity.setDependentSignTypeEntity(newEntity);
				}
			}
		}

		ArrayList ndGlossEntities = ndGlsWrapper.getGlossField().getEntities();
		for (Object obj : ndGlossEntities) {
			GlossChainedEventsEntity ndGlsEntity = (GlossChainedEventsEntity) obj;
			if (ndGlsEntity.getDependentGlossEntity() == null && ndGlsEntity.getDependentSignTypeEntity() == null) {
				SignTypeEntity newEntity = getSignTypeEntity(ndGlsEntity, ndSignTypeField);
				ndGlsEntity.setDependentSignTypeEntity(newEntity);
				newEntity.setRefGlossEntity(ndGlsEntity);
			}
		}
	}

	private SignTypeEntity getSignTypeEntity(GlossChainedEventsEntity glsEntity, SignTypeField signTypeField) {
		SignTypeEvent newEvent = new SignTypeEvent(segmentPanel, glsEntity.getStartTimeInfo(),
				glsEntity.getEndTimeInfo());
		newEvent.setSignType(SignType.getSignTypeFromGloss(glsEntity));
		newEvent.setNameSign(glsEntity.getGlossEntityProperties().isNameSign());

		SignTypeEntity newEntity = new SignTypeEntity(newEvent, segmentPanel, signTypeField);
		newEntity.setID(Util.getLongUniqueNumber());
		newEntity.updateParentEntity();

		signTypeField.addEventsEntity(newEntity);
		return newEntity;
	}

	private ChainedEventsEntity getHandEntity(HandShapeEntity handShapeEntity, DomNdomHandFieldWrapper handWrp) {

		String refHandID = handShapeEntity.getRefEntityId();

		EventsEntity entity = null;

		// first try to get the hand entity based on the hs entity ref entity id
		if (refHandID != null)
			entity = handWrp.getField().getEntity(refHandID);

		// if the ref entity was blank or the hand entity in question is missing from
		// the wrapper for whatever, try by midpoint coord of handShapeEntity
		if (entity == null) {

			double startCoordinate = handShapeEntity.getStartTimeCoordinate();
			double endCoordinate = handShapeEntity.getEndTimeCoordinate();
			double midCoordinate = startCoordinate + (endCoordinate - startCoordinate) / 2;

			entity = handWrp.getField().getEntityAt(midCoordinate); // this eventually makes use of checking start ==
			// end (one frame)
		}

		// if we still haven't found our hand entity, look in the wrapper for an entity
		// matching the start/end points of this handShapeEntity
		if (entity == null) {
			ArrayList list = handWrp.getField().getEntities();

			for (int i = 0; i < list.size(); i++) {
				EventsEntity ent = (EventsEntity) list.get(i);
				Time startTimeInfo = ent.getStartTimeInfo();
				Time endTimeInfo = ent.getEndTimeInfo();

				Time hsStartTimeInfo = handShapeEntity.getStartTimeInfo();
				Time hsEndTimeInfo = handShapeEntity.getEndTimeInfo();

				if (startTimeInfo.equals(hsStartTimeInfo) && endTimeInfo.equals(hsEndTimeInfo)) {
					entity = ent;
				}
			}
		}

		if (entity != null && isValid(handShapeEntity, entity))
			return (ChainedEventsEntity) entity;

		return null;
	}

	private GestureChainedEventsEntity getHandEntity(GestureHandShapeEntity handShapeEntity,
			GestureDomNdomHandFieldWrapper handWrp) {

		String refHandID = handShapeEntity.getRefEntityId();

		EventsEntity entity = null;

		// first try to get the hand entity based on the hs entity ref entity id
		if (refHandID != null)
			entity = handWrp.getField().getEntity(refHandID);

		// if the ref entity was blank or the hand entity in question is missing from
		// the wrapper for whatever, try by midpoint coord of handShapeEntity
		if (entity == null) {

			double startCoordinate = handShapeEntity.getStartTimeCoordinate();
			double endCoordinate = handShapeEntity.getEndTimeCoordinate();
			double midCoordinate = startCoordinate + (endCoordinate - startCoordinate) / 2;

			entity = handWrp.getField().getEntityAt(midCoordinate); // this eventually makes use of checking start ==
			// end (one frame)
		}

		// if we still haven't found our hand entity, look in the wrapper for an entity
		// matching the start/end points of this handShapeEntity
		if (entity == null) {
			ArrayList list = handWrp.getField().getEntities();

			for (int i = 0; i < list.size(); i++) {
				EventsEntity ent = (EventsEntity) list.get(i);
				Time startTimeInfo = ent.getStartTimeInfo();
				Time endTimeInfo = ent.getEndTimeInfo();

				Time hsStartTimeInfo = handShapeEntity.getStartTimeInfo();
				Time hsEndTimeInfo = handShapeEntity.getEndTimeInfo();

				if (startTimeInfo.equals(hsStartTimeInfo) && endTimeInfo.equals(hsEndTimeInfo)) {
					entity = ent;
				}
			}
		}

		if (entity != null && isValid(handShapeEntity, entity))
			return (GestureChainedEventsEntity) entity;

		return null;
	}

	private GestureChainedEventsEntity getHandEntity(GestureOrientationEntity orientationEntity,
			GestureDomNdomHandFieldWrapper handWrp) {

		String refHandID = orientationEntity.getRefEntityId();

		EventsEntity entity = null;

		// first try to get the hand entity based on the orn entity ref entity id
		if (refHandID != null)
			entity = handWrp.getField().getEntity(refHandID);

		// if the ref entity was blank or the hand entity in question is missing from
		// the wrapper for whatever, try by midpoint coord of orientationEntity
		if (entity == null) {

			double startCoordinate = orientationEntity.getStartTimeCoordinate();
			double endCoordinate = orientationEntity.getEndTimeCoordinate();
			double midCoordinate = startCoordinate + (endCoordinate - startCoordinate) / 2;

			entity = handWrp.getField().getEntityAt(midCoordinate); // this eventually makes use of checking start ==
			// end (one frame)
		}

		// if we still haven't found our hand entity, look in the wrapper for an entity
		// matching the start/end points of this orientationEntity
		if (entity == null) {
			ArrayList list = handWrp.getField().getEntities();

			for (int i = 0; i < list.size(); i++) {
				EventsEntity ent = (EventsEntity) list.get(i);
				Time startTimeInfo = ent.getStartTimeInfo();
				Time endTimeInfo = ent.getEndTimeInfo();

				Time ornStartTimeInfo = orientationEntity.getStartTimeInfo();
				Time ornEndTimeInfo = orientationEntity.getEndTimeInfo();

				if (startTimeInfo.equals(ornStartTimeInfo) && endTimeInfo.equals(ornEndTimeInfo)) {
					entity = ent;
				}
			}
		}

		if (entity != null && isValid(orientationEntity, entity))
			return (GestureChainedEventsEntity) entity;

		return null;
	}

	private boolean isValid(EventsEntity entity1, EventsEntity entity2) {
		
		return true;
		// TODO: not sure what the point of this ever was, or whether it was ever checking properly
		
//		if (entity2 == null)
//			return false;
//
//		// GREG TODO: this is assuming both timeinfo objs are of the same unit
//		double startTime = entity1.getStartTimeInfo().doubleValue();
//		double endTime = entity1.getEndTimeInfo().doubleValue();
//
//		Line2D domLine = new Line2D.Double(startTime, 0, endTime, 0);
//
//		double startMovieTime = entity2.getStartTimeInfo().doubleValue();
//		double w = entity2.getEndTimeInfo().doubleValue() - startMovieTime;
//
//		boolean isValid = domLine.intersects(startMovieTime, 0, w, 1);
//
//		if (startTime == startMovieTime) {
//			isValid = true;
//		}
//
//		return isValid;
	}

	private GlossChainedEventsEntity getGlossEntity(EventsEntity hsEntity, GlossChainedEventsEntityCollection glsColl) {

		double hsStartTime = hsEntity.getStartTimeInfo().doubleValue();
		double hsEndTime = hsEntity.getEndTimeInfo().doubleValue();

		Line2D domLine = new Line2D.Double(hsStartTime, 0, hsEndTime, 0);

		ArrayList<GlossChainedEventsEntity> glosses = glsColl.getEntities();

		GlossChainedEventsEntity entity = null;

		ArrayList<GlossChainedEventsEntity> entitiesToDelete = new ArrayList<GlossChainedEventsEntity>();
		for (GlossChainedEventsEntity gloss : glosses) {
			if (gloss == null || gloss.getStartTimeInfo() == null || gloss.getEndTimeInfo() == null) {
				entitiesToDelete.add(gloss);
			}
		}

		for (GlossChainedEventsEntity toDelete : entitiesToDelete) {
			glsColl.deleteEntity(toDelete);
		}

		for (GlossChainedEventsEntity gloss : glosses) {

			double startMovieTime = gloss.getStartTimeInfo().doubleValue();
			double endMovieTime = gloss.getEndTimeInfo().doubleValue();
			double w = endMovieTime - startMovieTime;

			if (domLine.intersects(startMovieTime, 0, w, 1)) {
				entity = gloss;
			}
		}
		return entity;
	}

	private GestureEventsEntity getGestureEntity(EventsEntity depEntity, GestureEventsEntityCollection gestColl) {

		double depStartTime = depEntity.getStartTimeInfo().doubleValue();
		double depEndTime = depEntity.getEndTimeInfo().doubleValue();

		Line2D domLine = new Line2D.Double(depStartTime, 0, depEndTime, 0);

		ArrayList<GestureEventsEntity> glosses = gestColl.getEntities();

		GestureEventsEntity entity = null;

		ArrayList<GestureEventsEntity> entitiesToDelete = new ArrayList<GestureEventsEntity>();
		for (GestureEventsEntity gloss : glosses) {
			if (gloss == null || gloss.getStartTimeInfo() == null || gloss.getEndTimeInfo() == null) {
				entitiesToDelete.add(gloss);
			}
		}

		for (GestureEventsEntity toDelete : entitiesToDelete) {
			gestColl.deleteEntity(toDelete);
		}

		for (GestureEventsEntity gloss : glosses) {

			double startMovieTime = gloss.getStartTimeInfo().doubleValue();
			double endMovieTime = gloss.getEndTimeInfo().doubleValue();
			double w = endMovieTime - startMovieTime;

			if (domLine.intersects(startMovieTime, 0, w, 1)) {
				entity = gloss;
			}
		}
		return entity;
	}

	private GlossChainedEventsEntity getGlossEntity(ChainedEventsEntity handEntity, GlossFieldWrapper glsFieldWrapper) {

		if (handEntity == null)
			return null;

		GlossChainedEventsEntityCollection coll = glsFieldWrapper.getGlossField()
				.getGlossChainedEventsEntityCollection();

		GlossChainedEventsEntity entity = coll.getEntity(handEntity.getRefEntityId());

		// first try to get the gloss entity based on the hand entity ref entity id
		if (entity == null)
			entity = coll.getEntity(handEntity.getRefEntityId());

		// if it was not found by ref entity id, try next by coordinate
		if (entity == null) {
			EventsEntity ent = glsFieldWrapper.getGlossField()
					.getEntityAt(handEntity.getValueOrTextValueEvent().getStartTimeCoordinate());

			if (ent != null)
				entity = (GlossChainedEventsEntity) ent;
		}

		// if still not found, call getGlossEntity
		if (entity == null) {
			entity = getGlossEntity(handEntity, coll);
		}

		if (isValid(handEntity, entity))
			return entity;

		return null;
	}

	private GestureEventsEntity getGestureEntity(GestureChainedEventsEntity handEntity,
			GestureFieldWrapper gestFieldWrapper) {

		if (handEntity == null)
			return null;

		GestureEventsEntityCollection coll = gestFieldWrapper.getGestureField().getGestureEventsEntityCollection();

		GestureEventsEntity entity = coll.getEntity(handEntity.getRefEntityId());

		// first try to get the gloss entity based on the hand entity ref entity id
		if (entity == null)
			entity = coll.getEntity(handEntity.getRefEntityId());

		// if it was not found by ref entity id, try next by coordinate
		if (entity == null) {
			EventsEntity ent = gestFieldWrapper.getGestureField()
					.getEntityAt(handEntity.getValueOrTextValueEvent().getStartTimeCoordinate());

			if (ent != null)
				entity = (GestureEventsEntity) ent;
		}

		// if still not found, call getGlossEntity
		if (entity == null) {
			entity = getGestureEntity(handEntity, coll);
		}

		if (isValid(handEntity, entity))
			return entity;

		return null;
	}

	private void setDependencies(GlossChainedEventsEntity glsEntity, ChainedEventsEntity handEntity,
			HandShapeEntity handShapeEntity) {

		glsEntity.setDependentHandEntity(handEntity);
		glsEntity.setDependentHandShapeEntity(handShapeEntity);

		if (!glsEntity.isLockedGlossPhonAlignments())
			return;

		// check if gloss and hand entities are displayed correctly
		Event value = glsEntity.getValueOrTextValueEvent();
		ChainedEvent initialHold = glsEntity.getInitialHold();
		ChainedEvent finalHold = glsEntity.getFinalHold();

		Event hndValue = handEntity.getValueOrTextValueEvent();

		ChainedEvent hndText = (ChainedEvent) handEntity.getValueOrTextValueEvent();
		hndText.setText(glsEntity.getText());

		Time valueStartTimeInfo = value.getStartTimeInfo();
		Time valueEndTimeInfo = value.getEndTimeInfo();
		Time initialHoldStartTimeInfo, finalHoldEndTimeInfo;

		// if initial hold exists, then set the entity in the RightLeftHandFieldWrapper
		// to its start time
		if (initialHold != null) {
			initialHoldStartTimeInfo = initialHold.getStartTimeInfo();
			hndValue.setStartTimeInfo(initialHoldStartTimeInfo);
			handEntity.setInitialHoldEvent(initialHold.cloneChainedEvent());
		} else {
			hndValue.setStartTimeInfo(valueStartTimeInfo);
		}

		// if final hold exists, then set the entity in the RightLeftHandFieldWrapper to
		// its end time
		if (finalHold != null) {
			finalHoldEndTimeInfo = finalHold.getEndTimeInfo();
			hndValue.setEndTimeInfo(finalHoldEndTimeInfo);
			handEntity.setFinalHoldEvent(finalHold.cloneChainedEvent());
		} else {
			hndValue.setEndTimeInfo(valueEndTimeInfo);
		}
	}

	private void setDependencies(GestureEventsEntity gestEntity, GestureChainedEventsEntity handEntity,
			GestureHandShapeEntity handShapeEntity) {

		gestEntity.setDependentHandEntity(handEntity);
		gestEntity.setDependentHandShapeEntity(handShapeEntity);
		handShapeEntity.setRefGestureEntity(gestEntity);

		// check if gloss and hand entities are displayed correctly
		Event value = gestEntity.getValueOrTextValueEvent();
//		GestureChainedEvent initialHold = gestEntity.getInitialHold();
//		GestureChainedEvent finalHold = gestEntity.getFinalHold();

		Event hndValue = handEntity.getValueOrTextValueEvent();

		GestureChainedEvent hndText = (GestureChainedEvent) handEntity.getValueOrTextValueEvent();
		hndText.setText(gestEntity.getText());

		Time valueStartTimeInfo = value.getStartTimeInfo();
		Time valueEndTimeInfo = value.getEndTimeInfo();
		Time initialHoldStartTimeInfo, finalHoldEndTimeInfo;

//		// if initial hold exists, then set the entity in the DomNdomHandFieldWrapper
//		// to its start time
//		if (initialHold != null) {
//			initialHoldStartTimeInfo = initialHold.getStartTimeInfo();
//			hndValue.setStartTimeInfo(initialHoldStartTimeInfo);
//			handEntity.setInitialHoldEvent(initialHold.cloneGestureChainedEvent());
//		} else {
//			hndValue.setStartTimeInfo(valueStartTimeInfo);
//		}
//
//		// if final hold exists, then set the entity in the DomNdomHandFieldWrapper to
//		// its end time
//		if (finalHold != null) {
//			finalHoldEndTimeInfo = finalHold.getEndTimeInfo();
//			hndValue.setEndTimeInfo(finalHoldEndTimeInfo);
//			handEntity.setFinalHoldEvent(finalHold.cloneGestureChainedEvent());
//		} else {
//			hndValue.setEndTimeInfo(valueEndTimeInfo);
//		}
	}

	private void setDependencies(ArrayList<GestureEventsEntity> gestEntity, ArrayList<GestureChainedEventsEntity> handEntity,
			ArrayList<GestureOrientationEntity> orientationEntity, boolean isPalmOrientation) {

		for (int x = 0; x < orientationEntity.size(); x++) {
			if (isPalmOrientation) {
				gestEntity.get(x).setDependentOrientationPalmEntity(orientationEntity.get(x));
				orientationEntity.get(x).setRefGestureEntity(gestEntity.get(x));
			} else {
				gestEntity.get(x).setDependentOrientationFingerTipEntity(orientationEntity.get(x));
				orientationEntity.get(x).setRefGestureEntity(gestEntity.get(x));
			}

			GestureChainedEvent hndText = (GestureChainedEvent) handEntity.get(x).getValueOrTextValueEvent();
			hndText.setText(gestEntity.get(x).getText());
		}

/*		if (orientationEntity.isPalmEntity()) {
			gestEntity.setDependentOrientationPalmEntity(orientationEntity);
			orientationEntity.setRefGestureEntity(gestEntity);
		} else {
			gestEntity.setDependentOrientationEntity(orientationEntity);
			orientationEntity.setRefGestureEntity(gestEntity);
		}*/

		// check if gloss and hand entities are displayed correctly
//		GestureChainedEvent initialHold = gestEntity.getInitialHold();
//		GestureChainedEvent finalHold = gestEntity.getFinalHold();


/*		GestureChainedEvent hndText = (GestureChainedEvent) handEntity.getValueOrTextValueEvent();
		hndText.setText(gestEntity.getText());*/

//		// if initial hold exists, then set the entity in the DomNdomHandFieldWrapper
//		// to its start time
//		if (initialHold != null) {
//			initialHoldStartTimeInfo = initialHold.getStartTimeInfo();
//			hndValue.setStartTimeInfo(initialHoldStartTimeInfo);
//			handEntity.setInitialHoldEvent(initialHold.cloneGestureChainedEvent());
//		} else {
//			hndValue.setStartTimeInfo(valueStartTimeInfo);
//		}
//
//		// if final hold exists, then set the entity in the DomNdomHandFieldWrapper to
//		// its end time
//		if (finalHold != null) {
//			finalHoldEndTimeInfo = finalHold.getEndTimeInfo();
//			hndValue.setEndTimeInfo(finalHoldEndTimeInfo);
//			handEntity.setFinalHoldEvent(finalHold.cloneGestureChainedEvent());
//		} else {
//			hndValue.setEndTimeInfo(valueEndTimeInfo);
//		}

//		glsEntity.setLockedGesturePhonAlignments(true);
//		handEntity.setLockedGesturePhonAlignments(true);
//		orientationEntity.setLockedGesturePhonAlignments(true);
	}
}
