package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;

import javax.swing.BorderFactory;
import javax.swing.BoundedRangeModel;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.observer.SSObservable;
import edu.bu.signstream.observer.SSObserver;

public class GlossNRelatedView extends JPanel implements SSObserver, SSBasicPanel {

	protected GlossFieldsLabelView labelView;
	protected JScrollPane pictureScrollPane;
	protected JSplitPane sp;

	private Segment segment;
	private SignStreamSegmentPanel panel;
	private SegmentGraphicRepresentation rep = null;
	private PresentationField parentField = null;
	private final String panelID;
	
	private FieldWrappersCollection fwc = new FieldWrappersCollection();

	private LabelObject selectedField;

	public GlossNRelatedView(Segment segment, SignStreamSegmentPanel panel, UtteranceView view, PresentationField parentField, String panelID) {
		this.segment = segment;
		this.panel = panel;
		this.parentField = parentField;
		this.panelID = panelID;

		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
		labelView = new GlossFieldsLabelView(segment, view, this, fwc);

		rep = new SegmentGraphicRepresentation(panel, segment, fwc);
		createUI();
	}

	public String getPanelID() {
		return this.panelID;
	}

	
	public void update(Segment segment) {
		this.segment = segment;
		labelView.update(segment);
	}

	public GlossFieldsLabelView getLabelView() {
		return labelView;
	}

	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}
	public JSplitPane getSplitPane() {
		return sp;
	}

	public void createUI() {
		double mDuration = SegmentGraphReprUtil.utteranceInterval().duration().as(TimeUnit.TIMESTAMP, Rounding.NEAREST);
		int length = panel.getZoomer().getCoordinateTimeConvertor().getScreenWidth(mDuration);

		length = Math.max(length, rep.getPreferredDimension().width);

		rep.setPreferredSize(new Dimension(length, (int) rep.getPreferredDimension().getHeight()));

		if (rep != null) {
			labelView.setPreferredSize(new Dimension(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH,	rep.getPreferredDimension().height));
			labelView.setMinimumSize(new Dimension(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH,0));
		} else {
			labelView.setPreferredHeight(480);
		}

		rep.setPreferredSize(new Dimension(rep.getWidth(), (int) labelView.getPreferredSize().getHeight()));

		pictureScrollPane = new JScrollPane(rep, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,	ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.getHorizontalScrollBar().setPreferredSize(new Dimension(0, 0));

		pictureScrollPane.setPreferredSize(new Dimension(300, 250));
		pictureScrollPane.setViewportBorder(BorderFactory.createLineBorder(Color.black));
		//pictureScrollPane.setRowHeaderView(labelView);

		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER,  new Corner());
		pictureScrollPane.setCorner(ScrollPaneConstants.LOWER_LEFT_CORNER,  new Corner());
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER,	new Corner());
		labelView.setVerticalScrollModel(getVerticalScrollModel());
		pictureScrollPane.getVerticalScrollBar().addAdjustmentListener(e -> {
			labelView.setVerticalScrollValue(e.getValue());
		});



		if (SegmentGraphReprUtil.utteranceStart != null) {
			var x = SegmentGraphReprUtil.utteranceStart.absolute().as(TimeUnit.COORDINATE, Rounding.NEAREST);
			scrollRectToVisible(new Point((int)x, 10));
		}

		sp = new JSplitPane();
		sp.setLeftComponent(labelView);
		sp.setRightComponent(pictureScrollPane);

		add(sp);
	}

	public Segment getSegment() {
		return segment;
	}

	public void updatePanelHeights(int height) {
		if (rep != null) {
			rep.setPreferredSize(new Dimension(rep.getPreferredDimension().width, height + SegmentGraphReprUtil.generalGlossGestureOffset));
			pictureScrollPane.getViewport().setView(rep);
		}
		if (labelView != null) {
			labelView.setPreferredHeight(height);
		}
	}



	public void scrollRectToVisible(Point point) {
//		JViewport vp = pictureScrollPane.getViewport();
//		vp.setViewPosition(point);
		pictureScrollPane.invalidate();
		pictureScrollPane.revalidate();
	}

	public FieldWrappersCollection getFieldWrappersCollection() {
		return fwc;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return panel;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRep() {
		return rep;
	}

	public void update() {
		labelView.update();

		int height = labelView.getPreferredSize().height;
		updatePanelHeights(height);

		labelView.invalidate();
		labelView.revalidate();
	}

	@Override
	public void update(SSObservable subj, Object o) {
		rep.update();
		labelView.update();

		labelView.setPreferredSize(new Dimension(150,
				rep.getPreferredSize().height));

		pictureScrollPane.setRowHeaderView(labelView);
		repaint();
	}
	
	private BoundedRangeModel getHorizontalScrollModel() {
		return getJScrollPane().getHorizontalScrollBar().getModel();
	}
	
	private BoundedRangeModel getVerticalScrollModel() {
		return getJScrollPane().getVerticalScrollBar().getModel();
	}

	public LabelObject getSelectedField() {
		return selectedField;
	}

	public void setVerticalScrollValue(int value) {
		pictureScrollPane.getVerticalScrollBar().setValue(value);
	}

	/**
	 * @return
	 */
	@Override
	public String getID() {
		return this.panelID;
	}
}
