package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.navigation.DbNavigationPanel;

public class LabelViewTemporalPartitions extends JPanel implements
		TreeSelectionListener, KeyListener {

	private static final long serialVersionUID = 1L;

	private JTree tree1 = null;

	//private PresentationUpperLeftCorner1 corner = null;

	protected Segment segment;
	protected DatabaseView view;
	
	private boolean deleteNode = false;

	private Color color;
	private CustomTreeRenderer renderer;
	private JScrollPane jsp;

	public LabelViewTemporalPartitions(Segment segment, final DatabaseView view) {	
		this.view = view;
		int fontSize = 12, fontStyle = Font.PLAIN;
		String fontFamily = "Times-Roman";
		Font font = new Font(fontFamily, fontStyle, fontSize);
		this.segment = segment;

		color = new Color(255, 247, 200);

		// renderer handles display of default segment tiers
		renderer = new CustomTreeRenderer(Palette.BACKGROUND_COLOR,
				200, SegmentGraphReprUtil.cellHeight);
		renderer.setLeafIcon(null);
		renderer.setIcon(null);
		renderer.setClosedIcon(null);
		renderer.setOpenIcon(null);
		renderer.setBackground(UIPalette.getBackground());
		renderer.setBackgroundNonSelectionColor(UIPalette.getBackground());
		renderer.setBackgroundSelectionColor(UIPalette.getBackground()); // .getBackgroundSelection());
		renderer.setBorderSelectionColor(UIPalette.getBackground()); // .getBackgroundSelection());
		renderer.setTextSelectionColor(Color.red);
		renderer.setFont(font);

		this.tree1 = new JTree(new Vector());

		tree1.addTreeExpansionListener(new TreeExpansionListener() {
			@Override
			public void treeCollapsed(TreeExpansionEvent e) {
				if (view == null)
					return;

				Object object = e.getPath().getLastPathComponent();

				if (object == null)
					return;

				DefaultMutableTreeNode show = (DefaultMutableTreeNode) object;

				Enumeration children = show.children();
				while (children.hasMoreElements()) {
					DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) children
							.nextElement();
					Object nodeInfo = treeNode.getUserObject();
					LabelObject lb = (LabelObject) nodeInfo;
					lb.getWrapper().setVisible(false);
				}

				view.repaint();
				resizeView();
			}

			@Override
			public void treeExpanded(TreeExpansionEvent e) {
				if (view == null)
					return;

				Object object = e.getPath().getLastPathComponent();

				if (object == null)
					return;

				DefaultMutableTreeNode show = (DefaultMutableTreeNode) object;

				Enumeration children = show.children();
				while (children.hasMoreElements()) {
					DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) children
							.nextElement();
					Object nodeInfo = treeNode.getUserObject();
					LabelObject lb = (LabelObject) nodeInfo;
					lb.getWrapper().setVisible(true);
				}

				view.repaint();
				resizeView();
			}
		});

		// do not remove!
		tree1.setCellRenderer(renderer);
		tree1.setBackground(UIPalette.getBackground());
		tree1.setForeground(UIPalette.getBackground());
		tree1.setRowHeight(SegmentGraphReprUtil.cellHeight);
		tree1.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree1.addTreeSelectionListener(this);
		tree1.addKeyListener(this);
		tree1.addKeyListener(SS3Singleton.getVideoControlKeyListener());
		tree1.setFocusable(false);

		populateTree();
	}

	/**
	 * The populateTree() function requires segment.getRootLabelObject() to be set
	 * prior to entering it.
	 * It takes a LabelObject structure and creates a DefaultTreeModel
	 */
	public void populateTree() {
		int originalRow = tree1.getLeadSelectionRow();		// returns -1 if nothing selected
		
		removeAll();

		// require segment.getRootLabelObject() to be set before entering this function
		RootLabelObject rootLabelObject = segment.getRootLabelObject();
		LabelObject presentationLabelObject = rootLabelObject.getPresentationLabelObject();

		DefaultTreeModel treeModel = new DefaultTreeModel(new DefaultMutableTreeNode());
		// this will "reset" the tree to a new root and populated it below
		tree1.setModel(treeModel);
		
		DefaultMutableTreeNode parent = (DefaultMutableTreeNode) treeModel.getRoot();

		// objects to be populated into tree model under parent
		ArrayList<LabelObject> objects = presentationLabelObject.getChildrenAsList();

		DefaultMutableTreeNode timePeriodNode = new DefaultMutableTreeNode(objects.get(0));
		treeModel.insertNodeInto(timePeriodNode, parent, 0);	// insert Display Time Period line at index 0 always

		for (int i = 1; i < objects.size(); i++) {
			LabelObject partitionObj = objects.get(i);
			DefaultMutableTreeNode partitionNode = new DefaultMutableTreeNode(partitionObj);
			treeModel.insertNodeInto(partitionNode, parent, i); // temporal partition partitionNode is the i-th child of parent

			ArrayList<LabelObject> segmentLabelObjects = partitionObj.getChildrenAsList();
			for (int k = 0; k < segmentLabelObjects.size(); k++) {
				LabelObject segmentLabelObject = segmentLabelObjects.get(k);
				DefaultMutableTreeNode segmentNode = new DefaultMutableTreeNode(segmentLabelObject);
				treeModel.insertNodeInto(segmentNode, partitionNode, k); // segment tier segmentNode is the k-th child of partitionNode
			}
			
			TreePath partitionPath = new TreePath(partitionNode.getPath());
			tree1.setSelectionPath(partitionPath);
			tree1.expandPath(partitionPath);
		}

		if (originalRow != -1) {
			tree1.setSelectionRow(originalRow);
			tree1.expandRow(originalRow);
		}

		JPanel p = new JPanel();
		p.setBackground(UIPalette.getBackground());
		p.setPreferredSize(new Dimension(getWidth(),
				SegmentGraphReprUtil.axesBottomHeight));

		setLayout(new BorderLayout());	
		
		add(p, BorderLayout.NORTH);
		
		jsp = new JScrollPane(tree1);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		jsp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		add(jsp, BorderLayout.CENTER);
		
		repaint();
		updateUI();
	}

	public void updateTree() {
		populateTree();
		resizeView();
	}
	
	/** 
	 * Returns the result of this JTree.getLastSelectedPathComponent()
	 * 
	 * @return DefaultMutableTreeNode representing selected path
	 */
	public DefaultMutableTreeNode getSelectedNode() {
		return (DefaultMutableTreeNode) tree1.getLastSelectedPathComponent();
	}
	
	/**
	 * Wrapper for JTree.setSelectionPath(TreePath path)
	 * @param path
	 */
	public void setSelectedNode(TreePath path) {
		tree1.setSelectionPath(path);
	}

	public void selectPartition(String label) {
        var root = (DefaultMutableTreeNode)tree1.getModel().getRoot();

        for (int i = 0; i < root.getChildCount(); i++) {
            var partition = (DefaultMutableTreeNode)root.getChildAt(i);
            if (((LabelObject)partition.getUserObject()).getLabel().equals(label)) {
                setSelectedNode(new TreePath(partition.getPath()));
                return;
            }
        }
    }

    public void selectSegment(String segmentID) {
        var root = (DefaultMutableTreeNode)tree1.getModel().getRoot();

        for (int i = 0; i < root.getChildCount(); i++) {
            var partition = (DefaultMutableTreeNode)root.getChildAt(i);
            for (int j = 0; j < partition.getChildCount(); j++) {
                var segment = (DefaultMutableTreeNode)partition.getChildAt(j);

                var label = (LabelObject)segment.getUserObject();
                var field = (PresentationField)label.getField();
                if (field.getSS3GlossWindowSegment().getID().equals(segmentID)) {
                    setSelectedNode(new TreePath(segment.getPath()));
                    return;
                }
            }
        }
    }

//	public void setPresentationUpperLeftCorner(
//			PresentationUpperLeftCorner1 corner) {
//		this.corner = corner;
//	}

	public void createTemporalPartition() {
		RootLabelObject rootObject = segment.getRootLabelObject();
		LabelObject presentationLabelObject = rootObject
				.getPresentationLabelObject();
		ArrayList<LabelObject> objects = presentationLabelObject
				.getChildrenAsList();
		/*
		 * SignStreamSegmentPanel panel = segment.getSignStreamSegmentPanel();
		 * //panel.get int counter = partitions.size() + 1; String
		 * particionLabel = "Temporal Partition " + counter; String segmentLabel
		 * = "Segment 1";
		 * 
		 * while (partitions.containsKey(particionLabel)) { counter++;
		 * particionLabel = "Temporal Partition " + counter; }
		 * 
		 * SS3TemporalPartition partition = new SS3TemporalPartition();
		 * partition.setLabel(particionLabel); //TemporalPartitionSegment
		 * tpSegment = new TemporalPartitionSegment();
		 * //tpSegment.setTemporalPartitionSegmentLabel(segmentLabel);
		 * //partitions.put(partition.getTemporalPartitionLabel(), partition);
		 * 
		 * PresentationField pf1 = new PresentationField(partition,
		 * view.getSignStreamSegmentPanel()); PresentationField pf2 = new
		 * PresentationField( new SS3TemporalPartition(), view
		 * .getSignStreamSegmentPanel()); PresentationFieldWrapper wrp1 = new
		 * PresentationFieldWrapper(pf1, view.getSignStreamSegmentPanel());
		 * SegmentFieldWrapper sw1 = new SegmentFieldWrapper(pf2, view
		 * .getSignStreamSegmentPanel(), particionLabel, segmentLabel);
		 * wrp1.addSegmentFieldWrapper(sw1); partitionObj.setWrapper(wrp1);
		 * segmentObj.setWrapper(sw1);
		 * 
		 * LabelObject partitionObj = new LabelObject("", particionLabel);
		 * LabelObject segmentObj = new LabelObject("", segmentLabel);
		 * 
		 * DefaultTreeModel treeModel = (DefaultTreeModel) tree1.getModel();
		 * DefaultMutableTreeNode parent = (DefaultMutableTreeNode) treeModel
		 * .getRoot();
		 * 
		 * DefaultMutableTreeNode partitionNode = new DefaultMutableTreeNode(
		 * partitionObj); treeModel.insertNodeInto(partitionNode, parent,
		 * parent.getChildCount()); DefaultMutableTreeNode segmentNode = new
		 * DefaultMutableTreeNode( segmentObj);
		 * treeModel.insertNodeInto(segmentNode, partitionNode, segmentNode
		 * .getChildCount());
		 * 
		 * TreePath partitionPath = new TreePath(partitionNode.getPath());
		 * tree1.setSelectionPath(partitionPath);
		 * tree1.expandPath(partitionPath);
		 * 
		 * if (view != null) { PresentationField pf1 = new PresentationField(
		 * new SS3TemporalPartition(), view .getSignStreamSegmentPanel());
		 * PresentationField pf2 = new PresentationField( new
		 * SS3TemporalPartition(), view .getSignStreamSegmentPanel());
		 * PresentationFieldWrapper wrp1 = new PresentationFieldWrapper(pf1,
		 * view.getSignStreamSegmentPanel()); SegmentFieldWrapper sw1 = new
		 * SegmentFieldWrapper(pf2, view .getSignStreamSegmentPanel(),
		 * particionLabel, segmentLabel); wrp1.addSegmentFieldWrapper(sw1);
		 * partitionObj.setWrapper(wrp1); segmentObj.setWrapper(sw1);
		 * 
		 * segment.getFieldWrappersCollection().addFieldWrapper(wrp1);
		 * segment.getFieldWrappersCollection().addFieldWrapper(sw1);
		 * view.repaint(); }
		 */
	}

	public void moveSelectedNodeUp() {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree1
				.getLastSelectedPathComponent();
		DefaultMutableTreeNode prevNode = node.getPreviousSibling();
		TreeNode parentNode = node.getParent();
		DefaultTreeModel treeModel = (DefaultTreeModel) tree1.getModel();
		int index = treeModel.getIndexOfChild(parentNode, node);
		boolean expanded = tree1.isExpanded(new TreePath(node.getPath()));

		if (parentNode == null || prevNode == null
				|| (node.getLevel() == 1 && index <= 1))
			return;

		treeModel.removeNodeFromParent(node);
		treeModel.insertNodeInto((MutableTreeNode) node,
				(MutableTreeNode) parentNode, index - 1);

		TreePath path = new TreePath(node.getPath());
		tree1.setSelectionPath(path);

		if (expanded)
			tree1.expandPath(new TreePath(node.getPath()));
	}

	public void moveSelectedNodeDown() {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree1
				.getLastSelectedPathComponent();
		DefaultMutableTreeNode nextNode = node.getNextSibling();
		TreeNode parentNode = node.getParent();
		DefaultTreeModel treeModel = (DefaultTreeModel) tree1.getModel();
		int index = treeModel.getIndexOfChild(parentNode, node);
		boolean expanded = tree1.isExpanded(new TreePath(node.getPath()));

		if (parentNode == null || nextNode == null || index == 0
				|| index >= parentNode.getChildCount())
			return;

		treeModel.removeNodeFromParent(node);
		treeModel.insertNodeInto((MutableTreeNode) node,
				(MutableTreeNode) parentNode, index + 1);

		TreePath path = new TreePath(node.getPath());
		tree1.setSelectionPath(path);

		if (expanded)
			tree1.expandPath(new TreePath(node.getPath()));
	}

	private void resizeView() {
		int height = tree1.getPreferredSize().height;
		try {
			;
			// view.getSegmentView2().updatePanelHeights(height);
		} catch (Exception ex) {
			;
		}
	}

	@Override
	public void keyTyped(KeyEvent e) {
	}

	@Override
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == 8 || e.getKeyCode() == 127)
			deleteNode = true;
	}

	@Override
	public void keyReleased(KeyEvent e) {
		deleteNode = false;
	}

	/*
	 * deletes node if 'DELETE' button is pressed and mouse clicked
	 */
	@Override
	public void valueChanged(TreeSelectionEvent e) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree1
				.getLastSelectedPathComponent();

		if (node == null)
			return;

		Object obj = node.getUserObject();
		LabelObject lObj = (LabelObject) obj;
		// FieldWrapper wrapper = lObj.getWrapper();

		DbNavigationPanel dbPanel = segment.getSignStreamSegmentPanel()
				.getDbNavigationPanel();
		Excerpt pExcerpt = segment.getSignStreamSegmentPanel().getExcerpt();
		TreeModel model1 = dbPanel.getJTree().getModel();
		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) model1
				.getRoot();

		if (rootNode.getChildCount() == 0)
			return;

		DefaultMutableTreeNode firstChildNode = (DefaultMutableTreeNode) rootNode
				.getFirstChild();

		DefaultMutableTreeNode nodeToAdd = null;

		Enumeration children = firstChildNode.children();
		while (children.hasMoreElements()) {
			DefaultMutableTreeNode childElement = (DefaultMutableTreeNode) children
					.nextElement();

			Excerpt excerpt = (Excerpt) childElement.getUserObject();
			// String label = excerpt.getExcerpt();
			if (excerpt.equals(pExcerpt)) {
				nodeToAdd = childElement;
			}
		}

		// if (nodeToAdd != null && wrapper != null && wrapper instanceof
		// PresentationFieldWrapper) {
		// PresentationField pField = (PresentationField) wrapper.getField();
		// ArrayList<SS3GlossWindowItem> statements =
		// pField.getSS3GlossWindowItems();
		// for (SS3GlossWindowItem statement : statements) {
		// DefaultMutableTreeNode sNode = new DefaultMutableTreeNode(
		// statement);
		// nodeToAdd.add(sNode);
		// }
		// dbPanel.getJTree().expandPath(new TreePath(nodeToAdd));
		// } else if (wrapper != null && wrapper instanceof SegmentFieldWrapper)
		// {
		// ;
		// }

		if (deleteNode) {
			DefaultTreeModel model = (DefaultTreeModel) tree1.getModel();
			model.removeNodeFromParent(node);

			int[] childIndices = new int[node.getChildCount()];
			Object[] removedChildren = new Object[node.getChildCount()];

			int i = 0;
			Enumeration enum8 = node.children();
			while (enum8.hasMoreElements()) {
				TreeNode treeNode = (TreeNode) enum8.nextElement();
				childIndices[i] = node.getIndex(treeNode);
				removedChildren[i] = treeNode;
				i++;
			}

			model.nodesWereRemoved(node, childIndices, removedChildren);
			model.nodeChanged(node);

			resizeView();
			view.repaint();
		} 
//		else {
//			if (corner == null)
//				return;
//
//			int level = node.getLevel();
//			if (level == 1) {
//				corner.setMode(PresentationUpperLeftCorner1.TEMPORAL_PARTITION_MODE);
//			} else if (level == 2) {
//				corner.setMode(PresentationUpperLeftCorner1.SEGMENT_MODE);
//			} else {
//				corner.setMode(PresentationUpperLeftCorner1.TIME_PERIOD_MODE);
//			}
//		}
	}

	private void loadDependentFields(JTree.DynamicUtilTreeNode node) {
		Enumeration enum7 = node.children();
		while (enum7.hasMoreElements()) {
			Object obj = enum7.nextElement();
			JTree.DynamicUtilTreeNode tNode = (JTree.DynamicUtilTreeNode) obj;
			Object userObj = tNode.getUserObject();

			TreeNode[] path = tNode.getPath();
			TreePath treePath = new TreePath(path);
			if (tree1.isVisible(treePath)) {
				// add to the collection
				if (!tNode.isLeaf()) {
					TreeNodeVector vector = (TreeNodeVector) tNode
							.getUserObject();
					LabelObject cObj = vector.getObject();
					segment.getFieldWrappersCollection().addFieldWrapper(
							cObj.getWrapper());
					loadDependentFields(tNode);
				} else {
					LabelObject cObj = (LabelObject) tNode.getUserObject();
					segment.getFieldWrappersCollection().addFieldWrapper(
							cObj.getWrapper());
				}
			}
		}
	}

	private void rearangeFieldWrapersNodeCollapsed(
			JTree.DynamicUtilTreeNode node, ArrayList list) {
		Enumeration enum5 = node.children();
		while (enum5.hasMoreElements()) {
			Object obj = enum5.nextElement();
			JTree.DynamicUtilTreeNode tNode = (JTree.DynamicUtilTreeNode) obj;
			Object userObj = tNode.getUserObject();

			FieldWrapper wrapper = null;

			if (userObj instanceof TreeNodeVector) {
				TreeNodeVector vector = (TreeNodeVector) userObj;
				LabelObject lb1 = vector.getObject();

				wrapper = segment.getFieldWrappersCollection()
						.getFieldWrapperWithFieldID(lb1.getFieldID());

				if (wrapper == null)
					return;

				wrapper.setVisible(false);

				rearangeFieldWrapersNodeCollapsed(tNode, list);

			} else {
				LabelObject lb2 = (LabelObject) userObj;
				wrapper = segment.getFieldWrappersCollection()
						.getFieldWrapperWithFieldID(lb2.getFieldID());
				wrapper.setVisible(false);
			}
			list.add(wrapper);
		}
	}

	public void setPreferredHeight(int ph) {
		int size = getPreferredSize().width;
		setPreferredSize(new Dimension(size, ph));
	}

	public void setPreferredWidth(int pw) {
		int size = getPreferredSize().height;
		setPreferredSize(new Dimension(pw, size));
	}
}

