package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.Point;

import javax.swing.BoundedRangeModel;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.time.Rounding;
import edu.bu.signstream.time.TimeUnit;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.observer.SSObservable;
import edu.bu.signstream.observer.SSObserver;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldDialog;

public class NonmanualFieldsView extends JPanel implements SSObserver, SSBasicPanel {

	private static final long serialVersionUID = 1L;

	protected LinenView linenView;
	protected NonManualFieldsLabelView labelView;
	protected JScrollPane pictureScrollPane;
	protected JSplitPane sp;
	protected JSplitPane spFinal;
	private Segment segment;
	private SignStreamSegmentPanel panel;
	private SegmentGraphicRepresentation rep = null;
	private Corner upperLeft = null;
	private FieldWrappersCollection fwc = new FieldWrappersCollection();
	private final String panelID;

	public NonmanualFieldsView(Segment segment, SignStreamSegmentPanel panel, SegmentControlPanel controlPanel,
			GlossNRelatedView glossNRelatedView, CustomFieldsView customView, GestureNRelatedView gestureView, 
			PresentationField parentField, String panelID) {

		this.segment = segment;
		this.panel = panel;
		this.panelID = panelID;
		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

		CSFieldDialog csFieldDialog = new CSFieldDialog(panel, segment, this, glossNRelatedView, customView, gestureView, parentField);

		labelView = new NonManualFieldsLabelView(segment, panel, controlPanel, 
				glossNRelatedView, customView, gestureView, this, csFieldDialog, fwc);
		rep = labelView.getSegmentGraphicRepresentation();
//		upperLeft = new UpperLeftCorner(panel);

        double mDuration = SegmentGraphReprUtil.utteranceInterval().duration().as(TimeUnit.TIMESTAMP, Rounding.NEAREST);
		int length = panel.getZoomer().getCoordinateTimeConvertor().getScreenWidth(mDuration);

		length = Math.max(length, rep.getPreferredDimension().width);

		rep.setPreferredSize(new Dimension(length, (int) rep.getPreferredDimension().getHeight()));

		if (rep != null) {
			labelView.setPreferredSize(new Dimension(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH,	rep.getPreferredDimension().height));
			labelView.setMinimumSize(new Dimension(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH, 0));
		} else {
			labelView.setPreferredHeight(480);
		}

		pictureScrollPane = new JScrollPane(rep, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,	ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.getHorizontalScrollBar().setPreferredSize(new Dimension(0, 0));

		labelView.getJScrollPane().getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		labelView.setVerticalScrollModel(getVerticalScrollModel());
		pictureScrollPane.getVerticalScrollBar().addAdjustmentListener(e -> {
			labelView.setVerticalScrollValue(e.getValue());
		});
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER, upperLeft);
		pictureScrollPane.setCorner(ScrollPaneConstants.LOWER_LEFT_CORNER, new Corner());
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER, new UpperRightCorner());

        if (SegmentGraphReprUtil.utteranceStart != null) {
            var x = SegmentGraphReprUtil.utteranceStart.absolute().as(TimeUnit.COORDINATE, Rounding.NEAREST);
            scrollRectToVisible(new Point((int) x, 10));
        }

		linenView = new LinenView(LinenView.HORIZONTAL, true, rep.getPreferredDimension().width, panel);
		sp = new JSplitPane();
		sp.setLeftComponent(labelView);
		sp.setRightComponent(pictureScrollPane);

		add(sp);
	}

	public String getPanelID() {
		return this.panelID;
	}

	public void update(Segment segment) {
		this.segment = segment;
		update();

        double mDuration = SegmentGraphReprUtil.utteranceInterval().duration().as(TimeUnit.TIMESTAMP, Rounding.NEAREST);

		int length = panel.getZoomer().getCoordinateTimeConvertor().getScreenWidth(mDuration);

		length = length > rep.getPreferredDimension().width ? length : rep.getPreferredDimension().width;

		rep.setPreferredSize(new Dimension(length, (int) rep.getPreferredDimension().getHeight()));

		linenView.update(length);

		if (rep != null) {
			linenView.setPreferredWidth(length);
			labelView.setPreferredSize(new Dimension(125, rep.getPreferredDimension().height));
		} else {
			linenView.setPreferredWidth(320);
			labelView.setPreferredHeight(480);
		}

	}

	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}

	public JSplitPane getSplitPane() {
		return sp;
	}

	public Segment getSegment() {
		return segment;
	}

	public void setScrollPaneHeight(int height) {
		rep.setPreferredSize(new Dimension(rep.getPreferredDimension().width, height));
		labelView.setPreferredHeight(height);

		pictureScrollPane.getViewport().setView(rep);
	}

	public void scrollRectToVisible(Point point) {

//		if (point == null)
//			return;

//		JViewport vp = pictureScrollPane.getViewport();
//		if (vp != null) {
//			vp.setViewPosition(point);
			pictureScrollPane.invalidate();
			pictureScrollPane.revalidate();
//		}
	}

	public FieldWrappersCollection getFieldWrappersCollection() {
		return fwc;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return panel;
	}

	public Corner getUpperLeftCorner() {
		return upperLeft;
	}

	public NonManualFieldsLabelView getLabelView() {
		return labelView;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRep() {
		return rep;
	}

	public CSFieldDialog getCSFieldDialog() {
		return getLabelView().getCSFieldDialog();
	}

	public void update() {
		labelView.update(segment);

		int height = labelView.getLabels().size() * SegmentGraphReprUtil.axesTopHeight * 2;

		setScrollPaneHeight(height);
	}

	public LinenView getLinenView() {
		return linenView;
	}

	@Override
	public void update(SSObservable subj, Object o) {
	}

	private BoundedRangeModel getVerticalScrollModel() {
		return getJScrollPane().getVerticalScrollBar().getModel();
	}

	public void setVerticalScrollValue(int value) {
		pictureScrollPane.getVerticalScrollBar().setValue(value);
	}

	/**
	 * @return
	 */
	@Override
	public String getID() {
		return this.getPanelID();
	}
}
