package edu.bu.signstream.grepresentation.view;

import edu.bu.signstream.common.util.CustomPanelizedSplitPane;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.ViewMode;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;
import java.util.logging.Level;

public class SegmentTemplate extends JPanel {
	private static final long serialVersionUID = 1L;

	private SignStreamSegmentPanel segmentPanel = null;
	private Segment segment = null;
	private SegmentControlPanel controlPanel = null;

	private NonmanualFieldsView nonmanView = null;
	private CustomFieldsView customView = null;
	private GlossNRelatedView signView = null;
	private GestureNRelatedView speechView = null;

	private NonManualFieldsButtonPanel fieldsButtonPanel = null;

	private HashMap<String, SSBasicPanel> panels;

	private ViewMode viewMode;
	private ViewDataManager viewDataManager;

	private final JPanel leftHandPane;
	private final CustomPanelizedSplitPane customPanelizedSplitPane;

	private GridBagConstraints c = new GridBagConstraints();
	private boolean isHidden = false;
	private boolean selected = false;
	private JSplitPane sp;

	public SegmentTemplate(Segment segment, UtteranceView view, SignStreamSegmentPanel segmentPanel, PresentationField parentField, ViewMode viewMode, ViewDataManager viewDataManager) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;
		this.viewMode = viewMode;
		this.viewDataManager = viewDataManager;

		controlPanel = new SegmentControlPanel(view, segmentPanel, segment, this);
		customView = new CustomFieldsView(segment, segmentPanel, controlPanel, parentField, Constants.DEFAULT_PANEL_ID); // Is this default panel ID or...?
		signView = new GlossNRelatedView(segment, segmentPanel, view, parentField, Constants.SIGN_PANEL_ID);
		speechView = new GestureNRelatedView(segment, segmentPanel,  view, parentField, Constants.GESTURE_PANEL_ID);
		nonmanView = new NonmanualFieldsView(segment, segmentPanel, controlPanel, signView, customView, speechView, parentField, Constants.NONMANUAL_PANEL_ID);

		this.panels = new HashMap<>();

		this.panels.put(this.customView.getPanelID(), this.customView);
		this.panels.put(this.signView.getPanelID(), this.signView);
		this.panels.put(this.speechView.getPanelID(), this.speechView);
		this.panels.put(this.nonmanView.getPanelID(), this.nonmanView);


		controlPanel.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH,
				this.getHeight()));
		controlPanel.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH,
				this.getHeight()));

		this.customPanelizedSplitPane = this.initializeCustomPanelizedSplitPane();
		this.customPanelizedSplitPane.setPreferredSize(new Dimension(
				(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH +
						SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH),
				(this.getHeight()/2)));

		this.customPanelizedSplitPane.addResizeListener(event -> {
			ArrayList<Double> panelRatios = SegmentTemplate.this.customPanelizedSplitPane.getPanelRatios();
			ArrayList<JPanel> panels = SegmentTemplate.this.customPanelizedSplitPane.getPanels();
			ViewDataManager viewDataManager1 = SegmentTemplate.this.viewDataManager;

			if (panels.size() != panelRatios.size()) {
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Panel size list and panel ratio size do not match");
				throw new RuntimeException("Panel size list and panel ratio size do not match");
			}

			for (int x = 0; x < panelRatios.size(); x++) {
				if (panels.get(x) instanceof SSBasicPanel) {
					viewDataManager1.getViewData(((SSBasicPanel) panels.get(x)).getID()).setPanelPercentage(panelRatios.get(x) * 100.0);
				}
			}
		});


		controlPanel.setBackground(Palette.BACKGROUND_COLOR);
		controlPanel.getTop().setBackground(Palette.BACKGROUND_COLOR);

		setBackground(Palette.BACKGROUND_COLOR);
		setBorder(BorderFactory.createLineBorder(Color.BLACK));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		fieldsButtonPanel = new NonManualFieldsButtonPanel(segmentPanel, nonmanView.getLabelView(), signView, speechView, customView.getLabelView());

//		fieldsButtonPanel.setPreferredSize(new Dimension(
//				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH,
//				this.getHeight()));
//		fieldsButtonPanel.setMinimumSize(new Dimension(
//				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH,
//				this.getHeight()));
		
		sp = new JSplitPane();
		
		leftHandPane = new JPanel();

//		this.leftHandPane.setLayout(new BoxLayout(this.leftHandPane, BoxLayout.PAGE_AXIS));
		this.leftHandPane.setLayout(new GridBagLayout());
		
		c.gridy = 0;
		c.weightx = 1.0;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		this.leftHandPane.add(fieldsButtonPanel, c);

		c.weighty = 1.0;
		c.fill = GridBagConstraints.BOTH;
		c.gridy++;		
		this.leftHandPane.add(controlPanel, c);
		
		this.leftHandPane.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH,
				this.getHeight()));
		this.leftHandPane.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH,
				this.getHeight()));

		sp.setLeftComponent(this.leftHandPane);
		sp.setRightComponent(this.customPanelizedSplitPane);

		add(sp);


		this.getSegment().getParentField().getSS3GlossWindowSegment().setViewDataManager(this.viewDataManager);
	}

	/**
	 * @return CustomPanelizedSplitPane with all the panels from the viewDataManager loaded previously
	 */
	private CustomPanelizedSplitPane initializeCustomPanelizedSplitPane() {
		ArrayList<Double> panelRatios = new ArrayList<>();
		ArrayList<JPanel> panels = new ArrayList<>();

		for (ViewData viewData : this.viewDataManager.getViewDataList()) {
			panelRatios.add(viewData.getPanelPercentage());
			JPanel panelToAdd = (JPanel) this.panels.get(viewData.getPanelID());

			panelToAdd.setPreferredSize(new Dimension(
					(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH +
							SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH),
					(this.getHeight()/2))); // Default panel size


			panels.add(panelToAdd);
		}


		return new CustomPanelizedSplitPane(panels, panelRatios);
	}


	/**
	 * Revalidate the outer and inner panels
	 */
	public void resizePanel() {
		super.revalidate();
		super.repaint();
		this.setSize(this.getPreferredSize());
		this.customPanelizedSplitPane.setSize(this.customPanelizedSplitPane.getPreferredSize()); // This means the segment was just selected and loaded and we need to set the size of the split pane
		this.customPanelizedSplitPane.resizeComponents();
	}


	private int viewCount() {
		return this.viewDataManager.getViewDataList().size();
	}

	/**
	 * This method refreshes the ViewDataManager after data to the viewDataManager might have been updated like a new panel being added and we need to refresh the panel based on the viewDataManager
	 * @param viewDataManager The viewDataManager to set
	 */
	public void setViewDataManager(ViewDataManager viewDataManager) {
		this.viewDataManager = viewDataManager;
		ArrayList<JPanel> panels = new ArrayList<>();
		ArrayList<Double> panelRatios = new ArrayList<>();

		for (ViewData viewData : this.viewDataManager.getViewDataList()) {
			panels.add((JPanel) this.panels.get(viewData.getPanelID()));
			panelRatios.add(viewData.getPanelPercentage());
		}

		this.customPanelizedSplitPane.setPanels(panels, panelRatios); // Rebuilds the split pane with the new panels and ratios
		this.getSegment().getParentField().getSS3GlossWindowSegment().setViewDataManager(this.viewDataManager);
	}


	/**
	 * Is this segment template pane currently hidden? Returns boolean
	 * Called by UtteranceView.renderSegmentTemplatePanes()
	 */
	public boolean isHidden() {
		return isHidden;
	}

	public void selectTemplate() {
		segmentPanel.getUtteranceView().unselectAllSegmentTemplates();
		this.selected = true;
		segmentPanel.getUtteranceView().renderSegmentTemplatePanes();
		this.resizePanel();
	}

	/**
	 * Is this segment the currently mouse-selected item?
	 * Called by UtteranceView.renderSegmentTemplatePanes()
	 * @return selected - boolean value activated by a mouse click on the panel, or default from UtteranceView
	 */
	public boolean isSelected() {
		return selected;
	}

	public void unselect() {
		this.selected = false;
	}

	public void update(Segment segment) {
		controlPanel.update(segment);
		nonmanView.update(segment);
		signView.update(segment);
		speechView.update(segment);
		customView.update(segment);
		SS3Singleton.getMediaToolBar().deselectLabel();
	}

	public Segment getSegment() {
		return segment;
	}

	public String getPartitionLabel() {
		return getSegment().getParentField().getTemporalPartition().getLabel();
	}

	public String getTierLabel() {
		return getSegment().getParentField().getSS3GlossWindowSegment().getLabel();
	}

	public SegmentControlPanel getSegmentControlPanel() {
		return controlPanel;
	}

	public NonmanualFieldsView getNonmanView() {
		return nonmanView;
	}

	public CustomFieldsView getCustomView() {
		return customView;
	}

	public GlossNRelatedView getSignView() {
		return signView;
	}

	public GestureNRelatedView getSpeechView() {
		return speechView;
	}

	public ViewDataManager getViewDataManager() {
		return this.viewDataManager;
	}

	public JSplitPane getSplitPane() {
		return sp;
	}

	public EventsEntity getSelectedUtterance() {
		return getSegmentControlPanel().getSelectedUtterance();
	}

	public int getSelectedUtteranceIndex() {
		return getSegmentControlPanel().getSelectedUtteranceIndex();
	}

	public ViewMode getViewMode() {
		return this.viewMode;
	}

	public void setViewMode(ViewMode viewMode) {
		this.viewMode = viewMode;
	}

	public CustomPanelizedSplitPane getCustomPanelizedSplitPane() {
		return this.customPanelizedSplitPane;
	}

	public NonManualFieldsButtonPanel getButtonPanel() { return fieldsButtonPanel; }
}