package edu.bu.signstream.grepresentation.view;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.panels.SegmentSelectionDialog;

public class UtteranceViewFooter extends JPanel {

	private static final long serialVersionUID = 1L;

	private SignStreamSegmentPanel segmentPanel;
	private UtteranceView utteranceView;
	
	private JPanel buttonPanel;
	
	private JButton addPaneButton;
	private JButton removePaneButton;
	private JButton movePaneUpButton;
	private JButton movePaneDownButton;
	
	private ScrollBarView barView;
	
	private JSplitPane sp;
	
	public UtteranceViewFooter(SignStreamSegmentPanel segmentPanel,
			UtteranceView utteranceView) {
		this.segmentPanel = segmentPanel;
		this.utteranceView = utteranceView;

		buttonPanel = new JPanel();
		buttonPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
		buttonPanel.setBackground(Palette.BACKGROUND_COLOR);
		buttonPanel.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		buttonPanel.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		buttonPanel.setMaximumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
				
		String addPaneButtonText = "Add";
		String removePaneButtonText = "Remove";
		String movePaneUpButtonText = "Up";
		String movePaneDownButtonText = "Dn";
		
		addPaneButton = new JButton(addPaneButtonText);
		addPaneButton.addActionListener(new NewPaneListener(utteranceView));
		addPaneButton.setToolTipText("Add Segment Tier");
	    UserInterfaceUtil.setButtonsLNF(addPaneButton);
		removePaneButton = new JButton(removePaneButtonText);
		removePaneButton.addActionListener(new RemovePaneListener(utteranceView));
		removePaneButton.setToolTipText("Remove Segment Tier");
	    UserInterfaceUtil.setButtonsLNF(removePaneButton);
		movePaneUpButton = new JButton(movePaneUpButtonText);
		movePaneUpButton.addActionListener(new MovePaneUpListener(utteranceView));
		movePaneUpButton.setToolTipText("Shift Selected Segment Tier Up");
	    UserInterfaceUtil.setButtonsLNF(movePaneUpButton);
		movePaneDownButton = new JButton(movePaneDownButtonText);
		movePaneDownButton.addActionListener(new MovePaneDownListener(utteranceView));
        movePaneDownButton.setToolTipText("Shift Selected Segment Tier Down");
	    UserInterfaceUtil.setButtonsLNF(movePaneDownButton);

		barView = new ScrollBarView(segmentPanel);
		barView.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		barView.setMaximumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH, 
				SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));
		
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
		buttonPanel.add(addPaneButton);
		buttonPanel.add(removePaneButton);
		buttonPanel.add(movePaneUpButton);
		buttonPanel.add(movePaneDownButton);
		
		buttonPanel.setAlignmentY(Component.TOP_ALIGNMENT);
		barView.setAlignmentY(Component.TOP_ALIGNMENT);
		
		sp = new JSplitPane();
		
		sp.setLeftComponent(buttonPanel);
		sp.setRightComponent(barView);
		
		add(sp);
	}
	
	public ScrollBarView getScrollBarView() {
		return barView;
	}
	
	public JSplitPane getSplitPane() {
		return sp;
	}


	static class NewPaneListener implements ActionListener {
		private UtteranceView utteranceView;

		public NewPaneListener(UtteranceView utteranceView) {
			this.utteranceView = utteranceView;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			SegmentSelectionDialog segmentSelectionDialog = new SegmentSelectionDialog(utteranceView);
			segmentSelectionDialog.setLocation(250, 250);
			segmentSelectionDialog.setVisible(true);

			ArrayList<SegmentTemplate> segmentTemplates = this.utteranceView.getSegmentTemplates();

			for (SegmentTemplate segmentTemplate : segmentTemplates) {
				segmentTemplate.resizePanel();
			}
		}
	}

	static class RemovePaneListener implements ActionListener {
		private UtteranceView utteranceView;

		public RemovePaneListener(UtteranceView utteranceView) {
			this.utteranceView = utteranceView;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			this.utteranceView.removeSelectedPane();

			ArrayList<SegmentTemplate> segmentTemplates = this.utteranceView.getSegmentTemplates();

			for (SegmentTemplate segmentTemplate : segmentTemplates) {
				segmentTemplate.resizePanel();
			}
		}
	}

	static class MovePaneUpListener implements ActionListener {
		private UtteranceView utteranceView;

		public MovePaneUpListener(UtteranceView utteranceView) {
			this.utteranceView = utteranceView;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			utteranceView.moveSelectedPaneUp();
		}

	}

	static class MovePaneDownListener implements ActionListener {
		private UtteranceView utteranceView;

		public MovePaneDownListener(UtteranceView utteranceView) {
			this.utteranceView = utteranceView;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			utteranceView.moveSelectedPaneDown();
		}

	}
}
