package edu.bu.signstream.media.data;

import org.jtransforms.fft.DoubleFFT_1D;

public class FFT extends Samples {
    FFT(Samples samples, Window window, int i, int j, boolean forward) {
        super(new double[j - i], samples.rate);

        // Create an array of twice the FFT size, filling the lower half
        // with the windowed samples.
        //
        // Leave zeros in the upper half since the FFT produces both a
        // real and imaginary part for each sample, which takes twice
        // the space.
        var real = new double[length * 2];
        var sum = 0.0;
        for (int s = 0; s < length; s++) {
            var c = window.coeff(s, length);
            sum += c;
            real[s] = c * samples.get(i + s);
        }

        var fft = new DoubleFFT_1D(length);
        if (forward)
            fft.realForwardFull(real);
        else
            fft.realInverseFull(real, true);

        // Store the norm of the complex frequency values, dividing out the window sum.
        for (int s = 0; s < length; s++)
            set(s, norm(real[s * 2], real[s * 2 + 1]) / sum);
    }
    FFT(Samples samples, Window window, boolean forward) { this(samples, window, 0, samples.length, forward); }

    private double norm(double re, double im) {
        return Math.sqrt((re * re) + (im * im));
    }

    public int bin(double freq) {
        return (int)(freq / (rate / length));
    }
    public double freq(int bin) {
        return bin * ((double)rate / length);
    }
}
