package edu.bu.signstream.media.ui.painter;

import edu.bu.signstream.common.SSMath;
import edu.bu.signstream.time.Duration;
import edu.bu.signstream.time.TimeRange;
import edu.bu.signstream.media.data.Samples;
import edu.bu.signstream.media.ui.color.ColorMap;

import java.awt.*;

public class WaveformPainter extends SamplesPainter {
    public WaveformPainter(Samples samples, ColorMap colors, TimeRange range, Duration page) {
        super(samples, colors, range, page);
    }

    @Override
    protected void paint(Graphics g, int w, int h, int s0, int s1) {
        // Draw the background
        g.setColor(colors.map(0));
        g.drawRect(0, 0, w, h);

        // Get the max absolute amplitude over the entire buffer
        var ceil = samples.ceil();

        g.setColor(colors.map(1));
        for (int i = 0; i < w; i++) {
            // Get the sample range for this column
            int j0 = (int)SSMath.project(s0, s1, 0, w, i);
            int j1 = (int)SSMath.project(s0, s1, 0, w, i + 1);

            if (j0 >= samples.length || j1 >= samples.length)
                continue;

            // Get the min/max amplitudes for this column normalized to the ceiling
            var max = samples.max(j0, j1) / ceil;
            var min = samples.min(j0, j1) / -ceil;

            // Draw a line from the min to the max normalized
            var h2 = h / 2.0;
            var y0 = (int)SSMath.lerp(h2, 0, max);
            var y1 = (int)SSMath.lerp(h2, h, min);
            g.drawLine(i, y0, i, y1);
        }
    }
}
