package edu.bu.signstream.navigation;

import java.awt.*;
import java.util.logging.Level;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import edu.bu.signstream.common.util.*;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class DbNavigationPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTree tree = new JTree();
	private DatabaseDelegate dbDelegate = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public DbNavigationPanel(SignStreamSegmentPanel segmentPanel,
			DatabaseDelegate dbDelegate) {
		this.segmentPanel = segmentPanel;

		if (dbDelegate == null) {
			this.dbDelegate = new DatabaseDelegate(segmentPanel);
		} else {
			this.dbDelegate = dbDelegate;
		}

		try {
			DefaultMutableTreeNode treeNode = dbDelegate.getMutableTreeNode();
			this.tree = new JTree(treeNode);

			initTreeUI();
			setTree();
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	private void initTreeUI() {
		Color treeColor = new Color(255, 247, 200);

		int fontSize = 12, fontStyle = Font.PLAIN;
		String fontFamily = "Times-Roman";
		Font font = new Font(fontFamily, fontStyle, fontSize);

		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setLeafIcon(null);
		renderer.setIcon(null);
		renderer.setClosedIcon(null);
		renderer.setOpenIcon(null);

		renderer.setBackground(treeColor);
		renderer.setBackgroundNonSelectionColor(treeColor);
		renderer.setBackgroundSelectionColor(treeColor);
		renderer.setBorderSelectionColor(treeColor);
		renderer.setTextSelectionColor(Color.RED);
		renderer.setFont(font);
		
		tree.getInputMap().put(KeyStroke.getKeyStroke("LEFT"), "none");
		tree.getInputMap().put(KeyStroke.getKeyStroke("RIGHT"), "none");

		tree.addKeyListener(SS3Singleton.getVideoControlKeyListener());		
		tree.setCellRenderer(renderer);
		tree.setBackground(treeColor);
		tree.setForeground(treeColor);
		tree.putClientProperty("JTree.lineStyle", "Horizontal");

		UIManager.put("Tree.line", Palette.AXIS_COLOR);
	}

	public DbNavigationPanel(TreeModel model) {
		this.tree = new JTree(model);
		this.dbDelegate = new DatabaseDelegate(segmentPanel);
		initTreeUI();

		try {
			setTree();
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
		}
	}

	public void setSignStreamSegmentPanel(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	public DatabaseDelegate getDatabaseDelegate() {
		return dbDelegate;
	}

	public JTree getJTree() {
		return tree;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	public void refresh(TreeNode treeNode) {
		tree = new JTree(treeNode);
		initTreeUI();
		setTree();
		updateUI();
	}

	public void setTree() {
		removeAll();

		JScrollPane hashPane = new JScrollPane(tree);
		hashPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		tree.addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {
			}
		});

		tree.expandPath(new TreePath(tree.getModel().getRoot()));

		var toUtterance = new JButton("Go to Utterance View");
        UserInterfaceUtil.setButtonsLNF(toUtterance);
        toUtterance.addActionListener(l -> enterUtteranceView());

        var toUtterancePanel = new JPanel();
        toUtterancePanel.setBackground(UIPalette.getBackground());
        toUtterancePanel.add(toUtterance);
        toUtterancePanel.setMaximumSize(new Dimension(
            9999, // full width
            toUtterancePanel.getPreferredSize().height));

		var vsplit = new JPanel();
		vsplit.setLayout(new BoxLayout(vsplit, BoxLayout.Y_AXIS));
		vsplit.add(toUtterancePanel);
		vsplit.add(hashPane);

        setLayout(new BorderLayout());
		add(vsplit, BorderLayout.CENTER);
	}

	private void enterUtteranceView() {
        var node = segmentPanel.getDatabaseView().getLabelView().getSelectedNode();
        var level = node.getLevel();

        PresentationField field;

/*		SegmentFieldWrapper fieldWrapper = (SegmentFieldWrapper) segmentPanel.getDatabaseView().getSegment().getFieldWrapper(Constants.SegmentFieldWrapper);
		segmentPanel.setSelectedFieldWrapper(fieldWrapper);*/
		/*
		The above code is commented because there was this issue where the SegmentFieldWrapper was null when setting the start time of the Gesture.
		The code above was originally made to fix that, and it did. But where that NullPointerException was occurring appeared to not be used, we we commented it
		it out along with this code that was originally made to fix that null pointer exception. If there is still a NullPointer in start time or end time for gestures,
		you can re-enable this code and the type setting in SegmentFieldWrapper, DisplayFieldWrapper, and PresentationFieldWrapper.
		 */

        if (node.getPreviousSibling() == null && level == 1) {
            // Display Time Period is selected, ignore.
            SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
            return;

        } else if (level == 1) {
            // Temporal Partition is selected, find the default segment tier
            var def = Util.stream(node.children())
                .map(c -> (LabelObject)((DefaultMutableTreeNode)c).getUserObject())
                .filter(LabelObject::isDefaultSegmentTier)
                .findFirst()
                .orElseThrow();

            field = (PresentationField)def.getField();

        } else if (level == 2) {
            // Segment Tier is selected, use it
            var label = (LabelObject)node.getUserObject();
            field = (PresentationField)label.getField();

        } else {
            /// ??? is selected
            return;
        }

        segmentPanel.loadUtteranceView(field, null);
    }
}

class Perspective {
	// Construct the panel with the toggle buttons.

	public Perspective() {
		JRadioButton showDescendant = new JRadioButton("segmentsnts", true);
		JRadioButton showAncestor = new JRadioButton("utterances");

		ButtonGroup bGroup = new ButtonGroup();
		bGroup.add(showDescendant);
		bGroup.add(showAncestor);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(showDescendant);
		buttonPanel.add(showAncestor);
	}
}
