package edu.bu.signstream.time;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.requestsHandler.SS3Singleton;

public class TimeUnit {
    public enum Type {
        SECOND,
        FRAME,
        @Deprecated
        TS,
        @Deprecated
        COORD,
    }

    private final String id;
    private final String name;
    private final String suffix;
    private final Type type;
	private double base;

	public TimeUnit(String id, String name, String suffix, Type type, double base) {
	    this.id = id;
	    this.name = name;
	    this.suffix = suffix;
	    this.type = type;
		this.base = base;
	}
    public TimeUnit(String name, String suffix, Type type, double base) {
	    this(Util.uuid(), name, suffix, type, base);
    }

    public String id() { return this.id; }
	public String name() { return this.name; }
	public String suffix() { return this.suffix; }
	public Type type() { return this.type; }
	public double base() { return this.base; }

    public void setBase(double base) {
        this.base = base;
    }

	public double factor() {
	    if (this.type == Type.SECOND)
	        return Math.pow(10, this.base);

	    if (this.type == Type.FRAME)
	        return this.base;

	    // JACK TODO: Remove this
	    if (this.type == Type.TS) {
            // Special case for calculating the deprecated packetDuration multiple timestamps.
            var converter = SS3Singleton.getVideoControlManager()
                .getZoomer()
                .getCoordinateTimeConvertor();

            return converter.timebase;
        }

	    // JACK TODO: Remove this
	    if (this.type == Type.COORD) {
            // Special case for calculating SegmentPanel coordinates
            var converter = SS3Singleton.getVideoControlManager()
                .getZoomer()
                .getCoordinateTimeConvertor();

            return (converter.timebase / converter.timestep) * converter.frameWidth();
        }

	    throw new IllegalStateException("Invalid TimeUnit Type!");
	}

    /**
     * Converts a value in this `TimeUnit` to the target `TimeUnit`.
     */
    public double convert(double value, TimeUnit unit) {
        double seconds = value * unit.factor();
        return seconds / this.factor();
    }

    /**
     * Converts a value in this `TimeUnit` to an integer factor of the target `TimeUnit`.
     * rounding with the provided strategy.
     */
	public long convert(double value, TimeUnit unit, Rounding rounding) {
	    return rounding.round(this.convert(value, unit));
    }

    public String label() {
        if (this.type == Type.FRAME) {
            var str = this.base == (int) this.base
                ? String.valueOf((int) this.base)
                : String.format("%,.2f", this.base);
            return str + " FPS";
        } else {
            return this.name();
        }
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof TimeUnit && this.id.equals(((TimeUnit)obj).id);
    }

    public static TimeUnit NANOSECOND = new TimeUnit("9", "Nanoseconds", "NS", Type.SECOND, 9);
    public static TimeUnit MICROSECOND = new TimeUnit("6", "Microseconds", "US", Type.SECOND, 6);
    public static TimeUnit MILLISECOND = new TimeUnit("3", "Milliseconds", "MS", Type.SECOND, 3);
    public static TimeUnit SECOND = new TimeUnit("0", "Seconds", "S", Type.SECOND, 0);
    public static TimeUnit FRAME = new TimeUnit("Frame", "FRAMES", Type.FRAME, Double.NaN);

    @Deprecated
    public static TimeUnit TIMESTAMP = new TimeUnit("Timestamp", "TS", Type.TS, Double.NaN);
    @Deprecated
    public static TimeUnit COORDINATE = new TimeUnit("Pixels", "PX", Type.COORD, Double.NaN);
}
