package edu.bu.signstream.ui.frame;

import javax.swing.*;
import javax.swing.plaf.LayerUI;
import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

public class Blocker extends JPanel implements Blockable {
    public final JComponent view;

    private final JLayer<JComponent> layer;
    private final LayerUI<JComponent> filter;
    private final LayerUI<JComponent> empty;

    int n = 0;

    public Blocker(JComponent view, Filter filter) {
        super(new BorderLayout());

        this.view = view;

        this.layer = new JLayer<>(view);
        layer.setLayerEventMask(
            AWTEvent.KEY_EVENT_MASK
                | AWTEvent.MOUSE_EVENT_MASK
                | AWTEvent.MOUSE_MOTION_EVENT_MASK
                | AWTEvent.MOUSE_WHEEL_EVENT_MASK);

        this.filter = filter;
        this.empty = new LayerUI<>();

        add(layer);
    }


    @Override
    public void block() {
        n++;
        layer.setUI(filter);
    }

    @Override
    public void unblock() {
        n = Math.max(0, n - 1);
        if (n == 0)
            layer.setUI(empty);
    }

    @Override
    public boolean blocked() {
        return n > 0;
    }

    private static class Filter extends LayerUI<JComponent> {
        @Override
        public void eventDispatched(AWTEvent e, JLayer<? extends JComponent> l) {
            if (e instanceof InputEvent)
                ((InputEvent) e).consume();
        }
    }

    public static Filter Darken(float alpha) {
        return new Filter() {
            @Override
            public void paint(Graphics g, JComponent c) {
                super.paint(g, c);

                var g2 = (Graphics2D)g;
                g2.setComposite(AlphaComposite.SrcOver);
                g2.setColor(new Color(0, 0, 0, alpha));
                g2.fillRect(0, 0, c.getWidth(), c.getHeight());
            }
        };
    }
    public static Filter Darken() { return Darken(0.4f); }
}
