package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractCellEditor;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

public class ColorEditor extends AbstractCellEditor
                         implements TableCellEditor,
                     ActionListener {
	
	private static final long serialVersionUID = 1L;
	
	private Color currentColor;
    private JButton button;
    private JColorChooser colorChooser;
    private JDialog dialog;
    protected static final String EDIT = "edit";  

    public ColorEditor() {}

    /**
     * Handles events from the editor button and from
     * the dialog's OK button.
     */
    @Override
	public void actionPerformed(ActionEvent e) {
        if (EDIT.equals(e.getActionCommand())) {
            //The user has clicked the cell, so bring up the dialog.
            button.setBackground(currentColor);
            colorChooser.setColor(currentColor);
            dialog.setVisible(true);

            //Make the renderer reappear.
            fireEditingStopped();

        } else { //User pressed dialog's "OK" button.
            currentColor = colorChooser.getColor();           
            fireEditingStopped();
        }
    }

    //Implement the one CellEditor method that AbstractCellEditor doesn't.
    @Override
	public Object getCellEditorValue() {
        return currentColor;
    }
    
    //Implement the one method defined by TableCellEditor.
    @Override
	public Component getTableCellEditorComponent(JTable table,
                                                 Object value,
                                                 boolean isSelected,
                                                 int row,
                                                 int column) {   
        currentColor = (Color)value;
        button = new ColorizedButton();
        button.setActionCommand(EDIT);
        button.addActionListener(this);
        button.setBorderPainted(false);

        //Set up the dialog that the button brings up.
        colorChooser = new JColorChooser();
        dialog = JColorChooser.createDialog(button,
                                        "Pick a Color",
                                        true,  //modal
                                        colorChooser,
                                        this,  //OK button handler
                                        null); //no CANCEL button handler
        return button;
    }
}

class ColorizedButton extends JButton {
	private static final long serialVersionUID = 1L;

	public ColorizedButton() {
    	super("");
    }
 
    @Override
	public void paintComponent(final Graphics g) {
    	g.setColor(getBackground());
    	g.fillRect(0, 0, getWidth(), getHeight());
    }
}
