package edu.bu.signstream.ui.panels.dependentSortModel;

import java.util.*;
import java.text.Collator;

public class Sorter {
   private ArrayList sortColumns = new ArrayList();
   private ArrayList unsortedRows = new ArrayList();
   private ArrayList rows = new ArrayList();

   public Sorter(ArrayList rows) {
      this.rows = rows;
      
      for(int i = 0; i < rows.size(); i++) {
         unsortedRows.add(i, rows.get(i));
      }
   }

   public void addSortColumn(SortColumn column) {

      SortDirection dir = column.getSortDirection();
      if (dir.equals(SortDirection.ASCENDING)) {

         column.setSortDirection(SortDirection.DESCENDING);
      } else if (dir.equals(SortDirection.DESCENDING)) {

         column.setSortDirection(SortDirection.UNSORTED);
      } else if (dir.equals(SortDirection.UNSORTED)) {

         column.setSortDirection(SortDirection.ASCENDING);
      }

      if (sortColumns.contains(column)) {
         int index = sortColumns.indexOf(column);

         if (column.getSortDirection().equals(SortDirection.UNSORTED)) {
            ArrayList toRemove = new ArrayList();
            for (int j = index; j < sortColumns.size(); j++) {
               SortColumn c = (SortColumn) sortColumns.get(j);
               c.setSortDirection(SortDirection.UNSORTED);
               toRemove.add(c);
            }

            for (int i = 0; i < toRemove.size(); i++) {
               sortColumns.remove(toRemove.get(i));
            }
            sortColumns.trimToSize();
         }
      } else {
         sortColumns.add(column);
      }

      sortByGroups();
   }

   public ArrayList getRows() {
      return rows;
   }

   private void sortInInterval(
      int colNumber,
      int lineStart,
      int lineEnd,
      SortDirection direction) {

      if (direction.equals(SortDirection.ASCENDING)) {
         for (int j = lineStart; j < lineEnd - 1; j++) {
            for (int i = lineEnd - 2; i > lineStart - 1; i--) {
               sortTwoLines(colNumber, i, direction);
            }
         }
      }

      if (direction.equals(SortDirection.DESCENDING)) {
         for (int j = lineStart; j < lineEnd - 1; j++) {
            for (int i = lineStart; i < lineEnd - 1; i++) {
               sortTwoLines(colNumber, i, direction);
            }
         }
      }

      if (direction.equals(SortDirection.UNSORTED)) {
         ;
      }
   }

   private void sortTwoLines(
      int colNumber,
      int lineStart,
      SortDirection direction) {

      Row row1 = (Row) rows.get(lineStart);
      Row row2 = (Row) rows.get(lineStart + 1);

      String value1 = row1.getColumn(colNumber).toString();
      String value2 = row2.getColumn(colNumber).toString();

      Collator collator = Collator.getInstance();
      int compared = collator.compare(value1, value2);

      if ((direction.equals(SortDirection.ASCENDING) && compared > 0)
         || (direction.equals(SortDirection.DESCENDING) && compared < 0)) {
         Collections.swap(rows, lineStart, lineStart + 1);
      }
   }

   private int getGroupEndForSingleGroup(
      int groupLevel,
      int lineStart,
      int prevGroupLineEnd) {

      if (groupLevel == 1)
         return rows.size();

      SortColumn column = (SortColumn) sortColumns.get(groupLevel);
      int colNumber = column.getColumnNumber();

      Row r = (Row) rows.get(lineStart);
      String toCompare = r.getColumn(colNumber).toString();

      for (int i = lineStart; i < rows.size(); i++) {
         Row row = (Row) rows.get(i);
         String str = row.getColumn(colNumber).toString();

         if (!str.equals(toCompare) && i <= prevGroupLineEnd)
            return i;
      }

      return prevGroupLineEnd;
   }

   // group level starts from 1
   private int getGroupEnd(int groupLevel, int lineStart) {
      if (groupLevel == 1)
         return rows.size();

      int lineEnd = rows.size();
      for (int i = 0; i < groupLevel; i++) {
         lineEnd = getGroupEndForSingleGroup(i, lineStart, lineEnd);
         i++;
      }

      return lineEnd;
   }
 
   private void unsortRows() {     
      ArrayList toRemove = new ArrayList();
      toRemove.addAll(rows); 
      
      for (int i = 0; i < toRemove.size(); i++) {
         rows.remove(toRemove.get(i));
      }
      sortColumns.trimToSize();

      for (int j = 0; j < unsortedRows.size(); j++) {
         rows.add(j, unsortedRows.get(j));
      }
   }

   public void sortByGroups() {
      unsortRows();

      for (int i = 0; i < sortColumns.size(); i++) {
         int groupLevel = i + 1;
         int lineStart = 0;

         while (lineStart < rows.size()) {
            int lineEnd = getGroupEnd(groupLevel, lineStart);

            SortColumn column = (SortColumn) sortColumns.get(groupLevel - 1);
            int colNumber = column.getColumnNumber();
            SortDirection direction = column.getSortDirection();
            sortInInterval(colNumber, lineStart, lineEnd, direction);
            lineStart = lineEnd;
         }
      }
   }
}
