package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;

//this class is based on src/edu/bu/signstream/ui/panels/newDatabase/VideoPanelScrolledView
public class GraphPanelScrolledView extends JPanel implements ItemListener {

	private static final long serialVersionUID = 1L;

	private ArrayList<GraphFileRow> graphPanels = new ArrayList<GraphFileRow>();

	private GraphFileRow graphPanel1 = null;
	private GraphFileRow graphPanel2 = null;
	private GraphFileRow graphPanel3 = null;
	private GraphFileRow graphPanel4 = null;

	private JScrollPane scroller = null;
	private GraphFilesPane graphFilesPane;

	private JLabel fileSelectLabel = new JLabel(" Graph File Select ");
	private JLabel graphNameLabel = new JLabel(" Graph Name ");

	public GraphPanelScrolledView(ArrayList<LoadedMediaFile> mFiles, Dimension d,
			JFrame parentFrame) {
		super(new BorderLayout());

		UserInterfaceUtil.setLabelLNF(fileSelectLabel);
		UserInterfaceUtil.setLabelLNF(graphNameLabel);

		if (mFiles.size() >= 1) {
			graphPanel1 = new GraphFileRow(mFiles.get(0), parentFrame);
		} else {
			graphPanel1 = new GraphFileRow(null, parentFrame);
		}
		graphPanels.add(graphPanel1);

		if (mFiles.size() >= 2) {
			graphPanel2 = new GraphFileRow(mFiles.get(1), parentFrame);
		} else {
			graphPanel2 = new GraphFileRow(null, parentFrame);
		}
		graphPanels.add(graphPanel2);

		if (mFiles.size() >= 3) {
			graphPanel3 = new GraphFileRow(mFiles.get(2), parentFrame);
		} else {
			graphPanel3 = new GraphFileRow(null, parentFrame);
		}
		graphPanels.add(graphPanel3);

		if (mFiles.size() >= 4) {
			graphPanel4 = new GraphFileRow(mFiles.get(3), parentFrame);
		} else {
			graphPanel4 = new GraphFileRow(null, parentFrame);
		}
		graphPanels.add(graphPanel4);

		graphFilesPane = new GraphFilesPane();
		graphFilesPane.setBackground(Color.white);

		scroller = new JScrollPane(graphFilesPane);
		scroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		scroller.setPreferredSize(d);

		add(scroller, BorderLayout.CENTER);
	}

	public void update(ArrayList<LoadedMediaFile> mFiles) {
		if (mFiles.size() >= 1) {
			graphPanel1.setMediaFile(mFiles.get(0));
		} else {
			graphPanel1.setMediaFile(null);
		}
		graphPanel1.repaint();

		if (mFiles.size() >= 2) {
			graphPanel2.setMediaFile(mFiles.get(1));
		} else {
			graphPanel2.setMediaFile(null);
		}
		graphPanel2.repaint();

		if (mFiles.size() >= 3) {
			graphPanel3.setMediaFile(mFiles.get(2));
		} else {
			graphPanel3.setMediaFile(null);
		}
		graphPanel3.repaint();

		if (mFiles.size() >= 4) {
			graphPanel4.setMediaFile(mFiles.get(3));
		} else {
			graphPanel4.setMediaFile(null);
		}
		graphPanel4.repaint();
	}

	public boolean isAtLeastOneValidGraphFileAvailable() {
		for (GraphFileRow vp : graphPanels) {
			LoadedMediaFile lFile = vp.getMediaFile();
			if (lFile != null && lFile.getMediaFile() != null && lFile.getMediaFile().getFile() != null
					&& lFile.getMediaFile().getFile().exists()) {
				return true;
			}
		}
		return false;
	}

	public ArrayList<GraphFileRow> getGraphRows() {
		return graphPanels;
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
	}

	/** The component inside the scroll pane. */
	public class GraphFilesPane extends JPanel {

		private static final long serialVersionUID = 1L;

		public GraphFilesPane() {
			JPanel graphPanel = new JPanel();
			GridBagConstraints c = new GridBagConstraints();
			graphPanel.setLayout(new GridBagLayout());
			graphPanel.setBackground(Color.WHITE);
			graphPanel.setForeground(Color.white);

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(0, 1, 0, 1);

			c.gridx = 1;
			c.gridwidth = 3;
			graphPanel.add(fileSelectLabel, c);
			c.gridx = 4;
			c.gridwidth = 1;
			graphPanel.add(graphNameLabel, c);

			addComponent(graphPanel1, c, 1, graphPanel);
			addComponent(graphPanel2, c, 2, graphPanel);
			addComponent(graphPanel3, c, 3, graphPanel);
			addComponent(graphPanel4, c, 4, graphPanel);

			setLayout(new FlowLayout(FlowLayout.LEFT));
			add(graphPanel);
		}
	}

	private void addComponent(GraphFileRow vp, GridBagConstraints c, int gridy, JPanel graphPanel) {
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 1;
		c.gridy = gridy;
		graphPanel.add(vp.getInfoBtn(), c);
		c.gridx = 2;
		graphPanel.add(vp.getSelectBtn(), c);
		c.gridx = 3;
		graphPanel.add(vp.getRemoveBtn(), c);
		c.gridx = 4;
		c.anchor = GridBagConstraints.WEST;
		graphPanel.add(vp.getTextField(), c);
	}
}
