package edu.bu.signstream.ui.panels.table.treetable.dependent;

import java.util.*;

public class FieldNode {

   /** Parent FieldNode of the receiver. */
   private FieldNode parent;

   /** Children of the receiver. */
   private ArrayList children = new ArrayList();

   private boolean display = false;
   private boolean data = false;
   private String fieldName = "";
   private String fieldLabel = "";
   private String fieldID = "";

   private int rowNumber = 0;

   public FieldNode() {}

   public void setRowNumber(int rowNumber) {
      this.rowNumber = rowNumber;
   }

   public int getRowNumber() {
      return rowNumber;
   }

   /**
    * Returns the parent of the receiver.
    */
   public FieldNode getParent() {
      return parent;
   }

   /**
    * Returns true if the receiver represents a leaf, that is it is
    * isn't a directory.
    */
   public boolean isLeaf() {
      return children.size() == 0;
   }

   /**
    * Loads the children, caching the results in the children
    * instance variable.
    */
   protected FieldNode[] getChildren() {
      FieldNode[] obj = new FieldNode[children.size()];
      for (int i = 0; i < children.size(); i++) {
         obj[i] = (FieldNode) children.get(i);
      }
      return obj;
   }

   public void addChild(FieldNode node) {
      children.add(node);
   }

   public void removeChild(FieldNode node) {
      children.remove(node);
   }

   /**
    * Returns true if the children have been loaded.
    */
   protected boolean loadedChildren() {
      return (children != null);
   }

   public boolean isData() {
      return data;
   }

   public boolean isDisplay() {
      return display;
   }

   public String getFieldID() {
      return fieldID;
   }

   public String getFieldLabel() {
      return fieldLabel;
   }

   public String getFieldName() {
      return fieldName;
   }

   public void setChildren(ArrayList list) {
      children = list;
   }

   public void setData(boolean b) {
      data = b;
   }

   public void setDisplay(boolean b) {
      display = b;
   }

   public void setChildrenDisplay(boolean b) {
      for (int i = 0; i < children.size(); i++) {
         FieldNode cNode = (FieldNode) children.get(i);
         cNode.setDisplay(b);
         cNode.setChildrenDisplay(b);
      }
   }

   public void setParentDisplay(boolean b) {
      if (parent != null)
         parent.setDisplay(b);
   }

   public void setParentDisplayRecursive(boolean b) {
      setParentDisplay(b);
      if (parent.getParent() != null)
         parent.setParentDisplayRecursive(b);
   }

   public void setFieldID(String string) {
      fieldID = string;
   }

   public void setFieldLabel(String string) {
      fieldLabel = string;
   }

   public void setFieldName(String string) {
      fieldName = string;
   }

   public void setParent(FieldNode node) {
      parent = node;
   }

   @Override
public String toString() {
      return fieldName;
   }
}
